/****************************************************************************

  Copyright (c) 2006, Dell Inc
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

  - Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
    
  - Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution. 

  - Neither the name of Dell Inc nor the names of its contributors
    may be used to endorse or promote products derived from this software 
    without specific prior written permission. 

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. 

*****************************************************************************/


/*
 * 
 */
#include <common.h>		/*must be first in file*/
#include <command.h>

#ifdef CONFIG_CMC_10G	/*only compile in for a CMC_10G build*/
/******************************************************************************
*
*	FILE: util_abstraction.c
*
*	This file contains functions that provide an abstraction for API's that
*	operate in the monitor and Linux environment.  The "util" top layer is 
*	common code between the monitor and Linux.   Example, a call to 
*	is_vendor_mode().  In the monitor we can directly read a memory mapped
*	location and test a bit.  Under Linux we have to make a driver call to 
*	access the memory mapped registers.  Another example, to erase a flash 
*	sector, the monitor driver and Linux MTD driver are very different 
*	interfaces, so API's in this file abstract the end result to keep the common
*	code common. 
*
******************************************************************************/


/******************************************************************************
*  Include files
******************************************************************************/
#include <util_defs.h>
#include <util_ret_codes.h>
#include <util_flashmap.h>


#include <nand.h>
#include <linux/mtd/mtd.h>
#include <linux/mtd/nand.h>
#include <linux/mtd/compat.h>


/******************************************************************************
*  Local Defines
******************************************************************************/

/******************************************************************************
*  Globals
******************************************************************************/

/******************************************************************************
*  Prototypes/Externs
******************************************************************************/
extern int abbrev_spec (char *str, flash_info_t ** pinfo, int *psf, int *psl);
extern nand_info_t nand_info[];

/******************************************************************************
*	List all structures defines
******************************************************************************/




/*****************************************************************************
*	FUNCTION: ABS_erase_nor()
*	DATE:	10-17-06
*	BUILD:	u-boot
*
*	Abstraction API to erase a sector, or range of sectors in NOR flash.  
*	For u-boot this is nothing more than an interface into existingu-boot 
*	API's.  Interface code ported from RACMON.
*
*	INPUT:	s_start - starting sector to eerase
*			s_end	- ending sector to erase
*			verbo_flag  - 0=silent, 1=msg indicating sector and elapsed time
*			verify_flag - 0=erase and exit, 1=erase and verify sector for F's.
*			msg - 
*****************************************************************************/
s32 ABS_erase_nor(s32 s_start, s32 s_end, u32 verbo_flag, u32 verify_flag, 
																	char *msg)
{
s32 rc = 0;
s32 s_current, ss, se, last_sect;	/*sector vars*/
u32	start_tstamp, last_tstamp;		/*time stamp vars*/
char tstamp_str[MAX_TSTAMP_STR_LEN];
flash_info_t *info;					/*flash information struct*/
char v_str[15];						
char s_str[SECTOR_STR_OPT_MAX_LEN];	/* "1:xxx-xxx" max + null*/

start_tstamp = ABS_get_timer(0);
last_tstamp = start_tstamp;

/*	Make sure the starting sector is less than the ending*/
if(s_start > s_end)
	{	
	sprintf(msg, "Starting sector %d cannot be greater than ending %d", 
											s_start, s_end);
	return(E_FLASH_START_GT_END);
	}

/*correct sentence structure*/
if(verify_flag)						
	strcpy(v_str, "with verify ");	/*11 chars*/
else
	strcpy(v_str,  "");

s_current = s_start;
while(s_current <= s_end)
	{
	if(verbo_flag)
		{
		last_tstamp = calc_tstamp_str(last_tstamp, tstamp_str);
		printf("Erase %sat 0x%x, sector %.2d [%d:%d], %s      \r",
								v_str, SECTOR_TO_PHYSADDR(s_current), 
								s_current, s_start, s_end, tstamp_str);
		}

	/*call u-boot API's*/
	sprintf(s_str, "1:%d-%d", s_current, s_current);
	rc = abbrev_spec (s_str, &info, &ss, &se);
	if(rc < 0 )
		{
		sprintf(msg, "abbrev_spec(), bank, sector, or end < start\n");
		return(rc);
		}

	if( (rc = flash_erase(info, ss, se)) != 0)
		{
		sprintf(msg, "ERROR: from flash_erase() RC=0x%x\n", rc);
		return(rc);
		}

#if 0 
	if(verify_flag)
		{
		if( (rc = SAL_comp_const((void*)SECTOR_TO_KSEG1ADDR(s_current), 0xffff, 
								FLASH_SECTOR_SIZE, WIDTH_HALFWORD, msg)) != 0)
			return(rc);
		}
#endif

	++s_current;
	}/*end while*/


if(verbo_flag)
	{
	calc_tstamp_str(start_tstamp, tstamp_str);
	last_sect = 
		(s_end < CFG_FLASH_1ST_8K_SECT)?(CFG_SECT_SIZE-1):(CFG_SECT_SIZE_8K-1);
	printf("Erased %s 0x%x-to-0x%x, sectors [%d:%d], %s      \n",
									v_str, SECTOR_TO_PHYSADDR(s_start), 		
									(SECTOR_TO_PHYSADDR(s_end) + last_sect),		
									s_start, s_end, tstamp_str);
	}

return(0);
}/*end FUNCTION: ABS_erase_nor() */


/*****************************************************************************
*	FUNCTION: ABS_flash_write_nor()
*	DATE:	10-18-06
*	BUILD:	u-boot
*
*	Abstraction API to copy data from memory -> to NOR flash.  
*	For u-boot this is nothing more than an interface into existing u-boot 
*	API's.  Interface code ported from RACMON.
*
*	INPUT:	src - memory address
*			addr- NOT flash address
*			cnt - byte count
*			msg - 
*
*	RETURN:	0 - write passed
*			non-0 write failed
*
*****************************************************************************/
s32 ABS_flash_write_nor(char *mem_addr, u32 flash_addr, u32 cnt, char *msg)
{
s32 rc;			
char tmp[40];

sprintf(msg, "mem addr=0x%x flash addr=0x%x bcnt=0x%x  ",
												mem_addr, flash_addr, cnt);

rc = flash_write (mem_addr, flash_addr, cnt);
switch(rc)
	{
	case ERR_OK:	/*nothing to do*/
		break;
	case ERR_TIMOUT:
		strcat(msg, "timeout");
		break;
	case ERR_NOT_ERASED:
		strcat(msg, "not erased");
		break;
	case ERR_PROTECTED:
		strcat(msg, "protected sectors");
		break;
	case ERR_INVAL:
		strcat(msg, "address not in flash");
		break;
	case ERR_ALIGN:
		strcat(msg, "address not aligned");
		break;
	default:
		sprintf(tmp, "flash_write() RC=0x%x", rc);
		strcat(msg, tmp);
		break;
	}

return(rc);
}/*end FUNCTION: ABS_flash_write_nor()*/


/*****************************************************************************
*	FUNCTION: ABS_erase_nand()
*	DATE:	12-20-06
*	BUILD:	u-boot
*
*	Abstraction API to erase a block, or range of blocks in NAND flash.  
*	For u-boot this is nothing more than an interface into existing u-boot 
*	API's.  Interface code ported from RACMON.
*
*	INPUT:	b_start - starting block to erase
*			b_end	- ending block to erase
*			verbo_flag  - 0=silent, 1=msg indicating block and elapsed time
*			verify_flag - 0=erase and exit, 1=erase and verify block for F's.
*			msg - 
*****************************************************************************/
s32 ABS_erase_nand(s32 b_start, s32 b_end, u32 verbo_flag, u32 verify_flag, 
																	char *msg)
{
s32 rc = 0;
s32 b_current;						/*block vars*/
u32	start_tstamp, last_tstamp;		/*time stamp vars*/
char tstamp_str[MAX_TSTAMP_STR_LEN];

/*global structs containing nand info, cmc will always have 1 NAND*/
nand_info_t *nand	= &nand_info[0];		
nand_erase_options_t opts;

char v_str[15];						

start_tstamp = ABS_get_timer(0);
last_tstamp = start_tstamp;

/*	Make sure the starting sector is less than the ending*/
if(b_start > b_end)
	{	
	sprintf(msg, "Starting block %d cannot be greater than ending %d", 
											b_start, b_end);
	return(E_FLASH_START_GT_END);
	}

/*correct sentence structure*/
if(verify_flag)						
	strcpy(v_str, "with verify ");	/*11 chars*/
else
	strcpy(v_str,  "");

b_current = b_start;
memset(&opts, 0, sizeof(opts));
opts.length = CFG_NAND_BLOCK_SIZE;
opts.jffs2  = 0;
opts.quiet  = 1;

while(b_current <= b_end)
	{
	if(verbo_flag)
		{
		last_tstamp = calc_tstamp_str(last_tstamp, tstamp_str);
/* HACK  The nand erase code messes up "timestamp . interrupt.c !!!!
		printf("Erase %sat 0x%x, block %.3d [%d:%d], %s             \r",
									v_str, BLOCK_TO_PHYSADDR(b_current), 
									b_current, b_start, b_end, tstamp_str);
*/
		printf("Erase %sat 0x%x, block %.3d [%d:%d]                 \r",
									v_str, BLOCK_TO_PHYSADDR(b_current), 
									b_current, b_start, b_end);
		}

	/*call u-boot API's*/
	opts.offset = BLOCK_TO_PHYSADDR(b_current) & 0x0fffffff;
	rc = nand_erase_opts(nand, &opts);
	if(rc)
		printf("ERROR: from nand_erase_opts() RC=0x%x\n", rc);


#if 0  /*HACK different for nand*/
	if(verify_flag)
		{
		if( (rc = SAL_comp_const((void*)SECTOR_TO_KSEG1ADDR(s_current), 0xffff, 
								FLASH_SECTOR_SIZE, WIDTH_HALFWORD, msg)) != 0)
			return(rc);
		}
#endif

	++b_current;
	}/*end while*/


if(verbo_flag)
	{
	calc_tstamp_str(start_tstamp, tstamp_str);
/*
	printf("Erased %s 0x%x-to-0x%x, blocks [%d:%d], %s              \n",
							v_str, BLOCK_TO_PHYSADDR(b_start), 
							BLOCK_TO_PHYSADDR(b_end) + CFG_NAND_BLOCK_SIZE -1, 
							b_start, b_end, tstamp_str);
*/
	printf("Erased %s 0x%x-to-0x%x, blocks [%d:%d]                  \n",
							v_str, BLOCK_TO_PHYSADDR(b_start), 
							BLOCK_TO_PHYSADDR(b_end) + CFG_NAND_BLOCK_SIZE -1, 
							b_start, b_end);
	}

return(0);
}/*end FUNCTION: ABS_erase_nand() */


/*****************************************************************************
*	FUNCTION: ABS_write_nand()
*	DATE:	12-20-06
*	BUILD:	u-boot
*
*	Abstraction API to write data from a memory location-to- nand flash.
*	For u-boot this is nothing more than an interface into existing u-boot 
*	API's.  
*
*	INPUT:	b_start - starting block to erase
*			b_end	- ending block to erase
*			verbo_flag  - 0=silent, 1=msg indicating block and elapsed time
*			verify_flag - 0=erase and exit, 1=erase and verify block for F's.
*			msg - 
*****************************************************************************/
s32 ABS_write_nand(u32 mem_p, s32 nand_off, u32 bcnt, char *msg)
{
s32 rc = 0;
ulong size = bcnt;

/*global structs containing nand info, cmc will always have 1 NAND*/
nand_info_t *nand	= &nand_info[0];		

/*	Make sure the nand offset is page aligned*/
if(nand_off % CFG_NAND_PAGE_SIZE)
	{	
	sprintf(msg, "Starting nand addr 0x%x is NOT page aligned", nand_off);
	return(E_FLASH_NAND_OFF_NOT_PAGED_ALIGNED);
	}

/*	Make sure the data will fit*/
if( (nand_off + bcnt) > CFG_FLASH_NAND_SIZE )
	{	
	sprintf(msg, 
		"Transfer size of 0x%x will not fit in nand starting at 0x%x",
															bcnt, nand_off);
	return(E_FLASH_NAND_WRITE_BCNT_WONT_FIT);
	}

/*	Write to nand*/
if( (rc = nand_write(nand, nand_off, &size, (u_char *)mem_p)) != 0)
	{
	sprintf(msg, "ERROR: nand write failure RC=0x%x\n", rc);
	return(rc);
	}

if(size != bcnt)
	{
	sprintf(msg, "ERROR: expected to write 0x%x bytes, actual 0x%x", bcnt, size);
	return(E_FLASH_NAND_EXP_BCNT_NOT_WRITTEN);
	}

return(0);
}/*end FUNCTION: ABS_write_nand() */


/*****************************************************************************
*	FUNCTION: ABS_read_nand()
*	DATE:	12-20-06
*	BUILD:	u-boot
*
*	Abstraction API to read data from nand flash to memory.  For u-boot this 
*	is nothing more than an interface into existing u-boot API's.  
*	We will read a block at a time and check for errors so we can verify the 
*	the offset against the bad block list. The caller will get a contigious 
*	array of read data.  The caller will know if bad blocks were encounterd as
*	the requested number of read bytes will not match the retured actual byte
*	count.
*
*	INPUT:	mem_p - memory location to store the read data.
*			nand-off - nand offset address to start reading from 
*			req_bcnt - requested number of bytes to read
*			msg - 
*	RETURN: act_bcnt -actual byte count read
*			0 - read passed
*			non-zero - failure reading nand
*****************************************************************************/
s32 ABS_read_nand(u32 mem_p, s32 nand_off, u32 req_bcnt, u32 *act_bcnt, char *msg)
{
s32 rc = 0;
ulong xfer_size, saved_size;

*act_bcnt = 0;

/*global structs containing nand info, cmc will always have just 1 NAND*/
nand_info_t *nand	= &nand_info[0];		

/*Block align 1st read, following reads will be on block boundaries*/
xfer_size = CFG_NAND_BLOCK_SIZE - (nand_off & CFG_NAND_BLOCK_SIZE_MASK);
if(xfer_size >= req_bcnt)
	xfer_size = req_bcnt;		/*the read will not cross block boundries*/

do
	{
	saved_size = xfer_size;	/*nand_read may modify size parameter*/
#if 0
	printf("Read offset 0x%x  xfer_size=0x%x mem_p=0x%x        \r", 
								nand_off, xfer_size, mem_p);
#endif
	if( (rc = nand_read(nand, nand_off, &xfer_size, (u_char *)mem_p)) != 0)
		{
		if( is_on_bad_blocklist(nand_off) )
			{
			printf("Block %d at 0x%x is on the bad list                  \n", 
										OFFSET_TO_BLOCK(nand_off), nand_off);
			}
		else
			{
			sprintf(msg, "ERROR: nand read failure, offset 0x%x RC=0x%x\n", 
																nand_off, rc);
			return(rc);
			}
		/*do not bump the mem_p, or actual bytes read*/
		}
	else
		{
		if(xfer_size != saved_size)
			{
			sprintf(msg, "ERROR: expected to read 0x%x bytes, actual 0x%x", 
														xfer_size, saved_size);
			return(E_FLASH_NAND_EXP_BCNT_NOT_READ);
			}

		mem_p	 += saved_size;
		*act_bcnt = *act_bcnt + saved_size;
		}

	req_bcnt	-= saved_size;
	nand_off	+= saved_size;
	
	if(req_bcnt > CFG_NAND_BLOCK_SIZE)
		xfer_size = CFG_NAND_BLOCK_SIZE;
	else
		xfer_size = req_bcnt;
	}
while(req_bcnt);

return(0);
}/*end FUNCTION: ABS_read_nand() */

/*****************************************************************************
*	FUNCTION: ABS_get_timer()
*	DATE:	10-17-06
*	BUILD:	u-boot
*
*	Abstraction API to get time in milli seconds. 
*
*	INPUT:	base -	Passing in a 0 will generate a string from the beginning
*					of u-boot time.  A non-0 value will generate a string of
*					elapsed time with respect of a previously saved "base".
*
*****************************************************************************/
u32 ABS_get_timer(u32 base)
{
return(get_timer(base));
}/*end FUNCTION: ABS_get_timer()*/


/*****************************************************************************
*	FUNCTION: ABS_read_fpga()
*	DATE:	2-1-07
*	BUILD:	u-boot
*
*	Abstraction API to read a 16 bit FPGA register. 
*
*	INPUT:	reg - Full register address within the FPGA. (base+offset)
*	RETURN:	16 bit read value
*****************************************************************************/
u16 ABS_read_fpga(u32 reg)
{
return( *((u16*)reg) );
}/*end FUNCTION: ABS_read_fpga()*/


/*****************************************************************************
*	FUNCTION: ABS_write_fpga()
*	DATE:	2-1-07
*	BUILD:	u-boot
*
*	Abstraction API to write a 16 bit FPGA register. 
*
*	INPUT:	reg  - Full register address within the FPGA. (base+offset)
*			data - 16 bit data value to write
*	RETURN:	16 bit read value
*****************************************************************************/
u16 ABS_write_fpga(u32 reg, u16 data)
{
return( *((u16*)reg) = data );
}/*end FUNCTION: ABS_write_fpga()*/



#endif /*CONFIG_CMC_10G*/



