/****************************************************************************

  Copyright (c) 2006, Dell Inc
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

  - Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
    
  - Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution. 

  - Neither the name of Dell Inc nor the names of its contributors
    may be used to endorse or promote products derived from this software 
    without specific prior written permission. 

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. 

*****************************************************************************/


/*
 * Utility support
 */
#include <common.h>		/*must be first in file*/
#include <command.h>
#include <asm/processor.h>
#include <ppc4xx.h>
#include <miiphy.h>

#ifdef CONFIG_CMC_10G	/*only compile in for a CMC_10G build*/

/******************************************************************************
*
*	FILE: util_chas.c
*	This command supports options to allow user level chassis component 
*	manipulation.
*
******************************************************************************/


/******************************************************************************
*  Include files
******************************************************************************/
#include <util_defs.h>
#include <util_ret_codes.h>
#include <util_fpga.h>


/******************************************************************************
*  Local Defines
******************************************************************************/
#define BLADE_ACTION_ON				0x00000001
#define BLADE_ACTION_OFF			0x00000002
#define BLADE_ACTION_TRI			0x00000004
#define BLADE_ACTION_STATUS			0x00000008
#define BLADE_ACTION_IMC_RST		0x00000010
#define BLADE_ACTION_IMC_RST_N		0x00000020
#define BLADE_ACTION_BLD_RST		0x00000040
#define BLADE_ACTION_BLD_RST_N		0x00000080
#define BLADE_ACTION_THROTTLE		0x00000100
#define BLADE_ACTION_THROTTLE_N		0x00000200
#define BLADE_ACTION_BLD_PWRDN		0x00000400
#define BLADE_ACTION_BLD_PWRDN_N	0x00000800
#define BLADE_ACTION_NTWRK_ISO		0x00001000
#define BLADE_ACTION_NTWRK_ISO_N	0x00002000
#define BLADE_ACTION_POST_STATUS	0x00004000


#define PSU_ACTION_ON				0x00000001
#define PSU_ACTION_OFF				0x00000002
#define PSU_ACTION_STATUS			0x00000004
#define PSU_ACTION_POST_STATUS		0x00000008

#define LKVM_ACTION_ON				0x00000001
#define LKVM_ACTION_OFF				0x00000002
#define LKVM_ACTION_TRI				0x00000004
#define LKVM_ACTION_STATUS			0x00000008
#define LKVM_ACTION_POST_STATUS		0x00000010

#define IOM_ACTION_STATUS			0x00000001
#define IOM_ACTION_POST_STATUS		0x00000002

#define FAN_ACTION_OFF				0x00000001
#define FAN_ACTION_LOW				0x00000002
#define FAN_ACTION_MED				0x00000004
#define FAN_ACTION_HIGH				0x00000008
#define FAN_ACTION_STATUS			0x00000010
#define FAN_ACTION_POST_STATUS		0x00000020
#define FAN_ACTION_STBY				0x00000040
#define FAN_ACTION_STBY_MISSING		0x00000080


#define MAX_FAN_TYPES				6
#define FAN_SPEEDS					6	/*off, low, med, high, stby, stby_missing*/
#define FAN_SPEED_IDX_OFF			0	/*index into canned_fan_speeds array*/
#define FAN_SPEED_IDX_LOW			1	
#define FAN_SPEED_IDX_MED			2	
#define FAN_SPEED_IDX_HIGH			3
#define FAN_SPEED_IDX_STBY			4	/*virtual standby for fans 7-9*/
#define FAN_SPEED_IDX_STBY_MISSING	5	/*all fans if 7|8|9 missing*/

#define FAN_TYPE_BLANK_INSTALLED	0	/*no fan, blank cover is installed*/
#define FAN_TYPE_NOT_INSTALLED		5	/*no fan, no blank cover is installed*/

	
#define MAX_SLOT_NUMS	16
#define END_MARKER		-1

#define MAX_BLADES	16
#define MAX_PSUS	6
#define MAX_FANS	9
#define MAX_LKVMS	1
#define MAX_IOMS	6

/******************************************************************************
*  Globals
******************************************************************************/


/******************************************************************************
*  Prototypes
******************************************************************************/
extern s32	mcoptind;
extern char	*mcoptarg[];

static s32 blade_action(u32 slot_list[], u32 action, u32 verbose, char *msg);
static s32 psu_action(u32 slot_list[], u32 action, u32 verbose, char *msg);
static s32 fan_action(u32 slot_list[], u32 action, u32 verbose, char *msg);
static s32 lkvm_action(u32 action, u32 verbose, char *msg);
static s32 iom_action(u32 slot_list[], u32 action, u32 verbose, char *msg);
static s32 find_action(char *comp_str, char *num_str, char *action_str, 
									u32 slot_list[], u32 *action, char *msg);
static s32 psus_powered_up(u32 *present, u32 *psus_on, char *msg);
static s32 reset_i2c_interfaces(void);



/******************************************************************************
*	List all structures defines
******************************************************************************/
t_options util_chas_options[] = 
	{
	/*Options*/
	{"blade",	"Blade operation."},
	{"psu",		"Power Supply operation."},
	{"fan",		"Fan speed operation."},
	{"lkvm",	"LKVM operation."},
	{"iom",		"IOM status"},
	{"status",	"Display chassis status."},
	{"help",	"Display available actions."},
	{"verbose", "Display pre/post register values."},
	{"active",  "Set CMC active, hold foreign in reset, enable CMC_BUF_EN"},
	{"passive", "Set CMC passive, release foreign reset, disable CMC_BUF_EN"},
	{ NULL, NULL }
	};


t_usage util_chas_usage = { 
	"chas", 
					
	/*Syntax*/
	"chas   [-help]\n"
	"       [-active | -passive]\n"
	"       [-status]\n"
	"       [-blade <blade # | \"all\"> <action>] [-verbose]\n"
	"       [-psu <psu # | \"all\"> <action>] [-verbose]\n"
	"       [-lkvm <action>] [-verbose]\n"
	"       [-iom <iom # | \"all\"> <action>] [-verbose]\n"
	"       [-fan <fan # | \"all\"> <action>] [-verbose]\n",

	/*Description*/
	"This command allows user control on chassis components.\n"
	"Examples:\n"
	"util chas -active\n"
	"util chas -blade 2 off\n"
	"util chas -fan 9 low\n"
	"util chas -help\n",
	
	util_chas_options
	}; /*end t_usage util_chas_usage*/


typedef struct 
	{
	char *name;
	u32	 action;
	} t_component_actions;

typedef struct 
	{
	char *name;					/*component name, blade, psu, etc*/
	u32	 max_comp;				/*max number of blades, psu, etc*/
	t_component_actions *comp;	/*struct of supported component actions*/
	} t_chassis_actions;



t_component_actions  blade_actlist[] =	{
	{ "on",				BLADE_ACTION_ON				},
	{ "off",			BLADE_ACTION_OFF			},
	{ "tri",			BLADE_ACTION_TRI			},
	{ "status",			BLADE_ACTION_STATUS			},
	{ "imc_rst",		BLADE_ACTION_IMC_RST		},
	{ "imc_rst_n",		BLADE_ACTION_IMC_RST_N		},
	{ "bld_rst",		BLADE_ACTION_BLD_RST		},
	{ "bld_rst_n",		BLADE_ACTION_BLD_RST_N		},
	{ "throttle",		BLADE_ACTION_THROTTLE		},
	{ "throttle_n",		BLADE_ACTION_THROTTLE_N		},
	{ "pwrdn",			BLADE_ACTION_BLD_PWRDN		},
	{ "pwrdn_n",		BLADE_ACTION_BLD_PWRDN_N	},
	{ "ntwrk_iso",		BLADE_ACTION_NTWRK_ISO		},
	{ "ntwrk_iso_n",	BLADE_ACTION_NTWRK_ISO_N	},
	{ 0, 0 } };

t_component_actions  psu_actlist[] =	{
	{ "on",				PSU_ACTION_ON		},
	{ "off",			PSU_ACTION_OFF		},
	{ "status",			PSU_ACTION_STATUS	},
	{ 0, 0 } };

t_component_actions  lkvm_actlist[] =	{
	{ "on",				LKVM_ACTION_ON		},
	{ "off",			LKVM_ACTION_OFF		},
	{ "tri",			LKVM_ACTION_TRI		},
	{ "status",			LKVM_ACTION_STATUS	},
	{ 0, 0 } };

t_component_actions  iom_actlist[] =	{
	{ "status",			IOM_ACTION_STATUS	},
	{ 0, 0 } };

t_component_actions  fan_actlist[] =	{
	{ "off",			FAN_ACTION_OFF			},
	{ "low",			FAN_ACTION_LOW			},
	{ "med",			FAN_ACTION_MED			},
	{ "high",			FAN_ACTION_HIGH			},
	{ "status",			FAN_ACTION_STATUS		},
	{ "stby",			FAN_ACTION_STBY			},
	{ "stby_missing",	FAN_ACTION_STBY_MISSING	},
	{ 0, 0 } };

t_chassis_actions  chas_actlist[] = {
	{ "blade",	MAX_BLADES,		blade_actlist	},
	{ "psu",	MAX_PSUS,		psu_actlist		},
	{ "fan",	MAX_FANS,		fan_actlist		},
	{ "lkvm",	MAX_LKVMS,		lkvm_actlist	},
	{ "iom",	MAX_IOMS,		iom_actlist		},
	{ 0, 0, 0 } };


/*
*	Default fan PWM values for:
*	1)off, 2)low, 3)med, 4)high, 5)[7:9] stby, 6)[1:9] stby missing fan
*	For type 0, 5 (no fan) we still need to set a PWM values (especially 
*	standby).  Then if a fan is installed later it will not scream high, 
*	but run at the same standby speed as the remaining ones(or user setting).
*/
u32	canned_fan_speeds[MAX_FAN_TYPES][FAN_SPEEDS] = {
		{ 0x19, 0x64, 0x80, 0xff, 0x60, 0x7f },	    /*TYPE 0 fan blank installed*/
		{ 0x19, 0x64, 0x80, 0xff, 0x60, 0x7f },		/*TYPE 1*/
		{ 0x19, 0x64, 0x80, 0xff, 0x60, 0x7f },		/*TYPE 2*/
		{ 0x19, 0x64, 0x80, 0xff, 0x60, 0x7f },		/*TYPE 3*/
		{ 0x19, 0x64, 0x80, 0xff, 0x60, 0x7f },		/*TYPE 4*/
		{ 0x19, 0x64, 0x80, 0xff, 0x60, 0x7f } };	/*TYPE 5 no fan, no blank*/
	 

static u32 blade_in_addrs[MAX_BLADES] =		
	{	FPGA_REG_BLADE1_GPI,	FPGA_REG_BLADE2_GPI,	FPGA_REG_BLADE3_GPI,
		FPGA_REG_BLADE4_GPI,	FPGA_REG_BLADE5_GPI,	FPGA_REG_BLADE6_GPI,
		FPGA_REG_BLADE7_GPI,	FPGA_REG_BLADE8_GPI,	FPGA_REG_BLADE9_GPI,	
		FPGA_REG_BLADE10_GPI,	FPGA_REG_BLADE11_GPI,	FPGA_REG_BLADE12_GPI,		
		FPGA_REG_BLADE13_GPI,	FPGA_REG_BLADE14_GPI,	FPGA_REG_BLADE15_GPI,	
		FPGA_REG_BLADE16_GPI };

static u32 blade_out_addrs[MAX_BLADES] =			
	{	FPGA_REG_BLADE1_GPO	,	FPGA_REG_BLADE2_GPO,	FPGA_REG_BLADE3_GPO,
		FPGA_REG_BLADE4_GPO,	FPGA_REG_BLADE5_GPO,	FPGA_REG_BLADE6_GPO,
		FPGA_REG_BLADE7_GPO,	FPGA_REG_BLADE8_GPO,	FPGA_REG_BLADE9_GPO,
		FPGA_REG_BLADE10_GPO,	FPGA_REG_BLADE11_GPO,	FPGA_REG_BLADE12_GPO,
        FPGA_REG_BLADE13_GPO,	FPGA_REG_BLADE14_GPO,	FPGA_REG_BLADE15_GPO,
		FPGA_REG_BLADE16_GPO };


static u32 psu_in_addrs[MAX_PSUS] =		
	{	FPGA_REG_GPI_PSU1_IN,	FPGA_REG_GPI_PSU2_IN,	FPGA_REG_GPI_PSU3_IN,
		FPGA_REG_GPI_PSU4_IN,	FPGA_REG_GPI_PSU5_IN,	FPGA_REG_GPI_PSU6_IN };


static u32 fan_pwm_ctrl_addrs[MAX_FANS] =		
	{	FPGA_REG_FAN1_PWM_CTRL,	FPGA_REG_FAN2_PWM_CTRL,	FPGA_REG_FAN3_PWM_CTRL,
		FPGA_REG_FAN4_PWM_CTRL,	FPGA_REG_FAN5_PWM_CTRL,	FPGA_REG_FAN6_PWM_CTRL,
		FPGA_REG_FAN7_PWM_CTRL, FPGA_REG_FAN8_PWM_CTRL, FPGA_REG_FAN9_PWM_CTRL };

static u32 fan_type_addrs[MAX_FANS] =		
	{	FPGA_REG_FAN1_TYPE,	FPGA_REG_FAN2_TYPE,	FPGA_REG_FAN3_TYPE,
		FPGA_REG_FAN4_TYPE,	FPGA_REG_FAN5_TYPE,	FPGA_REG_FAN6_TYPE,
		FPGA_REG_FAN7_TYPE, FPGA_REG_FAN8_TYPE, FPGA_REG_FAN9_TYPE };

static u32 fan_tac1_addrs[MAX_FANS] = 
	{	FPGA_REG_FAN1_TACH1, FPGA_REG_FAN2_TACH1, FPGA_REG_FAN3_TACH1,
		FPGA_REG_FAN4_TACH1, FPGA_REG_FAN5_TACH1, FPGA_REG_FAN6_TACH1,
		FPGA_REG_FAN7_TACH1, FPGA_REG_FAN8_TACH1, FPGA_REG_FAN9_TACH1 };

static u32 fan_tac2_addrs[MAX_FANS] = 
	{	FPGA_REG_FAN1_TACH2, FPGA_REG_FAN2_TACH2, FPGA_REG_FAN3_TACH2,
		FPGA_REG_FAN4_TACH2, FPGA_REG_FAN5_TACH2, FPGA_REG_FAN6_TACH2,
		FPGA_REG_FAN7_TACH2, FPGA_REG_FAN8_TACH2, FPGA_REG_FAN9_TACH2 };

static u32 fan_state_addrs[MAX_FANS] = 
	{	FPGA_REG_FAN1_STATE, FPGA_REG_FAN2_STATE, FPGA_REG_FAN3_STATE,
		FPGA_REG_FAN4_STATE, FPGA_REG_FAN5_STATE, FPGA_REG_FAN6_STATE,
		FPGA_REG_FAN7_STATE, FPGA_REG_FAN8_STATE, FPGA_REG_FAN9_STATE };


/******************************************************************************
*
*	FUNCTION: util_chas()
*	DATE:	12-11-06
*
******************************************************************************/
s32 util_chas(int argc, char *argv[])
{
u32  i, j;
u32  rc				= 0;
u32  index			= 0;;	
u32	 action			= 0;			/*action to perform*/
u32	 blade_flag		= 0;			/*-blade option*/
u32  psu_flag		= 0;			/*-psu option*/
u32  fan_flag		= 0;			/*-fan option*/
u32  lkvm_flag		= 0;			/*-lkvm option*/
u32  iom_flag		= 0;			/*-iom option*/
u32	 verbose		= VERBOSE_OFF;
u32	 slot_list[MAX_SLOT_NUMS + 1];	/*blade number, ps number, fan, etc*/
char msg[MAX_MSG_LEN];				/*message buffer*/
char *name;
int addr;


//	Get any options
mcoptind = 2;	
while( (index = mcgetopt(argc, argv, 
	  "-help -blade:2 -psu:2 -fan:2 -verbose -active -passive "
	  "-lkvm:1 -status -iom:2")) != MCGETOPT_EOF)
	{
	switch(index) 
		{
		case 0 :	/*help*/
			printf("Chassis actions:\n");
			for(i = 0; chas_actlist[i].name != 0; ++i)
				{
				for(j = 0; chas_actlist[i].comp[j].name != 0; ++j)
					printf("%s  \"%s\"\n", 
						chas_actlist[i].name, chas_actlist[i].comp[j].name);
				}
			break;

		case 1 :		/*blade operation*/
			blade_flag = 1;
			if( (rc = find_action("blade", mcoptarg[0], mcoptarg[1], 
											slot_list, &action, msg)) != 0)
				{
				printf("%s %s:ERROR:%s", argv[0], argv[1], msg);
				return(rc); 
				}
			break;

		case 2 :		/*psu*/
			psu_flag = 1;
			if( (rc = find_action("psu", mcoptarg[0], mcoptarg[1], 
											slot_list, &action, msg)) != 0)
				{
				printf("%s %s:ERROR:%s", argv[0], argv[1], msg);
				return(rc); 
				}
			break;

		case 3 :		/*fan*/
			fan_flag = 1;
			if( (rc = find_action("fan", mcoptarg[0], mcoptarg[1], 
											slot_list, &action, msg)) != 0)
				{
				printf("%s %s:ERROR:%s", argv[0], argv[1], msg);
				return(rc); 
				}
			break;

		case 4 :		/*	runtime verbose*/
			verbose = VERBOSE_ON;
			break;

		case 5 :		/* -active option*/
			set_cmcf_reset();
			printf(	"\nForeign CMC IS held in reset.\n"
					"This CMC is actively driving Mid-plane\n\n");
			set_local_active();
			set_cmc_buf_en();
			udelay(5000);
			reset_i2c_interfaces();		/*CR146327*/
			/* force PHYs to 100MB/FDX */
			set_micrel_en(1);
			name = miiphy_get_current_dev();
			for (addr = 0; addr <= 23; addr++)
				miiphy_write(name, addr, 0, 0x2100);
			break;

		case 6 :		/* -passive option*/
			release_cmcf_reset();
			printf(	"\nForeign CMC is NOT held in reset.\n"
					"This CMC is passive, NOT driving Mid-plane\n\n");
			clear_local_active();
			clear_cmc_buf_en();
			/* force PHYs */
			set_micrel_en(1);
			name = miiphy_get_current_dev();
			for (addr = 0; addr <= 23; addr++) {
				switch (addr) {
				/* blades */
				default:
					/* 10/HDX */
					miiphy_write(name, addr, 0, 0x0040);
					break;
				/* IOM, LKVM */
				case 0: case 3: case 6: case 10:
				case 13: case 18: case 21: case 22:
					/* 100/FDX */
					miiphy_write(name, addr, 0, 0x2100);
					break;
				}
			}
			break;

		case 7 :		/*lkvm operation*/
			lkvm_flag = 1;
			/*Only one LKVM, set a default number string to one*/
			if( (rc = find_action("lkvm", "1", mcoptarg[0], 
											slot_list, &action, msg)) != 0)
				{
				printf("%s %s:ERROR:%s", argv[0], argv[1], msg);
				return(rc); 
				}
			break;

		case 8 :		/*display chassis status*/
			post_cmc_status(msg);
			printf("\n\nCMC Status:  %s\n", msg);

			post_blade_status(msg);
			printf("Blade Status:%s\n", msg);
		
			post_psu_status(msg);
			printf("PSU Status:  %s\n", msg);

			post_fan_status(msg);
			printf("Fan Status:  %s\n", msg);

			post_iom_status(msg);
			printf("IOM Status:  %s\n", msg);
	
			post_lkvm_status(msg);
			printf("LKVM Status: %s\n", msg);
			break;

		case 9 :		/*iom*/
			iom_flag = 1;
			if( (rc = find_action("iom", mcoptarg[0], mcoptarg[1], 
											slot_list, &action, msg)) != 0)
				{
				printf("%s %s:ERROR:%s", argv[0], argv[1], msg);
				return(rc); 
				}
			break;

		default:		/*not good*/
			rc = mcgetopt_error(index, msg, mcoptarg[0]);
			printf("%s %s:%s\n", argv[0], argv[1], msg);
			return(rc);
			break;
		}
	}/*end mcgetopt while*/



if( (blade_flag && psu_flag)  ||
    (blade_flag && fan_flag)  ||
	(blade_flag && lkvm_flag) ||
	(blade_flag && iom_flag)  ||
	(fan_flag   && psu_flag)  ||
	(fan_flag   && lkvm_flag) ||
	(fan_flag   && iom_flag)  ||
	(psu_flag   && iom_flag)  ||
	(psu_flag   && lkvm_flag) )
	{
	printf("%s %s:ERROR:single component only.\n", argv[0], argv[1]);
	return(E_CHAS_TOO_MANY_OPTS);
	}


/*****************************************************************************
*	Blade option
*****************************************************************************/
if(blade_flag)
	{
	if( (rc = blade_action(slot_list, action, verbose, msg)) != 0)
		{
		printf("%s %s:%s\n", argv[0], argv[1], msg);
		return(rc);
		}
	}


/*****************************************************************************
*	PSU option
*****************************************************************************/
if(psu_flag)
	{
	if( (rc = psu_action(slot_list, action, verbose, msg)) != 0)
		{
		printf("%s %s:%s\n", argv[0], argv[1], msg);
		return(rc);
		}
	}

/*****************************************************************************
*	FAN option
*****************************************************************************/
if(fan_flag)
	{
	if( (rc = fan_action(slot_list, action, verbose, msg)) != 0)
		{
		printf("%s %s:%s\n", argv[0], argv[1], msg);
		return(rc);
		}
	}

/*****************************************************************************
*	LKVM option
*****************************************************************************/
if(lkvm_flag)
	{
	if( (rc = lkvm_action(action, verbose, msg)) != 0)
		{
		printf("%s %s:%s\n", argv[0], argv[1], msg);
		return(rc);
		}
	}

/*****************************************************************************
*	IOM option
*****************************************************************************/
if(iom_flag)
	{
	if( (rc = iom_action(slot_list, action, verbose, msg)) != 0)
		{
		printf("%s %s:%s\n", argv[0], argv[1], msg);
		return(rc);
		}
	}

return(0);
}/*end FUNCTION: util_reg()*/



/*****************************************************************************
*	FUNCTION: post_blade_status()
*	DATE:	2-12-07
*
*	Build an info line of blade status.
*****************************************************************************/
s32 post_blade_status(char *msg)
{
u32 i;
u32	slot_list[MAX_BLADES + 1];	

/*Get the current status for all blades*/
for(i = 0; i < MAX_BLADES; ++i)		
	slot_list[i] = i+1;
slot_list[i] = END_MARKER;

blade_action(slot_list, BLADE_ACTION_POST_STATUS, 0, msg);
return(LOG_INFO);
}/*end FUNCTION:post_blade_status() */

/*****************************************************************************
*	FUNCTION: post_psu_status()
*	DATE:	2-12-07
*
*	Build an info line of PSU status.
*****************************************************************************/
s32 post_psu_status(char *msg)
{
u32 i;
u32	slot_list[MAX_PSUS + 1];	

/*Get the current status for all PSU's*/
for(i = 0; i < MAX_PSUS; ++i)		
	slot_list[i] = i+1;
slot_list[i] = END_MARKER;

psu_action(slot_list, PSU_ACTION_POST_STATUS, 0, msg);
return(LOG_INFO);
}/*end FUNCTION:post_psu_status() */


/*****************************************************************************
*	FUNCTION: post_fan_status()
*	DATE:	2-12-07
*
*	Build an info line of PSU status.
*****************************************************************************/
s32 post_fan_status(char *msg)
{
u32 i;
u32	slot_list[MAX_FANS + 1];	

for(i = 0; i < MAX_FANS; ++i)		
	slot_list[i] = i+1;
slot_list[i] = END_MARKER;

fan_action(slot_list, FAN_ACTION_POST_STATUS, 0, msg);
return(LOG_INFO);
}/*end FUNCTION:post_fan_status() */

/*****************************************************************************
*	FUNCTION: post_cmc_status()
*	DATE:	2-12-07
*
*	Build an info line of CMC status.
*****************************************************************************/
s32 post_cmc_status(char *msg)
{
char *local_str;
char *vendor_str;
u16  slot = ABS_read_fpga(FPGA_REG_CMC_SLOT_ID) & FPGA_CMC_SLOT_ID;


if( ABS_read_fpga(FPGA_REG_ID) == FPGA_ID)
	{
	if(is_local_active())
		local_str = "active";
	else
		local_str = "passive";

	if( !(ABS_read_fpga(FPGA_REG_VENDOR_MODE_N) & FPGA_VENDOR_MODE_N) )
		vendor_str = "detected";
	else  
		vendor_str = "not detected";
	}
else
	{
	local_str = "unknown";
	vendor_str = "unknown";
	}


sprintf(msg, "CMC%d=%s    FPGA vendor mode=%s", 
								slot+1, local_str, vendor_str );

return(LOG_INFO);
}/*end FUNCTION:post_cmc_status() */

/*****************************************************************************
*	FUNCTION: post_lkvm_status()
*	DATE:	2-13-07
*
*	Build an info line for LKVM status.
*****************************************************************************/
s32 post_lkvm_status(char *msg)
{
lkvm_action(LKVM_ACTION_POST_STATUS, 0, msg);
return(LOG_INFO);
}/*end FUNCTION:post_lkvm_status() */

/*****************************************************************************
*	FUNCTION: post_iom_status()
*	DATE:	2-14-07
*
*	Build an info line of IOM status.
*****************************************************************************/
s32 post_iom_status(char *msg)
{
u32 i;
u32	slot_list[MAX_IOMS + 1];	

for(i = 0; i < MAX_IOMS; ++i)		
	slot_list[i] = i+1;
slot_list[i] = END_MARKER;

iom_action(slot_list, IOM_ACTION_POST_STATUS, 0, msg);
return(LOG_INFO);
}/*end FUNCTION:post_fan_status() */


/*****************************************************************************
*	FUNCTION: post_chas_setup()
*	DATE:	2-7-07
*
*	U-boot needs to enable at minimum a PS and three fans.
*****************************************************************************/
s32 post_chas_setup(char *msg)
{
s32  i;
u16  slot;
u32  psu_pres;
u32  psus_on;
u16  psu_out;
u32  verbose = 0;
char *psu_123num;
char *psu_456num;
char *env_p;
char tmp[MAX_MSG_LEN];		/*message buffer*/

/*	Run time verbose/debug flag*/
env_p = getenv("post_chas");
if(env_p)
	{
	if( strcmp(env_p, "verbose") )
		verbose = 0;
	else
		verbose = 1;
	}


/*	
*	If "any" PS's on, system has been previously powered. Let Linux worry
*	about active/passive.  If none are present, nothing we can do, get out.
*	U-Boot need to be as minimal as possible.
*/
psus_powered_up(&psu_pres, &psus_on, msg);

if(psu_pres == 0)
	{
	sprintf(msg, "No power supplies detected, no monitor negotiation.");
	return(I_CHAS_SETUP_NO_PSUS);
	}

if(psus_on)
	{
	sprintf(msg, "At least one PSU is powered up, no monitor negotiation.");
	return(I_CHAS_SETUP_PSU_ALREADY_ON);
	}
	

if(verbose)
	printf("post_chas_setup():psu_pres=0x%x psus_on=0x%x\n", 
														psu_pres, psus_on);
/*
*	Using the foreign -> local active line, determine who shoud drive the bus.
*	slot0=CMC1=Right     slot1=CMC2=Left
*/
slot =  ABS_read_fpga(FPGA_REG_CMC_SLOT_ID) & FPGA_CMC_SLOT_ID;
if(slot)
	{/* slot1=CMC2=Left */
	/*	If CMC1 is NOT present, set CMC2 local active, do not delay 4 seconds*/
	if( (*(u16*)FPGA_REG_CMCF_CMCL_PRES & FPGA_CMCF_CMCL_PRES) )
		{/*pres is low active*/
		set_local_active();
		}
	else
		{
		for(i = 0; i < 20 ; ++i)	/*20 *100msec = 2sec*/
			{
			if( is_foreign_active() )
				{
				sprintf(msg, "This CMC2 Passive, foreign CMC1 detected active");
				return(I_CHAS_SETUP_CMC2_PASSIVE);
				}
			udelay(100000);			/*delay 100milli seconds*/
			}
	
		set_local_active();
		for(i = 0; i < 20 ; ++i)	/*20 *100msec = 2sec*/
			{
			if(is_foreign_active() )
				{
				clear_local_active();
				sprintf(msg, 
					"This CMC2 Passive, foreign CMC1 detected active...2nd check");
				return(I_CHAS_SETUP_CMC2_PASSIVE_2);
				}
			udelay(100000);			/*delay 100milli seconds*/
			}
		}

	/* If we get here CMC2 is active*/
	set_cmc_buf_en();
	udelay(5000);
	reset_i2c_interfaces();		/*CR146327*/
	}
else
	{/* slot0=CMC1=Right */
	if(is_foreign_active() )
		{
		sprintf(msg, "This CMC1 Passive, foreign CMC2 detected active");
		return(I_CHAS_SETUP_CMC1_PASSIVE);
		}

	/* If we get here CMC1 will be active, drive the mid-plane*/
	set_local_active();
	set_cmc_buf_en();
	udelay(5000);
	reset_i2c_interfaces();		/*CR146327*/
	}


/*	Turn on a PSU in the 1-3 range, and one PSU in the 4-6 range*/
if(psu_pres)			/*should not get here if none are present*/
	{
	psu_out = 0xffff;	/*0=PSU on,  1=PSU off*/
	psu_123num = "";
	psu_456num = "";
	if(psu_pres & FPGA_PSU_OUT_1)	
		{
		psu_out &= ~FPGA_PSU_OUT_1;
		psu_123num = "1";
		}
	else if(psu_pres & FPGA_PSU_OUT_2)
		{
		psu_out &= ~FPGA_PSU_OUT_2;
		psu_123num = "2";
		}
	else if(psu_pres & FPGA_PSU_OUT_3)
		{
		psu_out &= ~FPGA_PSU_OUT_3;
		psu_123num = "3";
		}


	if(psu_pres & FPGA_PSU_OUT_4)
		{
		psu_out &= ~FPGA_PSU_OUT_4;
		psu_456num = "4";
		}
	else if(psu_pres & FPGA_PSU_OUT_5)
		{
		psu_out &= ~FPGA_PSU_OUT_5;
		psu_456num = "5";
		}
	else if(psu_pres & FPGA_PSU_OUT_6)
		{
		psu_out &= ~FPGA_PSU_OUT_6;
		psu_456num = "6";
		}

	*(u16*)FPGA_REG_PSU_OUT = psu_out;	/*turn on PSUs*/
	sprintf(msg, "CMC%d turn on PSU %s %s", slot + 1, psu_123num, psu_456num);
	}

if(verbose)
	printf("post_chas_setup(): %s\n", msg);

/*	
*	Do NOT check the rc of post_fan_setup(). IF we have a fan type that THIS 
*	u-boot does not support we will get a selftest failure and die. We do NOT
*	want this to happen.  Just try and turn on the necessary fans and let Linux
*	power/thermal do its thing.
*
*	DF158017	 The CPLD changed so that you have to be active BEFORE you can 
*	read the fan types.  It takes time for the CPLD to serial shift in the 
*	9 fan types so add a 15mill sec delay.  I padded it to 20m
*/
udelay(20000);
post_fan_setup(tmp, verbose);
 
strcat(msg, "\n             ");
strcat(msg, tmp);
return(0);
}/*end FUNCTION: post_chas_setup() */


/*****************************************************************************
*	FUNCTION: post_chas_setup_psubug()
*	DATE:	2-13-07
*
*	Early HW bugs cause all the PSU's to turn on as AC is applied.
*	Fans get rotor lock, etc.   In order for this funcion to get called the 
*	user has created a env variable "setenv post_chas psubug".  It is NOT
*	created by the FW, so no chance of being set by default.
*
*	CMC1 is active if installed.  CMC2 is actice if CMC1 is absent.
*	Turn on All PSU's
*	Turn on ALL fans to low.
*****************************************************************************/
s32 post_chas_setup_psubug(char *msg)
{
u32 rc;
u32 slot;
s32 fan_idx;
u32 fan_list[MAX_FANS+1];	/*+1 for end marker*/	
char tmp[MAX_MSG_LEN];		/*message buffer*/

/*	If CMC1 present it will be active*/
slot =  ABS_read_fpga(FPGA_REG_CMC_SLOT_ID) & FPGA_CMC_SLOT_ID;
if(slot)
	{/* slot1=CMC2=Left */
	if( !(*(u16*)FPGA_REG_CMCF_CMCL_PRES & FPGA_CMCF_CMCL_PRES) )
		{
		sprintf(msg, "post_chas=psubug: This CMC2 is passive");
		return(LOG_INFO);	/*CMC1 is present, ALWAYS allow CMC1 to be active*/
		}

	set_local_active();
	set_cmc_buf_en();
	udelay(5000);
	reset_i2c_interfaces();		/*CR146327*/
	sprintf(tmp, "post_chas=psubug: This CMC2 is active");
	}
else
	{/* slot0=CMC1=Right */
	set_local_active();
	set_cmc_buf_en();
	udelay(5000);
	reset_i2c_interfaces();		/*CR146327*/
	sprintf(tmp, "post_chas=psubug: This CMC1 is active");
	}

/*turn on all PSUs*/
*(u16*)FPGA_REG_PSU_OUT = ~FPGA_PSU_OUT_1_6_MASK;	

/*	Turn on All fans*/
for(fan_idx = 0; fan_idx < MAX_FANS; ++fan_idx)
	fan_list[fan_idx] = fan_idx + 1;	
fan_list[fan_idx] = END_MARKER;

if( (rc = fan_action(fan_list, FAN_ACTION_STBY, 0, msg)) != 0)
	return(rc);
	
/*	Turn on LKVM*/
if( (rc = lkvm_action(LKVM_ACTION_ON, 0, msg)) != 0)
	return(rc);

strcpy(msg, tmp);
strcat(msg, "\n             Set PSU slots 1-6 = ON, Set fan slots 1-9 = LOW");
return(LOG_INFO);
}/*end FUNCTION: post_chas_setup_psubug() */

/*****************************************************************************
*	FUNCTION: post_fan_setup()
*	DATE:	2-13-07
*
*	This function is call when the CMC POSTS, and no Power Supplies were
*	detected as powerd on.
*	If the bottom fans 7,8,9 are present, turn them on MED, and turn off
*	fans 1-6.
*	If any of the three (7,8,9) are missing, turn ALL fans on MED.	
*	
*****************************************************************************/
s32 post_fan_setup(char *msg, u32 verbose)
{
u32  i;
u32  rc;
u32  fan_idx;
u16  fan_type;
u32  fan_cnt;
u32 fan_list[MAX_FANS+1];	/*+1 for end marker*/	

/*	Test fans 7,8,9 for not installed*/
for(i = 7, fan_idx = 0, fan_cnt = 0; i < 10; ++i, ++fan_idx)
	{
	fan_type = *(u16*)fan_type_addrs[i-1];
	if( !((FAN_TYPE_BLANK_INSTALLED == fan_type) || 
		 (FAN_TYPE_NOT_INSTALLED   == fan_type)) )
		{
		fan_list[fan_idx] = i;	/*add bottom fan to TURN ON list*/
		 ++fan_cnt;
		}
	}
fan_list[fan_idx] = END_MARKER;

if(fan_cnt == 3)
	{/* Turn on fans 7-9*/
	if( (rc = fan_action(fan_list, FAN_ACTION_STBY, verbose, msg)) != 0)
		return(rc);

	/*	Build fan list 1-6*/
	for(fan_idx = 0; fan_idx < 6; ++fan_idx)	
		fan_list[fan_idx] = fan_idx + 1;	
	fan_list[fan_idx] = END_MARKER;

	/*	Turn off fans 1-6*/
	if( (rc = fan_action(fan_list, FAN_ACTION_OFF, verbose, msg)) != 0)
		return(rc);

	sprintf(msg, "Set fan slots 1-6 = OFF	Set fan slots 7-9 = LOW");
	}
else
	{/* set ALL fans on*/
	for(fan_idx = 0; fan_idx < MAX_FANS; ++fan_idx)
		fan_list[fan_idx] = fan_idx + 1;	
	fan_list[fan_idx] = END_MARKER;

	if( (rc = fan_action(fan_list, FAN_ACTION_STBY_MISSING, verbose, msg)) != 0)
		return(rc);
	sprintf(msg, "Set fan slots 1-9 = LOW");
	}
	
return(0);
}/*end FUNCTION: post_fan_setup()*/

/*****************************************************************************
*	FUNCTION: psus_powered_up
*	DATE:	2-7-07
*
*	Determins if ANY Power Supply Units are turned on.  The must be present
*	with AC and DC good indications.
*
*	INPUT:  present - bit mask of PSU present bit [5:0]= PSU 6,5,4,3,2,1
*			psus_on - bit mask of PSU's power on [5:0]= PSU 6,5,4,3,2,1
*			msg - 
*	RETURN:	PSU's powered/present [5:0]= PSU 6,5,4,3,2,1
*****************************************************************************/
static s32 psus_powered_up(u32 *present, u32 *psus_on, char *msg)
{
u16  rec;
s32  bit, i;

*present = 0;
*psus_on = 0;
for(i = 0, bit = 1; i < MAX_PSUS; ++i, bit <<= 1)
	{
	rec = *(u16*)psu_in_addrs[i];
	if( !(rec & FPGA_PSU_GPI_PRESENT) )
		{
		*present = *present | bit;
		if( (rec & FPGA_PSU_PGOOD_MASK) == FPGA_PSU_PGOOD_MASK )
			*psus_on = *psus_on | bit;
		}
	}/*end for*/

return(0);
}/*end FUNCTION: psus_powered_up() */


/*****************************************************************************
*	FUNCTION: post_passive_setup()
*	DATE:	4-30-07
*
*	DF125823
*	At RTS the passive CMC is NOT running as a hot backup/standby.  Module
*	managment is not running on the passive CMC.  So at u-boot time on the
*	passive CMC we need to enable "all" fans, IMC's, PSU's, and the vkvm.
*	(NOTE: documentation for the FPGA inplies blades. We cannot turn on 
*	blades, but it is the IMC that we are really turning on).
*
*	When we switch over and this passive CMC then becomes active, the module
*	managment will be enabled on the newly active CMC and turn off what is not
*	needed.   If these are not enabled the default settings will cause the 
*	IMC's, PSU's, and LKVM to all turn off.
*
*	blb 9-12-07
*	NOTE: FPGA 14 added a tristate register for each blade.  For the comment
*	above, we do NOT set the IMC's to be on, but tristate.
*	
*	If any of the action commands return an error nothing we can do anyway.
*	Just attempt to set all the values and get out.
*
*	INPUT:	msg - message buffer for info and error messaging
*	RETURN:	0
*	
*****************************************************************************/
s32 post_passive_setup(char *msg)
{
u32  i;
u32  rc;	
u32	slot_list[MAX_BLADES + 1];		/*+1 for end marker*/	
									/*Reuse the array, the largest component*/
									/* is for the blades*/


/*	Tristate all blades (this is really tristate all IMC's)*/
for(i = 0; i < MAX_BLADES; ++i)		
	slot_list[i] = i+1;
slot_list[i] = END_MARKER;

rc = blade_action(slot_list, BLADE_ACTION_TRI, 0, msg);


/*	Turn on all PSU's*/
for(i = 0; i < MAX_PSUS; ++i)		
	slot_list[i] = i+1;
slot_list[i] = END_MARKER;

rc = psu_action(slot_list, PSU_ACTION_ON, 0, msg);


/*	Enable on all fans*/
for(i = 0; i < MAX_FANS; ++i)		
	slot_list[i] = i+1;
slot_list[i] = END_MARKER;

rc = fan_action(slot_list, FAN_ACTION_HIGH, 0, msg);

/*	Enable lkvm*/
rc = lkvm_action(LKVM_ACTION_TRI, 0, msg);

return(0);
}/*end FUNCTION: post_passive_setup()*/



/*****************************************************************************
*	FUNCTION: find_action
*	DATE:	12-12-06
*
*	INPUT:	comp_str - The component string contains "blade", "psu", etc
*			slotnum_str - Contains the component number, or slot to operate 
*						  on.  The user can list several space seperated 
*						  numbers in one command, etc: 
*						  -blade "1 4 7" on   would turn on 3 blades at once.
*			action_str - action to perform to the target component.
*			num - slotnum_str converts to hex, then moved to this array.
*			action hex value of the action.
*
*****************************************************************************/
static s32 find_action(char *comp_str, char *slotnum_str, char *action_str, 
									u32 slot_list[], u32 *action, char *msg)
{
u32  i;
u32  j;
u32  act_index;
u32  more_nums;
char *p1;
char *cptr1;


/*	Make sure we have a valid component, we need the index later*/
for(i = 0, act_index = -1; chas_actlist[i].name != 0; ++i)
	{
	if( !strcmp(comp_str, chas_actlist[i].name) )
		{
		act_index = i;
		break;
		}
	}

if(act_index == -1)
	{
	sprintf(msg, "\"%s\" invalid component name.\n", comp_str);
	return(E_CHAS_FIND_INVALID_NAME);
	}

/*	Check for valid action, NOTE: only one action per cmd line*/
for(j = 0; chas_actlist[act_index].comp[j].name != 0; ++j)
	{
	if( !strcmp(chas_actlist[act_index].comp[j].name, action_str) )
		{
		*action = chas_actlist[act_index].comp[j].action;
		break;
		}
	}
			
if(chas_actlist[act_index].comp[j].name == 0)
	{
	sprintf(msg, "\"%s\" invalid action.\n", action_str);
	return(E_CHAS_FIND_INVALID_ACTION);
	}

/*	Convert the slot number string. One or more slot number possable*/
if(!strcmp(slotnum_str, "all") )
	{/* Set all slots*/
	for(i = 0; i < chas_actlist[act_index].max_comp; ++i)
		slot_list[i] = i + 1;
	}
else
	{/* Set selected slots*/
	for(i = 0, more_nums = 1; (more_nums)  && (i < MAX_SLOT_NUMS); ++i)
		{
		if( (p1 = strchr(slotnum_str, ' ')) == 0)
			{
			more_nums = 0;		/*last number*/
			p1 = (char *)((u32)p1 + strlen(slotnum_str));
			}
		else
			*p1 = '\0';		/*null the string*/

		slot_list[i] = (u32)simple_strtol( slotnum_str, &cptr1, 0 );
		if( (s32)cptr1 < (s32)p1 )
			{	
			sprintf(msg, "\"%s\" cannot be converted to int\n", slotnum_str);
			return(E_CHAS_FIND_INT_CONVERSION);
			}

		/*	Make sure it is in range*/
		if( (slot_list[i] < 1) || 
			(slot_list[i] > chas_actlist[act_index].max_comp) )
			{
			sprintf(msg, "\"%d\" is out of range, valid range 1 -to- %d\n", 
							slot_list[i], chas_actlist[act_index].max_comp);
			return(E_CHAS_FIND_INVALID_SLOT);
			}

		if(p1)
			slotnum_str = ++p1;		/*point to next ascii number*/
		}/*end for*/
	}

slot_list[i] = END_MARKER;
return(0);
}/*end FUNCTION: find_action*/




/*****************************************************************************
*	FUNCTION:	blade_action()
*	Date:	12-11-06
*
*	Perform an action to a chassis blade.
*
*	INPUT:  slot_list - blade/s number to access.
*			action - action to be taken (see t_component_actions).
*			verbose - Display FPGA registers before and after action.
*			msg		- error message buffer 
*
*	RETURN: 0 
*****************************************************************************/ 
static s32 blade_action(u32 slot_list[], u32 action, u32 verbose, char *msg)
{
char yes_str[] = "yes";		/*variables used in status action*/
char no_str[]  = "no";
char *p1, *stat_str;
u32  alert, link, ack;

u32 i;
u32 blade_cnt = 0;
u16 *out_p;
u16 *in_p;
u16 *pwr_p;
u16 *pwr_en_p;
u16  bld_pwr_reg;
u16  bld_pwr_en_reg;
u16  bld_out_reg;
u16  bld_in_reg;
char tmp[20];

msg[0] = '\0';	/*null the message string*/

for(i = 0; slot_list[i] != END_MARKER; ++i)
	{
	out_p = (u16*)blade_out_addrs[slot_list[i] - 1];
	in_p  = (u16*)blade_in_addrs[slot_list[i] - 1];
	pwr_p = (u16*)FPGA_REG_BLADE_PWR_OUT;
	pwr_en_p = (u16*)FPGA_REG_BLADE_PWR_OUT_EN;
	bld_pwr_reg		= *pwr_p;
	bld_pwr_en_reg	= *pwr_en_p;
	bld_out_reg		= *out_p;
	bld_in_reg		= *in_p;

	if(verbose)
		printf(	"Entry values slot: %d\n"
				"Blade out_p    = 0x%x, read data = 0x%04lx\n"
				"Blade in_p     = 0x%x, read data = 0x%04lx\n"
				"Blade pwr_p    = 0x%x, read data = 0x%04lx\n"
				"Blade pwr_en_p = 0x%x, read data = 0x%04lx\n",
				slot_list[i], out_p, bld_out_reg, in_p, bld_in_reg, 
				pwr_p, bld_pwr_reg, pwr_en_p, bld_pwr_en_reg);

	switch(action)
		{
		case BLADE_ACTION_ON :
			bld_pwr_reg |= (0x0001 << (slot_list[i] - 1));	    /*1=ps on*/
			*pwr_p = bld_pwr_reg;
	
			bld_pwr_en_reg |= (0x0001 << (slot_list[i] - 1));	/*1=en*/
			*pwr_en_p = bld_pwr_en_reg;
			break;

		case BLADE_ACTION_OFF :	
			bld_pwr_reg &= ~(0x0001 << (slot_list[i] - 1));		/*0=ps off*/
			*pwr_p = bld_pwr_reg;

			bld_pwr_en_reg |= (0x0001 << (slot_list[i] - 1));	/*1=en*/
			*pwr_en_p = bld_pwr_en_reg;
		break;

		case BLADE_ACTION_TRI :	
			bld_pwr_en_reg &= ~(0x0001 << (slot_list[i] - 1));	/*0=tri*/
			*pwr_en_p = bld_pwr_en_reg;
		break;

		case BLADE_ACTION_IMC_RST :
			bld_out_reg |= FPGA_BLD_GPO_IMC_RST;
			*out_p = bld_out_reg;
			break;

		case BLADE_ACTION_IMC_RST_N :
			bld_out_reg &= ~FPGA_BLD_GPO_IMC_RST;
			*out_p = bld_out_reg;
			break;

		case BLADE_ACTION_BLD_RST :
			bld_out_reg |= FPGA_BLD_GPO_BLD_RST;
			*out_p = bld_out_reg;
			break;

		case BLADE_ACTION_BLD_RST_N :
			bld_out_reg &= ~FPGA_BLD_GPO_BLD_RST;
			*out_p = bld_out_reg;
			break;

		case BLADE_ACTION_THROTTLE :
			bld_out_reg |= FPGA_BLD_GPO_THROTTLE;
			*out_p = bld_out_reg;
			break;

		case BLADE_ACTION_THROTTLE_N :
			bld_out_reg &= ~FPGA_BLD_GPO_THROTTLE;
			*out_p = bld_out_reg;
			break;

		case BLADE_ACTION_BLD_PWRDN :
			bld_out_reg |= FPGA_BLD_GPO_BLD_PWRDN;
			*out_p = bld_out_reg;
			break;

		case BLADE_ACTION_BLD_PWRDN_N :
			bld_out_reg &= ~FPGA_BLD_GPO_BLD_PWRDN;
			*out_p = bld_out_reg;
			break;

			case BLADE_ACTION_NTWRK_ISO :
			bld_out_reg |= FPGA_BLD_GPO_NTWRK_ISO;
			*out_p = bld_out_reg;
			break;

		case BLADE_ACTION_NTWRK_ISO_N :
			bld_out_reg &= ~FPGA_BLD_GPO_NTWRK_ISO;
			*out_p = bld_out_reg;
			break;

		case BLADE_ACTION_POST_STATUS :
			/*
			*	FPGA14 9-14-07: meaning of blade_in, bits 15:14 has changed:
			*	0 0 = Blade is NOT present
			*	0 1 = Blade is present but not powered
			*	1 0 = Blade is present and powered
			*	1 1 = Cannot happen
			*/
			if( !(bld_in_reg & FPGA_BLD_GPI_PRESENT) &&
				!(bld_in_reg & FPGA_BLD_GPI_BLANK_PRESENT) )
				stat_str = "--";	/*slot is empty*/
			else if( !(bld_in_reg & FPGA_BLD_GPI_PRESENT) &&
				      (bld_in_reg & FPGA_BLD_GPI_BLANK_PRESENT) )
				stat_str = "pr";	/*present, not powered*/
			else if(  (bld_in_reg & FPGA_BLD_GPI_PRESENT) &&
				     !(bld_in_reg & FPGA_BLD_GPI_BLANK_PRESENT) )
				stat_str = "on";	/*present, and powered*/
			
			sprintf(tmp, "BL%02d=%s ", slot_list[i], stat_str);

			if(++blade_cnt == 9)
				strcat(msg, "\n             ");	/*start a new line*/

            strcat(msg, tmp);
			break;

		case BLADE_ACTION_STATUS :
			if( !(bld_in_reg & FPGA_BLD_GPI_PRESENT) &&
				!(bld_in_reg & FPGA_BLD_GPI_BLANK_PRESENT) )
				p1 = "not present";			/*slot is empty*/
			else if( !(bld_in_reg & FPGA_BLD_GPI_PRESENT) &&
				      (bld_in_reg & FPGA_BLD_GPI_BLANK_PRESENT) )
				p1 = "present-powered off";	/*present, not powered*/
			else if(  (bld_in_reg & FPGA_BLD_GPI_PRESENT) &&
				     !(bld_in_reg & FPGA_BLD_GPI_BLANK_PRESENT) )
				p1 = "present-powered on";	/*present, and powered*/

			alert	= (bld_in_reg & FPGA_BLD_GPI_ALERT)		? 1 : 0;
			link	= (bld_in_reg & FPGA_BLD_GPI_LINK_STAT)	? 1 : 0;
			ack		= (bld_in_reg & FPGA_BLD_GPI_ACK)		? 1 : 0;

			printf("Slot %02d: Blade=%s   Alert=%d "
					"Link Status=%d Ack=%d\n", 
					slot_list[i], p1, alert, link, ack);
			break;
		}

	if(verbose)
		printf(	"Write back values: slot %d\n"
			"Blade out = 0x%04lx  Blade in = 0x%04lx\n"
			"Blade pwr = 0x%04lx  Blade pwr en(tristate=0) = 0x%04lx\n",
					slot_list[i], bld_out_reg, bld_in_reg, 
					bld_pwr_reg, bld_pwr_en_reg);
	}/*end for*/

return(0);
}/*end FUNCTION: blade_action*/



/*****************************************************************************
*	FUNCTION:	psu_action()
*	Date:	12-12-06
*
*	Perform an action to a chassis power supply.
*
*	INPUT:  slot_list - Power supply/s numner to access.
*			action  - action to be taken (see t_component_actions).
*			verbose - Display FPGA registers before and after action.
*			msg		- error message buffer 
*
*	RETURN: 0 
*****************************************************************************/ 
static s32 psu_action(u32 slot_list[], u32 action, u32 verbose, char *msg)
{
u32  i;
u16  *in_p;
u16  *pwr_p;
u16  psu_pwr_reg;
u16  psu_in_reg;
char *pres_p, *acgood_p, *dcgood_p;	/*variables used in status action*/
char *stat_str;
char tmp[20];
char yes_str[] = "yes";				/*variables used in status action*/
char no_str[]  = "no";

msg[0] = '\0';	/*null the message string*/

for(i = 0; slot_list[i] != END_MARKER; ++i)
	{
	in_p  = (u16*)psu_in_addrs[slot_list[i] -1 ];
	pwr_p = (u16*)FPGA_REG_PSU_OUT;
	psu_pwr_reg = *pwr_p;
	psu_in_reg  = *in_p;

	if(verbose)
		printf(	"Entry values: slot %d\n"
				"PSU pwr_p = 0x%x, read data = 0x%04lx\n"
				"PSU in_p  = 0x%x, read data = 0x%04lx\n",
				slot_list[i], pwr_p, psu_pwr_reg, in_p, psu_in_reg);

	switch(action)
		{
		case PSU_ACTION_ON :
			psu_pwr_reg &= ~(0x0001 << (slot_list[i] - 1));	/*0=ps on*/
			*pwr_p = psu_pwr_reg;
			break;

		case PSU_ACTION_OFF :	
			psu_pwr_reg |= (0x0001 << (slot_list[i] - 1));	/*1=ps off*/
			*pwr_p = psu_pwr_reg;
			break;

		case PSU_ACTION_POST_STATUS :
			if( (!(psu_in_reg & FPGA_PSU_GPI_PRESENT)) &&
				  (psu_in_reg & FPGA_PSU_GPI_ACGOOD)   &&
				  (psu_in_reg & FPGA_PSU_GPI_DCGOOD) )
				stat_str = "on";	/*powered up*/
			else if( (!(psu_in_reg & FPGA_PSU_GPI_PRESENT)) &&
				       (psu_in_reg & FPGA_PSU_GPI_ACGOOD)   &&
				     (!(psu_in_reg & FPGA_PSU_GPI_DCGOOD)) )
				stat_str = "ac";	/*present and ac good*/
			else if( (!(psu_in_reg & FPGA_PSU_GPI_PRESENT)) &&
				     (!(psu_in_reg & FPGA_PSU_GPI_ACGOOD))  &&
				     (!(psu_in_reg & FPGA_PSU_GPI_DCGOOD)) )
				stat_str = "pr";	/*present, no ac, no dc good*/
			else 
				stat_str = "--";	/*not present, no ac, no dc*/			

			sprintf(tmp, "PSU%d=%s ", slot_list[i], stat_str);
            strcat(msg, tmp);
			break;

		case PSU_ACTION_STATUS :
			pres_p	 = (psu_in_reg & FPGA_PSU_GPI_PRESENT)	? no_str : yes_str;
			acgood_p = (psu_in_reg & FPGA_PSU_GPI_ACGOOD)	? yes_str : no_str;
			dcgood_p = (psu_in_reg & FPGA_PSU_GPI_DCGOOD)	? yes_str : no_str;
			printf("Slot %d: PSU pres=%-3s ACGOOD=%-3s DCGOOD=%-3s\n", 
									slot_list[i], pres_p, acgood_p, dcgood_p);
			break;
		}

	if(verbose)
		printf(	"Write back values: slot %d\n"
				"PSU in = 0x%04lx  PSU pwr = 0x%04lx\n", 
				slot_list[i], psu_in_reg, psu_pwr_reg);
	}/*end for*/

return(0);
}/*end FUNCTION: psu_action*/



/*****************************************************************************
*	FUNCTION:	fan_action()
*	Date:	1-15-07
*
*	Perform an action to a chassis fan.
*
*	INPUT:  slot_list - fan/s to access.
*			action  - action to be taken (see t_component_actions).
*			verbose - Display FPGA registers before and after action.
*			msg		- error message buffer 
*
*	RETURN: 0 
*****************************************************************************/ 
static s32 fan_action(u32 slot_list[], u32 action, u32 verbose, char *msg)
{
u32  state, tac1, tac2;			/*variables used in status action*/
u32  i;
u32  fan_cnt = 0;
u16  *type_p;
u16  *pwm_p;
u16  *fan_state_p;
u16  type;
u16  pwm;
char *p;
char *stat_str;
char tmp[20];

msg[0] = '\0';	/*null the message string*/

for(i = 0; slot_list[i] != END_MARKER; ++i)
	{
	type_p	= (u16*)fan_type_addrs[slot_list[i] -1 ];
	pwm_p	= (u16*)fan_pwm_ctrl_addrs[slot_list[i] -1 ];
	fan_state_p = ((u16*)fan_state_addrs[slot_list[i] -1]);
	type	= *type_p;
	pwm		= *pwm_p;


	if( (type < 0) || (type > (MAX_FAN_TYPES - 1)) )
		{
		sprintf(msg, "ERROR: type %d, valid types 0 -to- %d, addr=0x%x\n", 
									type, (MAX_FAN_TYPES -1), (u32)type_p );
		return(E_CHAS_FAN_INVALID_TYPE);
		}
		
 
	if(verbose)
		printf(	"Entry values: fan %d\n"
				"type_p = 0x%x, type = 0x%04lx\n"
				"pwm_p  = 0x%x, pwm  = 0x%04lx\n",
				slot_list[i], type_p, type, pwm_p, pwm);

	switch(action)
		{
		case FAN_ACTION_OFF :
			pwm = canned_fan_speeds[type][FAN_SPEED_IDX_OFF];
			*pwm_p = pwm;
			*fan_state_p = FPGA_FAN_STATE_OFF;
			break;

		case FAN_ACTION_LOW :	
			pwm = canned_fan_speeds[type][FAN_SPEED_IDX_LOW];
			*pwm_p = pwm;
			*fan_state_p = FPGA_FAN_STATE_GOOD;
			break;

		case FAN_ACTION_MED :	
			pwm = canned_fan_speeds[type][FAN_SPEED_IDX_MED];
			*pwm_p = pwm;
			*fan_state_p = FPGA_FAN_STATE_GOOD;
			break;

		case FAN_ACTION_HIGH :	
			pwm = canned_fan_speeds[type][FAN_SPEED_IDX_HIGH];
			*pwm_p = pwm;
			*fan_state_p = FPGA_FAN_STATE_GOOD;
			break;

		case FAN_ACTION_STBY :	
			pwm = canned_fan_speeds[type][FAN_SPEED_IDX_STBY];
			*pwm_p = pwm;
			*fan_state_p = FPGA_FAN_STATE_GOOD;
			break;

		case FAN_ACTION_STBY_MISSING :	
			pwm = canned_fan_speeds[type][FAN_SPEED_IDX_STBY_MISSING];
			*pwm_p = pwm;
			*fan_state_p = FPGA_FAN_STATE_GOOD;
			break;
			

		case FAN_ACTION_POST_STATUS :
			tac1  = *((u16*)fan_tac1_addrs[slot_list[i] -1]);
			tac2  = *((u16*)fan_tac2_addrs[slot_list[i] -1]);

			if(FAN_TYPE_BLANK_INSTALLED == type)
				stat_str = "bk";	/*blank cover installed*/
			else if(FAN_TYPE_NOT_INSTALLED == type)
				stat_str = "--";	/*slot is empty, no fan, no blank*/
			else if( (tac1 > 0) && (tac1 < 0xffff) )
				stat_str = "on";	/*fans are spinning*/
			else
				stat_str = "pr";	/*present, but not spinning*/
			
			sprintf(tmp, "FAN%d=%s ", slot_list[i], stat_str);

			if(++fan_cnt == 6)
				strcat(msg, "\n             ");	/*start a new line*/

            strcat(msg, tmp);
			break;

		case FAN_ACTION_STATUS :
			state = *fan_state_p;
			tac1  = *((u16*)fan_tac1_addrs[slot_list[i] -1]);
			tac2  = *((u16*)fan_tac2_addrs[slot_list[i] -1]);

			if(FAN_TYPE_BLANK_INSTALLED == type)
				p = "blank cover";								
			else if(FAN_TYPE_NOT_INSTALLED == type)
				p = "not-present"; 
			else
				{
				sprintf(tmp, "%d", type);
				p = tmp;
				}

			printf("Fan %d: TYPE=%-11s STATE=0x%04lx PWM=0x%04lx "
					"TAC1=0x%04lx TAC2=0x%04lx\n",
					slot_list[i], p, state, pwm, tac1, tac2);
			break;
		}

	if(verbose)
		printf(	"Write back values: fan %d\n"
				"pwm_reg = 0x%04lx\n", 	slot_list[i], pwm);

		if( ((FAN_TYPE_BLANK_INSTALLED == type)  ||
		 (FAN_TYPE_NOT_INSTALLED   == type)) && 
		 (FAN_ACTION_STATUS != action) )
			{
			if(verbose)
				printf("NOTE: Fan %d: no physical fan is detected.\n", 
																slot_list[i]);
			}

	}/*end for*/

return(0);
}/*end FUNCTION: fan_action*/


/*****************************************************************************
*	FUNCTION:	lkvm_action()
*	Date:	2-9-07
*
*	Perform an action to the LKVM
*
*	INPUT: 	action  - action to be taken (see t_component_actions).
*			verbose - Display FPGA registers before and after action.
*			msg		- error message buffer 
*
*	RETURN: 0 
*****************************************************************************/ 
static s32 lkvm_action(u32 action, u32 verbose, char *msg)
{
u16  *in_p  = (u16*)FPGA_REG_GPI_KVM_IN;
u16  *pwr_p = (u16*)FPGA_REG_I2C_KVM_PWR_OUT;
u16  lkvm_pwr_reg = *pwr_p;
u16  lkvm_in_reg  = *in_p;
char *pres_p, *blank_p, *powered_p;
char *stat_str;
char yes_str[] = "yes";				/*variables used in status action*/
char no_str[]  = "no";

if(verbose)
	printf(	"Entry values:\n"
				"LKVM pwr_p = 0x%x, read data = 0x%04lx\n"
				"LKVM in_p  = 0x%x, read data = 0x%04lx\n",
				pwr_p, lkvm_pwr_reg, in_p, lkvm_in_reg);

switch(action)
	{
	case LKVM_ACTION_ON :
		lkvm_pwr_reg &= ~FPGA_I2C_KVM_PWR_OUT;		/*0=on*/
		lkvm_pwr_reg |= FPGA_I2C_KVM_PWR_CONTROL;	/*enable to drive bit*/
		*pwr_p = lkvm_pwr_reg;
		break;

	case LKVM_ACTION_OFF :	
		/*1=off*/
		lkvm_pwr_reg |= (FPGA_I2C_KVM_PWR_OUT | FPGA_I2C_KVM_PWR_CONTROL); 
		*pwr_p = lkvm_pwr_reg;
		break;

	case LKVM_ACTION_TRI :
		lkvm_pwr_reg &= ~(FPGA_I2C_KVM_PWR_OUT | FPGA_I2C_KVM_PWR_CONTROL);
		*pwr_p = lkvm_pwr_reg;
		break;

	case LKVM_ACTION_POST_STATUS :
		if( (!(lkvm_in_reg & FPGA_I2C_KVM_IN_PRESENT)) &&
			(!(lkvm_pwr_reg & FPGA_I2C_KVM_PWR_INPUT)) )
			stat_str = "on";		/*powered up*/
		else if( !(lkvm_in_reg & FPGA_I2C_KVM_IN_PRESENT) )
			stat_str = "pr";	/*present*/
		else if( !(lkvm_in_reg & FPGA_I2C_KVM_IN_BLANK) )
			stat_str = "bk";	/*blank*/
		else 
			stat_str = "--";	/*not present, no blank*/			
			
		sprintf(msg, "LKVM=%s ", stat_str);
		break;

	case LKVM_ACTION_STATUS :
		pres_p	  =(lkvm_in_reg  & FPGA_I2C_KVM_IN_PRESENT) ? no_str : yes_str;
		blank_p	  =(lkvm_in_reg  & FPGA_I2C_KVM_IN_BLANK)	? no_str : yes_str;
		powered_p =(lkvm_pwr_reg & FPGA_I2C_KVM_PWR_INPUT)	? no_str : yes_str;
		printf("LKVM pres=%-3s blank=%-3s powered=%-3s\n", 
												pres_p, blank_p, powered_p);
		break;
	}

if(verbose)
	printf(	"Write back values: \n"
			"LKVM in = 0x%04lx  LKVM pwr = 0x%04lx\n", 
			lkvm_in_reg, lkvm_pwr_reg);

return(0);
}/*end FUNCTION: lkvm_action*/


/*****************************************************************************
*	FUNCTION:	iom_action()
*	Date:	2-14-07
*
*	Perform an action to the IOM.
*
*	INPUT: 	action  - action to be taken (see t_component_actions).
*			verbose - Display FPGA registers before and after action.
*			msg		- error message buffer 
*
*	RETURN: 0 
*****************************************************************************/ 
static s32 iom_action(u32 slot_list[], u32 action, u32 verbose, char *msg)
{
u32  i;
u16  *in_p  = (u16*)FPGA_REG_GPI_IOM_IN;
u16  iom_in_reg  = *in_p;
char *pres_p, *blank_p;
char *stat_str;
char yes_str[] = "yes";				/*variables used in status action*/
char no_str[]  = "no";
char tmp[20];

msg[0] = '\0';	/*null the message string*/

if(verbose)
	printf(	"Entry values:\nIOM in_p  = 0x%x, read data = 0x%04lx\n",
													in_p, iom_in_reg);
for(i = 0; slot_list[i] != END_MARKER; ++i)
	{
	switch(action)
		{
		case IOM_ACTION_POST_STATUS :
			if(!(iom_in_reg & (FPGA_IOM1_PRES << (slot_list[i] - 1))) )
				stat_str = "pr";	/*present*/
			else if(!(iom_in_reg & (FPGA_IOM1_BLANK << (slot_list[i] - 1))) )
				stat_str = "bk";	/*blank*/
			else
				stat_str = "--";	/*not present, no blank*/		
				
			sprintf(tmp, "IOM%d=%s ", slot_list[i], stat_str);
			strcat(msg, tmp);
			break;

		case IOM_ACTION_STATUS :
			pres_p  = (iom_in_reg & (FPGA_IOM1_PRES << (slot_list[i] - 1)))
															? no_str : yes_str;
			blank_p	  =(iom_in_reg & (FPGA_IOM1_BLANK << (slot_list[i] - 1)))
															? no_str : yes_str;
			printf("Slot %d: IOM pres=%-3s blank=%-3s "	
					"(cannot determine IOM power)\n", 
					slot_list[i], pres_p, blank_p);
			break;
		}
	}/*end for*/

return(0);
}/*end FUNCTION: iom_action*/


/*****************************************************************************
*	FUNCTION:	reset_i2c_interfaces()
*	DATE:		7-9-07
*
*	CR146327 FPGA I2C cores need to be reset when uboot claims mgmt owner.
*	Reset each of the 5 I2C busses off the FPGA.  Called AFTER the CMC has
*	taken ownership and become active.
*	
*****************************************************************************/
static s32 reset_i2c_interfaces(void)
{
ABS_write_fpga((FPGA_BASE_IIC_1 + FPGA_OFF_IIC_IPIFSRR), FPGA_IIC_RESET_KEY);
ABS_write_fpga((FPGA_BASE_IIC_2 + FPGA_OFF_IIC_IPIFSRR), FPGA_IIC_RESET_KEY);
ABS_write_fpga((FPGA_BASE_IIC_3 + FPGA_OFF_IIC_IPIFSRR), FPGA_IIC_RESET_KEY);
ABS_write_fpga((FPGA_BASE_IIC_4 + FPGA_OFF_IIC_IPIFSRR), FPGA_IIC_RESET_KEY);
ABS_write_fpga((FPGA_BASE_IIC_5 + FPGA_OFF_IIC_IPIFSRR), FPGA_IIC_RESET_KEY);
xiic_init(FPGA_BASE_IIC_1);
xiic_init(FPGA_BASE_IIC_2);
xiic_init(FPGA_BASE_IIC_3);
xiic_init(FPGA_BASE_IIC_4);
xiic_init(FPGA_BASE_IIC_5);
return(0);
}/*end FUNCTION: reset_i2c_interfaces()*/

#endif /*CONFIG_CMC_10G*/



