/****************************************************************************

  Copyright (c) 2006, Dell Inc
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

  - Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
    
  - Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution. 

  - Neither the name of Dell Inc nor the names of its contributors
    may be used to endorse or promote products derived from this software 
    without specific prior written permission. 

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. 

*****************************************************************************/


/*
 * Utility support
 */
#include <common.h>		/*must be first in file*/
#include <command.h>
#include <asm/cache.h>
#include <net.h>


#ifdef CONFIG_CMC_10G	/*only compile in for a CMC_10G build*/

/******************************************************************************
*
*	FILE: util_cpld.c
*
******************************************************************************/


/******************************************************************************
*  Include files
******************************************************************************/
#include <util_defs.h>
#include <util_fpga.h>
#include <util_ret_codes.h>
#include <util_flashmap.h>

#include <malloc.h>


/******************************************************************************
*  Local Defines
******************************************************************************/
#define CPLD_IMAGE_MAX_SIZE			(CFG_SECT_SIZE * 1)	/*sector allocation*/
#define CPLD_BS_MAX_SIZE			(CPLD_IMAGE_MAX_SIZE - IMAGE_HEADER_SIZE)

#define	CPLD_FLAG_VERBOSE_ON		0x00000001
#define	CPLD_FLAG_VERBOSE_OFF		0x00000002
#define	CPLD_FLAG_ERASE_FLASH		0x00000004	/*remove vme file from flash*/


/******************************************************************************
*  Globals
******************************************************************************/


/******************************************************************************
*  Prototypes
******************************************************************************/
extern s32	mcoptind;
extern char	*mcoptarg[];
extern char	BootFile[128];				/*TFTP Boot File name*/

static s32 get_cpld_count(void);

/******************************************************************************
*	List all structures defines
******************************************************************************/
t_options util_cpld_options[] = 
	{
	/*Options*/
	{"bs2flash","Copy a previously downloaded binary into flash."},	
	{"load",    "Program the CPLD from the flash contents"},	
	{"pingpong","Program CPLD using alternate images"},
	{ NULL, NULL }
	};


t_usage util_cpld_usage = { 
	"cpld", 
					
	/*Syntax*/
	"cpld   [-load] [-verbose]\n"
	"       [-bs2flash <exp rev>] [-verbose]\n"
	"		[-pingpong]\n",

	/*Description*/
	"The cpld command allows the user to flash the CPLD bit stream into the\n"
	"deticated NOR flash sector, then re-program the CPLD.\n\n"
	"By setting userexe you can loop on programming the same image found in flash,\n"
	"or pingpong between two different images in which you will need to set up two\n"
	"env variables indicating the expected version and the name of the file/s to tftp\n"
	"\nEXAMPLES:\n\n"
	"tftp cpld.vme <TFTP server IP>\n"
	"util cpld -bs2flash  4\n"
	"util cpld -load\n\n"
	"setenv post_cpld1 <ver>:<cpld_a10.vme>\n"
	"setenv post_cpld2 <ver>:<cpld_a11.vme>\n"
	"setenv userexe util cpld -pingpong\n"
	"saveenv\n",
	
	util_cpld_options
	}; /*end t_usage util_cpld_usage*/


/******************************************************************************
*
*	FUNCTION: util_cpld()
*	DATE:	9-6-07
*	
*	(See usage statment above)
******************************************************************************/
s32 util_cpld(int argc, char *argv[])
{
u32  index		= 0;
u32	 rc			= 0;
u32  bs_flag	= 0;
u32  load_flag  = 0;
u32  cpld_flag  = 0;
u32  pp_flag	= 0;
u32  exp_rev	= 0;
char *cptr1;
char msg[MAX_MSG_LEN];		

//	Get any options
mcoptind = 2;	
while( (index = mcgetopt(argc, argv, 
	  "-bs2flash:1 -load -verbose -pingpong")) != MCGETOPT_EOF)
	{
	switch(index) 
		{
		case 0 :		/*copy bit stream from memory to flash*/
			bs_flag = 1;

			/*convert revision*/
			exp_rev = (u32)simple_strtol( mcoptarg[0], &cptr1, 0 );
			if( (s32)cptr1 < (strlen(mcoptarg[0]) + (s32)mcoptarg[0]) ) 
				{	
				printf("%s %s:ERROR: converting \"%s\" int.\n", 
											argv[0], argv[1], mcoptarg[0]);
				return(E_OPTARG_INT_CONVERSION);
				}

			break;

		case 1 :		/*load cpld*/
			load_flag = 1;
			break;

		case 2 :		/*Add verbose to flag*/
			cpld_flag |= CPLD_FLAG_VERBOSE_ON;
			break;

		case 3 :		/*pingpong test*/
			pp_flag = 1;
			break;

		default:		/*not good*/
			rc = mcgetopt_error(index, msg, mcoptarg[0]);
			printf("%s %s:%s\n", argv[0], argv[1], msg);
			return(rc);
			break;
		}
	}/*end mcgetopt while*/



/****************************************************************************
*	Copy the bit stream from memory to flash
****************************************************************************/
if(bs_flag)
	{	
	/*	
	*	Using the "filesize" and "fileaddr", take a previously loaded
	*	bit stream and build it into a flash image.
	*/
	if( (rc = cpld_bs2flash(exp_rev, cpld_flag, msg)) != 0)
		{
		printf("%s %s:ERROR:%s\n", argv[0], argv[1], msg);
		return(rc);
		}
	else
		{
		printf("%s %s:%s\n", argv[0], argv[1], msg);
		return(rc);
		}
	}


/****************************************************************************
*	Load the bit stream into the CPLD
****************************************************************************/
if(load_flag)
	{	
	printf("\n");
	if( (rc = cpld_load(cpld_flag, msg)) != 0)
		{
		printf("%s %s:ERROR:%s\n", argv[0], argv[1], msg);
		return(rc);
		}
	else
		{
		printf("%s %s:%s\n", argv[0], argv[1], msg);
		return(rc);
		}
	
	printf("\n");
	}


/****************************************************************************
*	Using env variables tftp an alternate iamge and program the CPLD
****************************************************************************/
if(pp_flag)
	{
	printf("\n");
	if( (rc = cpld_pingpong(cpld_flag, msg)) != 0)
		{
		printf("%s %s:ERROR:%s\n", argv[0], argv[1], msg);
		return(rc);
		}
	else
		{
		printf("%s %s:%s\n", argv[0], argv[1], msg);
		return(rc);
		}
	
	printf("\n");
	}

return(0);
}/* end FUNCTION: util_cpld()*/


/****************************************************************************
*
*	FUNCTION: cpld_bs2flash()
*	DATE:	9-6-07
*
*	In normal operation the build process constructs a header and appends the
*	CPLD bit stream.  Then fwupd will program it into the correct sector.
*	For lab development the "-bs2flash" option takes the CPLD binary file 
*	previously downloaded into memory, builds and prepends a working header,
*	then programs it into the flash sectors.
*
*	INPUT:	exp_ver - Expected CPLD revision to place into the header
*			msg     - Buffer used to return any messages.
*
*****************************************************************************/
s32 cpld_bs2flash(u32 exp_ver, u32 cpld_flag, char *msg)
{
s32		rc;
char	*p;
u32		bs_addr;
u32		bs_size;
u32		verbose;
t_image_header	*hd;

if(CPLD_FLAG_VERBOSE_ON & cpld_flag)
	verbose = VERBOSE_ON;
else
	verbose = VERBOSE_OFF;


/*	Get parameters from a previously tftp'ed bit stream */
if((rc = get_file_addr_and_size(&bs_addr, &bs_size, msg)) != 0)
	return(rc);

if(bs_size > CPLD_BS_MAX_SIZE)
	{
	sprintf(msg, "\"filesize=0x%x\" exceeds max of 0x%x", 
												bs_size, CPLD_BS_MAX_SIZE); 
	return(E_CPLD_INVALID_BYTE_CNT);
	}

/*	Obtain a memory buffer, set to F's,  init the header*/
p  = malloc(CPLD_IMAGE_MAX_SIZE);
memset( (void*)p, 0xff, CPLD_IMAGE_MAX_SIZE);

hd = (t_image_header *)p;
hd->header_type		= CURRENT_IMAGE_HEADER_TYPE;
hd->token			= IMAGE_TOKEN_CPLD;
hd->version			= exp_ver;
strncpy(hd->name, IMAGE_TEXTNAME_CPLD, MAX_IMAGE_NAME_SIZE);
hd->fstart			= FLASH_SECT_START_CPLD;	
hd->fcnt			= FLASH_SECT_CNT_CPLD;
hd->offset			= IMAGE_HEADER_SIZE;
hd->bcnt_zip		= 0;
hd->bcnt_unzip		= bs_size;	
hd->crc32 = crc32(0, (u8*)bs_addr, bs_size);

if(verbose)
	{
	printf("cpld_bs2flash(): header_type=0x%x token=0x%x version=0x%x\n"
					"name=%s offset=0x%x bcnt=0x%x crc32=0x%x\n"
					"start_sector=%d sector_count=%d\n\n", 
					hd->header_type, hd->token, hd->version, hd->name,
					hd->offset, hd->bcnt_unzip, hd->crc32, hd->fstart,
					hd->fcnt);

	printf("Malloc start=0x%x, load start=0x%x\n", p, bs_addr);
	}



/*	Append the bit stream after the header, erase, then program*/
memcpy((void *)((u32)p + IMAGE_HEADER_SIZE), (void *)bs_addr, bs_size);

if((rc = ABS_erase_nor(hd->fstart, 
									(hd->fstart + hd->fcnt -1),	
									verbose, 0, msg)) != 0)
	{
	free(p);
	return(rc);
	}	

if((rc = ABS_flash_write_nor(p, SECTOR_TO_PHYSADDR(hd->fstart), 
									(bs_size + IMAGE_HEADER_SIZE), msg)) != 0)
	{
	free(p);
	return(E_CPLD_BSTREAM_COPY_TO_FLASH);
	}

sprintf(msg, "Copied bs from mem 0x%x, to starting sector %d, for 0x%x bytes",
							p, hd->fstart, bs_size + IMAGE_HEADER_SIZE);
free(p);
return(0);
}/*end FUNCTION: cpld_bs2flash()*/


/****************************************************************************
*
*	FUNCTION: cpld_load()
*	DATE:	9-20-07
*
*	Load the CPLD vme image into the CPLD.  It will reset, and look like an
*	AC power up after the programming is done.
*
*	INPUT:	cpld_flag - Verbose option
*			msg   - Buffer used to return messages.
*	
*****************************************************************************/
s32 cpld_load(u32 cpld_flag, char *msg)
{
u32		rc = 0;
u32		bs_bcnt;
u32		flash_ver;
u32		saved_silent;
char	*bs_addr;
char	env_data[MAX_MSG_LEN];
t_image_header	*iheader;
/*	Just to shutup the u-boot version only*/
char *argv[] = {"cpld_update", "CPLD.vme"}; 


/*	
*	Verify a valid bit stream exists before attempting a load.
*	NOTE: bit stream will be relocated to memory.
*/
iheader=(t_image_header *)SECTOR_TO_PHYSADDR(FLASH_SECT_START_CPLD);
if( (rc = cpld_verify_bs(iheader, cpld_flag, &flash_ver, msg)) != 0)
	return(rc);

/*	Starting address and byte count of the bit stream from the header.*/
//bs_addr = (char*) (((u32)iheader) + iheader->offset);
bs_addr = (char*) CFG_LOAD_ADDR;
bs_bcnt = iheader->bcnt_unzip;

if(CPLD_FLAG_VERBOSE_ON & cpld_flag)
	printf("cpld_load(): bs_addr=0x%x bs_bcnt=0x%x \n", bs_addr, bs_bcnt);

/*	Keep an env count*/
sprintf(env_data, "%d", (get_cpld_count() + 1) );
setenv("post_cpld_count", env_data);				/*update count*/
saved_silent = set_save_silent_mode();
saveenv();
restore_silent_mode(saved_silent);

/*	If called from util_cmc_main(), remove flash image before programming*/
if(cpld_flag & CPLD_FLAG_ERASE_FLASH)
	ABS_erase_nor(FLASH_SECT_START_CPLD, FLASH_SECT_START_CPLD,	0, 0, msg);

/*	
*	The programming code was ported from a running OS version.	To minimize the
*	impact make it look like a argc, argv and pass in two aditional paramaters.
*
*	int cpld_main( int argc, char * argv[], char *start_p, char *end_p )
*
*	Should never return, CMC will reset upon completion.  
*/
printf("CMC CPLD Revision, current=%d  new=%d\n", 
					(ABS_read_fpga(FPGA_REG_CPLD_REV) & FPGA_CPLD_REV_MASK), 
					flash_ver);
rc = cpld_main( 2, argv, bs_addr, ((bs_addr + bs_bcnt)-1));
sprintf(msg, "CPLD did not program, reset did not occur RC=0x%x\n", rc);
return(E_CPLD_CMC_DID_NOT_RESET);
}/*end FUNCTION: cpld_load()*/




/****************************************************************************
*
*	FUNCTION: cpld_verify_bs()
*	DATE:	9-6-07
*
*	Verify the presence of a vaild header and crc it.
*
*	INPUT:	iheader -	pointer to the image header that is prepended to the
*						start of the bit stream
*			cpld_flag - verbose options
*			flash_ver - Version found in flash.
*			msg	-		char pointer to store any error messages
*
*	RETURN:	rc -		0 = verification passed
*						non-zero = verification failed, msg contains error msg
*****************************************************************************/
s32 cpld_verify_bs(t_image_header *iheader, u32 cpld_flag, u32 *flash_ver, char *msg)
{
u32		addr;
u32		crc = 0;
u32		verbose; 

if(CPLD_FLAG_VERBOSE_ON & cpld_flag)
	verbose = VERBOSE_ON;
else
	verbose = VERBOSE_OFF;


if(verbose)
	printf("cpld_verify_bs(): image name=%s, max size=0x%x\n"
	   "iheader points to=0x%x\n\n",
	   IMAGE_TEXTNAME_CPLD, FLASH_MAX_SIZE_CPLD, (u32)iheader );


/*
*	Verify that the image name is correct 
*/
if( strncmp(iheader->name, IMAGE_TEXTNAME_CPLD, MAX_IMAGE_NAME_SIZE)!= 0)
	{
	sprintf(msg,"%s expected header name missing", IMAGE_TEXTNAME_CPLD);
	return(E_CPLD_MISSING_IMAGENAME);
	}

/*
*	Early in dev. the iheader elements changed. To prevent a lockup of a huge 
*	bcnt where we cannot return to RACMON, put in a check. This should become
*	an obsolete check.
*/
if(iheader->bcnt_unzip > FLASH_MAX_SIZE_CPLD)
	{
	sprintf(msg,"Invalid CPLD byte count, requested=0x%x max=0x%x",
									iheader->bcnt_unzip, FLASH_MAX_SIZE_CPLD);
	return(E_CPLD_INVALID_BYTE_CNT);
	}

/*
*	Verify the bit streams crc32.  We must move the bs/vme file into memory 
*	before the crc32 as in normal operation we have to erase the sector first
*	since the operation automatically AC cycles when done.
*/
memcpy(CFG_LOAD_ADDR, ((u32)iheader) + iheader->offset, iheader->bcnt_unzip);

//addr = ((u32)iheader) + iheader->offset;
addr = CFG_LOAD_ADDR;
crc = crc32(0, (u8*)addr, iheader->bcnt_unzip);

if(verbose)
	printf("Found \"%s\"\nStart addr=0x%x bs bcnt=0x%x offset=0x%x \n"
	   "header->cksum=0x%x, crc=0x%x\n\n",
		IMAGE_TEXTNAME_CPLD, addr, iheader->bcnt_unzip, iheader->offset, 
		iheader->crc32, crc);

if(crc != iheader->crc32)
	{	
	sprintf(msg, 
		"CPLD bit stream crc32 failed at 0x%x, size of 0x%x\n"
		"exp crc 0x%x, rec crc 0x%x", 
		addr, iheader->bcnt_unzip, iheader->crc32, crc);
	return(E_CPLD_BSTREAM_CRC32_FAILED);
	}

sprintf(msg,
		"CPLD flash image header name and crc32 verified at 0x%x", iheader);

*flash_ver = iheader->version;
return(0);
}/*end FUNCTION: cpld_verify_bs()*/


/****************************************************************************
*	FUNCTION: cpld_check()
*	DATE:	9-24-07
*
*	Determine if the CPLD needs to be programmed.  Only program if the current
*	version is different than the flash version, AND the flash image is good.
*
*	NOTE: If we program the CPLD the CMC will automatically be reset, 
*	the code will NOT return.
*
*	INPUT:	msg	-		char pointer to store any error messages
*	RETURN:	rc -		0 = CPLD is NOT going to be programmed
*						non-zero = An error occured and the determination could
*								   not be reached. 
*****************************************************************************/
s32 cpld_check(char *msg)
{
s32	 rc;
char *env_cpld1		= getenv("post_cpld1");
char *env_cpld2		= getenv("post_cpld2");
u32  flash_ver		= 0xffffffff;
u32  current_ver	= (ABS_read_fpga(FPGA_REG_CPLD_REV) & FPGA_CPLD_REV_MASK);


/*	If any lab looping (post_cpldX) is on skip CPLD check*/
if( (env_cpld1 != 0) || (env_cpld2 != 0) )
	return(0);

if( (rc = cpld_verify_bs(
				(t_image_header *)SECTOR_TO_PHYSADDR(FLASH_SECT_START_CPLD), 
												0, &flash_ver, msg)) )
	return(0);		/*flash sector is invalid, nothing to update with*/
		
if(current_ver == flash_ver)
	return(0);		/*versions are equal, no need to update again*/

/*	Flash has a valid image, versions mismatch, so update, should NOT return.*/
return(cpld_load(CPLD_FLAG_ERASE_FLASH, msg));
}/*end FUNCTION: cpld_check() */


/*****************************************************************************
*	FUNCTION: cpld_pingpong()
*	DATE:	  9-25-07
*
*	Alternate programming the CPLD with two versions. To loop add to "userexe".
*****************************************************************************/
s32 cpld_pingpong(u32 cpld_flag, char *msg)
{
u32  rc;
u32  i;
char *p1;
char *cp;
char *env_cpld1 = getenv("post_cpld1");
char *env_cpld2 = getenv("post_cpld2");
u32  current_ver = (ABS_read_fpga(FPGA_REG_CPLD_REV) & FPGA_CPLD_REV_MASK);
char env_data[MAX_MSG_LEN];
u32	 new_ver[2];
char new_file[2][80];

/*	Must have two env variables with <ver:filename>*/
if( (env_cpld1 == 0) || (env_cpld2 == 0) )
	{
	sprintf(msg, "Env. post_cpld1 or post_cpld2 is missing");
	return(E_CPLD_POST_CPLD_SYNTAX);
	}

/*	Parse to find the two versions and file names*/
strcpy(env_data, env_cpld1);
for(i = 0; i < 2; ++i)
	{
	if( (p1 = strchr(env_data, ':')) == 0)
		{
		sprintf(msg, "post_cpldX is not : seperated");
		return(E_CPLD_POST_CPLD_SYNTAX);
		}
	else
		{
		*p1++ = '\0';
		strcpy(new_file[i], p1);
		new_ver[i] = (s32)simple_strtol(env_data, &cp, 0);
		if( (s32)cp < (strlen(env_data) + (s32)env_data) )
			{
			sprintf(msg, "ENV post_cpld has a bad version");
			return(E_CPLD_POST_CPLD_SYNTAX);
			}
		}
	strcpy(env_data, env_cpld2);
	}

if(current_ver != new_ver[0])
	i = 0;
else
	i = 1;

/*	Load file from TFTPserver*/
strcpy(BootFile, new_file[i]);	
printf("Loading file \"%s\" from TFTP server IP \"%s\"\n", 
											BootFile, getenv ("serverip") );

load_addr = CFG_LOAD_ADDR;		/*DFxxxxx set global load point*/	
if (NetLoop(TFTP) < 0) 
	{
	sprintf(msg, "TFTP load failure\n");
	return(E_CPLD_PINGPONG_TFTP_LOAD);
	}	


/*	Load into flash area*/
if( (rc = cpld_bs2flash(new_ver[i], cpld_flag, msg)) != 0)
	return(rc);	

return(cpld_load(cpld_flag, msg));					/*should not return*/
}/*end FUNCTION: cpld_pingpong() */



/*****************************************************************************
*
*****************************************************************************/
static s32 get_cpld_count(void)
{
char *env_p;
char *cp;
s32  count;

if( (env_p = getenv("post_cpld_count")) == 0)
	return(0);							/*env var does not exists, 0 count*/

count = (s32)simple_strtol(env_p, &cp, 10);
if( (s32)cp < (strlen(env_p) + (s32)env_p) )
	return(0);							/*conversion error, o count*/

return(count);
}/*end FUNCTION: get_cpld_count() */


#endif /*CONFIG_CMC_10G*/



