/****************************************************************************

  Copyright (c) 2006, Dell Inc
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

  - Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
    
  - Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution. 

  - Neither the name of Dell Inc nor the names of its contributors
    may be used to endorse or promote products derived from this software 
    without specific prior written permission. 

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. 

*****************************************************************************/



/*
 * Utility support
 */
#include <common.h>		/*must be first in file*/
#include <command.h>
#include <linux/ctype.h>    /* isalpha, isdigit */

#ifdef CONFIG_CMC_10G	/*only compile in for a CMC_10G build*/
/******************************************************************************
*	FILE: util_flash.c
*
*	For the Dell CMC build the util flash command allows the user to perform
*	various options of erasing, verifing, or testing a selected sector ranges.
*
*****************************************************************************/

/******************************************************************************
*  Include files
******************************************************************************/
#include <util_defs.h>
#include <util_ret_codes.h>
#include <util_flashmap.h>

#include <nand.h>
#include <linux/mtd/mtd.h>
#include <linux/mtd/nand.h>
#include <linux/mtd/compat.h>

/******************************************************************************
*  Local Defines
******************************************************************************/
#define FLASH_NOR				0x00000001
#define FLASH_NAND				0x00000002

#define PATTERN_RANDOM			0x00000001
#define PATTERN_INCREMENTING	0x00000002
#define PATTERN_CONSTANT		0x00000003


/******************************************************************************
*  Globals
******************************************************************************/
u32 g_force_ecc_addr = 0;

/******************************************************************************
*  Prototypes/Externs
******************************************************************************/
extern s32	mcoptind;
extern char	*mcoptarg[];

extern void nand_init(void);
extern nand_info_t nand_info[];
extern t_hw_inventory_nand	hw_inv_nand[];	/*def in util_cmc_main.c*/
extern t_hw_inventory_nor	hw_inv_nor[];	/*def in util_cmc_main.c*/

extern flash_info_t flash_info[];			/*nor flash info struct*/
extern int flash_get_manuf_dev_id(flash_info_t * info, u16 * Manuf_ID, u16 * Dev_ID);

static s32 create_bad_ecc_addrs(char *addr_list, char *msg);
static s32 parse_ecc_err_blocks(u8 *block_history);
static s32 nand_scan_markbad(u8 *block_history, u32 offset, u8 flag);
static s32 build_ecc_err_env(u8 *block_history);



/******************************************************************************
*	List all structures defines
******************************************************************************/
t_options util_flash_options[] = 
	{
	/*Options*/
	{ "s",  "Specify sector or range of NOR sectors.  Char\':\' specifies\n"
	  "      a begin:end range, \'#\' for a begin#number of sectors.\n"
	  "      An \"image\" name operates on sectors defined for that image."},

	{ "erase",	"Erase a specified flash range."},
	{ "verify",	"Used with erase to verify that every bit erased to Fs."},
	{ "rand",	"Perform pattern test, using random patterns."},
	{ "pat",	"Perform pattern test, using constant pattern."},
	{ "off2block", "Convert offset address to block number."},
	{ "block2off", "Convert block number to offset address."},
	{ "nand_init", "Initialize nand."},
	{ "nor_info",  "Dump flash_info_t struc."},
	{ "nor_id", "Read the Manufacturer and Device ID"},
	{ "l",		"Loop count, -1 loop forever."},
	{ "fecc",	"NAND addr to force ECC error."},
	{ "ecc_scan","Scan the NAND part for ECC read errors."},

	{ NULL, NULL }
	};


t_usage util_flash_usage = { 
	"flash", 
					
	/*Syntax*/
	"flash  [-s <start sector:end sector>|<start sector#count> |<image_name>]\n"
	"       [-b <start block:end block |<start block#count> |<image_name>]\n"
	"       [-erase  [-verify]  [-s <range>] | [-b <range>]\n"
	"       [-rand  <seed>]     [-s <range>] | [-b <range>]\n"
	"       [-pat <0x32bit pat> [-s <range>] | [-b <range>]\n"
	"       [-nor_id]\n"
	"       [-nand_init]\n"
	"       [-nor_info]\n"
	"       [-nor_id]\n"
	"       [-fecc <0xaddr>:<0xaddr>:](sequencially greater, block aligned)\n"
	"       [-ecc_scan\n"
	"       [-l <loopcount>]",

	/*Description*/
	"The flash subcommand allows the user to erase/test a single, or a range\n"
	"of sectors/blocks.  The -s or sector option specifies NOR flash.\n"
	"The -b or block option specifies NAND flash.\n"
	"Looping on any option allows verification and stress testing.\n\n"
	"EXAMPLES:\n"
	"util flash -fecc 0x20000:0x470000:\n"
	"util erase s bitstream\n"
	"util erase s 30:40  -verify\n"
	"util flash -b 0:1023 -rand 1 -l 10\n"
	"util flash -b 0:100 -pat 0x55555555 -l -1\n",
	
	util_flash_options
	};/*end t_usage util_flash_usage*/



/******************************************************************************
*
*	FUNCTION: util_flash()
*	DATE:	
*
******************************************************************************/
s32 util_flash(int argc, char *argv[])
{
u32  index = 0, rc = 0;
u32  erase_flag = 0, verify_flag = 0, pat_flag = 0, rand_flag = 0;;
u32  nor_flag	= 0, nand_flag = 0;
u32  loop_cnt	= 1, cur_loop = 1;
u32	 test_pat	= 0;
u32	 seed		= 0;
u32  block		= 0;
u32  offset		= 0;
u32  ecc_err_cnt;
char *cptr1;
char *protect_p;
u32  i, j;
char msg[MAX_MSG_LEN];			/*message buffer*/
s32  start = 0, end = 0;		/*starting and ending sector/block numbers*/

u32	start_tstamp;				/*time stamp vars*/
char tstamp_str[MAX_TSTAMP_STR_LEN];

flash_info_t *fi = (flash_info_t*)flash_info;


//	Get any options
mcoptind = 2;	
while( (index = mcgetopt(argc, argv, 
	  "-s:1 -b:1 -erase -verify -rand:1 -pat:1 -l:1 -nand_init "
	  "-off2block:1 -block2off:1 -nor_info -nor_id -fecc:1 -ecc_scan")) 
	  != MCGETOPT_EOF)
	{
	switch(index) 
		{
		case 0 :		/*-s sector option*/
			nor_flag = 1;
			if((rc = parse_sector(mcoptarg[0], &start, &end, FLASH_NOR, msg)) != 0)
				{
				printf("%s %s:%s\n", argv[0], argv[1], msg);
				return(rc);
				}
			break;

		case 1 :		/*-b block option*/
			nand_flag = 1;
			if((rc = parse_sector(mcoptarg[0], &start, &end, FLASH_NAND, msg)) != 0)
				{
				printf("%s %s:%s\n", argv[0], argv[1], msg);
				return(rc);
				}
			break;

		case 2 :		/*erase command*/
			erase_flag = 1;
			break;

		case 3 :		/*verify after erase*/
			verify_flag = 1;
			printf("Not yet supported\n");
			return(0);
			break;

		case 4 :		/*random pattern test*/
			rand_flag = 1;

			/* test_pat is the seed*/
			seed = (u32)simple_strtol( mcoptarg[0], &cptr1, 0 );

			if( (s32)cptr1 < (strlen(mcoptarg[0]) + (s32)mcoptarg[0]) ) 
				{	
				printf("%s %s:ERROR: -l argument \"%s\" cannot be converted "
						"to int.\n", argv[0], argv[1], mcoptarg[0]);
				return(E_OPTARG_INT_CONVERSION);
				}			
			break;

		case 5 :		/*pattern test*/
			pat_flag = 1;
			test_pat = (u32)simple_strtol( mcoptarg[0], &cptr1, 0 );

			if( (s32)cptr1 < (strlen(mcoptarg[0]) + (s32)mcoptarg[0]) ) 
				{	
				printf("%s %s:ERROR: -l argument \"%s\" cannot be converted "
						"to int.\n", argv[0], argv[1], mcoptarg[0]);
				return(E_OPTARG_INT_CONVERSION);
				}
			break;


		case 6 :		/*loop count*/
			loop_cnt = (u32)simple_strtol( mcoptarg[0], &cptr1, 0 );

			if( (s32)cptr1 < (strlen(mcoptarg[0]) + (s32)mcoptarg[0]) ) 
				{	
				printf("%s %s:ERROR: -l argument \"%s\" cannot be converted "
						"to int.\n", argv[0], argv[1], mcoptarg[0]);
				return(E_OPTARG_INT_CONVERSION);
				}
			break;


		case 7 :		/*Initialize NAND device*/
			printf("Initialize the NAND device.\n");
			nand_init();
			return(0);
			break;


		case 8 :		/*off2block*/
			offset = (u32)simple_strtol( mcoptarg[0], &cptr1, 0 );

			if( (s32)cptr1 < (strlen(mcoptarg[0]) + (s32)mcoptarg[0]) ) 
				{	
				printf("%s %s:ERROR: -off2block \"%s\" cannot be converted "
						"to int.\n", argv[0], argv[1], mcoptarg[0]);
				return(E_OPTARG_INT_CONVERSION);
				}

			printf("Offset 0x%x is in block %d\n", offset, OFFSET_TO_BLOCK(offset) );

			return(0);
			break;

		case 9 :		/*block2off*/
			block = (u32)simple_strtol( mcoptarg[0], &cptr1, 0 );

			if( (s32)cptr1 < (strlen(mcoptarg[0]) + (s32)mcoptarg[0]) ) 
				{	
				printf("%s %s:ERROR: -block2off \"%s\" cannot be converted "
						"to int.\n", argv[0], argv[1], mcoptarg[0]);
				return(E_OPTARG_INT_CONVERSION);
				}

			printf("Block %d is at offset 0x%x\n", block, BLOCK_TO_OFFSET(block) );

			return(0);
			break;

		case 10 :		/*dump nor info stuct*/
			printf(	"\nNOR flash_info_t struct:\n"
					"Size=0x%x  Sector count=%d  Flash ID=0x%x\n"
					"portwidth=%d    chipwidth=%d      buf_size=%d\n"
					"erase_blk_tout=%d  write_tout=%d\n"
					"buffer_write_tout=%d  vendor=%d  cmd_reset=0x%x\n"
					"interface=%d  legacy_unlock=%d\n",
					fi->size, fi->sector_count, fi->flash_id,
					fi->portwidth, fi->chipwidth, fi->buffer_size,
					fi->erase_blk_tout, fi->write_tout, 
					fi->buffer_write_tout, fi->vendor, fi->cmd_reset,
					fi->interface, fi->legacy_unlock);

			printf("\nPhysical sector start addresses:\n");
			for(i = 0, j = 0; i < CFG_MAX_FLASH_SECT; ++i)
				{
				if(fi->protect[i])
					protect_p = "RO";
				else
					protect_p = "  ";

				printf("S%02d:0x%08lx %s  ", i, fi->start[i], protect_p);

				if( j++ == 3)	/*4 per line*/
					{
					j = 0;
					printf("\n");
					}
				}

			printf("\n\n");
			return(0);
			break;

		case 11 :		/*ID the NOR device*/
			post_id_nor(msg);
			printf("%s\n", msg); 
			return(0);
			break;

		case 12 :		/*create a list of addrs to force ecc read errors*/
			if((rc = create_bad_ecc_addrs(mcoptarg[0], msg)) != 0)
				{
				printf("%s %s:ERROR: %s\n", argv[0], argv[1], msg);
				return(rc);
				}
			else
				{
				printf("%s\n", msg);
				return(0);
				}
			break;

		case 13 :		/*scan NAND for ECC errors*/
			if( (rc = nand_ecc_scan(CFG_FLASH_START_BLOCK, CFG_FLASH_NAND_SIZE,
							NAND_SCAN_NO_BAD, &ecc_err_cnt, NAND_SCAN_NOREBOOT, msg)) != 0)
				{
				printf("%s %s:ERROR: %s\n", argv[0], argv[1], msg);
				printf("Error Count returned = %d\n", ecc_err_cnt);
				return(rc);
				}
			else
				{
				printf("%s\n", msg);
				return(0);
				}
			
		default:		/*not good*/
			rc = mcgetopt_error(index, msg, mcoptarg[0]);
			printf("%s %s:%s\n", argv[0], argv[1], msg);
			return(rc);
			break;
		}
	}/*end mcgetopt while*/


/*****************************************************************************
*	User friendly checks
****************************************************************************/
if(nor_flag && nand_flag)
	{
	printf("%s %s:ERROR cannot specify both -s and -b options\n", 
															argv[0], argv[1]);
	return(E_FLASH_NOR_AND_NAND_FLAG);
	}

/*****************************************************************************
*	Run selected operations
*****************************************************************************/
start_tstamp = ABS_get_timer(0);
while(loop_cnt)
	{
	/*	-pat option, 32 bit pattern test*/
	if(pat_flag && nor_flag)
		if( (rc = flash_nor_pat_test(start, end, test_pat, msg)) != 0)
			break;

	if(pat_flag && nand_flag)
		if( (rc = flash_nand_pat_test(start, end, test_pat, msg)) != 0)
			break;

	/*	-rand option*/
	if(rand_flag && nor_flag)
		if( (rc = flash_nor_randpat_test(start, end, seed, msg)) != 0)
			break;

	if(rand_flag && nand_flag)
		if( (rc = flash_nand_randpat_test(start, end, seed, msg)) != 0)
			break;

	/*	-erase option*/
	if(erase_flag && nor_flag)
		if((rc=ABS_erase_nor(start, end, VERBOSE_ON, verify_flag, msg)) != 0)
			break;

	if(erase_flag && nand_flag)
		if((rc=ABS_erase_nand(start, end, VERBOSE_ON, verify_flag, msg)) != 0)
			break;

			
	calc_tstamp_str(start_tstamp, tstamp_str);
	printf("Completed loop %d,  Total time %s\n", cur_loop, tstamp_str);
	if(loop_cnt != -1)
		--loop_cnt;
	++cur_loop;
	++seed;
	}

/*	Any test failure?*/
if(rc)
	{
	printf("%s %s:%s\n", argv[0], argv[1], msg);
	return(rc);
	}



/*****************************************************************************
*	
*****************************************************************************/
return(0);
}/* end FUNCTION: util_flash()*/






/*****************************************************************************
*
*	FUNCTION: parse_sector()
*	DATE:	11-19-04
*
*	This function take the various user entered syntax'es of the "sector"
*	and "block" options and parses out the start and ending sector/block. 
*	The address syntax choices can be:
*	1)  <single sector>
*	2)  <start sector>:<end sector>		':' seperated
*	3)  <start sector>#<sector count>	'#' seperated
*	4)  <"image name">					Table lookup
*	
*	For hex values they must be proceeded by "0x".
*	INPUT:	str		- char string of sector syntax
*			start	- pointer for starting sector/block number
*			end		- pointer for ending sector/block number
*			flag	- FLASH_NOR or FLASH_NAND flag
*			msg		- error message return
*
******************************************************************************/
s32 parse_sector(char *str, s32 *start, s32 *end, u32 flags, 
																	char *msg)
{
char   s[PARSE_ADDR_MAX_LEN];
char   *start_p	= 0;			//start address
char   *end_p	= 0;			//end adress
char   *count_p	= 0;			//byte count
char   *bad_addr;				//used to determine a failure in simple_strtol
u32	   temp;
//t_image_sector_map *smap = (t_image_sector_map*)image_sector_table;

/*
*	Bring in the user string and make sure there is something there.
*	start_ptr allways points to start of the string
*/
strncpy(s, str, PARSE_ADDR_MAX_LEN);
s[PARSE_ADDR_MAX_LEN-1] = '\0';

if(strlen(s))
	start_p = s;			
else
	{
	sprintf(msg, "invalid user entered sector/block range.\n");
	return(E_PARSE_SECTOR_BAD_USER_STRING);
	}

/*****************************************************************************
*	If the first char is a letter, then the user has entered a image name so 
*	go search the image list to get the sector range.
*****************************************************************************/
if( isalpha(s[0]) ) 
	{
	if(strcmp(s, "help") == 0)
		{
		//HACK call util_info_flashmap()
		return(E_FLASH_IMAGENAME_HELP);
		}


	/*hack*/
	sprintf(msg, "invalid image name of \"%s\" ", s);
	return(E_FLASH_INVALID_IMAGENAME);

/*
	if( (rc = get_sector_by_name(s, start_sect, end_sect)) != 0)
		{
		sprintf(msg, "invalid image name of \"%s\" ", s);
		return(E_DIAG_FLASH_INVALID_IMAGENAME);
		}
	else
		return(0);

*/
	}


/*****************************************************************************
*	Check for an address seperator, or a count seperator
*****************************************************************************/
if( (end_p = strchr(s, ':')) != NULL)
	*end_p++	= '\0';
else if( (count_p = strchr(s, '#')) != NULL)
	*count_p++ = '\0';

/*	Get the hex value for start*/
temp = (u32)simple_strtol( start_p, &bad_addr, 0 );
if( (u32)bad_addr < (strlen(start_p) + (u32)start_p) ) 
	{
	sprintf(msg, "invalid start secto/blockr field.");
	return(E_PARSE_SECTOR_BAD_START_ADDR);
	}

*start = temp;

/*
*	If end_ptr, get the hex value for the ending sector/block.
*	If count_ptr, get the hex value and calc. the endng sector/block.
*	If neither then ending sector/block = starting sector/block.
*/
if(end_p != NULL)
	{
	temp = (u32)simple_strtol( end_p, &bad_addr, 0 );
	if( (u32)bad_addr < (strlen(end_p) + (u32)end_p) ) 
		{
		sprintf(msg, "invalid end address field.");
		return(E_PARSE_SECTOR_BAD_END_ADDR);
		}
	*end = temp;
	}
else if(count_p != NULL)
	{	
	temp = (u32)simple_strtol( count_p, &bad_addr, 0 );
	if( (u32)bad_addr < (strlen(count_p) + (u32)count_p) ) 
		{
		sprintf(msg, "invalid count field.");
		return(E_PARSE_SECTOR_BAD_COUNT);
		}

	if(temp)
		--temp;

	*end = *start + temp;
	}
else
	{/*only a single sector was entered*/
	*end = *start ;	
	}

/*User friendly check, make sure the sector/block numbers are in range.*/
if(flags & FLASH_NOR)
	{
	if( (*start < CFG_FLASH_START_SECT) || 
		(*start > CFG_FLASH_END_SECT) ||
		(*end < CFG_FLASH_START_SECT) || 
		(*end > CFG_FLASH_END_SECT) )
		{
		sprintf(msg, "sectors must be between %d and %d", 
								CFG_FLASH_START_SECT, CFG_FLASH_END_SECT);
		return(E_UTIL_SECTOR_OUT_OF_RANGE);
		}
	}
else
	{
	if( (*start < CFG_FLASH_START_BLOCK) || 
		(*start > CFG_FLASH_END_BLOCK) ||
		(*end < CFG_FLASH_START_BLOCK) || 
		(*end > CFG_FLASH_END_BLOCK) )
		{
		sprintf(msg, "blocks must be between %d and %d", 
								CFG_FLASH_START_BLOCK, CFG_FLASH_END_BLOCK);
		return(E_UTIL_SECTOR_OUT_OF_RANGE);
		}
	}


if(*start > *end)
	{
	sprintf(msg, "Starting sector/block cannot be greater than ending");
	return(E_UTIL_SECTOR_OUT_OF_RANGE);
	}

return(0);
}//end FUNCTION: parse_sector()




/*****************************************************************************/




/*****************************************************************************
*	FUNCTION:	post_id_nor()
*	DATE:		05-22-07
*	SM
*	
*	Call into existing nor flash structures and verify that the nor device
*	found is on the HW inventory list.
*
*	INPUT:	msg - pointer to return text message
*	RETURN: 0 - A valid ID was found
*			non-0 - A valid was NOT found
*****************************************************************************/
s32 post_id_nor(char *msg)
{
    s32 rc = 0;
    u16 manuf_id;
    u16 dev_id;
    u32 i;

    flash_info_t *finfo = &flash_info[0];
    
    if( (rc = flash_get_manuf_dev_id(finfo, &manuf_id, &dev_id)) != 0)
    {
		sprintf(msg, "ERROR:Reading the NOR flash ID, RC=0x%x", rc);
        return(W_FLASH_NOR_READ_ID_BUSY);
    }
    
    /*	See if NOR ID is in the HW inventory table*/
    for(i = 0; hw_inv_nor[i].idmanuf != 0; ++i)
	{
        if((hw_inv_nor[i].idmanuf == manuf_id) && (hw_inv_nor[i].iddev == dev_id))
            break;
        if((hw_inv_nor[i].idmanuf == manuf_id) && (hw_inv_nor[i].iddev != dev_id))
        {
            sprintf(msg, "Unknown %s device, ID=0x%x", hw_inv_nor[i].manuf, dev_id);
            return(W_FLASH_NOR_UNKNOWN_DEV_ID);
        }
	}

    if(hw_inv_nor[i].idmanuf != 0)
	{
        sprintf(msg, "Found %s %s NOR Flash", hw_inv_nor[i].manuf, hw_inv_nor[i].dev);
		rc = 0;
	}
    else
	{
        sprintf(msg, "Unknown NOR manufacturer, Manuf ID=0x%02x", manuf_id);
        rc = W_FLASH_NOR_UNKNOWN_MANUF_ID;
	}
    
	return(rc);

            
}/*end FUNCTION: post_id_nor()*/


/*****************************************************************************
*	FUNCTION:	post_id_nand()
*	DATE:		12-19-06
*	
*	Call into existing nand flash structures and verify that the nand device
*	found is on the HW inventory list.
*****************************************************************************/
s32 post_id_nand(char *msg)
{
u8	b1, b2, b3, b4;					/*4 bytes of ID*/
u32 i;
s32	rc = 0;

/*global structs containing nand info*/
nand_info_t *mtd	= &nand_info[0];		
struct nand_chip	*this = mtd->priv;

			
/*	Issue ID cmd and read back 4 id bytes*/
this->cmdfunc (mtd, NAND_CMD_READID, 0x00, -1);
b1 = this->read_byte(mtd);
b2 = this->read_byte(mtd);
b3 = this->read_byte(mtd);
b4 = this->read_byte(mtd);

/*	See if NAND ID is in the HW inventory table*/
for(i = 0; hw_inv_nand[i].id != 0; ++i)
	{
	if(hw_inv_nand[i].id == b1)
		break;
	}

if(hw_inv_nand[i].id != 0)
	{
	sprintf(msg, "Found %s %s", hw_inv_nand[i].manuf, mtd->name);
	}
else
	{
	sprintf(msg, "Unknown NAND manufacturer, ID=0x%x", b1);
	rc = W_FLASH_NAND_UNKNOWN_MANUF_ID;
	}


/*
*	Make selftest verify that we have < 20 bad blocks
*	This would be a vendor mode, as blocks go bad in the field
*	we cannot start failing ST.
*	Verify ID to make sure it is on the acceptable list.
*/
return(rc);		
}/*end FUNCTION: post_id_nand()*/



/*****************************************************************************
*	FUNCTION:	post_badblock_nand()
*	DATE:		12-19-06
*	
*	If in vendor mode we need to make sure the number of bad blocks do not
*	exceed the MAX_INVALID_BLOCKS_WHEN_NEW.  We cannot check this all the time
*	as blocks may go bad as time progresses in the field.  But we can make 
*	sure at PCBA when the part is new, that we are not starting with
*	abnormally high bad blocks.
*
*	If not in vendor mode we simply display an info message on the number of
*	bad blocks.  We will create an env variable with the starting bad block
*	count, then we can see if any are going bad over time.   If the env area
*	gets blown away, oh well, we did the best we can. This is info, not a test.
*****************************************************************************/
s32 post_badblock_nand(u32 *count, char *msg)
{
u32 badblock_cnt	= 0;
u32 offset			= 0;
char buf[20];
char *env_p;

/*global structs containing nand info, cmc will always have 1 NAND*/
nand_info_t *nand	= &nand_info[0];	

for (offset = 0; offset < nand->size; offset += nand->erasesize)
	if (nand_block_isbad(nand, offset))
		++badblock_cnt;

*count = badblock_cnt;


sprintf(buf, "%d", badblock_cnt);					/*make into string*/
if((env_p = (char*)getenv(BAD_BLOCK_ENV_NAME)) == 0)
	{
	env_p = buf;
	setenv(BAD_BLOCK_ENV_NAME, buf);
	saveenv();
	sprintf(msg, "NAND remapped blocks, new snapshot = %s, current = %d", 
														env_p, badblock_cnt);

	/*env missing=PCBA test, fail it exceed max bad blocks allowed when new*/
	if(is_vendor_mode())
		{
		if(badblock_cnt > MAX_INVALID_BLOCKS_WHEN_NEW)
			{/*	Only Dell/PCBA test can see this message*/
			sprintf(msg, "Current bad NAND blocks of %d, exceed max allowed %d",
									badblock_cnt, MAX_INVALID_BLOCKS_WHEN_NEW);
			return(E_FLASH_NAND_EXCESSIVE_BADBLOCKS);
			}
		}
	}
else
	{
	sprintf(msg, "NAND remapped blocks, original = %s, current = %d", 
														env_p, badblock_cnt);
	}

return(0);		
}/*end FUNCTION: post_badblock_nand()*/


/*****************************************************************************
*	FUNCTION:	flash_nand_pat_test()
*	DATE:		1-09-07
*	
*	The -pat option takes as user input a 32 bit value to test nand flash.
*	1) erase and verify that nand flash is erased. 
*	2) fill a memory buffer with the data pattern.
*	3) write block/page number at beginning of each page in memory buffer.
*	4) copy memory to nand.
*	5) clear memory buffer
*	6) read one nand block at a time, 
          Verify the unique block/page number and verify pattern
*	7) invert data pattern
*	8) repeat steps 1-6
*	9) erase and verify nand
*
*****************************************************************************/
s32 flash_nand_pat_test(s32 start_blk, s32 end_blk, u32 test_pat, char *msg)
{
u32 i; 
u32 rc;
s32 req_bcnt, act_bcnt;
u32	*mem_p;
u32 tmp_blk;
u32 tmp_page;
u32 page_marker;
u32 page_bytes;
u32 membuf;
u32 next_pass	= 2;
u32 start_phys	= BLOCK_TO_PHYSADDR(start_blk);
u32 end_phys	= (BLOCK_TO_PHYSADDR(end_blk) + CFG_NAND_BLOCK_SIZE) - 1;

while(next_pass)
	{
	/*	Erase and verify flash*/
	printf("Erase nand 0x%x -to- 0x%x, %d blocks                      \r", 
						start_phys, end_phys, ( (end_blk - start_blk) + 1) );

	if((rc = ABS_erase_nand(start_blk, end_blk, VERBOSE_OFF, 1, msg)) != 0)
		return(rc);
	
	/*fill memory buffer with 32 bit pattern*/
	req_bcnt = ( ((end_blk - start_blk) + 1) * CFG_NAND_BLOCK_SIZE);
	for(i = 0, mem_p = (u32*)CFG_LOAD_ADDR; i < req_bcnt; i += 4)
		*mem_p++ = test_pat;
	

	/*write 32 bit "[31:16]block [15:0]page number" at beginning of each page*/
	mem_p = (u32*)CFG_LOAD_ADDR; 
	for(tmp_blk = start_blk;  tmp_blk < end_blk;  ++tmp_blk)
		{
		page_marker = tmp_blk << 16;
		for(i = 0; i < CFG_NAND_PAGE_COUNT; ++i)
			{
			*mem_p = page_marker + i;
			mem_p = (u32*)((u32)mem_p + CFG_NAND_PAGE_SIZE);
			}
		}

	/*	Write a block at a time to nand*/
	for(membuf = CFG_LOAD_ADDR, tmp_blk  = start_blk;
		tmp_blk < end_blk;
		membuf += CFG_NAND_BLOCK_SIZE, ++tmp_blk)
		{
		printf("Write nand 0x%x block %d with 0x%x                 \r", 
								BLOCK_TO_OFFSET(tmp_blk), tmp_blk, test_pat);
		if((rc = ABS_write_nand(membuf, BLOCK_TO_OFFSET(tmp_blk), 
											CFG_NAND_BLOCK_SIZE, msg)) != 0)
			return(rc);

		if (ctrlc())		/* check for ctrl-c to abort... */
			{
			sprintf(msg, "CTRL-C early exit.                  ");
			return(EXIT_BY_CTRLC);  
			}
		}
	

	/*	Clear memory buffer, then read/verify a block at a time*/
	memset((void*)CFG_LOAD_ADDR, 0,  req_bcnt);
	tmp_blk  = start_blk;
	mem_p = (u32*)CFG_LOAD_ADDR;
	while(tmp_blk < end_blk)
		{
		printf("Read/verify nand 0x%x block %d buffer 0x%x                 \r", 
								BLOCK_TO_OFFSET(tmp_blk), tmp_blk, (u32)mem_p);
	
		if((rc = ABS_read_nand((u32)mem_p, BLOCK_TO_OFFSET(tmp_blk), 
									CFG_NAND_BLOCK_SIZE, &act_bcnt, msg)) != 0)
			return(rc);
	
		/*	Ignor bad blocks*/
		if( is_on_bad_blocklist(BLOCK_TO_OFFSET(tmp_blk)) )
			{
			++tmp_blk;
			mem_p = (u32*)((u32)mem_p + CFG_NAND_BLOCK_SIZE);
			continue;
			}
		
		/*	Compare each page in the block*/
		for(tmp_page = 0; tmp_page < CFG_NAND_PAGE_COUNT; ++tmp_page)
			{
			/*	Verify the unique block number, first 32 bits of page*/
			page_marker = (tmp_blk << 16) + tmp_page;
			if( *mem_p != page_marker )
				{
				sprintf(msg, 
						"ERROR:Nand offset=0x%x block=%d page=%d mem_p=0x%x "
						"exp=0x%x rec=0x%x",
						BLOCK_TO_OFFSET(tmp_blk), tmp_blk, tmp_page, 
						(u32)mem_p, page_marker, *mem_p);
				return(E_FLASH_NAND_BAD_BLOCKPAGE_NUMBER);
				}

			/*	Compare all bytes with a page*/
			++mem_p;			/*bump past unique page_marker*/
			page_bytes = 4;		/*start at next word*/
			while(page_bytes < CFG_NAND_PAGE_SIZE)
				{
				if(*mem_p != test_pat)
					{
					sprintf(msg,	"ERROR:Nand block=%d page=%d page offset="
									"0x%x exp=0x%x rec=0x%x", 
									tmp_blk, tmp_page, page_bytes, test_pat, *mem_p);
					return(E_FLASH_NAND_PATTERN_FAILURE);
					}
				page_bytes += 4;
				++mem_p;
				}/*end data while*/

			}/*end page for*/

		++tmp_blk;

		if (ctrlc())		/* check for ctrl-c to abort... */
			{
			sprintf(msg, "CTRL-C early exit.                  ");
			return(EXIT_BY_CTRLC);
			}
		}/*end block while*/

	test_pat = ~test_pat;	/*invert test pattern*/
	--next_pass;
	}

/*	Leave flash erased*/
printf("Pattern test passed 0x%x -to- 0x%x, %d blocks, re-erased           \n",
								start_phys, end_phys, ((end_blk-start_blk)+1));

if((rc = ABS_erase_nand(start_blk, end_blk, VERBOSE_OFF, 1, msg)) != 0)
	return(rc);

return(0);
}/*end FUNCTION: flash_nand_pat_test()*/






/*****************************************************************************
*	FUNCTION:	flash_nand_randpat_test()
*	DATE:		1-09-07
*	
*	The -rand option takes as user input a 32 bit seed value.  A random 
*	pattern is derived from this seed.
*	1) erase and verify that nand flash is erased. 
*	2) fill a memory buffer with the random data pattern.
*	3) copy memory to nand.
*	4) clear memory buffer, reseed the random generator
*	5) read one nand block, 
          Verify the random data pattern
*	6) erase and verify nand
*
*****************************************************************************/
s32 flash_nand_randpat_test(s32 start_blk, s32 end_blk, u32 seed, char *msg)
{
u32 i; 
u32 rc;
s32 req_bcnt, act_bcnt;
u32	*mem_p;
u32 tmp_blk;
u32 membuf;
u32 exp_pat;
u32 start_phys	= BLOCK_TO_PHYSADDR(start_blk);
u32 end_phys	= (BLOCK_TO_PHYSADDR(end_blk) + CFG_NAND_BLOCK_SIZE) - 1;

/*	Erase and verify flash*/
printf("Erase nand 0x%x -to- 0x%x, %d blocks                      \r", 
						start_phys, end_phys, ( (end_blk - start_blk) + 1) );

if((rc = ABS_erase_nand(start_blk, end_blk, VERBOSE_OFF, 1, msg)) != 0)
	return(rc);
	
/*fill memory buffer with random pattern*/
req_bcnt = ( ((end_blk - start_blk) + 1) * CFG_NAND_BLOCK_SIZE);
srand32(seed);
printf("Filling memory with random pattern, seed=0x%x            \r", seed);
for(i = 0, mem_p = (u32*)CFG_LOAD_ADDR; i < req_bcnt; i += 4)
	*mem_p++ = rand32();

/*	Write a block at a time to nand*/
for(membuf = CFG_LOAD_ADDR, tmp_blk  = start_blk;
	tmp_blk < end_blk;
	membuf += CFG_NAND_BLOCK_SIZE, ++tmp_blk)
	{
	printf("Write nand 0x%x block %d seed 0x%x                 \r", 
								BLOCK_TO_OFFSET(tmp_blk), tmp_blk, seed);
	if((rc = ABS_write_nand(membuf, BLOCK_TO_OFFSET(tmp_blk), 
											CFG_NAND_BLOCK_SIZE, msg)) != 0)
		return(rc);

	if (ctrlc())		/* check for ctrl-c to abort... */
		{
		sprintf(msg, "CTRL-C early exit.                  ");
		return(EXIT_BY_CTRLC); 
		}
	}
	

/*	Clear memory buffer*/
memset((void*)CFG_LOAD_ADDR, 0,  req_bcnt);

/*	Read/verify a block at a time*/
srand32(seed);	
tmp_blk  = start_blk;
mem_p = (u32*)CFG_LOAD_ADDR;
while(tmp_blk < end_blk)
	{
	printf("Read/verify nand 0x%x block %d buffer 0x%x                 \r", 
								BLOCK_TO_OFFSET(tmp_blk), tmp_blk, (u32)mem_p);
	
	if((rc = ABS_read_nand((u32)mem_p, BLOCK_TO_OFFSET(tmp_blk), 
									CFG_NAND_BLOCK_SIZE, &act_bcnt, msg)) != 0)
		return(rc);
	
	/*	Ignor bad blocks*/
	if( is_on_bad_blocklist(BLOCK_TO_OFFSET(tmp_blk)) )
		{
		++tmp_blk;
		mem_p = (u32*)((u32)mem_p + CFG_NAND_BLOCK_SIZE);

		/*	We must trash this block of random patterns*/
		for(i = 0; i < CFG_NAND_BLOCK_SIZE; i+=4)
			exp_pat = rand32();
		continue;
		}
		
	/*	Compare each word in a block*/
	for(i = 0; i < CFG_NAND_BLOCK_SIZE; i+=4, ++mem_p)
		{
		exp_pat = rand32();
		if(*mem_p != exp_pat)
			{
			sprintf(msg, 
				"ERROR:failing offset=0x%x block=%d mem_p=0x%x exp=0x%x rec=0x%x", 
				(BLOCK_TO_OFFSET(tmp_blk) + i), tmp_blk, (u32)mem_p, 
				exp_pat, *mem_p);
			return(E_FLASH_NAND_RANDOM_FAILURE);
			}
		}/*end page for*/

	++tmp_blk;
	if (ctrlc())		/* check for ctrl-c to abort... */
		{
		sprintf(msg, "CTRL-C early exit.                  ");
		return(EXIT_BY_CTRLC);
		}
	}/*end block while*/

/*	Leave flash erased*/
printf("Random test passed 0x%x -to- 0x%x, %d blocks, re-erased           \n",
								start_phys, end_phys, ((end_blk-start_blk)+1));

if((rc = ABS_erase_nand(start_blk, end_blk, VERBOSE_OFF, 1, msg)) != 0)
	return(rc);

return(0);
}/*end FUNCTION: flash_nand_randpat_test()*/


/*****************************************************************************
*	FUNCTION:	flash_nor_pat_test()
*	DATE:		2-02-07
*	
*	The -pat option takes as user input a 32 bit value to test nand flash.
*	1) erase and verify that nand flash is erased. 
*	2) fill a memory buffer with the data pattern.
*	3) write block/page number at beginning of each page in memory buffer.
*	4) copy memory to nand.
*	5) clear memory buffer
*	6) read one nand block at a time, 
          Verify the unique block/page number and verify pattern
*	7) invert data pattern
*	8) repeat steps 1-6
*	9) erase and verify nand
*
*****************************************************************************/
s32 flash_nor_pat_test(s32 start_sect, s32 end_sect, u32 test_pat, char *msg)
{
u32 i; 
u32 rc;
u32	*mem_p;
u32 tmp_sect;
u32 next_pass	= 2;
u32 start_phys	= SECTOR_TO_PHYSADDR(start_sect);
u32 end_phys	= (SECTOR_TO_PHYSADDR(end_sect) + CFG_SECT_SIZE) - 1;

while(next_pass)
	{
	/*	Erase and verify flash*/
	if((rc=ABS_erase_nor(start_sect, end_sect, VERBOSE_ON, 1, msg)) != 0)
		return(rc);
	
	/*	Create a sectors worth of data*/
	for(i = 0, mem_p = (u32*)CFG_LOAD_ADDR; i < CFG_SECT_SIZE; i += 4)
		*mem_p++ = test_pat;	

	/*	Write a sector at a time to nor*/
	for(tmp_sect = start_sect, mem_p = (u32*)CFG_LOAD_ADDR;
		tmp_sect < end_sect; ++tmp_sect)
		{
		printf("Write addr 0x%x sector %.2d [%d:%d] with 0x%x             \r", 
							SECTOR_TO_PHYSADDR(tmp_sect), tmp_sect, 
							start_sect, end_sect, test_pat);

		*mem_p = tmp_sect;	/*write unique sector number at the start*/
		if((rc = ABS_flash_write_nor((char*)CFG_LOAD_ADDR, 
					SECTOR_TO_PHYSADDR(tmp_sect), CFG_SECT_SIZE, msg)) != 0)
            return(rc);

		if (ctrlc())		/* check for ctrl-c to abort... */
			{
			sprintf(msg, "CTRL-C early exit.                  ");
			return(EXIT_BY_CTRLC);  
			}
		}
	

	/*	Read/verify a sector at a time*/
	for(tmp_sect  = start_sect; tmp_sect < end_sect; ++tmp_sect)
		{
		mem_p = (u32*)SECTOR_TO_PHYSADDR(tmp_sect);
		printf("Read/verify addr 0x%x sector %.2d [%d:%d]                  \r", 
								(u32)mem_p, tmp_sect, start_sect, end_sect);
	
		if(*mem_p != tmp_sect)
			{
			sprintf(msg, 
			"ERROR:Unique Sector Number addr=0x%x sector=%d exp=0x%x rec=0x%x",
			SECTOR_TO_PHYSADDR(tmp_sect), tmp_sect, tmp_sect, *mem_p); 
			return(E_FLASH_NOR_BAD_UNIQUE_SECTOR_NUM);
			}

		++mem_p;	/*start after the 1st word*/
		for(i = 4; i < CFG_SECT_SIZE; i += 4, ++mem_p)
			{
			if(*mem_p != test_pat)
				{
				sprintf(msg, "ERROR:addr 0x%x exp=0x%x rec=0x%x", 
									(u32)mem_p, test_pat, *mem_p);
				return(E_FLASH_NOR_PATTERN_FAILURE);
				}
			}

		if (ctrlc())		/* check for ctrl-c to abort... */
			{
			sprintf(msg, "CTRL-C early exit.                  ");
			return(EXIT_BY_CTRLC);
			}
		}/*end sector while*/

	test_pat = ~test_pat;	/*invert test pattern*/
	--next_pass;
	}

/*	Leave flash erased*/
printf("Pattern test passed 0x%x -to- 0x%x, [%d:%d]                       \n",
							start_phys, end_phys, start_sect, end_sect);

if((rc=ABS_erase_nor(start_sect, end_sect, VERBOSE_ON, 1, msg)) != 0)
	return(rc);

return(0);
}/*end FUNCTION: flash_nor_pat_test()*/



/*****************************************************************************
*	FUNCTION:	flash_nor_randpat_test()
*	DATE:		2-3-07
*	
*	The -rand option takes as user input a 16 bit seed value.  A random 
*	pattern is derived from this seed.
*	1) erase and verify that nor flash is erased. 
*	2) fill a memory buffer with the random data pattern.
*	3) copy memory to nor.
*	4) clear memory buffer, reseed the random generator
*	5) read one nor sector, 
          Verify the random data pattern
*	6) erase and verify nor
*
*****************************************************************************/
s32 flash_nor_randpat_test(s32 start_sect, s32 end_sect, u32 seed, char *msg)
{
u32 i; 
u32 rc;
u16	*mem_p;
u32 req_bcnt;
u32 tmp_sect;
u16 exp_data;
u32 start_phys	= SECTOR_TO_PHYSADDR(start_sect);
u32 end_phys	= (SECTOR_TO_PHYSADDR(end_sect) + CFG_SECT_SIZE) - 1;

/*	Erase and verify flash*/
if((rc=ABS_erase_nor(start_sect, end_sect, VERBOSE_ON, 1, msg)) != 0)
	return(rc);
	
/*fill memory buffer with random pattern*/
req_bcnt = ( ((end_sect - start_sect) + 1) * CFG_SECT_SIZE);
srand32(seed);
printf("Filling memory with random pattern, seed=0x%x            \r", seed);
for(i = 0, mem_p = (u16*)CFG_LOAD_ADDR; i < req_bcnt; i += 2)
	*mem_p++ = (u16)rand32();

/*	Write a sector at a time to nor*/
for(mem_p = (u16*)CFG_LOAD_ADDR, tmp_sect  = start_sect;
	tmp_sect < end_sect;
	mem_p = (u16*)((u32)mem_p + CFG_SECT_SIZE), ++tmp_sect)
	{
	printf("Write addr 0x%x sector %.2d [%d:%d] with seed 0x%x         \r", 
							SECTOR_TO_PHYSADDR(tmp_sect), tmp_sect, 
							start_sect, end_sect, seed);

	if((rc = ABS_flash_write_nor((char*)mem_p, SECTOR_TO_PHYSADDR(tmp_sect),
												CFG_SECT_SIZE, msg)) != 0)
           return(rc);

	if (ctrlc())		/* check for ctrl-c to abort... */
		{
		sprintf(msg, "CTRL-C early exit.                  ");
		return(EXIT_BY_CTRLC);  
		}
	}
	

/*	Read/verify a sector at a time*/
srand32(seed);
for(tmp_sect  = start_sect; tmp_sect < end_sect; ++tmp_sect)
	{
	mem_p = (u16*)SECTOR_TO_PHYSADDR(tmp_sect);
	printf("Read/verify addr 0x%x sector %.2d [%d:%d]                 \r", 
								(u32)mem_p, tmp_sect, start_sect, end_sect);
	
	for(i = 0; i < CFG_SECT_SIZE; i += 2, ++mem_p)
		{
		exp_data = (u16)rand32();
		if(*mem_p != exp_data)
			{
			sprintf(msg, "ERROR:addr 0x%x exp=0x%x rec=0x%x", 
									(u32)mem_p, exp_data, *mem_p);
			return(E_FLASH_NOR_RANDOM_FAILURE);
			}
		}

	if (ctrlc())		/* check for ctrl-c to abort... */
		{
		sprintf(msg, "CTRL-C early exit.                  ");
		return(EXIT_BY_CTRLC);
		}
	}/*end sector while*/


/*	Leave flash erased*/
printf("Random nor test passed 0x%x -to- 0x%x, [%d:%d]                   \n",
							start_phys, end_phys, start_sect, end_sect);

if((rc=ABS_erase_nor(start_sect, end_sect, VERBOSE_ON, 1, msg)) != 0)
	return(rc);


return(0);
}/*end FUNCTION: flash_nor_randpat_test()*/



/*****************************************************************************
*	FUNCTION:	is_on_bad_blocklist()
*	DATE:		12-21-06
*	
*	See if the input NAND offset address is within a bad block. 
*	RETURN:	0 - Not on the bad block list.
*			1 - Is a bad block.
*
*****************************************************************************/
s32 is_on_bad_blocklist(s32 nand_off)
{
nand_info_t *nand	= &nand_info[0];	

if( nand_block_isbad(nand, (nand_off & ~CFG_NAND_BLOCK_SIZE_MASK)) )
	return(1);
else
	return(0);
}/*end FUNCTION: is_on_bad_blocklist() */


/*****************************************************************************
*	FUNCTION:	create_bad_ecc_addrs()
*	DATE:		6-20-08
*	
*	Debug, called from "util flash -fecc 0xaddr1:0xaddr2:"
*	From a ":" seperated NAND address list create a env variable.
*	Address are verified to be within range, and MUST be accending in size.
*
*	RETURN:	0 - Valid NAND addresses.
*			1 - User error, addr range error.
*
*****************************************************************************/
static s32 create_bad_ecc_addrs(char *addr_list, char *msg)
{
char *p;
char *bad_addr;
s32	 temp;
u32  prev_addr = 0;
u32  cnt = 0;
char buf[20];
char force_addrs[MAX_FORCE_ECC_BUF];

memset((void*)force_addrs, 0,  MAX_FORCE_ECC_BUF);
while(1)
	{
	if( (p = strchr(addr_list, ':')) != NULL)
		*p++	= '\0';
	else 
		{
		if(strlen(force_addrs))	
			{
			setenv(FECC_ADDRS_ENV_NAME, force_addrs);
			saveenv();
			sprintf(msg, "Address to force ECC - %s", force_addrs);
			return(0);
			}
		else
			{
			sprintf(msg, "No valid force ECC addresses were entered.");
			return(E_FLASH_FECC_NO_ADDRS);
			}
		}

	/*	Get the hex value for block offset*/
	temp = (u32)simple_strtol( addr_list, &bad_addr, 0 );
	if( (u32)bad_addr < (strlen(addr_list) + (u32)addr_list) ) 
		{
		sprintf(msg, "Invalid hex address");
		return(E_FLASH_FECC_INVALID_HEX);
		}

	temp = ~CFG_NAND_BLOCK_SIZE_MASK & temp;	/*force block alligned*/
	if(temp <= prev_addr)
		{
		sprintf(msg, "Addresses must be sequentially greater and non 0");
		return(E_FLASH_FECC_NONSEQ_ADDR);
		}

	if(++cnt >= MAX_FORCE_ECC_ADDRS)
		{
		sprintf(msg, "A max of %d addresses allowed", MAX_FORCE_ECC_ADDRS);
		return(E_FLASH_FECC_EXCEED_MAX_ADDR);
		}

	if( (temp < CFG_FLASH_START_BLOCK) || (temp > CFG_FLASH_NAND_SIZE) )
		{
		sprintf(msg, "Adress must be in NAND range 0x%x - 0x%x", 
								CFG_FLASH_START_BLOCK, CFG_FLASH_NAND_SIZE);
		return(E_FLASH_FECC_ADDR_OUTOF_RANGE);
		}

	prev_addr = temp;
	sprintf(buf, "0x%x ", temp);
	strcat(force_addrs, buf);
	addr_list = p;
	}		

return(0);
}/*end FUNCTION: create_bad_ecc_addrs() */


/*****************************************************************************
*	FUNCTION:	nand_ecc_scan()
*	DATE:		6-20-08
*	
*	Read the NAND device from "start" to "end" a block at a time looking for 
*	read ECC failures.  Log failures into the env ECC_ERR_BLOCKS_ENV_NAME in 
*	the format of "0xblk:<errcnt>-0xblk:<errcnt>"
*	Convert NAND addresses to block numbers as it takes less space in the env
*	area, the errcnt is how many times the block as had an ECC read error.
*	A "B" replaces the errcnt IF the block has been previously marked bad by
*	this code.
*
*	INPUT:	start - starting NAND address offset
*			end -   ending NAND address offset
*			err_cnt - Address to write how many ECC errors were detected.
*			msg -   Return message
*
*	RETURN:	0 - nand scan completed
*		    non-0  Error return code. 
*****************************************************************************/
s32 nand_ecc_scan(u32 start, u32 end, u32 bad_offset, u32 *err_cnt, 
													u32 reboot_flag, char *msg)
{
u32  current;
s32  rc = 0;
u32  fecc_flag;
u32  existing_bad_blks = 0;
char buf[CFG_NAND_BLOCK_SIZE];		/*mem location for block list*/
u32  faddrs[MAX_FORCE_ECC_BUF];	/*NAND addresses to force ECC read error*/
u8	 block_history[CFG_FLASH_TOTAL_BLOCKS];
u32  i;
nand_read_options_t opts;
nand_info_t *nand	= &nand_info[0];

*err_cnt = 0;
g_force_ecc_addr = 0;	/*default to not force ECC errors*/

/*	Create a list of potiential addresses to force(ONLY in vendor mode)*/
memset((void*)faddrs, 0,  MAX_FORCE_ECC_BUF);
fecc_flag = parse_force_ecc_err_addrs(faddrs, 1);

/*	Create array of any previous ECC errors*/
memset((void*)block_history, 0,  (CFG_FLASH_TOTAL_BLOCKS * sizeof(u8)) );
parse_ecc_err_blocks(block_history);

/*
*	In order for this function to be called, a NAND read error has occured.
*	The caller needs to pass in this NAND offset to be marked bad first.
*	This way an intermittent will not cause a rescan with no bad blocks found.
*	(Had a Dev board flake between single and double bit errors)
*/
if(bad_offset != NAND_SCAN_NO_BAD)
	{
	nand_scan_markbad(block_history, bad_offset, 0x44);	/*0x44="D", double bit err*/
	++(*err_cnt);
	}

/*	Now read each block and let the ECC code do its thing.*/
memset(&opts, 0, sizeof(opts));
for(current = start, i = 0, existing_bad_blks = 0; 
	current < end; 
	current += CFG_NAND_BLOCK_SIZE)
	{
	/*	
	*	If the factory or the kernel has marked this block bad we would not 
	*	know it, put an "X" as a visual.
	*/
	if(is_on_bad_blocklist(current))
		{
		if(fecc_flag && (current == g_force_ecc_addr) )
			++i;	/*1st forced error is already on bad list, bump to next*/

		existing_bad_blks++;
		if(0 == block_history[OFFSET_TO_BLOCK(current)])
			block_history[OFFSET_TO_BLOCK(current)] = 0x58;	/*"X"*/
		continue;
		}

	if(fecc_flag)
		g_force_ecc_addr = faddrs[i];
		
	opts.buffer	= (u_char*) buf;
	opts.length	= CFG_NAND_BLOCK_SIZE;
	opts.offset	= current;
	opts.quiet	= 1;
	if(!(current & 0x000fffff))	/*print on 1meg boundry*/
		printf("NAND ECC scan at 0x%x [0x%x - 0x%x]\r", current, start, end);

	if((rc = nand_read_opts(nand, &opts)) != 0)
		{
		if(rc != -1)
			{
			if(current == g_force_ecc_addr)
				++i;
			if(i >= MAX_FORCE_ECC_ADDRS)
				{
				g_force_ecc_addr = 0;	/*disable further force checking*/
				fecc_flag = 0;			
				}

			++(*err_cnt);
			nand_scan_markbad(block_history, current, 0x44); /*0x44="D", double bit err*/
			}
		else
			{
			sprintf(msg, "\nA non-ECC related NAND error RC=0x%x     \n", rc);
			return(rc);
			}
		}
	}/*for*/


build_ecc_err_env(block_history);
if(*err_cnt)
	printf("Detected %d ECC read error/s                 \n", *err_cnt);
else	/*should never happen UNLESS called from "util flash -ecc_scan"*/
	printf("NAND scan completed with no ECC errors        \n");


if(NAND_SCAN_REBOOT == reboot_flag)
	reset_after_fwupd();
else
	return(0);
}/*end FUNCTION: nand_ecc_scan() */


/*****************************************************************************
*	FUNCTION:	nand_scan_markbad()
*	DATE:		12-9-08
*	
*	Mark a block bad from nand_ecc_scan()
*****************************************************************************/
static s32 nand_scan_markbad(u8 *block_history, u32 offset, u8 flag)
{
nand_info_t *nand	= &nand_info[0];

block_history[OFFSET_TO_BLOCK(offset)] = flag;	/*"D" double bit error*/
nand->block_markbad(nand, offset);
printf("Add ECC block 0x%x, offset 0x%x to remap list\n", 
												OFFSET_TO_BLOCK(offset), offset);
return(0);
}/*end FUNCTION:nand_scan_markbad()*/

/*****************************************************************************
*	FUNCTION:	parse_ecc_err_blocks()
*	DATE:		6-23-08
*	FUNCTION:	build_ecc_err_env()
*	DATE:		12-9-08
*	
*	The ECC_ERR_BLOCKS_ENV_NAME env contains a list of blocks that have had
*	previous ECC errors, or have been marked bad because of erase or double
*	bit errors.  The format of the env is "blk:D-blk:E-blk:X....". 
*	D=double bit ECC error,  E=erase error, X=previously marked bad by erase 
*	error, the vendor when the board was new, or by the kernel in which the 
*	monitor code would not know.
*	NAND block numbers are stored as it takes less space than offset addresses,
*	"-" seperates blocks as a space is parsed differently and only allows 
*	16 env entries. 
*
*	RETURN:	0
*
*****************************************************************************/
static s32 parse_ecc_err_blocks(u8 *block_history)
{
char *env_p;
char *bad_addr;
char *p;
char *buf_p;
u32  blk_idx;
char env_buf[MAX_ECC_BAD_BLOCK_ENVSTR];

if((env_p = (char*)getenv(ECC_ERR_BLOCKS_ENV_NAME)) != 0)
	{
	memset((void*)env_buf, 0,  MAX_ECC_BAD_BLOCK_ENVSTR);
	strcpy(env_buf, env_p);
	buf_p = env_buf;
	while(strlen(buf_p))
		{
		if( (p = strchr(buf_p, ':')) != NULL)
			{
			*p++	= '\0';
			/*env created programaticly, should not be conversion errors*/ 
			blk_idx = (u32)simple_strtol( buf_p, &bad_addr, 0 );
			block_history[blk_idx] = *p;		/*indicator why blk is bad*/
			buf_p = p;
			p = strchr(buf_p, '-');
			buf_p = ++p;			/*skip "-" char, move to next block field*/
			}
		}
	}

return(0);
}/*end FUNCTION: parse_ecc_err_blocks()*/




static s32 build_ecc_err_env(u8 *block_history)
{
char buf[100];		
char err_blocks_buf[MAX_ECC_BAD_BLOCK_ENVSTR];
s32  saved_silent;
u32  i;
u32  actual_bad;

memset((void*)err_blocks_buf, 0,  MAX_ECC_BAD_BLOCK_ENVSTR);
for(i = 0, actual_bad = 0; i < CFG_FLASH_TOTAL_BLOCKS; ++i)
	{
	if(block_history[i])
		{
		++actual_bad;
		if(actual_bad < MAX_BAD_BLOCKS)	
			{
			/*if not 1st entry, then seperate with "-" char*/
			if(strlen(err_blocks_buf))
				strcat(err_blocks_buf, "-");	
		
			sprintf(buf, "0x%x:%c", i, (u8)block_history[i]);
			strcat(err_blocks_buf, buf);
			}
		else
			/*
			* MAX_BAD_BLOCKS is used to calc the max buffer space needed for ecc_err env
			* areas, etc.  This number WELL exceeds any realistic/unrelistic expectation.  
			* If we have this error buffer maxed out we are so screwed by this time.
			* Throw anything away above this.  nand.py test script is the only thing that
			* should care.
			*/
			{
			strcat(err_blocks_buf, "-overflow");	
			break;
			}
		}
	}

saved_silent	= set_save_silent_mode();
setenv(ECC_ERR_BLOCKS_ENV_NAME, err_blocks_buf);
saveenv();
restore_silent_mode(saved_silent);
return(0);
}/*end FUNCTION: build_ecc_err_env()*/




/*****************************************************************************
*	FUNCTION:	add_to_err_blocks()
*	DATE:		12-11-08
*	
*	Add error blocks to an existing post_ecc_block_errors
*	RETURN:	0
*
*****************************************************************************/
s32 add_to_err_blocks(int *new_bad_blocks, u8 flag)
{
u8	 block_history[CFG_FLASH_TOTAL_BLOCKS];
u32  i, idx;

/*	Create array of any previous ECC errors*/
memset((void*)block_history, 0,  (CFG_FLASH_TOTAL_BLOCKS * sizeof(u8)) );
parse_ecc_err_blocks(block_history);

for(i = 0;  new_bad_blocks[i] != BAD_BLOCK_ARRAY_END_MARKER; ++i)
	{
	idx = OFFSET_TO_BLOCK(new_bad_blocks[i]);
	if( (idx < 0) || (idx > CFG_FLASH_TOTAL_BLOCKS))
		continue;	/*something bad, ignor*/
	if(0 == block_history[idx])
		block_history[idx] = flag;
	}

build_ecc_err_env(block_history);
return(0);
}/*end FUNCTION: add_to_err_blocks()*/

/*****************************************************************************
*	FUNCTION:	parse_force_ecc_err_addrs()
*	DATE:		6-23-08
*	
*	FECC_ADDRS_ENV_NAME contains a list of NAND addresses that will force an
*	ECC read. Format is "<0xaddr1> <0xaddr2>"
*
*	RETURN:	0 - FECC_ADDRS_ENV_NAME does not exists or no addresses to force
*			1 - At least one address to force bad
*
*****************************************************************************/
s32 parse_force_ecc_err_addrs(u32 *forced_addrs, u32 clear_env_flag)
{
char *env_p;
char *buf_p;
char *bad_addr;
char *p;
char env_buf[MAX_ECC_BAD_BLOCK_ENVSTR];
u32  fecc_flag = 0;
u32  i;
s32 saved_silent;
nand_info_t *nand	= &nand_info[0];

if(is_vendor_mode() )
	{
	if((env_p = (char*)getenv(FECC_ADDRS_ENV_NAME)) != 0)
		{
		strcpy(env_buf, env_p);
		buf_p = env_buf;
		for(i = 0; strlen(buf_p) && (i < MAX_FORCE_ECC_ADDRS);   )
			{
			if( (p = strchr(buf_p, ' ')) != NULL)
				{
				*p++	= '\0';
				/*env data has already been verified in create_bad_ecc_addrs()*/ 
				forced_addrs[i] = (u32)simple_strtol( buf_p, &bad_addr, 0 );
				if(nand_block_isbad(nand, forced_addrs[i]))
					{
					printf("NOTE:Address 0x%x is already remapped\n", forced_addrs[i]); 
					forced_addrs[i] = 0;	/*in case this is the last addr*/
					}
				else
					{
					fecc_flag = 1;
					++i;
					}
				
				buf_p = p;
				}
			}

		if(clear_env_flag)
			{
			/*	Remove the forced address env, never want this to linger around*/
			saved_silent = set_save_silent_mode();
			env_buf[0] = 0;
			setenv(FECC_ADDRS_ENV_NAME, env_buf);	
			saveenv();
			restore_silent_mode(saved_silent);
			}
		}
	}

return(fecc_flag);
}/*end FUNCTION: parse_force_ecc_err_addrs()*/








/*****************************************************************************
*	FUNCTION:	build_single_bit_env()
*	DATE:		1-15-09
*	
*	The SINGLE_BLOCKS_LIST_ENV_NAME env contains a list of blocks that have 
*	had a single bit error corrected against it, and a count on how many 
*	times it has been corrected.
*	NAND block numbers are stored instead of full offset addrs to save space
*	"-" seperates blocks as a space is parsed differently and only allows 
*	16 env entries. 
*
*	INPUT:  err_offset_list - is a list of NAND "page" offsets.  It is possible
*			to have multiple corrected errors on a single block as there are 
*			64 pages per block.
*	RETURN:	0
*
*****************************************************************************/
int build_single_bit_env(int *err_offset_list)
{
char *env_p;
char *bad_addr;
char *p;
char *e;
char *buf_p;
u32  offset_idx;
u32  single_cnt;
s32  saved_silent;
u32  i;
u32  idx;
u32  accumulated_singles;
u32  accumulated_blocks;
u32	 block_history[CFG_FLASH_TOTAL_BLOCKS];
char buf[100];		
char env_buf[MAX_ECC_BAD_BLOCK_ENVSTR];
char env_count[12];

/*	Parse out an existing single bit error env*/ 
memset((void*)env_buf, 0,  MAX_ECC_BAD_BLOCK_ENVSTR);
memset((void*)block_history, 0,  sizeof(block_history));
if((env_p = (char*)getenv(SINGLE_BLOCKS_LIST_ENV_NAME)) != 0)
	{
	strcpy(env_buf, env_p);
	buf_p = env_buf;
	while(strlen(buf_p))
		{
		if( (p = strchr(buf_p, ':')) != NULL)
			{
			*p++	= '\0';
			if( (e = strchr(p, '-')) != NULL)
				{
				*e++	= '\0';
				}
			else
				break;

			/*env created programaticly, should not be conversion errors*/ 
			offset_idx = (u32)simple_strtol( buf_p, &bad_addr, 0 );
			single_cnt = (u32)simple_strtol( p, &bad_addr, 0 );
			block_history[offset_idx] = single_cnt;	/*total single bit so far*/
			buf_p = e;
			}
		else
			break;
		}
	}

/*If the page offset exist in the passed in error offset list, increment the count*/
for(i = 0; ((err_offset_list[i] != BAD_BLOCK_ARRAY_END_MARKER) &&
		    (i < MAX_SBE_PAGES))
			; ++i)
	{
	idx = OFFSET_TO_BLOCK( (err_offset_list[i] & 0xfffe0000) ); /*page->offset->blk*/
	block_history[idx] += 1;
	}
	
/*Create the new env and write it out, reuse env_buf */
memset((void*)env_buf, 0,  MAX_ECC_BAD_BLOCK_ENVSTR);
for(i = 0, accumulated_singles = 0, accumulated_blocks = 0; 
	i < CFG_FLASH_TOTAL_BLOCKS; ++i)
	{
	if(block_history[i])
		{
		++accumulated_blocks;
		accumulated_singles += block_history[i]; 
		sprintf(buf, "0x%x:0x%x-", i, block_history[i]);
		if(	(strlen(env_buf) + strlen(buf)) < (MAX_ECC_BAD_BLOCK_ENVSTR-10))
			{
			strcat(env_buf, buf);
			}
		else
			/*
			* MAX_BAD_BLOCKS is used to calc the max buffer space needed for ecc_err env
			* areas, etc.  This number WELL exceeds any realistic/unrelistic expectation.  
			* If we have this error buffer maxed out we are so screwed by this time.
			* Throw anything away above this.  nand.py test script is the only thing that
			* should care.
			*/
			{
			strcat(env_buf, "overflow");	
			break;
			}
		}
	}

saved_silent	= set_save_silent_mode();
setenv(SINGLE_BLOCKS_LIST_ENV_NAME, env_buf);

sprintf(env_count, "%d-%d", accumulated_singles, accumulated_blocks);
setenv(SINGLE_BLOCKS_ENV_NAME, env_count);

saveenv();
restore_silent_mode(saved_silent);

return(0);
}/*end FUNCTION: build_single_bit_env()*/


#endif /*CONFIG_CMC_10G*/
