/****************************************************************************

  Copyright (c) 2006, Dell Inc
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

  - Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
    
  - Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution. 

  - Neither the name of Dell Inc nor the names of its contributors
    may be used to endorse or promote products derived from this software 
    without specific prior written permission. 

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. 

*****************************************************************************/


/*
 * Utility support
 */
#include <common.h>		/*must be first in file*/
#include <command.h>
#include <asm/cache.h>


#ifdef CONFIG_CMC_10G	/*only compile in for a CMC_10G build*/

/******************************************************************************
*
*	FILE: util_fpga.c
*
******************************************************************************/


/******************************************************************************
*  Include files
******************************************************************************/
#include <util_defs.h>
#include <util_ret_codes.h>
#include <util_flashmap.h>
#include <util_fpga.h>

#include <malloc.h>


/******************************************************************************
*  Local Defines
******************************************************************************/
#define THERMTRIP_ENV_COUNT		"post_thermtrip_count"

/******************************************************************************
*  Globals
******************************************************************************/


/******************************************************************************
*  Prototypes
******************************************************************************/
extern s32	mcoptind;
extern char	*mcoptarg[];

/******************************************************************************
*	List all structures defines
******************************************************************************/
t_options util_fpga_options[] = 
	{
	/*Options*/
	{"load",	"Load FPGA bit stream, verify DONE, ID, revision, uP I/F."},
	{"unload",	"Initialize/clear the FPGA."},
	{"bs2flash","Copy a previously downloaded binary into flash."},	
	{"verbose", "run time debug trace."},	
	{"l",		"Loop count, -1 loop forever."},
	{ NULL, NULL }
	};


t_usage util_fpga_usage = { 
	"fpga", 
					
	/*Syntax*/
	"fpga   [-load] [-unload] [-verbose]\n"
	"       [-bs2flash <exp rev> <S500E | S1200E>] [-verbose]\n"
	"       [-l <loopcount> | -1]\n",

	/*Description*/
	"The fpga command allows the user to parallel load the FPGA with a bit\n"
	"stream located in NOR flash.  The load is verified by:\n"
	"  a) DONE being set\n"
	"  b) a patern test to both scratch registers\n"
	"  c) the ID register is 0x1337\n"
	"  d) the revision must match the expected value\n"
	"The unload option will clear the FPGA\n"
	"The user may use the tftp | loadb commands to download a bit stream,\n"
	"then the bs2flash option moves the bit stream to flash.  The user\n"
	"MUST specify the expected bit stream revision, and the part type.\n"
	"Use the looping option for repetitive stress validation, a -1 count\n"
	"loops forever.\nEXAMPLES:\n\n"
	"tftp cmc_top.bit <TFTP server IP>\n"
	"util fpga -bs2flash  14 S1200E\n"
	"util fpga load -l 100\n"
	"util fpga -unload\n",
	
	util_fpga_options
	}; /*end t_usage util_fpga_usage*/


/******************************************************************************
*
*	FUNCTION: util_fpga()
*	DATE:	11-30-06
*	
*	(See usage statment above)
******************************************************************************/
s32 util_fpga(int argc, char *argv[])
{
u32  index		= 0;
u32	 rc			= 0;
u32  load_flag	= 0;
u32  bs_flag	= 0;
u32  exp_rev	= 0;
u32  fpga_token = 0;
u32  loop_cnt	= 1;
u32	 fpga_flag	= 0;
u32  current_loop = 1;
char *cptr1;
char msg[MAX_MSG_LEN * 2];		/*double msg, both bit streams are tried*/

//	Get any options
mcoptind = 2;	
while( (index = mcgetopt(argc, argv, 
	  "-load -unload -bs2flash:2 -l:1 -verbose")) != MCGETOPT_EOF)
	{
	switch(index) 
		{
		case 0 :		/*load FPGA*/
			load_flag = 1;
			break;

		case 1 :		/*unload/clear FPGA*/
			if( (rc = fpga_init(msg)) != 0 )
				{
				printf("%s %s:%s\n", argv[0], argv[1], msg);
				return(rc);
				}
			break;

		case 2 :		/*copy bit stream from memory to flash*/
			bs_flag = 1;

			/*convert revision*/
			exp_rev = (u32)simple_strtol( mcoptarg[0], &cptr1, 0 );
			if( (s32)cptr1 < (strlen(mcoptarg[0]) + (s32)mcoptarg[0]) ) 
				{	
				printf("%s %s:ERROR: converting \"%s\" int.\n", 
											argv[0], argv[1], mcoptarg[0]);
				return(E_OPTARG_INT_CONVERSION);
				}

			/*get type*/
			if( !strcmp(mcoptarg[1], IMAGE_SEARCHNAME_FPGA_S500E) )
				fpga_token = IMAGE_TOKEN_FPGA_S500E;
			else if( !strcmp(mcoptarg[1], IMAGE_SEARCHNAME_FPGA_S1200E) )
				fpga_token = IMAGE_TOKEN_FPGA_S1200E;
			else
				{
				printf("%s %s:ERROR: invalid type, exp %s or %s\n", 
					argv[0], argv[1],
					IMAGE_SEARCHNAME_FPGA_S1200E, IMAGE_SEARCHNAME_FPGA_S500E);
				return(E_FPGA_INVALID_ARG_T_OP);
				}

			swizzle(fpga_token);
			break;

		case 3 :		/*loop count*/
			loop_cnt = (u32)simple_strtol( mcoptarg[0], &cptr1, 0 );

			if( (s32)cptr1 < (strlen(mcoptarg[0]) + (s32)mcoptarg[0]) ) 
				{	
				printf("%s %s:ERROR: -l argument \"%s\" cannot be converted "
						"to int.\n", argv[0], argv[1], mcoptarg[0]);
				return(E_OPTARG_INT_CONVERSION);
				}
			break;

		case 4 :		/*Add verbose to flag*/
			fpga_flag |= FPGA_FLAG_VERBOSE_ON;
			break;


		default:		/*not good*/
			rc = mcgetopt_error(index, msg, mcoptarg[0]);
			printf("%s %s:%s\n", argv[0], argv[1], msg);
			return(rc);
			break;
		}
	}/*end mcgetopt while*/



/****************************************************************************
*	Copy the bit stream from memory to flash
****************************************************************************/
if(bs_flag)
	{	
	/*	
	*	Using the "filesize" and "fileaddr", take a previously loaded
	*	bit stream and build it into a flash image.
	*/
	if( (rc = fpga_bs2flash(exp_rev, fpga_token, fpga_flag, msg)) != 0)
		{
		printf("%s %s:ERROR:%s\n", argv[0], argv[1], msg);
		return(rc);
		}
	else
		{
		printf("%s %s:%s\n", argv[0], argv[1], msg);
		return(rc);
		}
	}


/****************************************************************************
*	Load the bit stream into the FPGA
****************************************************************************/
if(load_flag)
	{	
	printf("\n");
	while(	((current_loop <= loop_cnt) || (UTIL_LOOP_FOREVER == loop_cnt)) )
		{
		rc = fpga_preload(fpga_flag, msg);
		if(rc != 0)
			{
			printf("%s %s:ERROR:%s...loop %d\n", 
										argv[0], argv[1], msg, current_loop);
			return(rc);
			}
		
		printf("%s %s:%s...loop %d\r", 	argv[0], argv[1], msg, current_loop);
		++current_loop;

		if ( ctrlc() )		/* check for ctrl-c to abort... */
			return(EXIT_BY_CTRLC);
		}/*end while*/
	
	printf("\n");
	}


return(0);
}/* end FUNCTION: util_fpga()*/


/****************************************************************************
*
*	FUNCTION: fpga_preload()
*	DATE:	7-9-05
*	PORTED:	11-30-06
*
*	This function had to be added before the fpga_load.  HW wants to support 
*	both S500E and S1200E FPGA parts.  We do not have any way to detect what 
*	FPGA type is populated on the board, not via HW straps, GPIO's, 
*	register IDs, etc.  
*
*	1) Both FPGA images are installed during part programming, and FWupdate.
*	2) The first image is attempted to load.
*   3) If it passes (DONE is active) life is good we are finished.
*	   Leave the second image there, should never be attempted.
*   4) If 1st fails, the second image is attempted, and upon passing the first
*      image is removed so not to waste time attempting to load for each boot.
*	   E_FPGA_MISSING_IMAGENAME is returned.
*	5) If both fail then we are in the ditch and no image is removed.
*
*	INPUT:	msg   - Buffer used to return messages.
*	
*****************************************************************************/
s32 fpga_preload(u32 fpga_flag, char *msg)
{
s32  rc;
u32  verbose;
char *env_p;
u32  fpga_type_s1200e;
s32  saved_silent;				/*save quiet flag*/
char msg2[MAX_MSG_LEN];			/*message buffer*/
char tmp[MAX_MSG_LEN];			/*message buffer*/

/*	
*	Debug - Run time noload flag to skip FPGA loading.
*	Run time verbose.
*/
env_p = getenv("post_fpga");
if(env_p)
	{
	if( strstr(env_p, "noload") )
		fpga_flag |= FPGA_FLAG_SKIP_RELOAD;
	else
		fpga_flag &= ~(FPGA_FLAG_SKIP_RELOAD);


	if( strstr(env_p, "verbose") )
		fpga_flag |= FPGA_FLAG_VERBOSE_ON;
	else
		fpga_flag &= ~(FPGA_FLAG_VERBOSE_ON);
	}

/*	Set local verbose flag*/
if(FPGA_FLAG_VERBOSE_ON & fpga_flag)
	verbose = VERBOSE_ON;
else
	verbose = VERBOSE_OFF;


/*	Test for the S1200E env variable*/
fpga_type_s1200e = 0;
if( (env_p = getenv("fpga_type")) != 0)
	{
	if(!strcmp(env_p, IMAGE_SEARCHNAME_FPGA_S1200E))
		fpga_type_s1200e = 1;	/*previous load was a successful S1200E part*/
	}

/*	
*	If a previous S1200E load success, then do not attempt the S500E.
*	If the "fpga_type" cannot be found then we MUST try the S500E first.
*/
if(!fpga_type_s1200e)
	{
	if( (rc = fpga_load(IMAGE_TOKEN_FPGA_S500E, fpga_flag, msg)) == 0)
		return(rc);			/*Loaded correctly, must be an S500E part*/
	}

/*	To get here means the S500E part did NOT load, try the S1200E part*/
if( (rc = fpga_load(IMAGE_TOKEN_FPGA_S1200E, fpga_flag, msg2)) == 0)
	{	
	if(!fpga_type_s1200e)
		{
		/* 
		*	Ok this is the first time S1200E loaded.   Save env var for future
		*	reset passes.  This flag is now forever persistent, even across 
		*	fwupdates.
		*/
		setenv("fpga_type", IMAGE_SEARCHNAME_FPGA_S1200E);
		saved_silent = set_save_silent_mode();
		saveenv();
		restore_silent_mode(saved_silent);

		if(verbose)
			printf("fpga_preload(): Created fpga_type=%s\n", 
												IMAGE_SEARCHNAME_FPGA_S1200E);
		}

	strcpy(msg, msg2);	/*S1200E good msg, overwrite S500E failing msg*/
	return(rc);			/*Loaded correctly, must be an S1200E part*/
	}

/*if neither loaded we are screwed*/
strcpy(tmp, msg);
sprintf(msg,"%s\n             %s", tmp, msg2);
return(E_FPGA_S500_S1200_NOLOAD);		
}/*end FUNCTION: fpga_preload()*/



/****************************************************************************
*
*	FUNCTION: fpga_bs2flash()
*	DATE:	10-18-06
*
*	In normal operation the build process constructs a header and appends the
*	FPGA bit stream.  Then fwupd will program it into the correct sector.
*	For lab development the "-bs2flash" option takes the fpga binary file 
*	previously downloaded into memory, builds and prepends a working header,
*	then programs it into the flash sectors.
*
*	INPUT:	exp_ver - Expected FPGA revision to place into the header
*			fpga_type - FPGA type determines the token.
*			msg     - Buffer used to return any messages.
*
*****************************************************************************/
s32 fpga_bs2flash(u32 exp_ver, u32 fpga_token, u32 fpga_flag, char *msg)
{
s32		rc;
char	*p;
u32		bs_addr;
u32		bs_size;
u32		verbose;
t_image_header	*hd;

if(FPGA_FLAG_VERBOSE_ON & fpga_flag)
	verbose = VERBOSE_ON;
else
	verbose = VERBOSE_OFF;


/*	Get parameters from a previously tftp'ed bit stream */
if((rc = get_file_addr_and_size(&bs_addr, &bs_size, msg)) != 0)
	return(rc);

if(bs_size > FPGA_BS_MAX_SIZE)
	{
	sprintf(msg, "\"filesize=0x%x\" exceeds max of 0x%x", 
												bs_size, FPGA_BS_MAX_SIZE); 
	return(E_FPGA_INVALID_BYTE_CNT);
	}

/*	Obtain a memory buffer, set to F's,  init the header*/
p  = malloc(FPGA_IMAGE_MAX_SIZE);
memset( (void*)p, 0xff, FPGA_IMAGE_MAX_SIZE);

hd = (t_image_header *)p;
hd->header_type		= CURRENT_IMAGE_HEADER_TYPE;
hd->token			= fpga_token;
hd->version			= exp_ver;

if(fpga_token == IMAGE_TOKEN_FPGA_S500E)
	{
	strncpy(hd->name, IMAGE_TEXTNAME_FPGA_S500E, MAX_IMAGE_NAME_SIZE);
	hd->fstart			= FLASH_SECT_START_FPGA_S500E;	
	hd->fcnt			= FLASH_SECT_CNT_FPGA_S500E;
	}
else
	{
	strncpy(hd->name, IMAGE_TEXTNAME_FPGA_S1200E, MAX_IMAGE_NAME_SIZE);
	hd->fstart			= FLASH_SECT_START_FPGA_S1200E;	
	hd->fcnt			= FLASH_SECT_CNT_FPGA_S1200E;
	}

hd->offset			= IMAGE_HEADER_SIZE;
hd->bcnt_zip		= 0;
hd->bcnt_unzip		= bs_size;	
hd->crc32 = crc32(0, (u8*)bs_addr, bs_size);

if(verbose)
	{
	printf("fpga_bs2flash(): header_type=0x%x token=0x%x version=0x%x\n"
					"name=%s offset=0x%x bcnt=0x%x crc32=0x%x\n"
					"start_sector=%d sector_count=%d\n\n", 
					hd->header_type, hd->token, hd->version, hd->name,
					hd->offset, hd->bcnt_unzip, hd->crc32, hd->fstart,
					hd->fcnt);

	printf("Malloc start=0x%x, load start=0x%x\n", p, bs_addr);
	}



/*	Append the bit stream after the header, erase, then program*/
memcpy((void *)((u32)p + IMAGE_HEADER_SIZE), (void *)bs_addr, bs_size);

if((rc = ABS_erase_nor(hd->fstart, 
									(hd->fstart + hd->fcnt -1),	
									verbose, 0, msg)) != 0)
	{
	free(p);
	return(rc);
	}	

if((rc = ABS_flash_write_nor(p, SECTOR_TO_PHYSADDR(hd->fstart), 
									(bs_size + IMAGE_HEADER_SIZE), msg)) != 0)
	{
	free(p);
	return(E_FPGA_BSTREAM_COPY_TO_FLASH);
	}

sprintf(msg, "Copied bs from mem 0x%x, to starting sector %d, for 0x%x bytes",
							p, hd->fstart, bs_size + IMAGE_HEADER_SIZE);
free(p);
return(0);
}/*end FUNCTION: fpga_bs2flash()*/

/****************************************************************************
*
*	FUNCTION: fpga_init()
*	DATE:	11-21-06
*
*	Put the FPGA into a programming mode.  
*	If the FPGA was already loaded,  then it will be cleared. 
*
*****************************************************************************/
s32 fpga_init(char *msg)
{
	fpga_program_enable();
	udelay (20000);				/*20milli sec*/
	fpga_program_disable();
	udelay (50000);				/*50milli sec*/
	fpga_program_enable();
	udelay (20000);				/*20milli sec*/

	if( !is_fpga_init_b() )
		{
		sprintf(msg, "FPGA_INIT_B did not go active after PROG_B is set.");
		return(E_FPGA_INIT_B_NOT_ACTIVE);
		}

	if(is_fpga_done())
		{	
		sprintf(msg, "FPGA_DONE is still active after PROG_B toggle.");
		return(E_FPGA_DONE_ACTIVE_AFTER_INIT);
		}

return(0);
}/*end FUNCTION: fpga_init()*/


/****************************************************************************
*
*	FUNCTION: fpga_load()
*	DATE:	11-28-06
*
*	Load the FPGA bit stream from the dedicated flash sector.  The header is
*	expected and verified. Called from "util fpga" or from normal u-boot 
*	startup.  
*
*	NOTE: Writing bytes (u8) to the FPGA to program it did not work.  I hae to 
*	use u16 cycles.
*
*	INPUT:	token - Indicates a S500E or S1200E FPGA type
*			msg   - Buffer used to return messages.
*	
*****************************************************************************/
s32 fpga_load(u32 fpga_token, u32 fpga_flag, char *msg)
{
u32		i;
u32		rc;
u32		bs_bcnt;
char	*bs_addr;
u8		pre_swap;
u8		post_swap;
u16		*fpga_p = (u16*)FPGA_BASE;
u16		saved_mode0 = 0xffff;		/*default to a non-loaded condition*/
u16		saved_wdog = 0;	
s32		saved_silent;
u16		fpga_id;
char    therm_cnt_str[15];			/*0xXXXXXXXX+null, 11 max*/
t_image_header	*iheader;


if( fpga_token == IMAGE_TOKEN_FPGA_S500E )
	iheader=(t_image_header *)SECTOR_TO_PHYSADDR(FLASH_SECT_START_FPGA_S500E);
else 
	iheader=(t_image_header*)SECTOR_TO_PHYSADDR(FLASH_SECT_START_FPGA_S1200E);


/*	Verify a valid bit stream exists before attempting a load.*/
if( (rc = fpga_verify_bs(iheader,  fpga_token, fpga_flag, msg)) != 0)
	return(rc);

/*	Starting address and byte count of the bit stream from the header.*/
bs_addr = (char*) (((u32)iheader) + iheader->offset);
bs_bcnt = iheader->bcnt_unzip;

if(FPGA_FLAG_VERBOSE_ON & fpga_flag)
	printf("fpga_load(): bs_addr=0x%x bs_bcnt=0x%x \n", bs_addr, bs_bcnt);


/*
*	If bit stream was already loaded then save the mode register.
*	Rewrite the mode after the new load.
*/
if( (fpga_id = ABS_read_fpga(FPGA_REG_ID)) == FPGA_ID)
	{
	saved_mode0 = ABS_read_fpga(FPGA_REG_MODE0);
	saved_wdog  = ABS_read_fpga(FPGA_REG_WDT_CTRL);
	}

if(FPGA_FLAG_VERBOSE_ON & fpga_flag)
	printf("ID=0x%x saved_mode0=0x%x\n", ABS_read_fpga(FPGA_REG_ID), saved_mode0);


/*	For debug only, post_fpga=noload  */
if( (!(fpga_flag & FPGA_FLAG_SKIP_RELOAD)) || 
	(FPGA_ID != fpga_id) )
	{

	/*Init/clear the FPGA*/
	if( (rc = fpga_init(msg)) )
		return(rc);

	/*
	*	Swap the bits and write to the FPGA.	Should be able to do byte cycles.
	*	But we had to do u16 cycles,  PPC ??
	*/
	for(i = 0; i < bs_bcnt; ++i)
		{
		post_swap = 0;
		pre_swap  = bs_addr[i];; 
		post_swap |= (pre_swap & 0x01) << 7;
		post_swap |= (pre_swap & 0x02) << 5;
		post_swap |= (pre_swap & 0x04) << 3;
		post_swap |= (pre_swap & 0x08) << 1;
		post_swap |= (pre_swap & 0x10) >> 1;
		post_swap |= (pre_swap & 0x20) >> 3;
		post_swap |= (pre_swap & 0x40) >> 5;
		post_swap |= (pre_swap & 0x80) >> 7;
		*fpga_p = post_swap;	/*write to FPGA*/
		}

	/*	Verify the newly loaded FPGA part*/
	if( (rc = fpga_verify_load(iheader->version, fpga_token, fpga_flag, msg)) != 0)
		return(rc);
	}
else
	{
	sprintf(msg, "FPGA was NOT loaded, \"post_fpga=noload\" ");
	}


/******************************************************************************
*	Build up the mode register
*******************************************************************************/
/*wait up to 50milli for shifty to read vendor mode, and therm trip bits, etc*/
udelay(50000);	

if(saved_mode0 == 0xffff)
	{
	/*	*1ST PRIORITY* therm trip*/
	if(ABS_read_fpga(FPGA_REG_THERM_TRIP) & FPGA_THERM_TRIP)
		{
		saved_mode0 = MODE_RESET_THERM_TRIP;
		sprintf(therm_cnt_str, "%d", (get_thermtrip_count() + 1) );

		/*	Save off count for future diag*/
		setenv(THERMTRIP_ENV_COUNT, therm_cnt_str);	/*update count*/
		saved_silent = set_save_silent_mode();
		saveenv();
		restore_silent_mode(saved_silent);

		clearlow_gpio_or(GPIO_BANK0, GPIO_CPLD_THERMTRIP_CLEAR_N);
		sethigh_gpio_or(GPIO_BANK0, GPIO_CPLD_THERMTRIP_CLEAR_N);
		}		
	else	/*	*2nd PRIORITY* PON*/
		saved_mode0 = MODE_RESET_PON;		/*other reset choices are all 0*/
	}
else
	{
	/*	Clear ALL reset cause bits (except PEER)*/
	saved_mode0 &= ~(MODE_RESET_CORE_CHIP_BUTTON | MODE_RESET_PON | 
					 MODE_FPGA_WDOG  | MODE_RESET_BOARD | 
					 MODE_RESET_THERM_TRIP);

	if(saved_wdog & FPGA_WDT_CTRL_EXPIRED)
		{		/*	*3rd PRIORITY* Forced WDOG*/
		if(saved_mode0 & MODE_FPGA_FORCE_WDOG)
			{
			saved_mode0 |= MODE_RESET_BOARD;	/*board reset*/
			saved_mode0 &= ~MODE_FPGA_FORCE_WDOG;
			}
		else	/*	*4th PRIORITY* WDOG*/
			saved_mode0 = MODE_FPGA_WDOG;		/*watch dog expired*/
		}
	/*	
	*	Redundency added MODE_PEER_RESET reset reason. It will force a wdog 
	*	(RESET_BOARD) but also set the MODE_PEER_RESET bit.  MODE_PEER_RESET
	*	carries a higher priority IF both are set. I have no other way of 
	*	knowing MODE_PEER_RESET reset other than the mode bit, so do NOT clear
	*	the bit above.  Linux peerd is responsable to clear MODE_PEER_RESET.
	*/
	else if(saved_mode0 & MODE_PEER_RESET)
		{		/*	*5th PRIORITY* PEER reset*/
		saved_mode0 &= ~(MODE_RESET_CORE_CHIP_BUTTON | MODE_RESET_PON | 
					 MODE_FPGA_WDOG  | MODE_RESET_BOARD | 
					 MODE_RESET_THERM_TRIP);
		}
	else
		{		/*	*6th PRIORITY* anything else*/
		saved_mode0 |= MODE_RESET_CORE_CHIP_BUTTON;	/*nothing else left*/
      	}

	/*Clear ST and recover, last_stage_init() re-evaluates*/
	saved_mode0 &= ~(MODE_SELF_TEST_ERROR | MODE_IMAGE_RECOVER_CONDITION);
	}/*end NON-AC/Therm Trip else*/




if(is_hard_vendor_mode() || is_fpga_vendor_mode() )
	saved_mode0 |= MODE_VENDOR_HARD;
else
	saved_mode0 &= ~MODE_VENDOR_HARD;

if(is_hard_recover_mode())
	saved_mode0 |= MODE_RECOVER_HARD;
else
	saved_mode0 &= ~MODE_RECOVER_HARD;

ABS_write_fpga(FPGA_REG_MODE0, saved_mode0);
return(0);
}/*end FUNCTION: fpga_load()*/



/****************************************************************************
*
*	FUNCTION: fpga_verify_load() 
*	DATE:	11-28-06
*	
*	Verify that the FPGA has been programmed by:
*	1) The DONE bit is in a high state.
	2) A quick pattern test to the scratch registers.
*	3) The ID register is correct.
*	4) The revision register has the correct revision.
*
*	INPUT:	exp_rev - expected revision from the revision register.
*			msg  - Buffer used to return messages.
*
*****************************************************************************/
s32 fpga_verify_load(u32 exp_rev, u32 fpga_token, u32 fpga_flag, char *msg)
{
s32	 i, rc;
u16  rec_rev;
u16  rec_id;
u32  verbose;
u32	 *addr = (u32*)FPGA_REG_SCRATCH1;
u32	 exp[FPGA_PAT_TEST_CNT]={0xffff0000, 0xaaaa5555, 0x99996666, 0xc3c33c3c};
u32	 rec[FPGA_PAT_TEST_CNT];
char *image_textname;


if(FPGA_FLAG_VERBOSE_ON & fpga_flag)
	verbose = VERBOSE_ON;
else
	verbose = VERBOSE_OFF;


if(fpga_token == IMAGE_TOKEN_FPGA_S500E)
	image_textname = IMAGE_TEXTNAME_FPGA_S500E;
else
	image_textname = IMAGE_TEXTNAME_FPGA_S1200E;

/*	Check for FPGA_DONE	*/
if( (rc = is_fpga_done() != 1))
	{
	sprintf(msg, "%s FPGA_DONE did not go active", image_textname);
	return(E_FPGA_NO_DONE);
	}

if(verbose)
	printf("fpga_verify_load(): DONE is active for %s\n", image_textname);


/*
*	Do a quick pattern test of the scratch registers. These registers are 16
*	bits.  A 32bit access will cause two, 16bit write/read cycles.  This is
*	faster that individual cycles.
*/
for(i = 0; i < FPGA_PAT_TEST_CNT; ++i)
	{
	*addr = exp[i];		
	rec[i] = *addr;	
	}

for(i = 0; i < FPGA_PAT_TEST_CNT; ++i)
	{
	if(exp[i] != rec[i])
		{	
		sprintf(msg, "FPGA:Scratch reg failure, at 0x%x  exp=0x%x rec=0x%x",
					addr, exp[i], rec[i]);
		return(E_FPGA_SCRATCH_REG_PATTERN_TEST);
		}
	}
*addr = 0;	/*	Clear scratch registers*/ 

/*	Check for ID register*/
rec_id = *((u16*)FPGA_REG_ID);
if(FPGA_ID != rec_id)
	{
	sprintf(msg, "FPGA ID exp=0x%x, rec=0x%x", FPGA_ID, rec_id); 
	return(E_FPGA_INVALID_ID);
	}

/*	Check for valid revision*/
rec_rev = *((u16*)FPGA_REG_REV);
if(exp_rev != rec_rev)
	{
	sprintf(msg, "FPGA revision register, exp=%d, rec=%d", 
															exp_rev, rec_rev); 
	return(E_FPGA_INVALID_REVISION);
	}
else
	{
	sprintf(msg, 
		"%s loaded, revision %d", image_textname, rec_rev);
	}

return(0);
}//end FUNCTION: fpga_verify_load()


/****************************************************************************
*
*	FUNCTION: fpga_verify_bs()
*	DATE:	11-21-06
*
*	Simply verify the presence of a vaild header.  Do not waste time 
*	attempting a FPGA load if the stream does not exist.
*
*	INPUT:	iheader -	pointer to the image header that is prepended to the
*						start of the bit stream
*			fpga_token- Indicates FPGA type
*			msg	-		char pointer to store any error messages
*
*	RETURN:	rc -		0 = verification passed
*						non-zero = verification failed, msg contains error msg
*****************************************************************************/
s32 fpga_verify_bs(t_image_header *iheader, u32 fpga_token, u32 fpga_flag, char *msg)
{
u32		addr;
u32		crc = 0;
u32		fpga_max_size;
char    *image_textname;
u32		verbose; 


if(FPGA_FLAG_VERBOSE_ON & fpga_flag)
	verbose = VERBOSE_ON;
else
	verbose = VERBOSE_OFF;


if(fpga_token == IMAGE_TOKEN_FPGA_S500E)
	{
	image_textname = IMAGE_TEXTNAME_FPGA_S500E;
	fpga_max_size  = FLASH_MAX_SIZE_FPGA_S500E;
	}
else
	{
	image_textname = IMAGE_TEXTNAME_FPGA_S1200E;
	fpga_max_size  = FLASH_MAX_SIZE_FPGA_S1200E;
	}


if(verbose)
	printf("fpga_verify_bs(): image name=%s, max size=0x%x\n"
	   "iheader points to=0x%x\n\n",
	   image_textname, fpga_max_size, (u32)iheader );


/*
*	Verify that the image name is correct 
*/
if( strncmp(iheader->name, image_textname, MAX_IMAGE_NAME_SIZE)!= 0)
	{
	sprintf(msg,"%s expected header name missing", image_textname);
	return(E_FPGA_MISSING_IMAGENAME);
	}

/*
*	Verify the bit stream crc32
*/
addr = ((u32)iheader) + iheader->offset;

/*
*	Early in dev. the iheader elements changed. To prevent a lockup of a huge 
*	bcnt where we cannot return to RACMON, put in a check. This should become
*	an obsolete check.
*/
if(iheader->bcnt_unzip > fpga_max_size)
	{
	sprintf(msg,"Invalid FPGA byte count, requested=0x%x max=0x%x",
									iheader->bcnt_unzip, fpga_max_size);
	return(E_FPGA_INVALID_BYTE_CNT);
	}

crc = crc32(0, (u8*)addr, iheader->bcnt_unzip);

if(verbose)
	printf("Found \"%s\"\nStart addr=0x%x bs bcnt=0x%x offset=0x%x \n"
	   "header->cksum=0x%x, crc=0x%x\n\n",
		image_textname, addr, iheader->bcnt_unzip, iheader->offset, 
		iheader->crc32, crc);

if(crc != iheader->crc32)
	{	
	sprintf(msg, 
		"FPGA bit stream crc32 failed at 0x%x, size of 0x%x\n"
		"exp crc 0x%x, rec crc 0x%x", 
		addr, iheader->bcnt_unzip, iheader->crc32, crc);
	return(E_FPGA_BSTREAM_CRC32_FAILED);
	}

sprintf(msg,
		"FPAG flash image header name and crc32 verified at 0x%x", iheader);

return(0);
}/*end FUNCTION: fpga_verify_bs()*/


/****************************************************************************
*	FUNCTION: get_thermtrip_count()
*	DATE:	  6-6-07
*
*	Open THERMTRIP_ENV_COUNT if it exists, convert to decimal and return.
*	This is the total number of therm trips that THIS CMC has experienced.
*
*****************************************************************************/
s32 get_thermtrip_count(void)
{
char *env_p;
char *cp;
s32  count;

if( (env_p = getenv(THERMTRIP_ENV_COUNT)) == 0)
	return(0);							/*env var does not exists, 0 count*/

count = (s32)simple_strtol(env_p, &cp, 10);
if( (s32)cp < (strlen(env_p) + (s32)env_p) )
	return(0);							/*conversion error, 0 count*/

return(count);
}/*end FUNCTION: get_thermtrip_count() */



#endif /*CONFIG_CMC_10G*/



