/****************************************************************************

  Copyright (c) 2006, Dell Inc
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

  - Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
    
  - Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution. 

  - Neither the name of Dell Inc nor the names of its contributors
    may be used to endorse or promote products derived from this software 
    without specific prior written permission. 

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. 

*****************************************************************************/



#include <common.h>		/*must be first in file*/
#include <command.h>

#ifdef CONFIG_CMC_10G	/*only compile in for a CMC_10G build*/


/******************************************************************************
*
*	FILE: util_fru.c
*
******************************************************************************/


/******************************************************************************
*  Include files
******************************************************************************/
#include <util_defs.h>
#include <util_ret_codes.h>


/******************************************************************************
*  Local Defines
******************************************************************************/
#define BOARD_SERIALNUM_OFFSET 0x2A
#define BOARD_PARTNUM_OFFSET   0x39
#define PCBA_TEST_OFFSET       0x14
#define MAX_BUILD_DATE_LEN     0x0A
#define FRU_BIN_MAX_SIZE        256
#define BOARD_BOARDREV_OFFSET  0x3F

#define ENVNAME_CMC_REV			"fru_board_rev"

/******************************************************************************
*  Globals
******************************************************************************/


/******************************************************************************
*  Prototypes/Externs
******************************************************************************/
extern s32	mcoptind;
extern char	*mcoptarg[];

extern s32 mac_verify(char *str, s32 * mac_buf_ptr);
extern void format_print(unsigned char *data,long maxlength);
static s32 read_fru(char * fptr, s32 numbytes);
static s32 write_fru(char *fptr);
extern int i2c_read(uchar chip, uint addr, int alen, uchar *buffer, int len);
extern int i2c_write(uchar chip, uint addr, int alen, uchar *buffer, int len);


/******************************************************************************
*	List all structures defines
******************************************************************************/
t_options util_fru_options[] = 
	{
	/*Options*/
	{"mac0",    "Program MAC0."},
	{"mac1",	"Program MAC1."},
	{"barcode",	"Write the ASCII string to serial number field."},
	{"ift",     "Read, increment and write back the functional test count."},
	{"date",    "Store the board manufacture date in an envirinment variable."},
	{"boardrev", "Update the board revision number."},
    {"dump",    "Dump the 256 Byte CMC fru"},
    {"display", "Display the current values in the supported options."},
	{"write",   "Write a previously downloaded CMC FRU template"},
	{ NULL, NULL }
	};


t_usage util_fru_usage = { 
	"fru", 
					
	/*Syntax*/
	"fru  [-mac0 <string>] [-mac1 <string>] [-barcode <string>] [-ift] \n"
    "     [-date <string>] [-boardrev <string>][-dump] [-display] [-write] \n",

	/*Description*/
	"This command allows the user to manipulate the following fields in the CMC FRU\n"
    "MAC0, MAC1, dell barcode and IFT field. While writing back to the FRU, the command"
    "recalculates the checksum field\n"
    "util fru command \nExamples:\n\n"
    "util fru -mac0 112233445566\n"
	"util fru -mac1 aabbccddeeff\n"
    "util fru -barcode 000UJ92412401740002E\n"
    "util fru -ift\n"
    "util fru -date 07-12-2007\n"
    "util fru -boardrev A12\n"
    "util fru -dump\n"
	"tftp 200000 CMC.BIN; util fru -write\n"
    "util fru -display\n",
    	
	util_fru_options
	}; /*end t_usage util_fru_usage*/


/******************************************************************************
*
*	FUNCTION: util_fru()
*	DATE: 07/02/07	
*
******************************************************************************/
s32 util_fru(int argc, char *argv[])
{

s32     c,rc=0, i;
u32     index = 0;
uchar   *fru_ptr;
char    fru[CMCFRUSIZE];
char    msg[MAX_MSG_LEN];		/*message buffer*/
char    *eeprom_ptr;
char    *serialnum_ptr;
char    *partnum_ptr;
char    *boardrev_ptr;
char    *len_ptr;
char    *barcodestr;
char    *boardrevstr;
char    *env_p;
s32     str0[6],str1[6],*p0, *p1;
u32     board_info_area_offset;
u32 	bin_addr;
u32	bin_size;
u16     mac_offset, pcba_offset;
u8      internal_usr_area_offset, board_area_cksum = 0;
u8      pcba_cksum = 0, elem_len, mac_cksum = 0;
s32     display_flag = 0 ,barcode_flag = 0,ift_flag = 0, boardrev_flag =0;
s32     mac_flag = 0,ift_count, dump_flag = 0, date_flag =0;



    fru_ptr = &fru[0];
    p0 = str0;
    p1 = str1;
      
    /*Read the CMC Fru into a buffer, perform the various operations on it and write it back*/
    if((rc = read_fru(fru, 256))!= 0)
            {
                printf("I2C fru read error\n");
                return(rc);
            }
        
//	Get any options
mcoptind = 2;	
while( (index = mcgetopt(argc, argv, "-mac0:1 -mac1:1 -barcode:1 -ift -boardrev:1 -dump -display -date:1 -write")) != MCGETOPT_EOF)
	{
	switch(index) 
		{
        case 0 :		/*option "mac0" */

                if( (rc = mac_verify(mcoptarg[0], str0))!= 0)
                {
                    printf("Invalid MAC0, MAC format XXXXXXXXXXXX\n");
                    return(rc);
                }
                mac_flag++;
                eeprom_ptr = &fru[CMC_HEADER_LEN];
                internal_usr_area_offset = 8 * (*(eeprom_ptr+1));
                eeprom_ptr = eeprom_ptr + internal_usr_area_offset + BMC_MAC_OFFSET;
                mac_offset = *(eeprom_ptr + 1) << 8;
                mac_offset = mac_offset + *eeprom_ptr;
                eeprom_ptr = &fru[CMC_HEADER_LEN+ mac_offset];
                len_ptr = eeprom_ptr;
                eeprom_ptr = eeprom_ptr + 4;
                for(i = 0; i<6; i++)
                {
                    *eeprom_ptr++ = *p0++;
                }
                len_ptr++;
                elem_len = *(len_ptr + 1) << 8;
                elem_len = elem_len + *len_ptr;
                c = CMC_HEADER_LEN + mac_offset;
                mac_cksum = 0;
                for(i = c; i< (c + elem_len); i++)
                {   
                     if(i != (c+3)){
                        mac_cksum+= fru[i];
                     }
                }
                fru[c+3] = (u8)((~mac_cksum) +1);
                
             
             break;
        case 1 :		/*option "mac1" */

                if( (rc = mac_verify(mcoptarg[0], str1))!= 0)
                {
                    printf("Invalid MAC1, MAC format XXXXXXXXXXXX\n");
                    return(rc);
                }
            
                mac_flag++;
                eeprom_ptr = &fru[CMC_HEADER_LEN];
                internal_usr_area_offset = 8 * (*(eeprom_ptr+1));
                eeprom_ptr = eeprom_ptr + internal_usr_area_offset + BMC_MAC_OFFSET;
                mac_offset = *(eeprom_ptr + 1) << 8;
                mac_offset = mac_offset + *eeprom_ptr;
                eeprom_ptr = &fru[CMC_HEADER_LEN+ mac_offset];
                len_ptr = eeprom_ptr;
                eeprom_ptr = eeprom_ptr + 10;
                for(i = 0; i<6; i++)
                {
                    *eeprom_ptr++ = *p1++;
                }

                len_ptr++;
                elem_len = *(len_ptr + 1) << 8;
                elem_len = elem_len + *len_ptr;
                c = CMC_HEADER_LEN + mac_offset;
                //printf("MAC element \n");
                mac_cksum = 0;
                for(i = c; i< (c + elem_len); i++)
                {   //printf("%02X ", fru[i]);
                    if(i != (c+3)){
                        mac_cksum+= fru[i];
                        
                    }
                }
                //printf("\n");
                
                fru[c+3] = (u8)((~mac_cksum) +1);  
                //printf("\ncsum = %02X\n",fru[c+3]);

			break;
        case 2 :		/*option "barcode" */
                barcode_flag++;
                barcodestr = mcoptarg[0]; 
                if(strlen(barcodestr)!= 20)
                {
                    printf("barcodestr = %s\n",barcodestr);
                    printf("Invalid string length entered for barcode\n");
                    printf("Requires 20 Ascii characters\n");
                    return(1); //E_FRU_INVALID_BARCODE_LEN);
                }
                eeprom_ptr = &fru[CMC_HEADER_LEN];
                board_info_area_offset = 8*(*(eeprom_ptr+3));
                serialnum_ptr = eeprom_ptr + board_info_area_offset + BOARD_SERIALNUM_OFFSET;
                partnum_ptr = eeprom_ptr + board_info_area_offset + BOARD_PARTNUM_OFFSET;
                                         
                for(i = 0; i<20; i++)
                {   if(i<2 || i>7){
                        *serialnum_ptr++ = barcodestr[i];
                    }
                    else{
                        *partnum_ptr++ = barcodestr[i];
                    }
                }
                //printf("board info area\n");
                c = CMC_HEADER_LEN + board_info_area_offset;
                for(i = c ;i< (c+(8*fru[c+1]))-1 ;i++)
                {
                    board_area_cksum+= fru[i];
                }
                //printf("csum = %02X",board_area_cksum);
                fru[(c + (8*fru[c+1])) -1] = (u8)((~board_area_cksum)+1);
                //printf("\n");
                         
			break;
        case 3:         /*option ift*/
            /*Incrementing the Functional test count*/
            ift_flag++;
            eeprom_ptr = &fru[CMC_HEADER_LEN];
            internal_usr_area_offset = 8 * (*(eeprom_ptr+1));
            eeprom_ptr = eeprom_ptr + internal_usr_area_offset + PCBA_TEST_OFFSET;
            pcba_offset = *(eeprom_ptr+1) << 8;
            pcba_offset = pcba_offset + *eeprom_ptr;
            eeprom_ptr = &fru[CMC_HEADER_LEN+ pcba_offset];     //Element #2 
            len_ptr = eeprom_ptr;                               // store the pointer to element #2
            eeprom_ptr = eeprom_ptr+10;                        //The IFT position within element #2
            ift_count = *eeprom_ptr;                           //Read, increment and write back the IFT count 
            ift_count++;
            *eeprom_ptr = ift_count;
            len_ptr++;
            elem_len = *(len_ptr + 1) << 8;
            elem_len = elem_len + *len_ptr;
            c = CMC_HEADER_LEN + pcba_offset;
            for(i = c; i< (c + elem_len); i++)
            {
                if(i != (c+3)){
                    pcba_cksum+= fru[i];
                }
            }
            fru[c+3] = (u8)((~pcba_cksum) +1);               // the 4th byte in element #2 is the element checksum
            break;
            
	case 4:		    /*option "boardrev"*/
	    boardrev_flag++;		
	    boardrevstr = mcoptarg[0];
	    if(strlen(boardrevstr) != 3)
	    {
                 printf("boardrevstr = %s\n",boardrevstr);
                 printf("Invalid string length entered for barcode\n");
                 printf("Requires 3 Ascii characters\n");
                 return(E_FRU_INVALID_BOARDREV_LEN);
 
            }
 	    eeprom_ptr = &fru[CMC_HEADER_LEN];		
	    board_info_area_offset = 8 * ( *(eeprom_ptr + 3));
	    boardrev_ptr = eeprom_ptr + board_info_area_offset + BOARD_BOARDREV_OFFSET;
	    for(i= 0 ; i<3 ; i++)
	    {
        	*boardrev_ptr++ = boardrevstr[i];
            }
	    c = CMC_HEADER_LEN + board_info_area_offset;
            for(i = c ;i< (c+(8*fru[c+1]))-1 ;i++)
            {
                board_area_cksum+= fru[i];
            }
                //printf("csum = %02X",board_area_cksum);

	    fru[(c + (8*fru[c+1])) -1] = (u8)((~board_area_cksum)+1);
		
	    break;     
        case 5:             /*option "dump"*/
            
            if((rc = read_fru(fru, 256))!= 0)
            {
                printf("I2C fru read error\n");
                return(rc);
            }
            
            dump_flag++;
            break;
        case 6:             /*option display*/
                display_flag++;
                eeprom_ptr = &fru[CMC_HEADER_LEN];
                internal_usr_area_offset = 8*(*(eeprom_ptr+1));
                eeprom_ptr = eeprom_ptr + internal_usr_area_offset + BMC_MAC_OFFSET;
                mac_offset = *(eeprom_ptr + 1) << 8;
                mac_offset = mac_offset + *eeprom_ptr;
                eeprom_ptr = &fru[CMC_HEADER_LEN+ mac_offset];
                eeprom_ptr = eeprom_ptr +4;
               
                printf("\nMAC0 = ");
                for(i =0 ;i <6; i++)
                {
                    printf("%02X ",*eeprom_ptr);
                    eeprom_ptr++;
                }
                
                printf("\nMAC1 = ");
                for(i =0 ;i <6; i++)
                {
                    printf("%02X ",*eeprom_ptr);
                    eeprom_ptr++;
                }
               
                eeprom_ptr = &fru[CMC_HEADER_LEN];
                internal_usr_area_offset = 8 * (*(eeprom_ptr+1));
                eeprom_ptr = eeprom_ptr + internal_usr_area_offset + PCBA_TEST_OFFSET;
                pcba_offset = *(eeprom_ptr+1) << 8;
                pcba_offset = pcba_offset + *eeprom_ptr;
                eeprom_ptr = &fru[CMC_HEADER_LEN+ pcba_offset];     //Element #2 
                eeprom_ptr = eeprom_ptr+10;
                printf("\nift_count = %02X\n",*eeprom_ptr);

                eeprom_ptr = &fru[CMC_HEADER_LEN];
                board_info_area_offset = 8*(*(eeprom_ptr+3));
                serialnum_ptr = eeprom_ptr + board_info_area_offset + BOARD_SERIALNUM_OFFSET;
                partnum_ptr = eeprom_ptr + board_info_area_offset + BOARD_PARTNUM_OFFSET;

                printf("barcode = ");
                for(i = 0; i<20; i++)
                {   if(i<2 || i>7){
                        printf("%c",*serialnum_ptr++);
                    }
                    else{
                        printf("%c",*partnum_ptr++);
                    }
                }
               env_p = getenv(ENV_BUILD_DATE);
                                
                if(env_p != 0)
                {
                    printf("\nBuild date = %s\n\n",env_p);
                }
                
            break;
        case 7:     /*option Manuf Date*/
                date_flag++;
                if(strlen(mcoptarg[0]) != 10)
                {
                    printf("build date format mm-dd-yyyy\n");
                    return(1);
                }
                env_p = getenv(ENV_BUILD_DATE);
                /*                
                if(env_p != 0)
                {
                    printf("Build date = %s\n",env_p);
                }*/
                                
                printf("Saving build date\n");			
                setenv(ENV_BUILD_DATE, mcoptarg[0]);
                saveenv();
                			
            break;
	case 8:     /*option write(to tftp a fru template)*/
            if(!mac_flag || !display_flag || !ift_flag || !date_flag || !barcode_flag || !dump_flag || !boardrev_flag)
            {
            
                if((rc = get_file_addr_and_size(&bin_addr, &bin_size, msg)) != 0)
                {
                    printf(" Cannot find env variables for CMC FRU binary\n"
                           " tftp the binary to RAM\n");
                    printf(" Usage: tftp <addr> <file>\n eg:    tftp 200000 CMC.BIN\n");
                    return(rc);
                }
                else
                {
                    printf("Found binary\n");
                    if(bin_size > FRU_BIN_MAX_SIZE)
                    {
                        printf("filesize= %d, exceeds max of %d\n", 
												bin_size, FRU_BIN_MAX_SIZE); 
                        return(E_FRU_BIN_EXCEED_MAX_SIZE);
                    }
                    printf("bin addr = 0x%x\n",bin_addr);
                    fru_ptr = &fru[0];
                    memcpy( (void*)fru_ptr, (void*)bin_addr, bin_size);
                    fru_ptr = &fru[CMC_HEADER_LEN];
                    if( (rc = write_fru(fru_ptr))!= 0)
                    {
                        printf("I2C fru Write error\n");
                        return(rc);
                    }
                }
            }
            else
            {
                printf("Option fru -write cannot be used with any other option\n");
            }
            break;
        default:		/*not good*/

            rc = mcgetopt_error(index, msg, mcoptarg[0]);
			printf("%s %s:%s\n", argv[0], argv[1], msg);
			return(rc);
			break;
		}
     }/*end mcgetopt while*/
     if(dump_flag)
     {
         format_print(fru,256); //Hard coding the number of bytes to print to 256 bytes!!!
     }
     if(mac_flag || barcode_flag || ift_flag || boardrev_flag)
     {
         fru_ptr = &fru[CMC_HEADER_LEN];
         if( (rc = write_fru(fru_ptr)) != 0)
         {
             printf("I2C fru Write error\n");
             return(rc);
         }
     }

return(0);
}/* end FUNCTION: util_fru()*/

/***************************************************************************
*	Function: read_fru()
*	DATE:	07-03-07
*
*  Read the CMC FRU,including the first 48 bytes
*
*	INPUT:	
*	RETURN:	0 - I2C read success
*			non-zero - I2C read error
*
****************************************************************************/
s32 read_fru(char * fptr, s32 numbytes)
{
    s32 rc;
    u32 addr;
    /*Read 16 bytes at a time instead of the whole 256 bytes*/
            for(addr = 0; addr < numbytes; addr +=16,fptr +=16)
            {
                rc = i2c_read(CHIP_ADDR_FRU, addr, 1, fptr, 16);
                if (rc == 1)
                {
                    printf("Cannot read FRU: I2C read failed\n");
                    return(1);
                }
            }

return(0);
}/*end FUNCTION: read_fru*/


/*************************************************************************
*   Function: write_fru()
*   DATE:     07-10-07
*
*   This function writes the modified FRU data back into the CMC FRU.
*
*   INPUT:
*   RETURN: 0 - I2C Write Success
*           nonzero - I2C Write Error.
*************************************************************************/
s32 write_fru(char *fptr)
{
    s32 rc;
    u32 addr;

    for(addr = CMC_HEADER_LEN; addr < CMCFRUSIZE; addr++, fptr++)
    {
        rc = i2c_write(CHIP_ADDR_FRU, addr, 1, fptr, 1);
        if (rc != 0)
        {
            printf("Cannot Write FRU: I2C Write failed\n");
            return(1);
        }
        #ifdef CFG_EEPROM_PAGE_WRITE_DELAY_MS
		    udelay(CFG_EEPROM_PAGE_WRITE_DELAY_MS * 500);
        #endif
    }

    return(0);
}


/****************************************************************************
*	Function: get_cmc_rev_from_fru()
*	DATE:	11-2-07
*
*	Get the board revision that is contained in the CMC local FRU.  In an 
*	attempt to not abuse reading the FRU, cache it into a env variable so we 
*	will only read it once.
*
*	INPUT:	fru_rev_str - Pointer to a array for FRU_REV_STR_LEN bytes.
*	RETURN:	0
****************************************************************************/
s32 get_cmc_rev_from_fru(char *msg)
{ 
u32  rc = 0;
u32  i;
char c;
char fru[CMCFRUSIZE];
char *eeprom_ptr;
char *boardrev_ptr;
u32  board_info_area_offset;
char *env_p;
char fru_rev_str[FRU_REV_STR_LEN+1];				/* + null*/

/*	Clear str, env variable should always exists after 1st boot at board vendor*/
memset(fru_rev_str, 0, FRU_REV_STR_LEN+1);
if((env_p = getenv(ENVNAME_CMC_REV)) != 0)	
	{
	strncpy(fru_rev_str, env_p, FRU_REV_STR_LEN);
	}
else
	{
	/*	Must be the first time to boot, read the CMC FRU and get the rev field*/ 
	if((rc = read_fru(fru, 256))!= 0)
		{
		sprintf(msg, "Cannot read CMC FRU, I2C fru read error");
		return(rc);
		}

	eeprom_ptr = &fru[CMC_HEADER_LEN];		
	board_info_area_offset = 8 * ( *(eeprom_ptr + 3));
	boardrev_ptr = eeprom_ptr + board_info_area_offset + BOARD_BOARDREV_OFFSET;

	for(i = 0; i < FRU_REV_STR_LEN; i++)
		{
		c = boardrev_ptr[i];
		if( (c >= 0x20) && (c <= 0x7e) )
		    fru_rev_str[i] = boardrev_ptr[i];
		else	
			{/*dot for non-printable char, then WARN msg to flag bad FRU*/
			fru_rev_str[i] = 0x2e;		
			rc = W_POST_FRU_CMC_BOARDREV;
			}
		}

	/*	Now save it off as an env variable, IF it is good*/
	if(rc == 0)
		{
		setenv(ENVNAME_CMC_REV, fru_rev_str);
		saveenv();
		}
	}


/*If default of "rrr" then ICT never programmed these fields*/
if(!strcmp(fru_rev_str, "rrr"))
	rc = W_POST_FRU_CMC_BOARDREV_DEFAULT;

/* NOTE: Linux KEYS ON THIS TEXT, DO NOT CHANGE IT*/
sprintf(msg, "Board Revision=%s", fru_rev_str );
return(rc);
}/*end FUNCTION: get_cmc_rev_from_fru()*/

#endif /*CONFIG_CMC_10G*/



