/****************************************************************************

  Copyright (c) 2006, Dell Inc
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

  - Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
    
  - Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution. 

  - Neither the name of Dell Inc nor the names of its contributors
    may be used to endorse or promote products derived from this software 
    without specific prior written permission. 

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. 

*****************************************************************************/



#include <common.h>		/*must be first in file*/
#include <command.h>

#ifdef CONFIG_CMC_10G	/*only compile in for a CMC build*/
/******************************************************************************
*
*	FILE: util_fwupdate.c
*
*	This file contains API's used to support firmware update.  The CLI 
*	call into update is "recover fwupdate".  
*
******************************************************************************/


/******************************************************************************
*  Include files
******************************************************************************/
#include <util_defs.h>
#include <util_ret_codes.h>
#include <util_fpga.h>
#include <nand.h>


/******************************************************************************
*  Local Defines
******************************************************************************/
DECLARE_GLOBAL_DATA_PTR;

/******************************************************************************
*  Globals
******************************************************************************/

/******************************************************************************
*  Prototypes/Externs
******************************************************************************/
static s32 fwupd_unprotect(u32 start_sector, u32 sector_count, char *msg); 
static s32 how_many_bad_blocks(nand_info_t *nand, u32 fstart, u32 bcnt);

extern int new_bad_blocks_idx;						
extern int new_bad_blocks[];

extern int flash_sect_protect (int flag, ulong addr_first, ulong addr_last);
extern nand_info_t nand_info[];
extern s32 nand_ecc_scan(u32 start, u32 end, u32 bad_offset, u32 *err_cnt, 
												u32 reboot_flag, char *msg);
extern s32 add_to_err_blocks(int *new_bad_blocks, u8 flag);

/******************************************************************************
*	List all structures defines
******************************************************************************/


/****************************************************************************
*	FUNCTION: fwupd()
*	DATE:	  3-5-07
*
*	Call this API to take a firmimg.cmc update image file, walk the headers 
*	and apply to the flash.  Called from:
*	1. util_cmc_main() on power up.
*	2. recover_fwupdate() invoked from the recovery CLI
*
*	INPUT:	mem_start- Starting address of the firmimg.cmc data. 
*			msg      - Buffer used to return any messages.
*	RETURN: 0        - image was moved and verified to be correct.
*			non-zero - a valid image CANNOT be relocated to ram.
*
*****************************************************************************/
s32 fwupd(u32 mem_start, char *msg)
{
s32  rc;
u32  i;
u32  start_of_bin;
u32  bcnt;
u32  total_image_bcnt;
u32  crc;
u32  fstart;
u32	 nand_erased = 0;
#if DYNAMIC_IMAGE_START
s32  bad_blks;
u32  next_fstart = 0;	
#endif
s32  saved_silent;				/*save quiet flag*/
u32  nor_touched  = 0;
char env_name[40];
char env_data[MAX_MSG_LEN-40];
nand_write_options_t	w_opts;
nand_erase_options_t	e_opts;
/*global structs containing nand info, cmc will always have 1 NAND*/
nand_info_t *nand		= &nand_info[0];	

/*	Pointer to master and image headers*/
t_image_header			*ihdr;
t_master_image_header	*mhdr = (t_master_image_header*)mem_start;


/*	Verify integerty of the image file*/
if( (rc = fwupd_verify_image(mem_start, &total_image_bcnt, msg)) != 0)
	return(rc);

/*	Walk the header list and update flash, IF the verify passes*/
for(i = 0; (i < mhdr->num_of_images) && !rc; ++i)
	{
	ihdr = (t_image_header*)((u32)mhdr + mhdr->image_header_offset[i]);
	/*	Move after bcnt check, so to NOT print image name for 0 len file*/
	//printf("\nUpdating image \"%s\" \n",ihdr->name);

	if(ihdr->bcnt_zip)			/*	See if we need to unzip the image*/
		{
		/*TODO: unzip the image.  None of them are zipped at this time.*/
		printf("Unziping image\n");
		start_of_bin = ((u32)ihdr + ihdr->offset); //remove when unzip support
		/*start_of_bin = sizeof imagefile + base*/
		/*unzip_image(((u32)ihdr + ihdr->offset), start_of_bin, &bcnt);*/
		bcnt = ihdr->bcnt_unzip; //remove
		}
	else
		{/*	pick up the memory addr of the binary and its byte count*/
		start_of_bin = ((u32)ihdr + ihdr->offset);
		bcnt = ihdr->bcnt_unzip;
		}

	if(0 == bcnt)
		continue;	

	printf("\nUpdating image \"%s\" \n",ihdr->name);

	crc = crc32(0, (u8*)start_of_bin, bcnt);
	printf("Verify image crc32: bytes=0x%x, crc32=0x%x\n", bcnt, crc);
	if(ihdr->crc32 != crc)
		{
		sprintf(msg, "Image \"%s\" crc32 failed, exp=0x%x rec=0x%x\n", 
											ihdr->name, ihdr->crc32, crc);
		rc = E_FWUPD_CHECKSUM_FAILURE;
		break; /* do not use the dreaded "goto" */
		}

	/***************************************************************************
	*	NOR update
	***************************************************************************/
	if(ihdr->flash_type == FLASH_TYPE_NOR)
		{
		if( !is_nor_image_ok(ihdr->name, SECTOR_TO_PHYSADDR(ihdr->fstart), 
															bcnt, ihdr->crc32))
			{

			if((rc = fwupd_unprotect(ihdr->fstart, ihdr->fcnt, msg)) != 0)
				break;

			if((rc=ABS_erase_nor(ihdr->fstart, ihdr->fstart + ihdr->fcnt - 1, 
													VERBOSE_ON, 0, msg)) != 0)
				break;

			printf("Writing data 0x%x -to- 0x%x please wait -- ", 
								SECTOR_TO_PHYSADDR(ihdr->fstart), 
								SECTOR_TO_PHYSADDR(ihdr->fstart) + bcnt - 1);

			if((rc = ABS_flash_write_nor(	(char*)start_of_bin, 
											SECTOR_TO_PHYSADDR(ihdr->fstart),
											bcnt, msg)) != 0)
				break;

			++nor_touched;
			printf("complete\n");	
			}
		fstart = SECTOR_TO_PHYSADDR(ihdr->fstart);	/*need for env descriptor*/
		}
	/*************************************************************************
	*	NAND update
	*************************************************************************/
	else if(ihdr->flash_type == FLASH_TYPE_NAND)
		{
		/*
		*	2.6.25 Port   The image placment in the new kernel is now dynamic.
		*	The kernel starts at 0, rootfs picks up where kernel leaves off
		*	no matter how many bad blocks, var starts where rootfs leaves off 
		*	no matter how many bad blocks. Static tables will NOT work as a 
		*   concentration of bad blocks causes too many issues.
		*	NAND IMAGES *MUST* BE IN THIS ORDER AS DEFINED BY 
		*	../cmc_ip/Dell/image_tools/header.c and util_flashmap.h
		*	1) kernel, 2) rootfs, 3) var, 4) copy of firmimg.cmc 
		*/

		/*
		*	Cannot write to ihdr->fstart as it corrupts the CRC, and needed for
		*	env descriptor( different than NOR )
		*/
		fstart = ihdr->fstart;
#if DYNAMIC_IMAGE_START
		if( !strcmp(IMAGE_TEXTNAME_KERNEL, ihdr->name))
			next_fstart = fstart;
		else
			fstart = next_fstart;
#endif /*DYNAMIC_IMAGE_START*/
	
		/*	Erase NAND area
		*	The new 2.6.25 kernel has forced a few NAND changes.  We used to 
		*	erase just enough flash area to store the exact image size. If we 
		*	had "leftovers" (previous version was bigger) they were not erased. 
		*
		*	The unused flash MUST be erased for JFFS2 to be able to use the 
		*	remaining area assigned to it. Since we will be adding dynamic 
		*	starting points for the kernel images we are now forced to erase 
		*	the entire kernel image areas.  (The ability to update a single 
		*	kernel image for debug is no longer possible).	
		*/
		if(0 == nand_erased)
			{
			nand_erased = 1;		/*run this only once*/
			memset(&e_opts, 0, sizeof(e_opts));
			//e_opts.offset	= fstart;
			//e_opts.length	= ihdr->fcnt;
			e_opts.offset	= 0;	/*start of NAND, up to firmimg.cmc start*/
			e_opts.length	= FLASH_OFF_START_FWUPD_IMAGE - 1; 
			e_opts.quiet	= 0;
			//e_opts->jffs2	= 1;
			e_opts.scrub	= 0;	/*NEVER EVER WRITE A NON-ZERO !!!*/
			if((rc = nand_erase_opts(nand, &e_opts)) != 0)
				{
				sprintf(msg, 
					"ERROR:Erasing \"%s\" NAND_p=0x%x size=0x%x RC=0x%x\n",
					ihdr->name, fstart, ihdr->fcnt, rc);
				rc = E_FWUPD_NAND_ERASE_OPT;
				break;
				}

			/*	Log any new erase blocks*/
			if(new_bad_blocks[0] != BAD_BLOCK_ARRAY_END_MARKER)
				add_to_err_blocks(new_bad_blocks, 0x45); /*"E" erase block*/
			}

		/*	Write NAND*/
		memset(&w_opts, 0, sizeof(w_opts));
		w_opts.buffer	= (u_char*)start_of_bin;
		w_opts.length	= bcnt;
		w_opts.offset	= fstart;
		/* w_opts.forcejffs2 = 1; */
		w_opts.pad	= 1;
		w_opts.blockalign = 1;
		w_opts.quiet	= 0;
		if((rc = nand_write_opts(nand, &w_opts)) != 0)
			{
			sprintf(msg, 
				"ERROR:Writing \"%s\" mem_p=0x%x NAND_p=0x%x size=0x%x RC=0x%x\n",
				ihdr->name, start_of_bin, fstart, bcnt, rc);
			rc = E_FWUPD_NAND_WRITE_OPT;
			break;
			}
		}
	/*************************************************************************
	*	Not a NOR or NAND.....major screwup
	*************************************************************************/
	else
		{
		sprintf(msg, "Invalid flash type %d\n", ihdr->flash_type);
		rc = E_FWUPD_INVALID_FLASH_TYPE;
		break;
		}

	/*************************************************************************
	*	Create a environment variable as an image descriptor containing the
	*	<starting offset> <byte count> <CRC32>
	*	This will be used to CRC any OS image BEFORE jumping into it.
	*************************************************************************/
	/* 
	* To support bad block managment for NAND we cannot hard code the starting 
	* image address as there might be a concentration of bad blocks anywhere in
	* the chip. Prior to 2.6.25 kernel the allocitations were static, now they 
	* can be dynamic. It is too much trouble calculating the bad blocks and 
	* dividing up between rootfs and var. Rootfs is RD only in production, dev
	* likes RW for lib image downloading/debug.  Just give roofs X Meg beyond
	* its size IF in vendor mode.  As the firmimg.cmc gets to its max AND IF we
	* have lots of bad blocks at worst case we steal a few Meg from var IN DEV
	* ONLY NON-VENDOR MODE LEAVES ROOFS END ITS ACTUAL SIZE, VAR STAYS AT 12M.
	*/
#if DYNAMIC_IMAGE_START	/*	2.6.25 Port*/
	if(ihdr->flash_type == FLASH_TYPE_NAND)
		{
		/*
		* fstart and bcnt are of the image that was just flashed.
		* Preserve them for the env creation below.
		*/
		next_fstart = (fstart + bcnt + (2 * CFG_NAND_BLOCK_SIZE)); 
		next_fstart &= ~CFG_NAND_BLOCK_SIZE_MASK;		/*make block aligned*/
		bad_blks = how_many_bad_blocks(nand, fstart, bcnt);
		next_fstart += ( bad_blks * CFG_NAND_BLOCK_SIZE);
		printf("fstart=0x%x next_fstart=0x%x bcnt=0x%x remap_blks=%d\n", 
										fstart, next_fstart, bcnt, bad_blks);

		if( !strcmp(IMAGE_TEXTNAME_ROOTFS, ihdr->name))
			{
			/*
			* I wanted to add 4M to rootfs if in vendor mode ONLY for a 
			* dev play area. But since Foxconn programs in vendor mode the 
			* image placement would not be identical to what the Dell 
			* factory or the customer would do.
			*
			* With 2.1 we must pass up on the cmd line the "exact" size at 
			* build time, we cannot "tack " on any 
			* additional bytes as the CRC fails and fwupdate is forced.
			*
			* DF315976: Saga continues
			* The final fix, and to give dev a XMeg play area is to always 
			* leave a XMeg gap between rootfs and var durning ANY type of 
			* update, vendor or non-vendor mode.  Then in cmc_10g.c IF in 
			* vendor mode tack on XMeg to the size.  We do NOT CRC in vendor 
			* mode so this should be ok. IF you remove vendor mode and reboot 
			* you willl probably get a forced fwupdate (normal).  So the play
			* area is always there, but in non-vendor mode boot the kernal 
			* does have access to this space.
			*/
			next_fstart += ROOTFS_VMODE_PAD;	//always leave play area gap
			//DF315976 do NOT increase size      bcnt = next_fstart - fstart;
			printf("rootfs next_fstart=0x%x bcnt=0x%x remap_blks=%d\n",
										next_fstart, bcnt, bad_blks);
			}
		else if( !strcmp(IMAGE_TEXTNAME_VARFS, ihdr->name))
			{
			/*
			* Was going to allow var to grow UP TO firmimg.cmc backup. This 
			* would be a better wear leveling story.  BUT the fear of abuse
			* of var means chop it to the defined size and forget it.
			*/
			bcnt = FLASH_MAX_SIZE_VAR;
			if( (fstart + bcnt) >= FLASH_OFF_START_FWUPD_IMAGE)
				{/*build should catch, do NOT allow growth into firmimg.cmc*/
				bcnt = (FLASH_OFF_START_FWUPD_IMAGE - fstart) - CFG_NAND_BLOCK_SIZE;
				}
			
			printf("var bcnt=0x%x remap_blks=%d\n", bcnt, bad_blks);
			}
		}/*end if NAND*/

#endif /*DYNAMIC_IMAGE_START*/

	sprintf(env_name, "%s%s", ENV_FWUPD_PRETEXT, ihdr->name);
	sprintf(env_data, "0x%x 0x%x 0x%x", fstart, bcnt, crc);
	printf("Writing Env descriptor \"%s %s\" \n", env_name, env_data);
	setenv(env_name, env_data);


	/*	When we finish u-boot, and ANY NOR image was flashed we have to reset*/
	if( (!strcmp(ihdr->name, IMAGE_TEXTNAME_UBOOT)) && nor_touched)
		break;
	}/*end for*/


/*	flashing is over, did it work*/
if(rc)
	{
	setenv(FWUPD_ENV_STATE, FWUPD_ENV_STATE_FAIL);		/*set failed state*/
	setenv(FWUPD_ENV_ERROR_MSG, msg);					/*save error message*/
	}
else
	{
	if(nor_touched)
		{
		/*	
		*	To get here means one or more NOR images had been updated with 
		*	u-boot being the last of the NOR's.  We want to reset to run with
		*	the latest u-boot code.
		*	1st choice is board reset.  BUT if an FPGA problem we will be 
		*	screwed, so we have to hit the core to make sure.
		*	NOTE: the fwupd_state has NOT been changed, it is still in the 
		*	pending state to make sure we re-enter fwupd() to do the OS images.
		*/
		/*	HACK: do we need a throtle env, or a "pass1" state in case we have a
		*	bad NOR?  We will spin forever in re-attempting fwupd().
		*/
		printf("\n**Now resetting CMC to run with new monitor...fwupdate will continue**\n");
		reset_after_fwupd();	
		}
	else
		{/*must be finishing a NAND flash, clean up all triggers of fwupd*/
		setenv(FWUPD_ENV_FW_VERSION, mhdr->version_string);	/*new FW version*/
		setenv(FWUPD_ENV_STATE, FWUPD_ENV_STATE_PASS);		/*pass state*/
		/*	Do not remove env of previous error, use as a debug trail*/
		/*setenv(FWUPD_ENV_ERROR_MSG, "");*/

		sprintf(env_data, "%d", (get_fwupd_count() + 1) );
		setenv(FWUPD_ENV_COUNT, env_data);					/*update count*/
		clear_fwupd_mode();									/*clear FPGA mode*/
		sprintf(msg, "CMC%d: FWupdate successful to version %s", 
				(ABS_read_fpga(FPGA_REG_CMC_SLOT_ID) & FPGA_CMC_SLOT_ID) + 1,
				mhdr->version_string);
		}
	}


/*	Save all env variables, use this common exit to not abuse the saveenv()*/
saved_silent = set_save_silent_mode();
saveenv();
restore_silent_mode(saved_silent);
return(rc);
}/*end FUNCTION: fwupd()*/



/****************************************************************************
*	FUNCTION: fwupd_flash2ram()
*	DATE:	  3-1-07
*
*	This API relocates the fwupdate image file from the dedicated NAND 
*	location into ram located at "mem_addr".
*
*	INPUT:	mem_addr - Starting address of the update image file. 
*			msg      - Buffer used to return any messages.
*	RETURN: 0        - image was moved and verified to be correct.
*			non-zero - a valid image was NOT found in NAND.
*
*	NOTE:	RETURN CODES *MUST* BE INFORMATIONAL AS WE DO NOT BLOCK THE BOOT
*			ON THIS FAILURE AS WE WANT TO POST AN OS EVENT.
*
*****************************************************************************/
s32 fwupd_flash2ram(u32 mem_addr, char *msg)
{
s32  rc = 0;
u32  err_cnt;
nand_read_options_t opts;

/*global structs containing nand info, cmc will always have 1 NAND*/
nand_info_t *nand			= &nand_info[0];
t_master_image_header *mhdr = (t_master_image_header*)mem_addr;

sprintf(msg, "Retrieve image file from redundant backup.");

/*	Read master header to get total image size*/
memset(&opts, 0, sizeof(opts));
opts.buffer	= (u_char*) mem_addr;
opts.length	= MASTER_IMAGE_HEADER_SIZE;
opts.offset	= FLASH_OFF_START_FWUPD_IMAGE;
opts.quiet	= 1;
if((rc = nand_read_opts(nand, &opts)) != 0)
	{
	if(rc != -1)	/*does NOT return from nand_ecc_scan() */
		nand_ecc_scan(CFG_FLASH_START_BLOCK, CFG_FLASH_NAND_SIZE, rc, 
											&err_cnt, NAND_SCAN_REBOOT, msg);
	sprintf(msg, "ERROR: nand read master header failure RC=0x%x\n", rc);
	return(I_FLASH_FLASH2RAM_HDR_READ);
	}

/*
*	Verify the master header token and name string.  This prevents us from 
*	reading a lot of nand crap from a bad/corrup image. Also do some byte count
*	bounds checking as another precaution.
*/
if(swizzle(mhdr->token) != IMAGE_TOKEN_MASTER)
	return(E_FWUPD_NO_MASTER_TOKEN);

if( strncmp(mhdr->name, IMAGE_TEXTNAME_MASTER, MAX_IMAGE_NAME_SIZE) )
	return(E_FWUPD_NO_MASTER_TEXTNAME);

if( (mhdr->image_bcnt < 0) || (mhdr->image_bcnt > gd->ram_size) )
	return(E_FWUPD_MASTER_BCNT_OUT_OF_RANGE);


/*	Now read entire image*/
memset(&opts, 0, sizeof(opts));
opts.buffer	= (u_char*) mem_addr;
opts.length	= mhdr->image_bcnt;
opts.offset	= FLASH_OFF_START_FWUPD_IMAGE;
opts.quiet	= 1;
if((rc = nand_read_opts(nand, &opts)) != 0)
	{
	if(rc != -1)	/*does NOT return from nand_ecc_scan() */
			nand_ecc_scan(CFG_FLASH_START_BLOCK, CFG_FLASH_NAND_SIZE, rc, 
											&err_cnt, NAND_SCAN_REBOOT, msg);
	sprintf(msg, "ERROR: reading entire image from nand RC=0x%x\n", rc);
	return(I_FLASH_FLASH2RAM_IMAGE_READ);
	}

return(0);
}/*end FUNCTION: fwupd_flash2ram()*/


/****************************************************************************
*	FUNCTION: fwupd_ram2flash()
*	DATE:	  3-1-07
*
*	This API moves a firmimg.cmc file that was previously downloaded into ram
*	by "recover fwupdate", into a dedicated offset in the NAND.  
*	FWupdate under the OS will write the binary to this same location. 
*
*	INPUT:	mem_addr - Starting address of the firmimg.cmc data. 
*			msg      - Buffer used to return any messages.
*	RETURN: 0        - image was verified to be correct and moved.
*			non-zero - a valid image CANNOT be relocated to flash.
*
*****************************************************************************/
s32 fwupd_ram2flash(u32 mem_addr, u32 bcnt, char *msg)
{
s32 rc = 0;
nand_write_options_t	w_opts;
nand_erase_options_t	e_opts;
/*global structs containing nand info, cmc will always have 1 NAND*/
nand_info_t *nand		= &nand_info[0];


printf("\nSaving image file as redundant backup.\n");
if(bcnt > FLASH_MAX_SIZE_FWUPD_IMAGE)
	{
	sprintf(msg, "Image is too big for partition, max=0x%x actual=0x%x",															FLASH_MAX_SIZE_FWUPD_IMAGE, bcnt);
	return(E_FLASH_NAND_RAM2FLASH_TOO_LARGE);
	}

#if 0
printf("NAND start=0x%x erase size=0x%x bcnt=0x%x\n", 
			FLASH_OFF_START_FWUPD_IMAGE, FLASH_MAX_SIZE_FWUPD_IMAGE, bcnt);
#endif

/*	Erase NAND area*/
memset(&e_opts, 0, sizeof(e_opts));
e_opts.offset	= FLASH_OFF_START_FWUPD_IMAGE;
e_opts.length	= FLASH_MAX_SIZE_FWUPD_IMAGE;
e_opts.quiet	= 0;
//e_opts->jffs2	= 1;
e_opts.scrub	= 0;	/*NEVER EVER WRITE A NON-ZERO !!!*/
if((rc = nand_erase_opts(nand, &e_opts)) != 0)
	{
	sprintf(msg, "NAND erase at 0x%x RC=0x%x", FLASH_OFF_START_FWUPD_IMAGE, rc);
	return(E_FLASH_NAND_RAM2FLASH_ERASE);
	}
/*	Log any new erase blocks*/
if(new_bad_blocks[0] != BAD_BLOCK_ARRAY_END_MARKER)
	add_to_err_blocks(new_bad_blocks, 0x45); /*"E" erase block*/

/*	Write NAND*/
memset(&w_opts, 0, sizeof(w_opts));
w_opts.buffer	= (u_char*)mem_addr;
w_opts.length	= bcnt;
w_opts.offset	= FLASH_OFF_START_FWUPD_IMAGE;
/* w_opts.forcejffs2 = 1; */
w_opts.pad	= 1;
w_opts.blockalign = 1;
w_opts.quiet	= 0;
if((rc = nand_write_opts(nand, &w_opts)) != 0)
	{
	sprintf(msg, "ERROR: nand write failure RC=0x%x\n", rc);
	return(E_FLASH_NAND_RAM2FLASH_WRITE);
	}

return(0);
}/*end FUNCTION: fwupd_ram2flash()*/


/****************************************************************************
*	FUNCTION: fwupd_verify_image()
*	DATE:	  3-1-07
*
*	This API verifies the master header and the overall CRC32 which includes 
*	ALL the individual image headers and their binaries.
*
*	INPUT:	mem_start  - Starting memory location of the image file.
*			bcnt_p     - total byte count of image, all hdrs, all binaries
*			msg        - Buffer used to return any messages.
*
*	RETURN: 0        - image verified ok.
*			non-zero - image verification failed.
*
*****************************************************************************/
s32 fwupd_verify_image(u32 mem_start, u32 *bcnt_p, char *msg)
{
u32 start_addr;
u32 calc_crc;
t_master_image_header *mhdr = (t_master_image_header*)mem_start;

*bcnt_p = 0;	/*init to 0 in case of a failure*/

/*	
*	Look for valid token and name string. The build tools do not swizzle the 
*	token as this is a good debug 4 character string when going a octal dump
*/
if(swizzle(mhdr->token) != IMAGE_TOKEN_MASTER)
	{
	sprintf(msg, "Image token not found at 0x%x, exp=0x%x rec=0x%x", 
					&mhdr->token, IMAGE_TOKEN_MASTER, swizzle(mhdr->token) );
	return(E_FWUPD_NO_MASTER_TOKEN);
	}

if( strncmp(mhdr->name, IMAGE_TEXTNAME_MASTER, MAX_IMAGE_NAME_SIZE) )
	{
	sprintf(msg, "Image text name cannot be found at 0x%x.", &mhdr->name);
	return(E_FWUPD_NO_MASTER_TEXTNAME);
	}

/*
*	Verify the update image CRC32.	All image headers and binaries are 
*	included. Only the master header is not part of the CRC32.
*
*	Make some gross integrety checks for byte count and the starting offset 
*	where we start calculating the checksum.
*/
if( (mhdr->image_bcnt < 0) || (mhdr->image_bcnt > gd->ram_size) )
	{
	sprintf(msg, "Total image byte count is out of range, bcnt=0x%x",
														mhdr->image_bcnt);
	return(E_FWUPD_MASTER_BCNT_OUT_OF_RANGE);
	}

/*	Cannot be lower than sizeof mhdr struct or cannot run off the top*/
if( (mhdr->image_header_offset[0] < 0) ||
    (mhdr->image_header_offset[0] > gd->ram_size) )
	{
	sprintf(msg, "Starting image addr it out of range, offset=0x%x",
												mhdr->image_header_offset[0]);
	return(E_FWUPD_MASTER_HDR_OFF_OUT_OF_RANGE);
	}

start_addr = ((u32)mhdr + mhdr->image_header_offset[0]);
calc_crc = crc32(0, (u8*)start_addr, mhdr->total_bcnt);

sprintf(msg, "Verify FWupdate image crc32: bytes=0x%x, crc32=0x%x\n", 
												mhdr->total_bcnt, calc_crc);

if(mhdr->total_crc32 != calc_crc)
	{
	sprintf(msg, "Image checksum failure\n      "
		"start=0x%x total_bcnt=0x%x exp=0x%x calc=0x%x\n",
		start_addr, mhdr->total_bcnt, mhdr->total_crc32, calc_crc);
	return(E_FWUPD_CHECKSUM_FAILURE);
	}

*bcnt_p = mhdr->image_bcnt;
return(0);
}/*end FUNCTION: fwupd_verify_image()*/



/****************************************************************************
*	FUNCTION: fwupd_dump_hdr_info()
*	DATE:	  3-6-07
*
*	This API displays the master and image header information.  
*
*	INPUT:	mem_start- memory pointer to hold the master header.
*			msg      - Buffer used to return any messages.
*	RETURN: 0        - Header information was displayed
*			non-zero - Error identifing header.
*
*****************************************************************************/
s32 fwupd_dump_hdr_info(u32 mem_start, char *msg)
{
u32	i;
s32 rc;
u32 bcnt;
t_image_header			*ihdr;
t_master_image_header	*mhdr = (t_master_image_header*)mem_start;


/*	First verify integerty of the image file*/
if( (rc = fwupd_verify_image(mem_start, &bcnt, msg)) != 0)
	return(rc);


switch(mhdr->header_type)
	{
	case MASTER_HEADER_TYPE_1 :
	default:
		/*	Master header info (build tools do not swizzle token)*/
		printf("\nImage: %s\n", mhdr->name);
		printf("       Header Type: 0x%x  Token: 0x%x  Rac_HW_Type: 0x%x\n",
				mhdr->header_type, swizzle(mhdr->token), mhdr->rac_hw_type);
		printf("       Version: %s\n       Build: %s\n", 
					mhdr->version_string, mhdr->build_string);

		//	Image header info 
		for(i = 0; i < mhdr->num_of_images; ++i)
			{
			ihdr = (t_image_header*)((u32)mhdr + mhdr->image_header_offset[i]);
			printf("\nImage: %s\n", ihdr->name);
			printf("       Header Type: 0x%x  Token: 0x%x\n", 
									ihdr->header_type, swizzle(ihdr->token) );
			printf("       Byte count zip: %d  UNzip: %d\n",
								ihdr->bcnt_zip, ihdr->bcnt_unzip);
			printf("       Starting Sector: %d  Sector count: %d\n",
								ihdr->fstart, ihdr->fcnt);
			}
		break;
	}//end switch


return(0);
}//end FUNCTION: fwupd_dump_hdr_info()


/****************************************************************************
*	FUNCTION: fwupd_unprotect()
*	DATE:	  3-8-07
*
*	In NOR u-boot is protected.  Cannot risk any area being protected.
*	Make sure and always unprotect NOR before attempting to erase.
*	Remember we have a top boot device, the top 8 sectors are only 8K in size.
*
*	INPUT:	start_sector - Starting sector to erase.
*			sector_count - Number of sectors to erase.
*			msg      - Buffer used to return any messages.
*	RETURN: 0        - Flash area is unprotected.
*			non-zero - Error, cannot unprotect
*
*****************************************************************************/
static s32 fwupd_unprotect(u32 start_sector, u32 sector_count, char *msg)
{
s32 rc;
s32 saved_silent;
u32 sector     = start_sector;
u32 start_addr = SECTOR_TO_PHYSADDR(start_sector);
u32 end_addr   = start_addr;

while(sector_count--)
	{
	if(sector >= CFG_FLASH_1ST_8K_SECT)
		end_addr += CFG_SECT_SIZE_8K;
	else
		end_addr += CFG_SECT_SIZE;

	++sector;
	}
end_addr = end_addr -1;

saved_silent = set_save_silent_mode();
rc = flash_sect_protect(0, start_addr, end_addr);
restore_silent_mode(saved_silent);

if(rc)
	{
	sprintf(msg, "Cannot unprotect 0x%x -through- 0x%x RC=0x%x\n",  
													start_addr, end_addr, rc);
	return(E_FWUPD_CANNOT_UNPROTECT_RANGE);
	}


 
return(0);
}/*end FUNCTION: fwupd_unprotect() */


/****************************************************************************
*	FUNCTION: post_verify_os_images()
*	DATE:	  3-26-07
*
*	Do a CRC32 on the OS images that are read only.  We cannot CRC /var as 
*	it is dynamic.  The env formats is:
*	<Flash offset>	<byte count> <expected CRC32>
*
*	MOD: 12-16-08
*	Now that we put in NAND ECC double bit error re-scan.  We cannot CRC but
*	we can re-scan to flush out problems.
*
*****************************************************************************/
s32 post_verify_os_images(char *msg)
{
s32 rc;
u32 offset;
u32 bcnt;
u32 exp_crc;
u32 flash_crc;
char buf[60];
u32  err_cnt;
nand_read_options_t opts;

/*global structs containing nand info, cmc will always have 1 NAND*/
nand_info_t *nand			= &nand_info[0];

/*null buffer, used if file system CRC is omitted with vendor mode*/
buf[0] = '\0';				

/*	Get kernel parameters*/
if( (rc = parse_env_descriptor(ENV_FWUPD_KERNEL, &offset, &bcnt, &exp_crc, 
																	msg)) != 0)
	return(rc);

/*	Read in kernel image and crc it*/
memset(&opts, 0, sizeof(opts));
opts.buffer	= (u_char*) CFG_LOAD_ADDR;
opts.length	= bcnt;
opts.offset	= offset;
opts.quiet	= 1; 
if((rc = nand_read_opts(nand, &opts)) != 0)
	{
	if(rc != -1)	/*does NOT return from nand_ecc_scan() */
			nand_ecc_scan(CFG_FLASH_START_BLOCK, CFG_FLASH_NAND_SIZE, rc, 
											&err_cnt, NAND_SCAN_NOREBOOT, msg);
	sprintf(msg, "ERROR: nand read kernel failure RC=0x%x\n", rc);
	return(E_FLASH_READ_OS_KERNEL);
	}
	
flash_crc = crc32(0, (u8*)CFG_LOAD_ADDR, bcnt);
if(exp_crc != flash_crc)
	{
	sprintf(msg, "Kernal image failed CRC32 exp=0x%x rec=0x%x",
														exp_crc, flash_crc);
	return(E_FWUPD_OS_IMAGE_KERNEL);
	}

/*
*	If the file systems are marked RW for debug then the CRC WILL fail.
*	Only allow this CRC check to be skipped in vendor mode.
*/
if(!is_vendor_mode() )
	{
	/*	Get root file system parameters*/
	if( (rc = parse_env_descriptor(ENV_FWUPD_ROOTFS, &offset, &bcnt, &exp_crc, 
																	msg)) != 0)
		return(rc);
	
	/*	Read in root file system image and crc it*/
	memset(&opts, 0, sizeof(opts));
	opts.buffer	= (u_char*) CFG_LOAD_ADDR;
	opts.length	= bcnt;
	opts.offset	= offset;
	opts.quiet	= 1; 
	if((rc = nand_read_opts(nand, &opts)) != 0)
		{
		if(rc != -1)	/*does NOT return from nand_ecc_scan() */
			nand_ecc_scan(CFG_FLASH_START_BLOCK, CFG_FLASH_NAND_SIZE, rc, 
											&err_cnt, NAND_SCAN_NOREBOOT, msg);

		sprintf(msg, "ERROR: nand read root file system failure RC=0x%x\n", rc);
		return(E_FLASH_READ_ROOT_FS);
		}

	flash_crc = crc32(0, (u8*)CFG_LOAD_ADDR, bcnt);
	if(exp_crc != flash_crc)
		{
		sprintf(msg, "root_fs image failed CRC exp=0x%x rec=0x%x\n",
															exp_crc, flash_crc);
		return(E_FWUPD_OS_IMAGE_ROOT_FS);
		}
	}
else
	{
	sprintf(buf, "(Omit CRC check for %s)", ENV_FWUPD_ROOTFS);
	}
	

/*	Get var parameters*/
if( (rc = parse_env_descriptor(ENV_FWUPD_VARFS, &offset, &bcnt, &exp_crc, 
																	msg)) != 0)
	return(rc);

/* NOTE: If included this verification adds about 10 seconds to the boot */
#if 0	
/*	Read in var image and crc it*/
memset(&opts, 0, sizeof(opts));
opts.buffer	= (u_char*) CFG_LOAD_ADDR;
opts.length	= bcnt;
opts.offset	= offset;
opts.quiet	= 1; 
if((rc = nand_read_opts(nand, &opts)) != 0)
	{
	if(rc != -1)	/*does NOT return from nand_ecc_scan() */
		{
		/*force a reflash after reset*/
		set_fwupd_state_env(FWUPD_ENV_STATE_PENDING); 
		nand_ecc_scan(CFG_FLASH_START_BLOCK, CFG_FLASH_NAND_SIZE, rc, 
											&err_cnt, NAND_SCAN_NOREBOOT, msg);
		}
	sprintf(msg, "ERROR: nand read var failure RC=0x%x\n", rc);
	return(E_FLASH_READ_VAR_FS);
	}
#endif


sprintf(msg, "OS images verified via CRC32 %s", buf);
return(0);
}/*end FUNCTION: post_verify_os_images() */


/****************************************************************************
*	FUNCTION: parse_env_descriptor()
*	DATE:	  3-26-07
*
*	Parse the environment varible string for the specified image file.
*	The env formats is:
*	<Flash offset/starting sector>	<byte count> <expected CRC32>
*
*	INPUT:	env_desc - env name of fwupdate image
*			offset - flash offset for start of image
*			bcnt   - byte count of image
*			exp_crc- expected crc32 of image
*			msg      - Buffer used to return any messages.
*
*****************************************************************************/
s32 parse_env_descriptor(char *env_desc, u32 *offset, u32 *bcnt, u32 *exp_crc,
																	char *msg)
{
char *start_p;
char *bcnt_p;
char *crc_p;
char *cp;
char buf[100];


/*	Verify Kernel*/
if( (start_p = getenv(env_desc)) == 0)
	{
	sprintf(msg, "ENV descriptor for \"%s\" is missing",  env_desc);
	return(E_FWUPD_ENV_DESCRIPTOR_MISSING);
	}

/*Make a copy as I need to chop up and convert to hex, DONT touch the original*/
strcpy(buf, start_p);			

bcnt_p = strchr(buf, ' ');
*bcnt_p++ = '\0';
crc_p  = strchr(bcnt_p, ' ');
*crc_p++ = '\0';


*offset = (s32)simple_strtol(buf, &cp, 16);
if( (s32)cp < (strlen(buf) + (s32)buf) )
	{
	sprintf(msg, "ENV descriptor \"%s\" has a bad offset", env_desc);
	return(E_FWUPD_ENV_BAD_OFFSET);
	}

*bcnt = (s32)simple_strtol(bcnt_p, &cp, 16);
if( (s32)cp < (strlen(bcnt_p) + (s32)bcnt_p) )
	{
	sprintf(msg, "ENV descriptor \"%s\" has a bad byte count", env_desc);
	return(E_FWUPD_ENV_BAD_BYTE_COUNT);
	}

*exp_crc = (s32)simple_strtol(crc_p, &cp, 16);
if( (s32)cp < (strlen(crc_p) + (s32)crc_p) )
	{
	sprintf(msg, "ENV descriptor \"%s\" has a bad exp crc32", env_desc);
	return(E_FWUPD_ENV_BAD_EXP_CRC);
	}

return(0);
}/*end FUNCTION: parse_env_descriptor() */


/****************************************************************************
*	FUNCTION: post_verify_fwupd_image()
*	DATE:	  4-4-07
*
*	Verify the fwupdate image that is located in NAND flash.  The image must
*	first be copied from NAND to ram.  Leave the image in ram to be used by
*	the caller.
*	
*	INPUT:	mem_addr - Starting address of the firmimg.cmc data. 
*			msg      - Buffer used to return any messages.
*	RETURN:	0 - image verified ok
*			non-zero - image failed
*
*****************************************************************************/
s32 post_verify_fwupd_image(u32 mem_addr, char *msg)
{
s32 rc;
u32 bcnt;
t_master_image_header	*mhdr = (t_master_image_header*)CFG_LOAD_ADDR;

if( (rc = fwupd_flash2ram(mem_addr, msg)) != 0)
	{
	sprintf(msg, "Error reading, or invalid redundant update image RC=0x%x", rc);
	return(rc);									/*returns informational*/
	}

if( (rc = fwupd_verify_image(mem_addr, &bcnt, msg)) != 0)
	{
	sprintf(msg, "Cannot verify redundant update image RC=0x%x", rc);
	return(E_FLASH_FLASH2RAM_VERIFY_IMAGE);		/*change to informational*/
	}

sprintf(msg, "Verified redundant update image \"%s\" ", mhdr->version_string);
return(0);
}/*end FUNCTION: Retriev() */


/****************************************************************************
*	FUNCTION: is_fwupd_pending()
*	DATE:	  4-4-07
*
*	Linux will set an environment variable to indicate a fwupdate should be
*	performed.  This is a test of that variable.
*	INPUT: none
*	RETURN: 0 - env var is NOT set
*			1 - env var exists and indicates to update
*****************************************************************************/
s32 is_fwupd_env_pending(void)
{
char *env_p;

if( (env_p = getenv(FWUPD_ENV_STATE)) == 0)
	return(0);								/*env var does not exists*/

if(strcmp(env_p, FWUPD_ENV_STATE_PENDING))
	return(0);								/*exists but no start msg*/
else
	return(1);								/*flag is present*/
}/*end FUNCTION: is_fwupd_pending() */

/****************************************************************************
*	FUNCTION: set_fwupd_state_env()
*	DATE:	  4-4-07
*
*	Clear the env var flag that was set by Linux indicating a fwupdate 
*	was pending.
*****************************************************************************/
s32 set_fwupd_state_env(char *state)
{
s32  saved_silent;				/*save quiet flag*/

setenv(FWUPD_ENV_STATE, state);
saved_silent = set_save_silent_mode();
saveenv();
restore_silent_mode(saved_silent);
return(0);
}/*end FUNCTION: set_fwupd_state_env() */

/****************************************************************************
*	FUNCTION: get_fwupd_count()
*	DATE:	  4-4-07
*
*	Open FWUPD_ENV_COUNT if it exists, convert to decimal and return.
*
*****************************************************************************/
s32 get_fwupd_count(void)
{
char *env_p;
char *cp;
s32  count;

if( (env_p = getenv(FWUPD_ENV_COUNT)) == 0)
	return(0);							/*env var does not exists, 0 count*/

count = (s32)simple_strtol(env_p, &cp, 10);
if( (s32)cp < (strlen(env_p) + (s32)env_p) )
	return(0);							/*conversion error, o count*/

return(count);
}/*end FUNCTION: get_fwupd_count() */


/****************************************************************************
*	FUNCTION: is_nor_image_ok()
*	DATE:	  4-7-07
*
*	Calculate the CRC32 of an image in NOR flash and compare it against the
*	CRC32 of the update image.  
*
*	NOTE: We MUST calculate the CRC with the address and byte count from the
*	update image and NOT the env descriptor.  Just in case the new image is 
*	in a different location.
*
*	INPUT:	image_name - Image name for print message.
*			addr - NOR address to start the crc calculation
*			bcnt - number of bytes to crc
*			exp_crc - expected crc to determine if the image is different.
*
*	RETURN: 0 - NOR image is NOT ok and will require update
*			1 - NOR image IS ok and does NOT require updating.
*
*****************************************************************************/
s32 is_nor_image_ok(char *image_name, u32 addr, u32 bcnt, u32 exp_crc)
{
u32  flash_crc;

/*	If re-calculated CRC is different , we must update*/
flash_crc = crc32(0, (u8*)addr, bcnt);
if(exp_crc != flash_crc)
	return(0);
else
	{	
	printf("Omit flashing \"%s\" crc=0x%x is identical to update image.\n", 
														image_name, exp_crc); 
	return(1);
	}
}/*end FUNCTION: is_nor_image_ok() */


static s32 how_many_bad_blocks(nand_info_t *nand, u32 next_fstart, u32 bcnt)
{
s32 bbcnt = 0;
u32 off = next_fstart;
s32 cnt = 0;

while(cnt < bcnt)
	{
	if (nand_block_isbad(nand, off))
		++bbcnt;
	else
		cnt += CFG_NAND_BLOCK_SIZE;

	off += CFG_NAND_BLOCK_SIZE;
	}
return(bbcnt);

}/*end FUNCTION:how_many_bad_blocks()*/

#endif /*CONFIG_CMC_10G*/
