/****************************************************************************

  Copyright (c) 2006, Dell Inc
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

  - Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
    
  - Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution. 

  - Neither the name of Dell Inc nor the names of its contributors
    may be used to endorse or promote products derived from this software 
    without specific prior written permission. 

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. 

*****************************************************************************/

/************************************************************************
*
*	FILE:	util_getopt.c
*
*	
************************************************************************/
#include <common.h>		/*must be first in file*/

#ifdef CONFIG_CMC_10G	/*only compile in for a Dell build*/


/************************************************************************
*  Include files
************************************************************************/
#include <util_defs.h>
#include <util_ret_codes.h>


/*****************************************************************************
*	List all prototypes used in the utility commands,
*	and the external reference.
*****************************************************************************/



/*****************************************************************************
*	Globals for mcgetopt() 
*****************************************************************************/
u32		mcoptind = 1;
char	*mcoptarg[MCGETOPT_MAX_OPTARG];
/*
*	parsed_opts[] MUST be global for the ptrs in mcoptarg to have a valid
*	reference in which to access then from the caller.
*/
char	parsed_opts[MCGETOPT_MAX_PARSED_LIST_SIZE];

/******************************************************************************
*	List all structures defines
******************************************************************************/



/*****************************************************************************
*
*	FUNCTION: mcgetopt
*	DATE:	10-1-04
*
*	This "multi-character getopt()" call has some modifications from the 
*	traditional getopt().   
*	Differences are:
*	1) Supports multi character options. You can enter options like 
*	   "-32  -off -reset  -a -d -thisIsALongOption"  The option MUST start
*	   with a minus '-', and continues to the first space.  Options MUST
*	   be seperated with a space. 
* 
*	2) Supports multi arguments per option, mcoptarg is now an array, mcoptarg[].
*      Example "-a 0xa0100000 0xa010fff0"  The option for "-a" will then find
*	   the two arguments in mcoptarg[0], and mcoptarg[1].  The arg list in 
*	   traditional getopt() will contain a ':' character indicating that a 
*	   single argument is expected.  This mcgetopt will have ':x' where x = the 
*	   number of arguments expected.
*
*	   If you specify 2 arguments associated with the option.  Then two 
*	   arguments are required.  
*	
*	3) In mcgetopt upon finding the option return the index of where the option
*	   is found in the calling option list.  SO THE CALLER MUST MAKE THE ORDER
*	   OF THE CASE STATMENTS MATCH THE ORDER OF THE OPTION LIST.
*
*	4) The option list must also include the '-' char.
*	5) Example of caller:
*/
/*

// ex, "util <subcommand> -32 -a a0100000 100"
// mcoptind will need to start with 2, there are 2 items before the options start



char	msg[MAX_MISC_MSG_SIZE];			//misc message buffer
u32		index;

mcoptind = 2;		//Offset in argv[] to start the search
while( (index = mcgetopt(argc, argv, "-8 -16 -32 -reset -a:2 -l:1")) != MCGETOPT_EOF)
	{
	switch(index) {
		case 0 :	//process the "8"
			break;

		case 1 :	//process the "16"
			break;

		case 2 :	//process the "32"
			break;

		case 3 :	//process the "reset"
			break;

		case 4 :	//process the "a"  MUST have 2 arguments
			break;

		case 5 :	//process the "l"  MUST have 1 argument
			break;
				
		default:		//not good
			rc = mcgetoptsError(index, msg, mcoptarg[0]);
			printf("%s %s:%s\n", argv[0], argv[1], msg);
			return(rc);
			break;
	}
}//end mcgetopt while


*/
/*****************************************************************************/
u32 mcgetopt(int argc, char **argv, char *opts)
{
char	*ptr, *saved_op_ptr;
char	*parsed_opt_ptrs[MCGETOPT_MAX_OP_LIST_PTRS];
s32		opts_idx, parse_opts_idx, popt_ptr_idx, done, opt_arg_idx, rc;
s32		opt_arg_cnt, found = 0;

#if 0 //debug
printf("HACK argc = %d  mcoptind = %d\n", argc, mcoptind);
#endif

/*
*	If there are no arguments exit
*/
if(mcoptind >= argc)
	return(MCGETOPT_EOF);


/*
*	Make sure that a option list was passed in.
*/
if( (opts == NULL) || ( strlen(opts) == 0) )
	return(E_MCGETOPT_MISSING_OP_LIST);

/*
*	Since we support multi char options, and multi arguments for a single 
*	option, first parse the "option list" to break out the options, args 
*	indicator (":"), number of args, and option arguments themselves.
*	This makes the search easier.
*/
opts_idx		= 0;
parse_opts_idx	= 0;
popt_ptr_idx	= 0;
done			= 0;
ptr				= parsed_opts;

while (!done) 
	{
	switch (opts[opts_idx])
		{
		case ' ':
			parsed_opts[parse_opts_idx++] = '\0';	//terminate string
			parsed_opt_ptrs[popt_ptr_idx++] = ptr;	//add string ptr to list
			ptr = &parsed_opts[parse_opts_idx];	//start of new ptr
			break;

		case ':':
			parsed_opts[parse_opts_idx++] = '\0';	//terminate string
			parsed_opt_ptrs[popt_ptr_idx++] = ptr;	//add string ptr to list
			ptr = &parsed_opts[parse_opts_idx];	//start of new ptr

			parsed_opts[parse_opts_idx++] = ':';	//new string of ":"
			parsed_opts[parse_opts_idx++] = '\0';	//terminate string
			parsed_opt_ptrs[popt_ptr_idx++] = ptr;	//add ":" string ptr to list
			ptr = &parsed_opts[parse_opts_idx];	//start of new ptr

			//Sneak a peek ahead to make sure there is a arg count
			if(opts[opts_idx + 1] == '\0')
				return(E_MCGETOPT_NO_ARG_CNT_SPECIFIED);
			break;

		case '\0':
			parsed_opts[parse_opts_idx] = '\0';	//terminate string
			parsed_opt_ptrs[popt_ptr_idx++] = ptr;	//add string ptr to list
			parsed_opt_ptrs[popt_ptr_idx++] = '\0';	//null parsed list
			done = 1;
			break;

		default:
			parsed_opts[parse_opts_idx++] = opts[opts_idx];
			break;
		}

	++opts_idx;
	}//end while



#if 0	//debug
for(i = 0; parsed_opt_ptrs[i] != NULL; ++i)
	printf("%s\n", parsed_opt_ptrs[i]);	
#endif


/*
*	Search the opts list that was previously passed in and parsed, 
*	against the current argv pointed to by mcoptind.
*/
popt_ptr_idx	= 0;
opt_arg_idx		= 0;

// Pre-load with error, will be overwritten if a valid oarg is found
mcoptarg[opt_arg_idx++] = argv[mcoptind];
rc = -1;

while(parsed_opt_ptrs[popt_ptr_idx] != NULL)
	{
	// Must start off with a '-'
	if(argv[mcoptind][0] != '-')
		{
		opt_arg_idx = 0;
		mcoptarg[opt_arg_idx++] = argv[mcoptind];							
		return(E_MCGETOPT_MISSING_MINUS);
		}

	//rc is the index for the caller switch(), only inc for "-"opts
	if(parsed_opt_ptrs[popt_ptr_idx][0] == '-')
		++rc;	//inc rc which is the index for the caller switch()


	// see if this pass finds the option
	if( (strcmp(argv[mcoptind], parsed_opt_ptrs[popt_ptr_idx])) == 0)
		{
		saved_op_ptr = argv[mcoptind];	//save user option off for error returns
		found = 1;
		/*
		*	Found the option, see if there are args associated with it.
		*	If so make sure the correct number of arg are available.
		*/
		if(parsed_opt_ptrs[++popt_ptr_idx] == NULL)
			break;

		if(parsed_opt_ptrs[popt_ptr_idx][0] == ':')		// arg seperator
			{
						
			if(parsed_opt_ptrs[++popt_ptr_idx] == NULL)	// no expected args
				{
				opt_arg_idx = 0;
				mcoptarg[opt_arg_idx++] = saved_op_ptr;
				return(E_MCGETOPT_TO_FEW_ARGS);
				}

			// Pick up the number of arguments
			opt_arg_cnt = (u32)simple_strtoul( parsed_opt_ptrs[popt_ptr_idx], 0, 0 );

			if(opt_arg_cnt <= 0)
				{									//too few args entered
				opt_arg_idx = 0;
				mcoptarg[opt_arg_idx++] = saved_op_ptr;
				return(E_MCGETOPT_TO_FEW_ARGS);
				}
	
			opt_arg_idx = 0;
			while(opt_arg_cnt)
				{
	
				if(++mcoptind >= argc)
					{
					opt_arg_idx = 0;
					mcoptarg[opt_arg_idx++] = saved_op_ptr;
					return(E_MCGETOPT_TO_FEW_ARGS);
					}

				mcoptarg[opt_arg_idx++] = argv[mcoptind];	//save the found arg
#if 0 //debug
				printf("HACK argCnt=%d argc=%d mcoptind=%d argv=%s opt=%s rc=%d\n",
						opt_arg_cnt, argc, mcoptind, argv[mcoptind], saved_op_ptr, rc);
#endif
				--opt_arg_cnt;
				}//while
			}

		break;
		}
	else
		{
		++popt_ptr_idx;					//keep looking
		}
	}//end while


if(!found)
	return(E_MCGETOPT_INVALID_OP);

mcoptarg[opt_arg_idx] = 0;				//null terminate arg list
++mcoptind;								//adjust mcoptind
return(rc);
}/*end FUNCTION: mcgetopt()*/




/*****************************************************************************
*
*	FUNCTION: parse_addr()
*	DATE:	10-6-04
*
*	This function take the various user entered syntax'es of the address 
*	option and parses out the start, ending addresses, and/or the count.
*
*	The address syntax choices can be:
*	1)  <start address>
*	2)  <start address>:<end address>	':' seperated
*	3)  <start address>#<count>			'#' seperated
*	
*	For hex values they must be proceeded by "0x".
*	The count is always BYTES, reguardless of the buswidth of the command.
*
******************************************************************************/
u32 parse_addr(char *str, u32 *start_addr, u32 *end_addr, s32 *count,
												u32 width, char *msg)
{
char   s[PARSE_ADDR_MAX_LEN];
char   *start_ptr	= 0;		/*start address*/
char   *end_ptr		= 0;		/*end adress*/
char   *count_ptr	= 0;		/*byte count*/
char   *bad_addr;				/*used to determine failure in simple_strtoul*/
u32     temp;

/*
*	Bring in the user string and make sure there is something there.
*	start_ptr allways points to start of the string
*/
strncpy(s, str, PARSE_ADDR_MAX_LEN);
s[PARSE_ADDR_MAX_LEN-1] = '\0';

if(strlen(s))
	start_ptr = s;			
else
	{
	sprintf(msg, "invalid user entered address range.\n");
	return(E_PARSE_ADDR_BAD_USER_STRING);
	}


/*
*	Check for an address seperator, or a count seperator
*/
if( (end_ptr = strchr(s, ':')) != NULL)
	*end_ptr++	= '\0';
else if( (count_ptr = strchr(s, '#')) != NULL)
	*count_ptr++ = '\0';


/*
*	Get the hex value for start
*/
temp = (u32)simple_strtoul( start_ptr, &bad_addr, 0 );

if( (s32)bad_addr < (strlen(start_ptr) + (s32)start_ptr) ) 
	{
	sprintf(msg, "invalid start address field.");
	return(E_PARSE_ADDR_BAD_START_ADDR);
	}

*start_addr = temp;

/*
*	If end_ptr, get the hex value for the ending address, then calc the byte count
*	If count_ptr, get the hex value for the byte count, then calc the ending address.
*	If neither then ending address = starting address, count is one byte.
*/
if(end_ptr != NULL)
	{
	temp = (u32)simple_strtoul( end_ptr, &bad_addr, 0 );

	if( (s32)bad_addr < (strlen(end_ptr) + (s32)end_ptr) ) 
		{
		sprintf(msg, "invalid end address field.");
		return(E_PARSE_ADDR_BAD_END_ADDR);
		}

	*end_addr = temp;
	*count = (temp - *start_addr) + width;
	}
else if(count_ptr != NULL)
	{	
	temp = (u32)simple_strtoul( count_ptr, &bad_addr, 0 );

	if( (s32)bad_addr < (strlen(count_ptr) + (s32)count_ptr) ) 
		{
		sprintf(msg, "invalid count field.");
		return(E_PARSE_ADDR_BAD_COUNT);
		}

	*count = temp;
	*end_addr = (*start_addr + temp) - width;
	}
else
	{
	*end_addr	= *start_addr;
	*count		= width;				
	}


/*
*	Do some user friendly checks.
*	If the saddr and eaddr are the same, adjust the byte count as the width.  
*/
if(*start_addr == *end_addr)
	*count = width;


/*
*	Verify the address alignments
*/
if( ((*start_addr & 0x1) && (width == WIDTH_16))  ||
	((*start_addr & 0x3) && (width == WIDTH_32)) )
	{
	sprintf(msg, "misaligned starting address.");
	return(E_UTIL_ADDR_MISALIGNED);
	}


if( ((*end_addr & 0x1) && (width == WIDTH_16))  ||
	((*end_addr & 0x3) && (width == WIDTH_32)) )
	{
	sprintf(msg, "misaligned ending address.");
	return(E_UTIL_ADDR_MISALIGNED);
	}


// Make sure the ending address is bigger than the starting.
if(*start_addr > *end_addr)
	{
	sprintf(msg, "starting address is greater than ending.");
	return(E_UTIL_START_LESSTHAN_ENDING);
	}

return(0);
}//end FUNCTION: parse_addr()




/*****************************************************************************
*
*	FUNCTION:	mcgetopt_error()
*	DATE:	10-7-04
*
*	So that every command does not have to parse and print all of the 
*	possiable errors, make a common call for generic errors.
*
*****************************************************************************/
u32 mcgetopt_error(u32 index, char *msg, char *getoptErrorMsg)
{

switch(index)
	{
	case E_MCGETOPT_INVALID_OP:		
		sprintf(msg, 
				"mcgetopt:ERROR: option \"%s\" was not found.\n", 
				getoptErrorMsg);
		break;

	case E_MCGETOPT_NO_ARG_CNT_SPECIFIED:
		sprintf(msg, 
				"mcgetopt:ERROR: argument count after \':\' is mising.\n");
		break;
			
	case E_MCGETOPT_TO_FEW_ARGS:	
		sprintf(msg, 
				"mcgetopt:ERROR:too few option arguments entered for\"%s\"\n", 
				getoptErrorMsg);
		break;

	case E_MCGETOPT_MISSING_OP_LIST:
		sprintf(msg, 
				"mcgetopt:ERROR: option list was not passed in.\n");
		break;

	case E_MCGETOPT_MISSING_MINUS:
		sprintf(msg, 
				"mcgetopt:ERROR: missing \'-\' character for option \'%s\'\n",
				getoptErrorMsg);
		break;

	default:		
		sprintf(msg, 
				"mcgetopt:ERROR: unknown option error rc=0x%x.\n", index);
		break;
	}

return(index);
}//end FUNCTION: mcgetopt_error()

#endif /*CONFIG_CMC_10G*/


