/****************************************************************************

  Copyright (c) 2006, Dell Inc
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

  - Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.

  - Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.

  - Neither the name of Dell Inc nor the names of its contributors
    may be used to endorse or promote products derived from this software
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE.

*****************************************************************************/

/*
 * Utility support
 */
#include <common.h>		/*must be first in file*/
#include <command.h>

#include "xiic_l.h"

#ifdef CONFIG_CMC_10G	/*only compile in for a CMC_10G build*/


/******************************************************************************
*
*	FILE: util_i2c.c
*
******************************************************************************/


/******************************************************************************
*  Include files
******************************************************************************/
#include <util_defs.h>
#include <util_ret_codes.h>


/******************************************************************************
*  Local Defines
******************************************************************************/
#define ADT7460_INT_MASK1			0x74
#define ADT7460_INT_MASK2			0x75
#define ADT7460_REG_CFG1			0x40
#define ADT7460_REG_CFG1_START		0x01

#define ADT7460_REG_REMOTE1_TEMP	0x25		/*PPC temp*/
#define ADT7460_REG_LOCAL_TEMP		0x26		/*ambient temp*/


/******************************************************************************
*  Globals
******************************************************************************/


/******************************************************************************
*  Prototypes/Externs
******************************************************************************/
extern s32	mcoptind;
extern char	*mcoptarg[];

extern int i2c_probe (uchar chip);

extern int i2c_read(uchar chip, uint addr, int alen, uchar *buffer, int len);
extern int i2c_write(uchar chip, uint addr, int alen, uchar *buffer, int len);

/******************************************************************************
*	List all structures defines
******************************************************************************/
t_options util_i2c_options[] =
	{
	/*Options*/
	{"help",	"Display available actions."},
	{"probe",	"Find available I2C devices."},
	{"read",	"Read I2C device."},
	{"write",	"Write I2C device."},
	{"readat",	"Read I2C from a given offset."},
	{"writeat",	"Write I2C device at a given offset."},
	{ NULL, NULL }
	};


t_usage util_i2c_usage = {
	"i2c",

	/*Syntax*/
	"i2c  [-help]\n"
	"     [-probe <bus>]\n"
	"     [-read <bus> <address> <# bytes>]\n"
	"     [-write <bus> <address> <data1,data2,...,datan>]\n"
	"     [-readat <bus> <address> <offset1,offset2,...,offsetn> <# bytes>]\n"
	"     [-writeat <bus> <address> <offset1,offset2,...,offsetn> <data1,data2,...,datan>]\n",

	/*Description*/
	"This command allows user control of I2C busses\n\n"
	"Examples:\n"
	"util i2c -probe lkvm\n"
	"util i2c -read psu123 0x50 16\n"
	"util i2c -write iomset2 0x50 42\n"
	"util i2c -readat iomset1 0x50 0,0 42\n"
	"util i2c -writeat iomset2 0x50 0,0 0x1,0x2,0x3\n",

	util_i2c_options
	}; /*end t_usage util_i2c_usage*/

/******************************************************************************
*
*       FUNCTION: getArray(char *string, u8 *array, int max)
*       DATE:
*
******************************************************************************/

int getArray(char *string, u8 *array, int max)
{
	char *cptr1;
        char *results = NULL;
        const char delims[]=",";
        int len = 0;

        if(string == NULL) return 0;

        results = strtok(string,delims);
        while( (results != NULL) && (len<max) ) {
		*array++ = (u8)simple_strtol( results, &cptr1, 0 );
        	if( (s32)cptr1 < (strlen(results) + (s32)results) )
        	{
                	printf("ERROR: \"%s\" cannot be converted to int.\n", results);
                	return(0);
        	}
		len++;
                results = strtok(NULL, delims);
        }
        return len;
}

/******************************************************************************
*
*       FUNCTION: fpga_i2c_init(char *dev)
*       DATE:
*
******************************************************************************/

s32 fpga_i2c_init(char *dev)
{
	u32	fpga_addr;

	if(!strcmp("iomset1",dev))
                fpga_addr = CMC_I2C_IOMSET1;
        else if(!strcmp("iomset2",dev))
                fpga_addr = CMC_I2C_IOMSET2;
        else if(!strcmp("psu123",dev))
                fpga_addr = CMC_I2C_PSU123;
        else if(!strcmp("psu456",dev))
                fpga_addr = CMC_I2C_PSU456;
        else if(!strcmp("lkvm",dev))
                fpga_addr = CMC_I2C_LKVM;
        else {
                printf("Unknown bus\n");
                return 0;
        }

	xiic_init(fpga_addr);
	printf("%s i2c core initialized.\n",dev);
	return 1;
}

/******************************************************************************
*
*       FUNCTION: fpga_i2c_read(char *dev, char *address, char *len)
*       DATE:
*
******************************************************************************/

s32 fpga_i2c_read(char *dev, char *address, char *len)
{
	u16  i2c_addr;
	u16  read_len;
	u32  fpga_addr;
	u8 data[16];
	int i;
	char *cptr1;

	if(!strcmp("iomset1",dev))
		fpga_addr = CMC_I2C_IOMSET1;
	else if(!strcmp("iomset2",dev))
		fpga_addr = CMC_I2C_IOMSET2;
	else if(!strcmp("psu123",dev))
		fpga_addr = CMC_I2C_PSU123;
	else if(!strcmp("psu456",dev))
		fpga_addr = CMC_I2C_PSU456;
	else if(!strcmp("lkvm",dev))
		fpga_addr = CMC_I2C_LKVM;
	else {
		printf("Unknown bus\n");
		return 0;
	}

	i2c_addr = (u32)simple_strtol( address, &cptr1, 0 );
	if( (s32)cptr1 < (strlen(address) + (s32)address) )
        {
        	printf("ERROR: \"%s\" cannot be converted to int.\n", address);
        	return(E_REG_INT_CONVERSION);
        }

	read_len = (u32)simple_strtol( len, &cptr1, 0 );
	if( (s32)cptr1 < (strlen(len) + (s32)len) )
        {
        	printf("ERROR: \"%s\" cannot be converted to int.\n", len);
        	return(E_REG_INT_CONVERSION);
        }

	if(read_len>16) {
		printf("Reads of longer than 16 bytes are not supported.\n");
		return 0;
	}

        if(xiic_iic_recv(fpga_addr, i2c_addr, (u8 *)data, read_len,0)) {
             for(i=0;i<read_len;i++) printf("%.2X ", data[i]);
                  printf("\n");
             } else {
                  printf("I2C Bus error.\n");
             }
	return 0;
}

/******************************************************************************
*
*       FUNCTION: fpga_i2c_readat(char *dev, char *address, char *offset, char *len)
*       DATE:
*
******************************************************************************/

s32 fpga_i2c_readat(char *dev, char *address, char *o_string, char *len)
{
	u16  i2c_addr;
	u16  read_len;
	u32  fpga_addr;
	u8 data[16];
	u8 offset[16];
	int i, o_len;
	char *cptr1;

    if(!(o_len=getArray(o_string,offset,16))) return 0;

	if(!strcmp("iomset1",dev))
		fpga_addr = CMC_I2C_IOMSET1;
	else if(!strcmp("iomset2",dev))
		fpga_addr = CMC_I2C_IOMSET2;
	else if(!strcmp("psu123",dev))
		fpga_addr = CMC_I2C_PSU123;
	else if(!strcmp("psu456",dev))
		fpga_addr = CMC_I2C_PSU456;
	else if(!strcmp("lkvm",dev))
		fpga_addr = CMC_I2C_LKVM;
	else {
		printf("Unknown bus\n");
		return 0;
	}

	i2c_addr = (u32)simple_strtol( address, &cptr1, 0 );
	if( (s32)cptr1 < (strlen(address) + (s32)address) )
        {
        	printf("ERROR: \"%s\" cannot be converted to int.\n", address);
        	return(E_REG_INT_CONVERSION);
        }

	read_len = (u32)simple_strtol( len, &cptr1, 0 );
	if( (s32)cptr1 < (strlen(len) + (s32)len) )
        {
        	printf("ERROR: \"%s\" cannot be converted to int.\n", len);
        	return(E_REG_INT_CONVERSION);
        }

	if(read_len>16) {
		printf("Reads of longer than 16 bytes are not supported.\n");
		return 0;
	}

    if(!xiic_iic_send(fpga_addr, i2c_addr, (u8 *)offset, o_len,1))
        printf("I2C Bus error.\n");

    if(xiic_iic_recv(fpga_addr, i2c_addr, (u8 *)data, read_len,0)) {
        for(i=0;i<read_len;i++) printf("%.2X ", data[i]);
              printf("\n");
        } else {
              printf("I2C Bus error.\n");
        }
	return 0;
}


/******************************************************************************
*
*       FUNCTION: fpga_i2c_write(char *dev, char *address, char *val)
*       DATE:
*
******************************************************************************/

s32 fpga_i2c_write(char *dev, char *address, char *string)
{
        u16  i2c_addr;
        u32  fpga_addr;
        char *cptr1;
	u8 data[16];
	int len;

	if(!(len=getArray(string,data,16))) return 0;

        if(!strcmp("iomset1",dev))
                fpga_addr = CMC_I2C_IOMSET1;
        else if(!strcmp("iomset2",dev))
                fpga_addr = CMC_I2C_IOMSET2;
        else if(!strcmp("psu123",dev))
                fpga_addr = CMC_I2C_PSU123;
        else if(!strcmp("psu456",dev))
                fpga_addr = CMC_I2C_PSU456;
        else if(!strcmp("lkvm",dev))
                fpga_addr = CMC_I2C_LKVM;
        else {
                printf("Unknown bus\n");
                return 0;
        }

        i2c_addr = (u32)simple_strtol( address, &cptr1, 0 );
        if( (s32)cptr1 < (strlen(address) + (s32)address) )
        {
                printf("ERROR: \"%s\" cannot be converted to int.\n", address);
                return(E_REG_INT_CONVERSION);
        }

        if(xiic_iic_send(fpga_addr, i2c_addr, (u8 *)data, len,0)) {
		printf("Write complete.\n");
		return 1;
        } else
                  printf("I2C Bus error.\n");
        return 0;
}

/******************************************************************************
*
*       FUNCTION: fpga_i2c_writeat(char *dev, char *address, char *offset, char *val)
*       DATE:
*
******************************************************************************/

s32 fpga_i2c_writeat(char *dev, char *address, char *o_string, char *v_string)
{
        u16  i2c_addr;
        u32  fpga_addr;
        char *cptr1;
        u8 data[16];
        u8 offset[16];
        int o_len,d_len;

        if(!(o_len=getArray(o_string,offset,16))) return 0;
        if(!(d_len=getArray(v_string,data,16))) return 0;

        if(!strcmp("iomset1",dev))
                fpga_addr = CMC_I2C_IOMSET1;
        else if(!strcmp("iomset2",dev))
                fpga_addr = CMC_I2C_IOMSET2;
        else if(!strcmp("psu123",dev))
                fpga_addr = CMC_I2C_PSU123;
        else if(!strcmp("psu456",dev))
                fpga_addr = CMC_I2C_PSU456;
        else if(!strcmp("lkvm",dev))
                fpga_addr = CMC_I2C_LKVM;
        else {
                printf("Unknown bus\n");
                return 0;
        }

        i2c_addr = (u32)simple_strtol( address, &cptr1, 0 );
        if( (s32)cptr1 < (strlen(address) + (s32)address) )
        {
                printf("ERROR: \"%s\" cannot be converted to int.\n", address);
                return(E_REG_INT_CONVERSION);
        }

        if(!xiic_iic_send(fpga_addr, i2c_addr, (u8 *)offset, o_len,1))
            printf("I2C Bus error.\n");

        if(xiic_iic_send(fpga_addr, i2c_addr, (u8 *)data, d_len,0)) {
            printf("Write complete!\n");
            return 1;
        } else
            printf("Bus Error.\n");

        return 0;
}

/******************************************************************************
*
*       FUNCTION: fpga_i2c_probe(char *dev)
*       DATE:
*
******************************************************************************/

s32 fpga_i2c_probe(char *dev)
{
        u32  fpga_addr;
	u16  i2c_addr;
	u8   dummy_data;

        if(!strcmp("iomset1",dev))
                fpga_addr = CMC_I2C_IOMSET1;
        else if(!strcmp("iomset2",dev))
                fpga_addr = CMC_I2C_IOMSET2;
        else if(!strcmp("psu123",dev))
                fpga_addr = CMC_I2C_PSU123;
        else if(!strcmp("psu456",dev))
                fpga_addr = CMC_I2C_PSU456;
        else if(!strcmp("lkvm",dev))
                fpga_addr = CMC_I2C_LKVM;
        else {
                printf("Unknown bus\n");
                return 0;
        }

	printf("Valid chip addresses: ");
	for(i2c_addr = 0; i2c_addr <128; i2c_addr++) {
		if(xiic_iic_recv(fpga_addr, i2c_addr, &dummy_data, 1,0)) {
			printf("%X ");
		}
	}
	printf("\n");
	return 1;
}

/******************************************************************************
*
*	FUNCTION: util_i2c()
*	DATE:
*
******************************************************************************/
s32 util_i2c(int argc, char *argv[])
{
u32  index = 0, rc = 0;
char msg[MAX_MSG_LEN];		/*message buffer*/


//	Get any options
mcoptind = 2;
while( (index = mcgetopt(argc, argv, "-help -init:1 -read:3 -write:3 -probe:1 -readat:4 -writeat:4 -options")) !=
MCGETOPT_EOF)
	{
	switch(index)
		{
		case 0 :		/*help*/
			printf("-probe -read -write\n");
			break;
		case 1 : /*init*/
			 fpga_i2c_init(mcoptarg[0]);
			 break;
		case 2 : /*read*/
			 fpga_i2c_read(mcoptarg[0],mcoptarg[1],mcoptarg[2]);
			 break;
		case 3 : /*write*/
			 fpga_i2c_write(mcoptarg[0],mcoptarg[1],mcoptarg[2]);
			 break;
		case 4 : /*probe*/
			 fpga_i2c_probe(mcoptarg[0]);
			 break;
		case 5 : /*readat*/
			 fpga_i2c_readat(mcoptarg[0],mcoptarg[1],mcoptarg[2],mcoptarg[3]);
			 break;
		case 6 : /*writeat*/
			 fpga_i2c_writeat(mcoptarg[0],mcoptarg[1],mcoptarg[2],mcoptarg[3]);
			 break;
		default:		/*not good*/
			rc = mcgetopt_error(index, msg, mcoptarg[0]);
			printf("%s %s:%s\n", argv[0], argv[1], msg);
			return(rc);
			break;
		}
	}/*end mcgetopt while*/


return(0);
}/* end FUNCTION: util_fpagi2c()*/


/******************************************************************************
*
*	FUNCTION: post_i2c_bus0_scan()
*	DATE:	1-16-07
*
******************************************************************************/
s32 post_i2c_bus0_scan(char *msg)
{
u32 i, idx;

/* This list MUST be smallest addresses to largest*/
static u8 i2c_dev_list[] =
		{CHIP_ADDR_ADT7460, CHIP_ADDR_TEMP, CHIP_ADDR_FRU, CHIP_ADDR_RTC, -1};

for(i = 0, idx = 0; i < 127; ++i)
	{
	if( !i2c_probe(i) )
		{/*i2c device responded*/
		if(i != i2c_dev_list[idx++])
			{
			/*	Special case, SMBUS alert "may" respond, but not always*/
			if(i != CHIP_SMBUS_ALERT)
				{
				sprintf(msg, "Unknown I2C device 0x%x responded", i);
				return(E_I2C_UNKNOWN_DEVICE_RESPONDED);
				}
			else
				--idx;	/*CHIP_SMBUS_ALERT don't count, bump back*/
			}
		}
	}/*end for*/

strcpy(msg, "I2C Bus0 scan.");
return(0);
}/*end FUNCTION:  post_i2c_bus0_scan()*/


/*****************************************************************************
*	RTC
*****************************************************************************/
s32 post_id_rtc(char *msg)
{
u32 rc;

if((rc = i2c_probe(CHIP_ADDR_RTC)) != 0)
	{
	sprintf(msg, "Could not ID RTC at I2C addr 0x%x", CHIP_ADDR_RTC);
	return(E_I2C_RTC_ID_FAILURE);
	}
else
	{
	//sprintf(msg, "ID RTC at i2c addr 0x%x", CHIP_ADDR_RTC);
	sprintf(msg, "ID at i2c addr 0x%x RTC", CHIP_ADDR_RTC);
	return(0);
	}
}/*end FUNCTION: post_id_rtc()*/


/*****************************************************************************
*	TEMP sensor
*****************************************************************************/
s32 post_id_temp(char *msg)
{
u32 rc;

if((rc = i2c_probe(CHIP_ADDR_TEMP)) != 0)
	{
	sprintf(msg, "Could not ID TEMP sensor at I2C addr 0x%x", CHIP_ADDR_TEMP);
	return(E_I2C_TEMP_ID_FAILURE);
	}
else
	{
	//sprintf(msg, "ID TEMP sensor at i2c addr 0x%x", CHIP_ADDR_TEMP);
	sprintf(msg, "ID at i2c addr 0x%x TEMP", CHIP_ADDR_TEMP);
	return(0);
	}
}/*end FUNCTION: post_id_temp()*/

/*****************************************************************************
*	FRU
*****************************************************************************/
s32 post_id_fru(char *msg)
{
u32 rc;

if((rc = i2c_probe(CHIP_ADDR_FRU)) != 0)
	{
	sprintf(msg, "Could not ID FRU at I2C addr 0x%x", CHIP_ADDR_FRU);
	return(E_I2C_FRU_ID_FAILURE);
	}
else
	{
	//sprintf(msg, "ID FRU at i2c addr 0x%x", CHIP_ADDR_FRU);
	sprintf(msg, "ID at i2c addr 0x%x FRU", CHIP_ADDR_FRU);
	return(0);
	}
}/*end FUNCTION: post_id_fru()*/



/*****************************************************************************
*	PPC and ambient TEMP sensor
*****************************************************************************/
s32 post_id_adt7460(char *msg)
{
u32 rc;
u8	local;
u8  remote;

if((rc = post_adt7460_get_reg(ADT7460_REG_LOCAL_TEMP, &local)) != 0)
	{
	sprintf(msg, "Could not read ADT7460 local TEMP");
	return(rc);
	}

if((rc = post_adt7460_get_reg(ADT7460_REG_REMOTE1_TEMP, &remote)) != 0)
	{
	sprintf(msg, "Could not read ADT7460 remote PPC TEMP");
	return(rc);
	}

/*	ADT7460 TEMPs are in 1 degree C inctements when read 8 bit mode*/
sprintf(msg, "ID at i2c addr 0x%x ADT7460 TEMP, ambient %i C, uP %i C",
											CHIP_ADDR_ADT7460, local, remote);
return(0);

}/*end FUNCTION: post_id_temp()*/



/*****************************************************************************
*	FUNCTIONS:	post_adt7460_init()
*				post_adt7460_get_reg()
*
*	This code could be placed in the "cmc_u-boot1.1.5/ddt" dir to be all
*	things to all people.  But it is easier to maintain here.
*
*	If we just turn on the device we get a SMBalert, then address 0x0c shows
*	up with iprobe.  So mask all interrupts first before turning on the device.
*
*****************************************************************************/
s32 post_adt7460_init(char *msg)
{
s32 rc;
u8	mask	= 0xff;
u8  cfg_reg = ADT7460_REG_CFG1_START;


if((rc = i2c_probe(CHIP_ADDR_TEMP)) != 0)
	{
	sprintf(msg, "Could not ID ADT7460 TEMP sensor at I2C addr 0x%x",
															CHIP_ADDR_ADT7460);
	return(E_I2C_ADT7460_TEMP_ID_FAILURE);
	}

strcpy(msg, "Could not init the ADT7460");		/*common error message*/
/*	Mask both interrupt mask registers, remove the SMBus alerting adr of 0xc*/
if(i2c_write(CHIP_ADDR_ADT7460, ADT7460_INT_MASK1, 1, &mask, 1) != 0)
	return(E_I2C_ADT7460_WRITE_FAILURE);

if(i2c_write(CHIP_ADDR_ADT7460, ADT7460_INT_MASK2, 1, &mask, 1) != 0)
	return(E_I2C_ADT7460_WRITE_FAILURE);

/*	Enable the device and delay so a complete poll cycle occurs*/
if(i2c_write(CHIP_ADDR_ADT7460, ADT7460_REG_CFG1, 1, &cfg_reg, 1) != 0)
	return(E_I2C_ADT7460_WRITE_FAILURE);

udelay(50000);
strcpy(msg, "ADT7460 temp sensor init");
return(0);
}/*FUNCTION: post_adt7460_init() */



s32 post_adt7460_get_reg(u8 reg, u8 *data)
{
if (i2c_read(CHIP_ADDR_ADT7460, reg, 1, data, 1) != 0)
	return(E_I2C_ADT7460_READ_FAILURE);

return(0);
}/*FUNCTION: post_adt7460_get_reg()*/


#endif /*CONFIG_CMC_10G*/



