/****************************************************************************

  Copyright (c) 2006, Dell Inc
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

  - Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
    
  - Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution. 

  - Neither the name of Dell Inc nor the names of its contributors
    may be used to endorse or promote products derived from this software 
    without specific prior written permission. 

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. 

*****************************************************************************/


/*
 * Utility support
 */
#include <common.h>		/*must be first in file*/
#include <command.h>

#include <usb.h>

#ifdef CONFIG_CMC_10G	/*only compile in for a CMC_10G build*/

/******************************************************************************
*
*	FILE: util_post.c
*	
*	This file contains various interfaces to assist PCBA test and lab 
*	activities. 
*
*	U-boot has a log mechanism found in cmd_post.c  The u-boot POST test also
*	use this log.   We have simular methods from previous RAC products.  
*	It will be easier and faster to leave CONFIG_POST and CONFIG_LOGBUFFER 
*	un-defined and port what we already have.  This way our Linux utilities
*	will also work without change.
*	
*	Our previous log used an area at the top of RAM, the monitor was located in
*	low ran under 1M.   The u-boot CONFIG_PRAM will work perfectly for this 
*	design as the "mem" environment variable reflects this as a Linux boot arg.
*	The "pram" area is a lockout area unknown by Linux.  Existing OS utility's
*	from previous RAC code has access to structures located in the param area.
*
*	Messages that go into the pre-boot log will be moved into one or more of 
*	the following Linux logs:
*	1. trace
*	2. cmc (old RAC log)
*	3. BOARD/tmp/<prop>
*	4. fwupd status
*
*
******************************************************************************/


/******************************************************************************
*  Include files
******************************************************************************/
#include <util_defs.h>
#include <util_ret_codes.h>
#include <util_flashmap.h>


/******************************************************************************
*  Local Defines
******************************************************************************/
#define LED_TEST_DELAY	500000	/*in usec*/

/******************************************************************************
*  Globals
******************************************************************************/


/******************************************************************************
*  Prototypes/Externs
******************************************************************************/
extern s32	mcoptind;
extern char	*mcoptarg[];

extern u32	 g_post_rc;
extern char  g_post_msg[MAX_POST_MSG_SIZE];
extern u32	 g_warn_rc;			
extern char  g_warn_msg[MAX_POST_MSG_SIZE];

extern char * usb_get_class_desc(unsigned char dclass);


/******************************************************************************
*	List all structures defines
******************************************************************************/
t_options util_post_options[] = 
	{
	/*Options*/
	{"trace",	"Add msg to OS trace log."},
	{"cmc",		"Add msg to CMC log (old RAC log)."},
	{"prop",	"Add msg to BOARD/PROP/<propname>."},
	{"fwupd",	"Add a fwupdate status msg"},
	{"msg",		"Return code and message string to be added."},
	{"post_rc",	"Display the POST return code and message."},
	{"warn_rc",	"Display the WARN return code and message."},
	{"tod",		"Get the RTC Time-of-day."},
	{"usb",		"ID the USB controller to the SD card."},
	{"dump",	"Dump pre-boot log."},
	{"time",	"Dev test for get_utiltime() and ut struct."},
	{"led",		"Toggle LEDs for visual test."},
	
	{ NULL, NULL }
	};


t_usage util_post_usage = { 
	"post", 
					
	/*Syntax*/
	"post  [<-trace &| -cmc &| -prop &| -fwupd>  -msg <0xRC> \"<log msg>\"]\n"
	"post  [-post_rc | -warn_rc]\n"
	"post  [-tod]\n"
	"post  [-time]\n"
	"post  [-usb]\n"
	"post  [-led <debug | green | blue | amber | normal>]\n"
	"      [-dump]\n",

	/*Description*/
	"For testing the user can read and write entries in the pre-boot log\n"
	"as well as direct where the msg will go under the OS. Bit31=FAIL\n"
	"bit30=INFO bit 29=WARN.  Options also allow display of rc and text\n"
	"messages for POST results\n\n"
	"Examples:\n"
	"util post -dump\n"
	"util post -post_rc\n"
	"util post -trace -cmc -msg 0x0 \"<double quoted message>\"\n"
	"util post -trace -msg  0x80001234 \"test fail\"\n", 
	
	util_post_options
	}; /*end t_usage util_post_usage*/


/******************************************************************************
*
*	FUNCTION: util_post()
*	DATE:	12-6-06
*
******************************************************************************/
s32 util_post(int argc, char *argv[])
{
DECLARE_GLOBAL_DATA_PTR;	/*necessary to get info from gd_t*/
s32  index		= 0;
u32  rc			= 0;
u32	 i			= 0;
u32  rc_num		= 0;
u32	 flags		= 0;
u32  msg_flag	= 0;
u32  dump_flag	= 0;
u32  usb_flag	= 0;
u32  len		= 0;
u32  tod		= 0;
u32  gpio_reg;
char *cp;
char msg[MAX_MSG_LEN];								/*message buffer*/
char logmsg[MAX_MSG_LEN];
char tstamp_str[MAX_TSTAMP_STR_LEN];
bd_t *bd = gd->bd;
t_pram_preboot *pram = (t_pram_preboot*) (bd->bi_memsize-(CONFIG_PRAM*1024));
t_preboot_log  *log  = (t_preboot_log*)&pram->log;
struct usb_device *dev;


//	Get any options
mcoptind = 2;	
while( (index = mcgetopt(argc, argv, 
	  "-dump -trace -cmc -prop -fwupd -msg:2 -time -usb -post_rc -warn_rc -tod -led:1")) 
	  != MCGETOPT_EOF)
	{
	switch(index) 
		{
		case 0:			/*dump pre-boot log*/
			dump_flag = 1;
			break;

		case 1:			/*trace log*/
			flags |= LOG2TRACE;
			break;

		case 2:			/*cmc log*/
			flags |= LOG2RAC;
			break;

		case 3:			/*prop log*/
			flags |= LOG2PROP;
			break;

		case 4:			/*fwupdate status*/
			flags |= LOG2FWUPD_STATUS;
			break;

		case 5:			/*log message*/
			rc_num = (int)simple_strtol(mcoptarg[0], &cp, 10);
			if( (s32)cp < (strlen(mcoptarg[0]) + (s32)mcoptarg[0]) ) 
				{
				printf("%s %s:ERROR: invalid RC %s\n", 
												argv[0], argv[1], mcoptarg[0]);
				return(E_LOG_BAD_RC_ARG);					/*not found*/
				}

			if( (len = strlen(mcoptarg[1])) >= MAX_MSG_LEN)
				{
				printf("%s %s:ERROR:-msg string too long.\n", argv[0], argv[1]);
				return(E_LOGMSG_TOO_LONG);
				}

			strcpy(logmsg, mcoptarg[1]);
			++msg_flag;
			break;


		case 6:			/*test calc_tstamp_str()*/
			calc_tstamp_str(0, tstamp_str);
			printf("Time stamp string: %s\n", tstamp_str);
			printf("Time stamp from base 0: %d\n", ABS_get_timer(0) );
			return(0);
			break;

		case 7:			/*USB dev*/
			usb_flag = 1;
			break;

		case 8:			/*Display the POST RC and text message*/
			printf("0x%08lx=%s\n", g_post_rc, g_post_msg);
			return(0);
			break;

		case 9:			/*Display the WARN RC and text message*/
			printf("0x%08lx=%s\n", g_warn_rc, g_warn_msg);
			return(0);
			break;

		case 10:		/*Get time-of-day from RTC*/
			if((rc = get_rtc_tod(&tod)) != 0)
				{
				printf("%s %s:%s\n", argv[0], argv[1], "ERROR:reading RTC device\n");
				return(E_POST_RTC_READ_FAILURE);
				}

			printf("RTC-TOD=0x%08lx\n", tod);
			return(0);
			break;

		case 11:		/*toggle each LED for a visual test*/
			/*	Turn all LEDs off*/
			clearlow_gpio_or(GPIO_BANK0,  (GPIO_LED_BLUE | GPIO_LED_AMBER | 
										   GPIO_LED_POWER | GPIO_DEBUG_LED) );
			
			if(!strcmp(mcoptarg[0], "debug"))
				gpio_reg = GPIO_DEBUG_LED;
			else if(!strcmp(mcoptarg[0], "green"))
				gpio_reg = GPIO_LED_POWER;
			else if(!strcmp(mcoptarg[0], "blue"))
				gpio_reg = GPIO_LED_BLUE;
			else if(!strcmp(mcoptarg[0], "amber"))
				gpio_reg = GPIO_LED_AMBER;
			else	/*"normal"*/
				{
				sethigh_gpio_or(GPIO_BANK0, (GPIO_LED_POWER | GPIO_DEBUG_LED));
				return(0);
				}

			printf("Toggle \"%s\" LED\n", mcoptarg[0]);
			while(1)
				{
				sethigh_gpio_or(GPIO_BANK0, gpio_reg);
				udelay(LED_TEST_DELAY);
				clearlow_gpio_or(GPIO_BANK0, gpio_reg);
				udelay(LED_TEST_DELAY);

				if ( ctrlc() )		/* check for ctrl-c to abort... */
					return(EXIT_BY_CTRLC);
				}
			break;

		default:		/*not good*/
			rc = mcgetopt_error(index, msg, mcoptarg[0]);
			printf("%s %s:%s\n", argv[0], argv[1], msg);
			return(rc);
			break;
		}
	}/*end mcgetopt while*/


/*****************************************************************************
*	User frindly checks
*****************************************************************************/
if( (msg_flag && !flags) || (flags && !msg_flag) )
	{
	printf("%s %s:ERROR: must enter a log type with msg option\n", 
														argv[0], argv[1]);
	return(E_LOG_MISSING_ARGS);
	}


/*****************************************************************************
*	log a message
*****************************************************************************/
if(msg_flag)		
	{	
	if( (rc = log_preboot(flags, rc_num, logmsg)) )
		{
		printf("%s %s:ERROR:RC:0x%x log is full \n", argv[0], argv[1], rc);
		return(rc);
		}

	log_preboot_crc_gen();		/*	Re-calc the crc*/
	}


/*****************************************************************************
*	Dump the log
*****************************************************************************/
if(dump_flag)		
	{	
	printf("\n\nPre-Boot message log: crc32=0x%x total_bytes=0x%x", 
												log->crc, log->total_bytes);
	for(i = 0, msg[0] = '\0'; log->msg_p[i]; ++i)
		{
		strcpy(msg, "Log to: ");

		if(log->flags[i] & LOG2TRACE)
			strcat(msg, "Trace, ");
		if(log->flags[i] & LOG2RAC)
			strcat(msg, "RAC, ");
		if(log->flags[i] & LOG2PROP)
			strcat(msg, "BOARD/PROP/ ");
		if(log->flags[i] & LOG2FWUPD_STATUS)
			strcat(msg, "FWupdate status");
		if(log->flags[i] & LOG2OSEVENT)
			strcat(msg, "OS Event");

		printf("\n%s\n%s", msg, log->msg_p[i]);
		}
	}


/*****************************************************************************
*	Test USB controller for SD card
*****************************************************************************/
if(usb_flag)		
	{	
	post_id_usb(msg);
	printf("%s\n", msg);

	for(i = 0; i < USB_MAX_DEVICE; ++i)
		{
		dev = usb_get_dev_index(i);
		if (dev == NULL)
			break;
		printf("i=%d\n", i);

		if (dev->descriptor.bDescriptorType==USB_DT_DEVICE) 
			{
			printf("%d: %s,  USB Revision %x.%x\n",
				dev->devnum,
				usb_get_class_desc(dev->config.if_desc[0].bInterfaceClass),
				(dev->descriptor.bcdUSB>>8) & 0xff,
				dev->descriptor.bcdUSB & 0xff);

			if (strlen(dev->mf))
				printf("mf=%s\n", dev->mf);
            
			if(strlen(dev->prod))
				printf("prod=%s\n", dev->prod);

			if(strlen(dev->serial))
				printf("serial=%s\n", dev->serial);

	
			printf("Vendor: 0x%04x  Product 0x%04x Version %d.%d\n",
				dev->descriptor.idVendor,
				dev->descriptor.idProduct,
				(dev->descriptor.bcdDevice>>8) & 0xff,
				dev->descriptor.bcdDevice & 0xff);
			}
		else
			{
			printf("type=0x%x\n", dev->descriptor.bDescriptorType);
			}
		}/*end for*/
	}

/*****************************************************************************
*	
*****************************************************************************/


return(0);
}/*end FUNCTION: util_post() */

#endif /*CONFIG_CMC_10G*/



