/****************************************************************************

  Copyright (c) 2006, Dell Inc
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

  - Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
    
  - Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution. 

  - Neither the name of Dell Inc nor the names of its contributors
    may be used to endorse or promote products derived from this software 
    without specific prior written permission. 

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. 

*****************************************************************************/


/*
 * Recover support
 */
#include <common.h>		/*must be first in file*/
#include <command.h>
#include <net.h>

#ifdef CONFIG_CMC_10G	/*only compile in for a CMC_10G build*/


/******************************************************************************
*
*	FILE: util_recover.c
*
******************************************************************************/


/******************************************************************************
*  Include files
******************************************************************************/
#include <util_defs.h>
#include <util_ret_codes.h>
#include <util_fpga.h>
#include <ppc4xx_enet.h>


/******************************************************************************
*  Local Defines
******************************************************************************/
#define MAX_IP_STR_LEN	40

#define MFLAG_FLASH		1	/*Copy the memory image into the NAND*/
#define MFLAG_NOFLASH	2	/*Do NOT copy the memory image into the NAND*/

/******************************************************************************
*  Globals
******************************************************************************/


/******************************************************************************
*  Prototypes/Externs
******************************************************************************/
s32 calc_ecc_flag(u32 mem);


extern s32	mcoptind;
extern char	*mcoptarg[];

extern IPaddr_t string_to_ip(char *s);
extern IPaddr_t	NetPingIP;
extern char	BootFile[128];				/*TFTP Boot File name*/
extern u32 g_ecc_hack;



/******************************************************************************
*	List all structures defines
******************************************************************************/
t_options recover_fwupdate_options[] = 
	{
	//Options
	{"g",		"Get fwupdate image file from TFTP server"},
	{"a",		"IP address of the TFTP server to load the image file"},
    {"p",		"Push the fwupdate image file through the serial port"},
	
	{ NULL, NULL }
	};


t_usage recover_fwupdate_usage = { 
	"fwupdate", 
					
	//Syntax
	"fwupdate [-g -a <TFTP server IP>]\n"
	"         [-p <kermit | ymodem>]\n",

	//Description
	"The fwupdate command allows the user to load, then program the\n"
	"CMC firmware flash.  The -g option gets the update image from a\n"
	"tftp server, while the -a option specifies the IP address of the\n"
	"TFTP server. The TFTP server root path MUST be setup to point to\n"
	"the update image.\n\n"  
	"Note: The -p option for serial push will be a lengthy process, and\n"
	"is not recommended. You must specify the binary download of \"kermit\"\n"
	"or \"ymodem\", then send the file to the CMC serial port.\n\n"
	"Examples:\n"
	"  recover fwupdate -g -a 192.168.0.100\n"
	"  recover fwupdate -p kermit\n",
	
	recover_fwupdate_options
	};//end t_usage recover_fwupdate_usage              

/*****************************************************************************/
t_options recover_setniccfg_options[] = 
	{
	//Options
	{ "None",	"" },
	{ NULL,		NULL }
	};


t_usage recover_setniccfg_usage = { 
	"setniccfg", 
					
	//Syntax
	"setniccfg <IPaddress> <Subnetmask> <Gateway>\n",

	//Description
	"This command allows the user to setup a static network configuration\n"
	"so that the CMC update image can be downloaded from a TFTP server.\n\n"
	"Example:\n"
	"  recover setniccfg 192.168.0.200  255.255.255.0  192.168.0.1\n\n",
	
	recover_setniccfg_options
	};//end t_usage recover_setniccfg_usage              

/*****************************************************************************/
t_options recover_getniccfg_options[] = 
	{
	//Options
	{ "None",	"" },
	{ NULL,		NULL }
	};


t_usage recover_getniccfg_usage = { 
	"getniccfg", 
					
	//Syntax
	"getniccfg\n",

	//Description
	"This command displays the current static network setings of the\n"
	"CMC Flash Recover monitor.\n\n"
	"Example:\n"
	"  recover getniccfg",
	
	recover_getniccfg_options
	};//end t_usage recover_getniccfg_usage         


/****************************************************************************/
t_options recover_ping_options[] = 
	{
	//Options	
	{ "None",	"" },
	{ NULL,		NULL }
	};


t_usage recover_ping_usage = { 
	"ping", 
					
	//Syntax
	"ping <IP address>\n",

	//Description
    "This command is typically used to verify end-to-end network\n"
	"functionality and connectivity.\n"
	"NOTE: You cannot ping \"into\" the CMC at the recover prompt.\n\n"
	"Example:\n"
	"  recover ping 192.168.0.100\n\n", 
	
	recover_ping_options
	};//end t_usage recover_ping_usage              


/*****************************************************************************/
t_options recover_reset_options[] = 
	{
	//Options
	{ "None",	"" },
	{ NULL,		NULL }
	};


t_usage recover_reset_usage = { 
	"reset", 
					
	//Syntax
	"reset\n",

	//Description
	"This command allows the user to reset the CMC.  This may be required\n"
	"after setting up the static network configurations prior to fwupdate.\n\n"
	"Example:\n"
	"  recover reset",
	
	recover_getniccfg_options
	};//end t_usage recover_reset_usage         






/*****************************************************************************
*	FUNCTION:	recover_fwupdate()
*	DATE:	2-28-07
*****************************************************************************/
s32 recover_fwupdate(int argc, char *argv[])
{
u32	 index;
u32  rc;
u32	 gflag = 0, aflag = 0,  pflag = 0, cflag = 0, fflag = 0, mflag = 0;
u32  peer_flag = 0;
u32  tftptest_flag  = 0;
u32	 mem_start;
u32  loop_cnt		= 1;
u32  current_loop	= 1;
u32  total_image_bcnt;
u32  tftp_retry;
u32  tftp_image_bad;
char *serial_cmd = "loadb";		/*default*/
char *s;
char *cptr1;
IPaddr_t  ipaddr = 0;
char msg[MAX_MSG_LEN];
//hack char ip_str[MAX_IP_STR_LEN];

strcpy(BootFile, FWUPD_IMAGE_FILENAME);		/*-f option can override*/

/*	Get any options*/
mcoptind = 2;	
while( (index = mcgetopt(argc, argv, 
	  "-g -a:1 -p:1 -c -hdr:1 -f:1 -m:1 -l:1 -tftptest -peer")) 
															!= MCGETOPT_EOF)
	{
	switch(index) {
		case 0 :		/* -g option specifies a tftp "get"*/
			gflag = 1;
			break;
			
		case 1 :		/* IP address of the TFTP server*/
			aflag = 1;

			if( (rc = ip_verify(mcoptarg[0])) != 0)
				{
				printf("%s %s:ERROR:Invalid IP string \"%s\".\n", 
												argv[0], argv[1], mcoptarg[0]);
				return(E_RECOVER_INVALID_IP);
				}

			if( (ipaddr = string_to_ip(mcoptarg[0])) == 0)
				{
				printf("%s %s:ERROR:Invalid IP string \"%s\".\n", 
												argv[0], argv[1], mcoptarg[0]);
				return(E_RECOVER_INVALID_IP);
				}

			setenv ("serverip", mcoptarg[0]);
			break;

		case 2:			/*	-p sets up for a serial push of the update file*/
			pflag = 1;

			if(!strcmp("kermit", mcoptarg[0]))
				serial_cmd = "loadb";
			else if(!strcmp("ymodem", mcoptarg[0]))
				serial_cmd = "loady";
			else
				{
				printf("%s %s:ERROR:\"kermit\" or \"ymodem\" required\n", 
															argv[0], argv[1]);
				return(E_RECOVER_INVALID_P_ARGUMENT);
				}
			break;

/*****	Options that follow are undocumented and are vendor mode only *****/

		case 3:			/*Undocumented, current uses an existing NAND file*/ 
			if(!is_vendor_mode() )
				{
				printf("%s %s:ERROR:Invalid option \"%s\"\n", 
											argv[0], argv[1], argv[mcoptind] );
				return(E_RECOVER_VENDOR_MODE_OPTION);
				}

			cflag = 1;
			break;

		case 4:			/*Undocumented, hdr dumps master and image headers*/ 
			if(!is_vendor_mode() )
				{
				printf("%s %s:ERROR:Invalid option \"%s\"\n", 
											argv[0], argv[1], argv[mcoptind] );
				return(E_RECOVER_VENDOR_MODE_OPTION);
				}

			/* if "-hdr flash" move backup to memory, if "-hdr mem" fall through*/
			if( !strcmp("flash", mcoptarg[0]) )
				{
				if( (rc = post_verify_fwupd_image(CFG_LOAD_ADDR, msg)) != 0)
					{
					printf("%s %s:ERROR:%s\n", argv[0], argv[1], msg);
					return(rc);
					}	
				}

			if( (rc = fwupd_dump_hdr_info(CFG_LOAD_ADDR, msg)) != 0)
				{
				printf("%s %s:ERROR:%s\n", argv[0], argv[1], msg);
				return(rc);
				}

			return(0);
			break;

		case 5:			/*Undocumented option -f file name*/ 
			if(!is_vendor_mode() )
				{
				printf("%s %s:ERROR:Invalid option \"%s\"\n", 
											argv[0], argv[1], argv[mcoptind] );
				return(E_RECOVER_VENDOR_MODE_OPTION);
				}

			fflag = 1;
			if(strlen(mcoptarg[0]) < 128)
				strcpy(BootFile, mcoptarg[0]);	
			break;

		case 6:			/*Undocumented option -m, use data already in memory*/ 
			if(!is_vendor_mode() )
				{
				printf("%s %s:ERROR:Invalid option \"%s\"\n", 
											argv[0], argv[1], argv[mcoptind] );
				return(E_RECOVER_VENDOR_MODE_OPTION);
				}

			if( !strcmp("flash", mcoptarg[0]) )			/*save into NAND*/
				mflag = MFLAG_FLASH;
			else if(!strcmp("noflash", mcoptarg[0]) )	/*do NOT save into NAND*/
				mflag = MFLAG_NOFLASH;
			else
				{
				printf("%s %s:ERROR:%s\n", argv[0], argv[1], 
									"-m needs \"flash\" or \"noflash\" arg");
				return(E_RECOVER_OPTION_CHOICES);	
				}
			break;

		case 7 :		/*Undocumented option -l, loop count*/
			loop_cnt = (u32)simple_strtol( mcoptarg[0], &cptr1, 0 );

			if( (s32)cptr1 < (strlen(mcoptarg[0]) + (s32)mcoptarg[0]) ) 
				{	
				printf("%s %s:ERROR: -l argument \"%s\" cannot be converted "
						"to int.\n", argv[0], argv[1], mcoptarg[0]);
				return(E_OPTARG_INT_CONVERSION);
				}
			break;

		case 8 :		/* -tftptest option*/
			tftptest_flag = 1;
			break;

		case 9 :		/* -peer option*/
			peer_flag = 1;
			break;

		default:		/*not good*/
			rc = mcgetopt_error(index, msg, mcoptarg[0]);
			printf("%s %s:%s\n", argv[0], argv[1], msg);
			return(rc);
			break;
	}
}/*end mcgetopt while*/

/*****************************************************************************
*	User friendly checks
*****************************************************************************/
if( (!gflag && !pflag && !cflag && !mflag && !tftptest_flag && !peer_flag) ||
    (gflag  &&  pflag)	 ||
	(gflag  &&  cflag) ||
	(pflag  &&  cflag) )
	{
	printf("%s %s:ERROR:Must specify a -g | -p option.\n", 
															argv[0], argv[1]);
	return(E_RECOVER_OPTION_CHOICES);
	}

if( (gflag  && !aflag) ||
    (!gflag && aflag && !tftptest_flag) )
	{
	printf("%s %s:ERROR:Must specify both -g AND -a option.\n", argv[0], argv[1]);
	return(E_RECOVER_GET_AND_IP_OPTION);
	}


/*****************************************************************************
*	Load the image via TFTP
*	Error return code are found in net_api.h, under NET_error_ids, 
*	ERROR_NET=0xa000
*****************************************************************************/
if(gflag)
	{
	printf("Loading file \"%s\" from TFTP server IP \"%s\"\n", 
											BootFile, getenv ("serverip") );

	load_addr = CFG_LOAD_ADDR;		/*DFxxxxx set global load point*/
	mem_start = CFG_LOAD_ADDR;		
	if (NetLoop(TFTP) < 0) 
		{
		printf("%s %s:ERROR:TFTP load.\n", argv[0], argv[1]);
		return(E_RECOVER_TFTP_LOAD_FAILED);
		}	
	else
		printf("%s %s:TFTP Load passed.\n", argv[0], argv[1]);


	/*	Verify integerty of the image file (provides toal_image_bcnt)*/
	if( (rc = fwupd_verify_image(mem_start, &total_image_bcnt, msg)) != 0)
		{
		printf("%s %s:ERROR:%s\n", argv[0], argv[1], msg);
		return(rc);
		}

	/*	
	*	Need to know if we are crossing the 1.x <-> >2.1 barrier
	*	BEFORE fwupd_ram2flash()
	*/
	g_ecc_hack = calc_ecc_flag(mem_start);	/*sets g_ecc_hack*/
	if(1 == g_ecc_hack)
		printf("\nUsing new ECC algorithm.");
	if(-1 == g_ecc_hack)
		printf("\nUsing old ECC algorithm.");

	/*	Copy the image into flash for future use*/
	if( (rc = fwupd_ram2flash(mem_start, total_image_bcnt, msg)) != 0)
		{
		printf("%s %s:ERROR:%s\n", argv[0], argv[1], msg);
		return(rc);
		}	

	/*	
	*	This env will protect us upon an abnormal termination (EST clowns 
	*	pulling AC at every whim). If we detect this env flag, fwupd() will
	*	be called again to repair any incomplete or corrupt image, and will 
	*	only be cleared by fwupd() when complete.
	*/
	set_fwupd_state_env(FWUPD_ENV_STATE_PENDING);

	/*	Call the update*/
	if( (rc = fwupd(mem_start, msg)) != 0)
		{
		printf("%s %s:ERROR:%s\n", argv[0], argv[1], msg);
		return(rc);
		}	
	/*
	*	OK...to get here means fwupd did NOT return an error, AND also NOR 
	*	images did NOT require an update, else the process would have reset.
	*	So,  NOR was not updated and NAND is now finished so reset.
	*	I would prefer a forced wdog/board reset but then we depend on the 
	*	FPGA.  So just do a core reset.
	*/
	printf("\n\n** Recover FWupdate complete, booting OS **\n\n");
	reset_after_fwupd();
	}

/*****************************************************************************
*	Load the image via serial
*****************************************************************************/
if(pflag)
	{
	if( (rc = run_command( serial_cmd, 0 )) < 0)
		{
		printf("%s %s:ERROR:\"%s\" failed to execute. RC=0x%x\n", 
											argv[0], argv[1], serial_cmd, rc);
		return(E_RECOVER_SERIAL_LOAD_ERROR);
		}	

	/*	
	*	Get the starting memory address of a downloaded binary.
	*	see do_load_serial_bin()
	*/
	if((s = (char*)getenv("loadaddr")) == 0)
		{
		mem_start = CFG_LOAD_ADDR;	/*serial download does not set loadaddr*/
		}
	else
		{
		mem_start = (u32)simple_strtol( s, &cptr1, 16 ); /*u-boot does not prepend 0x*/
		if( (s32)cptr1 < (strlen(s) + (s32)s) )
			{	
			printf("%s %s:ERROR:\"fileaddr\" cannot be converted to int\n", 
															argv[0], argv[1]);
			return(E_RECOVER_CANNOT_CONVERT_MEMPTR);
			}
		}

	/*	Verify integerty of the image file (provides toal_image_bcnt)*/
	if( (rc = fwupd_verify_image(mem_start, &total_image_bcnt, msg)) != 0)
		{
		printf("%s %s:ERROR:%s\n", argv[0], argv[1], msg);
		return(rc);
		}	

	/*	
	*	Need to know if we are crossing the 1.x <-> >2.0 barrier
	*	BEFORE fwupd_ram2flash()
	*/
	g_ecc_hack = calc_ecc_flag(mem_start);	/*sets g_ecc_hack*/
	if(1 == g_ecc_hack)
		printf("\nUsing new ECC algorithm.");
	if(-1 == g_ecc_hack)
		printf("\nUsing old ECC algorithm.");


	/*	Copy the image into flash for future use*/
	if( (rc = fwupd_ram2flash(mem_start, total_image_bcnt, msg)) != 0)
		{
		printf("%s %s:ERROR:%s\n", argv[0], argv[1], msg);
		return(rc);
		}	

	/*	
	*	This env will protect us upon an abnormal termination (EST clowns 
	*	pulling AC at every whim). If we detect this env flag, fwupd() will
	*	be called again to repair any incomplete or corrupt image, and will 
	*	only be cleared by fwupd() when complete.
	*/
	set_fwupd_state_env(FWUPD_ENV_STATE_PENDING);

	/*	Call the update*/
	if( (rc = fwupd(mem_start, msg)) != 0)
		{
		printf("%s %s:ERROR:%s\n", argv[0], argv[1], msg);
		return(rc);
		}	
	/*
	*	OK...to get here means fwupd did NOT return an error, AND also NOR 
	*	images did NOT require an update, else the process would have reset.
	*	So,  NOR was not updated and NAND is now finished so reset.
	*	I would prefer a forced wdog/board reset but then we depend on the 
	*	FPGA.  So just do a core reset.
	*/
	printf("\n\n** Recover FWupdate complete, booting OS **\n\n");
	reset_after_fwupd();
	}

/*****************************************************************************
*	FWupdate using the current nand back up copy. (vendor mode only)
*****************************************************************************/
if(cflag)
	{
	/*	Bring the NAND backup into memory*/
	if( (rc = post_verify_fwupd_image(CFG_LOAD_ADDR, msg)) != 0)
		{
		printf("%s %s:ERROR:%s\n", argv[0], argv[1], msg);
		return(rc);
		}	

	/*	
	*	This env will protect us upon an abnormal termination (EST clowns 
	*	pulling AC at every whim). If we detect this env flag, fwupd() will
	*	be called again to repair any incomplete or corrupt image, and will 
	*	only be cleared by fwupd() when complete.
	*/
	set_fwupd_state_env(FWUPD_ENV_STATE_PENDING);

	/*	Call the update*/
	if( (rc = fwupd(CFG_LOAD_ADDR, msg)) != 0)
		{
		printf("%s %s:ERROR:%s\n", argv[0], argv[1], msg);
		return(rc);
		}	
	/*
	*	OK...to get here means fwupd did NOT return an error, AND also NOR 
	*	images did NOT require an update, else the process would have reset.
	*	So,  NOR was not updated and NAND is now finished so reset.
	*	I would prefer a forced wdog/board reset but then we depend on the 
	*	FPGA.  So just do a core reset.
	*/
	printf("\n\n** Recover FWupdate complete, booting OS **\n\n");
	reset_after_fwupd();
	}


/*****************************************************************************
*	The image has already been loaded into memory.  
*	NOTE: may need to  "setenv fileaddr;saveenv" BEFORE downloading.
*	Just go and program the image. (vendor mode only)
*****************************************************************************/
if(mflag)
	{
	/*	Get the starting memory address of a downloaded binary*/
	if((s = (char*)getenv("fileaddr")) == 0)
		{
		mem_start = CFG_LOAD_ADDR;	/*serial download does not set fileaddr*/
		}
	else
		{
		mem_start = (u32)simple_strtol( s, &cptr1, 16 ); /*u-boot does not prepend 0x*/
		if( (s32)cptr1 < (strlen(s) + (s32)s) )
			{	
			sprintf(msg, "\"fileaddr\" cannot be converted to int.\n");
			return(1);
			}
		}

	/*	Verify integerty of the image file (provides toal_image_bcnt)*/
	if( (rc = fwupd_verify_image(mem_start, &total_image_bcnt, msg)) != 0)
		{
		printf("%s %s:ERROR:%s\n", argv[0], argv[1], msg);
		return(rc);
		}	

	/*	
	*	Need to know if we are crossing the 1.x <-> >2.0 barrier
	*	BEFORE fwupd_ram2flash()
	*/
	g_ecc_hack = calc_ecc_flag(mem_start);	/*sets g_ecc_hack*/
	if(1 == g_ecc_hack)
		printf("\nUsing new ECC algorithm.");
	if(-1 == g_ecc_hack)
		printf("\nUsing old ECC algorithm.");

	if(mflag == MFLAG_FLASH)
		{
		/*	Copy the image into flash for future use*/
		if( (rc = fwupd_ram2flash(mem_start, total_image_bcnt, msg)) != 0)
			{
			printf("%s %s:ERROR:%s\n", argv[0], argv[1], msg);
			return(rc);
			}
		}

	/*	
	*	This env will protect us upon an abnormal termination (EST clowns 
	*	pulling AC at every whim). If we detect this env flag, fwupd() will
	*	be called again to repair any incomplete or corrupt image, and will 
	*	only be cleared by fwupd() when complete.
	*/
	set_fwupd_state_env(FWUPD_ENV_STATE_PENDING);

	/*	Call the update*/
	if( (rc = fwupd(mem_start, msg)) != 0)
		{
		printf("%s %s:ERROR:%s\n", argv[0], argv[1], msg);
		return(rc);
		}	
	/*
	*	OK...to get here means fwupd did NOT return an error, AND also NOR 
	*	images did NOT require an update, else the process would have reset.
	*	So,  NOR was not updated and NAND is now finished so reset.
	*	I would prefer a forced wdog/board reset but then we depend on the 
	*	FPGA.  So just do a core reset.
	*/
	printf("\n\n** Recover FWupdate complete, booting OS **\n\n");
	reset_after_fwupd();
	}

//	Fwupdates should not get here, resets after fwupdate

/*****************************************************************************
*	Use to test the ability to loop on tftp'ing in a file. 
*	Origional problem was with ECC enabled while TFTP firmimg.cmc
*
*	Load the image via TFTP
*	Error return code are found in net_api.h, under NET_error_ids, 
*	ERROR_NET=0xa000
*****************************************************************************/
if(tftptest_flag)
	{
	while(	((current_loop <= loop_cnt) || (UTIL_LOOP_FOREVER == loop_cnt)) )
		{
		printf("Loading file \"%s\" from TFTP server IP \"%s\"\n", 
											BootFile, getenv ("serverip") );

		load_addr = CFG_LOAD_ADDR;		/*DFxxxxx set global load point*/
		mem_start = CFG_LOAD_ADDR;
		if (NetLoop(TFTP) < 0) 
			{
			printf("%s %s:ERROR:TFTP load.\n", argv[0], argv[1]);
			return(E_RECOVER_TFTP_LOAD_FAILED);
			}	
		else
			printf("%s %s:TFTP Load passed.\n", argv[0], argv[1]);

		/*	Verify integerty of the image file (provides total_image_bcnt)*/
		if( (rc = fwupd_verify_image(mem_start, &total_image_bcnt, msg)) != 0)
			{
			printf("%s %s:ERROR:%s\n", argv[0], argv[1], msg);
			return(rc);
			}	

		printf("\nTFTP load test...loop %d\n\n", current_loop);
		++current_loop;

		if ( ctrlc() )		/* check for ctrl-c to abort... */
			return(EXIT_BY_CTRLC);
		}/*end while*/
	}/*end tftptest_flag*/

/*****************************************************************************
*	Load the image via TFTP from the peer CMC.
*	Error return code are found in net_api.h, under NET_error_ids, 
*	ERROR_NET=0xa000
*****************************************************************************/
if(peer_flag)
	{
	printf("CMC update from peer\n");
	for(tftp_retry = 0; tftp_retry < MAX_TFTP_RETRY; ++tftp_retry)
		{
		if((tftp_image_bad = tftp_from_peer(msg)) != 0)	
			printf("FWupdate from PEER failure retry-%d %s\n", 
															tftp_retry, msg);
		else				
			break;	/*tftp from peer, verify and commit to NAND worked*/
		}

	if(tftp_retry >= MAX_TFTP_RETRY)
		{
		tftp_image_bad = 1;
		printf("%s %s:ERROR:%s\n", argv[0], argv[1], "FWupdate from PEER failed");
		return(-1); //hack
		}


	/*	
	*	This env will protect us upon an abnormal termination (EST clowns 
	*	pulling AC at every whim). If we detect this env flag, fwupd() will
	*	be called again to repair any incomplete or corrupt image, and will 
	*	only be cleared by fwupd() when complete.
	*/
	set_fwupd_state_env(FWUPD_ENV_STATE_PENDING);

	/*	Call the update*/
	mem_start = CFG_LOAD_ADDR;
	if( (rc = fwupd(mem_start, msg)) != 0)
		{
		printf("%s %s:ERROR:%s\n", argv[0], argv[1], msg);
		return(rc);
		}	
	/*
	*	OK...to get here means fwupd did NOT return an error, AND also NOR 
	*	images did NOT require an update, else the process would have reset.
	*	So,  NOR was not updated and NAND is now finished so reset.
	*	I would prefer a forced wdog/board reset but then we depend on the 
	*	FPGA.  So just do a core reset.
	*/
	printf("\n\n** Recover FWupdate complete, booting OS **\n\n");
	reset_after_fwupd();
	}

return(0);
}/*end FUNCTION: recover_fwupdate()*/

/*****************************************************************************
*	FUNCTION:	recover_getniccfg()
*	DATE:		2-28-07
*****************************************************************************/
s32 recover_getniccfg(int argc, char *argv[])
{
char *env_p;
char tmp[40];
char msg[MAX_MSG_LEN];

sprintf(msg, "NIC settings: ip=");

if( (env_p = getenv("ipaddr")) == 0)
	strcat(msg, "<unknown> ");
else
	{
	sprintf(tmp, "%s netmask=", env_p);
	strcat(msg, tmp);
	}

if( (env_p = getenv("netmask")) == 0)
	strcat(msg, "<unknown> ");
else
	{
	sprintf(tmp, "%s gateway=", env_p);
	strcat(msg, tmp);
	}
	

if( (env_p = getenv("gatewayip")) == 0)
	strcat(msg, "<unknown>");
else
	{
	sprintf(tmp, "%s", env_p);
	strcat(msg, tmp);
	}

printf("%s\n", msg);
return(0);
}/*end FUNCTION: recover_getniccfg()*/

/*****************************************************************************
*	FUNCTION:	recover_ping()
*	DATE:		2-28-07
*****************************************************************************/
s32 recover_ping(int argc, char *argv[])
{

if(argc < 3)
	{
	printf("%s %s:ERROR: IP address missing\n", argv[0], argv[1]);
	return(E_RECOVER_MISSING_IP);
	}

if(ip_verify(argv[2]))
	{
	printf("%s %s:ERROR:Invalid IP string \"%s\".\n", 
												argv[0], argv[1], argv[2]);
	return(E_RECOVER_INVALID_IP);
	}

if( (NetPingIP = string_to_ip(argv[2])) == 0)
	{
	printf("%s %s:ERROR:Invalid IP string \"%s\".\n", 
												argv[0], argv[1], argv[2]);
	return(E_RECOVER_INVALID_IP);
	}

if (NetLoop(PING) < 0) 
	{
	printf("%s %s:ERROR:Ping failed to \"%s\".\n", 
												argv[0], argv[1], argv[2]);
	return(E_RECOVER_PING_FAILED);
	}
else
	printf("%s %s:Ping successful to \"%s\".\n", 
												argv[0], argv[1], argv[2]);
return(0);
}/*end FUNCTION: recover_ping()*/

/*****************************************************************************
*	FUNCTION:	recover_reset()
*	DATE:		2-28-07
*****************************************************************************/
s32 recover_reset(int argc, char *argv[])
{
reset_after_fwupd();		/*should never return*/
return(0);				
}/*end FUNCTION: recover_reset()*/

/*****************************************************************************
*	FUNCTION:	recover_setniccfg()
*	DATE:		2-28-07
*****************************************************************************/
s32 recover_setniccfg(int argc, char *argv[])
{
	s32 rc;

    if((rc = ip_verify(argv[2])) != 0)
    {
        printf("%s %s:ERROR:Invalid IP string \"%s\".\n", 
												argv[0], argv[1], argv[2]);
		return(E_RECOVER_INVALID_IP);
    }
    if((rc = ip_verify(argv[3])) != 0)
    {
        printf("%s %s:ERROR:Invalid netmask string \"%s\".\n", 
												argv[0], argv[1], argv[3]);
        return(E_RECOVER_INVALID_NETMASK);
    }
    if((rc = ip_verify(argv[4])) != 0)
    {
        printf("%s %s:ERROR:Invalid gateway string \"%s\".\n", 
												argv[0], argv[1], argv[4]);
        return(E_RECOVER_INVALID_GATEWAY);
    }
    setenv("ipaddr",argv[2]);
    setenv("netmask",argv[3]);
    setenv("gatewayip",argv[4]);
    saveenv();

return(0);
}/*end FUNCTION: recover_setniccfg()*/


/*****************************************************************************
*	FUNCTION:	calc_ecc_flag()
*	DATE:		5-8-08
*	
*	1.x uses old ECC algorithm, > 2.0 uses new ECC.  
*	
*****************************************************************************/
s32 calc_ecc_flag(u32 mem)
{
u32 flag;
char cur_ver[MAX_VERSION_STRING_SIZE];
char new_ver[MAX_VERSION_STRING_SIZE];
char cv[MAX_VERSION_STRING_SIZE];
char nv[MAX_VERSION_STRING_SIZE];
t_master_image_header *mhdr = (t_master_image_header*)mem;

memset(cur_ver, 0, MAX_VERSION_STRING_SIZE);
memset(new_ver, 0, MAX_VERSION_STRING_SIZE);
memset(cv, 0, MAX_VERSION_STRING_SIZE);
memset(nv, 0, MAX_VERSION_STRING_SIZE);

get_current_fwver(cur_ver);					/*get from env*/
strcpy(new_ver, mhdr->version_string);		/*read from mem copy firmimg.cmc*/

/* 
*	For 1.x this code does not exist in recover so firmimg.cmc will be saved
*	as normal and fwupd will update u-boot and reset.  Then the newly flashed 
*	u-boot is_fwupd_needed() code will set g_ecc_hack=1 and force a re-write 
*	of firmimg.cmc with new ECC algorithm.  
*	The fact that this code exists means we are at >= 2.1 NOR, or new ECC.
*
*	A bug made 1.0 into A02, and 1.01 into A05!!!  Cannot simply cmp the 
*	first byte.  Going forward this numbering CANNOT be allowed.  
*	So if 'A' then it MUST be 1.x
*	(See util_cmc_main.c   is_fwupd_needed()  )
*
*/
flag = 1;			/*force new ECC by default, IF env is NOT readable*/

strncpy(cv, cur_ver, 3);
strncpy(nv, new_ver, 3);
if( (cv[0] < 0x30) || (cv[0] > 0x39) )	/*convert A0X to 1.0*/
	strcpy(cv, "1.0");
if( (nv[0] < 0x30) || (nv[0] > 0x39) )	
	strcpy(nv, "1.0");

if( (strcmp(cv, "2.0") > 0) && (strcmp(nv, "2.1") < 0) )
	flag = -1;	/*moving from >= 2.1 to < 2.0*/
else if(strcmp(cur_ver, new_ver) == 0)
	flag = 0;

return(flag);
}/*end FUNCTION: calc_ecc_flag()*/



/*****************************************************************************
*	FUNCTION:	tftp_from_peer()
*	DATE:		9-15-08
*
*****************************************************************************/
s32 tftp_from_peer(char *msg)
{
s32  rc;
u32	 mem_start;
u32  slot;
u32  pres;
u32  total_image_bcnt;
char *ip_p, *serverip_p, *ethact_p;
char *serverip;
char ip_save[20];
char serverip_save[20];
char ethact_save[40];

/*	
*	See if we have a peer CMC present
*/
if((rc = is_peer_cmc_present(&pres, msg)) != 0)
	return(rc);

if((rc = get_cmc_slot(&slot, msg)) != 0)
	return(rc);

/*	
*	We have a peer, save off some env variables that we need to change so 
*	we can restore them IF PEER fwupd does NOT work.	
*	Now return via "error_exit:"
*/
if( (ip_p = (char*)getenv("ipaddr")) != 0)
	strcpy(ip_save, ip_p);

if( (serverip_p = (char*)getenv("serverip")) != 0)
	strcpy(serverip_save, serverip_p);

if( (ethact_p = (char*)getenv("ethact")) != 0)
	strcpy(ethact_save, ethact_p);


if(0 == slot)
	serverip = "169.254.32.2";	/*load from slot 1 CMC*/
else
	serverip = "169.254.32.1";	/*load from slot 0 CMC*/

setenv("serverip", serverip);
setenv("ipaddr", "169.254.32.120");
setenv("ethact", "ppc_4xx_eth1");

/*****************************************************************************
*	Load the image via TFTP from the peer CMC.
*	Error return code are found in net_api.h, under NET_error_ids, 
*	ERROR_NET=0xa000
*****************************************************************************/
strcpy(BootFile, FWUPD_IMAGE_FILENAME);		
load_addr = CFG_LOAD_ADDR;		/*DFxxxxx set global load point*/
mem_start = CFG_LOAD_ADDR;	


if (NetLoop(TFTP) < 0) 
	{
	strcpy(msg, "ERROR:TFTP load");
	rc = E_RECOVER_TFTP_LOAD_FAILED;
	goto cleanup_exit;
	}	


/*	Verify integerty of the image file (provides toal_image_bcnt)*/
if( (rc = fwupd_verify_image(mem_start, &total_image_bcnt, msg)) != 0)
	goto cleanup_exit;

/*	
*	Need to know if we are crossing the 1.x <-> >2.1 barrier
*	BEFORE fwupd_ram2flash()
*/
g_ecc_hack = calc_ecc_flag(mem_start);	/*sets g_ecc_hack*/
if(1 == g_ecc_hack)
	printf("\nUsing new ECC algorithm.");
if(-1 == g_ecc_hack)
	printf("\nUsing old ECC algorithm.");

/*	Copy the image into flash for future use*/
if( (rc = fwupd_ram2flash(mem_start, total_image_bcnt, msg)) != 0)
	goto cleanup_exit;



cleanup_exit:
if(ip_p)
	setenv("ipaddr", ip_save);
if(serverip_p)
	setenv("serverip", serverip_save);
if(ethact_p)
	setenv("ethact", ethact_save);

return(rc);
}/*end FUNCTION: tftp_from_peer()*/


#endif /*CONFIG_CMC_10G*/



