/****************************************************************************

  Copyright (c) 2006, Dell Inc
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

  - Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
    
  - Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution. 

  - Neither the name of Dell Inc nor the names of its contributors
    may be used to endorse or promote products derived from this software 
    without specific prior written permission. 

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. 

*****************************************************************************/


/*
 * Utility support
 */
#include <common.h>		/*must be first in file*/
#include <command.h>
#include <asm/processor.h>
//#include <asm/cache.h>
#include <ppc4xx.h>

#ifdef CONFIG_CMC_10G	/*only compile in for a CMC_10G build*/

/******************************************************************************
*
*	FILE: util_reg.c
*	This command supports options to allow user level register manipulation.
*
******************************************************************************/


/******************************************************************************
*  Include files
******************************************************************************/
#include <util_defs.h>
#include <util_ret_codes.h>
#include <util_fpga.h>

#include <ppc440.h>


/******************************************************************************
*  Local Defines
******************************************************************************/


/******************************************************************************
*  Globals
******************************************************************************/


/******************************************************************************
*  Prototypes
******************************************************************************/
extern s32	mcoptind;
extern char	*mcoptarg[];


static u32 reg_read(char *group_name, char *addr, char *msg);
static u32 reg_write(char *group_name, char *reg_str, char *data, char *msg);
static u32 reg_mod(char *group_name, char *reg_str, char *msg);
static u32 single_read(u32 type, u32 reg);
static u32 single_write(u32 type, u32 reg, u32 data);
static u32 get_group_index(char *group_name, u32 *gindex, char *msg);
static u32 get_reg_asc2hex(char *reg_str, u32 gindex, u32 *reg, u32 *rindex, 
																	char *msg);

static u32 get_spr(u32 reg);
static u32 set_spr(u32 reg, u32 data);

static u32 get_uic(u32 reg);
static u32 set_uic(u32 reg, u32 data);

/******************************************************************************
*	List all structures defines
******************************************************************************/
t_options util_reg_options[] = 
	{
	/*Options*/
	{"r",	 "Read a register from a specified group."},
	{"w",	 "Write a register to a specified group."},
	{"mm",	 "Read and allow modification."},
	{"dump", "Dump supported registers from a specified group."},
	{"help", "Display a list of supported groups."},
	{ NULL, NULL }
	};


t_usage util_reg_usage = { 
	"reg", 
					
	/*Syntax*/
	"reg    [-r <reg group> <0xReg | name>]\n"
	"       [-w <reg group> <0xReg | name> <0xdata>]\n"
	"       [-mm <reg group> <0xReg | name>]\n"
	"       [-dump <reg group>]\n"
	"       [-help]\n",

	/*Description*/
	"This command allows the user to manipulate PPC, and memory mapped\n"
	"registers.  The read/write options will access any register\n"
	"within the group, with no error checking.  BUT the group must be\n"
	"supported.   The dump option lists the regsisters that have been\n"
	"included within the internal table, and can be added as needed.\n"
	"Examples:\n"
	"util reg -help\n"
	"util reg -dump ebc\n"
	"util reg -r ebc 0x11\n"
	"util reg -w ebc pb1ap 0x04055400\n",
	
	util_reg_options
	}; /*end t_usage util_reg_usage*/


/******************************************************************************
*	Define supported register groupings
******************************************************************************/
typedef struct	{		/*Individual register group*/
	char *name;			/*name of register*/
	u32	 addr;			/*address/offset of register*/
	} t_regs;

typedef struct	{		/*List of register groups*/
	char *subsys;		/*where reg group is associated, PPC, FPGA, etc*/ 
	char *name;			/*text name of register group*/
	u32	 width;			/*register bus width 16, 32, etc*/
	u32	 type;			/*type id used to determine how to access reg*/
	u32  base;			/*Base offset where register set starts*/
	t_regs  *regs;		/*ptr to single group register list*/
	char *hdr_msg;		/*helpful header text used in dump cmd*/
	} t_reg_groups;

/*	Generic PPC dump header*/
char hdr_generic[] = "\nName addr/offset=read data";

t_regs  reg_ebc[] = 	{	/*list of DCR EBC registers*/
	{ "pb0cr", pb0cr },
	{ "pb1cr", pb1cr },
	{ "pb2cr", pb2cr },
	{ "pb3cr", pb3cr },
	{ "pb4cr", pb4cr },
	{ "pb5cr", pb5cr },
	{ "pb0ap", pb0ap },
	{ "pb1ap", pb1ap },
	{ "pb2ap", pb2ap },
	{ "pb3ap", pb3ap },
	{ "pb4ap", pb4ap },
	{ "pb5ap", pb5ap },
	{ "cfg",   xbcfg },
	{ "bcid",   xbcid },
	{ 0, 0}	};


t_regs   reg_sdr[] = 	{	/*list of DCR SDR registers*/
	{ "sdstp0", 0x0020 },
	{ "sdstp1", 0x0021 },
	{ "sdstp2", 0x4001 },
	{ "sdstp3", 0x4003 },
	{ "pinstp", 0x0040 },
	{ "sdcs0",  0x0060 },
	{ "ecid0",  0x0080 },
	{ "ecid1",  0x0081 },
	{ "ecid2",  0x0082 },
	{ "ecid3",  0x0083 },
	{ "jtag",   0x00c0 },	
	{ "cp440",   0x0180 },	
	{ "usb2d0cr",	0x0320 },	
	{ "usb2h0cr",	0x0340 },
	{ "pfc0",	0x4100 },
	{ "pfc1",	0x4101 },
	{ "usb2phy0cr",	0x4103 },
	{ "usb2host",	0x4600 },

	{ 0, 0}	};

t_regs   reg_gpio[] = 	{	/*list of GPIO registers*/
	{ "or",		0x0B00 },
	{ "tcr",	0x0B04 },
	{ "osrl",	0x0B08 },
	{ "osrh",	0x0B0C },
	{ "tsrl",	0x0B10 },
	{ "tsrh",	0x0B14 },
	{ "odr",	0x0B18 },
	{ "ir",		0x0B1c },
	{ "rr1",	0x0B20 },
	{ "rr2",	0x0B24 },
	{ "rr3",	0x0B28 },
	{ "isr1l",	0x0B30 },
	{ "isr1h",	0x0B34 },
	{ "isr2l",	0x0B38 },
	{ "isr2h",	0x0B3C },
	{ "isr23",	0x0B40 },
	{ "isr3h",	0x0B44 },
	{ "1or",	0x0C00 },
	{ "1tcr",	0x0C04 },
	{ "1osrl",	0x0C08 },
	{ "1osrh",	0x0C0C },
	{ "1tsrl",	0x0C10 },
	{ "1tsrh",	0x0C14 },
	{ "1odr",	0x0C18 },
	{ "1ir",	0x0C1c },
	{ "1rr1",	0x0C20 },
	{ "1rr2",	0x0C24 },
	{ "1rr3",	0x0C28 },
	{ "1isr1l",	0x0C30 },
	{ "1isr1h",	0x0C34 },
	{ "1isr2l",	0x0C38 },
	{ "1isr2h",	0x0C3C },
	{ "1isr3l",	0x0C40 },
	{ "1isr3h",	0x0C44 },
	{ 0, 0}	};

t_regs   reg_sdram[] = 	{	/*list of DCR SDRAM registers*/
	{ "ddr00", 0x0000 },
	{ "ddr01", 0x0001 },
	{ "ddr02", 0x0002 },
	{ "ddr03", 0x0003 },
	{ "ddr04", 0x0004 },
	{ "ddr05", 0x0005 },
	{ "ddr06", 0x0006 },
	{ "ddr07", 0x0007 },
	{ "ddr08", 0x0008 },
	{ "ddr09", 0x0009 },
	{ "ddr10", 0x000a },
	{ "ddr11", 0x000b },
	{ "ddr12", 0x000c },
	{ "ddr14", 0x000e },
	{ "ddr17", 0x0011 },
	{ "ddr18", 0x0012 },
	{ "ddr19", 0x0013 },
	{ "ddr20", 0x0014 },
	{ "ddr21", 0x0015 },
	{ "ddr22", 0x0016 },
	{ "ddr23", 0x0017 },
	{ "ddr24", 0x0018 },
	{ "ddr25", 0x0019 },
	{ "ddr26", 0x001a },
	{ "ddr27", 0x001b },
	{ "ddr28", 0x001c },
	{ "ddr31", 0x001f },
	{ "ddr32", 0x0020 },
	{ "ddr33", 0x0021 },
	{ "ddr34", 0x0022 },
	{ "ddr35", 0x0023 },
	{ "ddr36", 0x0024 },
	{ "ddr37", 0x0025 },
	{ "ddr38", 0x0026 },
	{ "ddr39", 0x0027 },
	{ "ddr40", 0x0028 },
	{ "ddr41", 0x0029 },
	{ "ddr42", 0x002a },
	{ "ddr43", 0x002b },
	{ "ddr44", 0x002c },
	{ 0, 0}	};

	
t_regs   reg_clk[] = 	{	/*list of clock and power-on reset registers*/
	{ "clkupd",	0x0020 },
	{ "pllc0",	0x0040 },
	{ "plld0",	0x0060 },
	{ "primad0",0x0080 },
	{ "primbd0",0x00a0 },
	{ "opbd0",	0x00c0 },
	{ "perd0",	0x00e0 },
	{ "mald",	0x0100 },
	{ "spcid",	0x0120 },
	{ "icfg",	0x0140 },
	{ 0, 0}	};


		
t_regs   reg_mal0[] = 	{	/*list of Memory Access Layer registers*/
	{ "cfg",		0x0180 },
	{ "esr",		0x0181 },
	{ "ier",		0x0182 },
	{ "txcasr",		0x0184 },
	{ "txcarr",		0x0185 },
	{ "txeobisr",	0x0186 },
	{ "txdeir",		0x0187 },
	{ "txtattrr",	0x0188 },
	{ "txbaddr",	0x0189 },
	{ "rxcasr",		0x0190 },
	{ "rxcarr",		0x0191 },
	{ "rxeobisr",	0x0192 },
	{ "rxdeir",		0x0193 },
	{ "rxtattrr",	0x0194 },
	{ "rxbaddr",	0x0195 },
	{ "txctpor",	0x01a0 },
	{ "txctp1r",	0x01a1 },
	{ "rxctpor",	0x01c0 },
	{ "rxctp1r",	0x01c1 },
	{ "rcbs0",		0x01e0 },
	{ "rcbs1",		0x01e1 },
	{ 0, 0}	};

t_regs   reg_dcrnull[] = {	/*DCR stub to allow read/write of any DCR*/
	{ 0, 0}	};

/*	NOTE: you MUST also have to modify get_spr()/set_spr()*/
t_regs   reg_spr[] = 	{	/*list of PPC Special Purpose Registers*/
	{ "ivor0",		0x00000190 },
	{ "ivor1",		0x00000191 },
	{ "ivor2",		0x00000192 },
	{ "ivor3",		0x00000193 },
	{ "ivor4",		0x00000194 },
	{ "ivor5",		0x00000195 },
	{ "ivor6",		0x00000196 },
	{ "ivor7",		0x00000197 },
	{ "ivor8",		0x00000198 },
	{ "ivor9",		0x00000199 },
	{ "ivor10",		0x0000019a },
	{ "ivor11",		0x0000019b },
	{ "ivor12",		0x0000019c },
	{ "ivor13",		0x0000019d },
	{ "ivor14",		0x0000019e },
	{ "ivor15",		0x0000019f },
	{ "ivpr",		0x0000003f },
	{ 0, 0}	};


	/*	NOTE: you MUST also have to modify get_uic()/set_uic()*/
t_regs   reg_uic[] = 	{	/*list of UIC registers*/
	{ "uic0_sr",	0x00000c0 },
	{ "uic0_er",	0x00000c2 },
	{ "uic0_cr",	0x00000c3 },
	{ "uic0_pr",	0x00000c4 },
	{ "uic0_tr",	0x00000c5 },
	{ "uic0_msr",	0x00000c6 },
	{ "uic0_vr",	0x00000c7 },

	{ "uic1_sr",	0x00000d0 },
	{ "uic1_er",	0x00000d2 },
	{ "uic1_cr",	0x00000d3 },
	{ "uic1_pr",	0x00000d4 },
	{ "uic1_tr",	0x00000d5 },
	{ "uic1_msr",	0x00000d6 },
	{ "uic1_vr",	0x00000d7 },

	{ "uic2_sr",	0x00000e0 },
	{ "uic2_er",	0x00000e2 },
	{ "uic2_cr",	0x00000e3 },
	{ "uic2_pr",	0x00000e4 },
	{ "uic2_tr",	0x00000e5 },
	{ "uic2_msr",	0x00000e6 },
	{ "uic2_vr",	0x00000e7 },
	{ 0, 0}	};


t_regs  reg_nand[] = 	{	/*list of DCR EBC registers*/
	{ "cmd",  (CFG_NAND_ADDR + 0x0000) },
	{ "addr", (CFG_NAND_ADDR + 0x0004) },
	{ "data", (CFG_NAND_ADDR + 0x0008) },
	{ "ecco", (CFG_NAND_ADDR + 0x0010) },
	{ "ecc1", (CFG_NAND_ADDR + 0x0014) },
	{ "ecc2", (CFG_NAND_ADDR + 0x0018) },
	{ "ecc3", (CFG_NAND_ADDR + 0x001c) },
	{ "ecc4", (CFG_NAND_ADDR + 0x0020) },
	{ "ecc5", (CFG_NAND_ADDR + 0x0024) },
	{ "ecc6", (CFG_NAND_ADDR + 0x0028) },
	{ "ecc7", (CFG_NAND_ADDR + 0x002c) },
	{ "b0cr", (CFG_NAND_ADDR + 0x0030) },
	{ "b1cr", (CFG_NAND_ADDR + 0x0034) },
	{ "b2cr", (CFG_NAND_ADDR + 0x0038) },
	{ "b3cr", (CFG_NAND_ADDR + 0x003c) },
	{ "cr",   (CFG_NAND_ADDR + 0x0040) },
	{ "sr",   (CFG_NAND_ADDR + 0x0044) },
	{ "hwctl",(CFG_NAND_ADDR + 0x0048) },
	{ "id",   (CFG_NAND_ADDR + 0x0050) },	
	{ 0, 0}	};

/*****************************************************************************
*	FPGA Registers
*****************************************************************************/
char hdr_fpga_gpr[] =	"[0]:VENDOR_MODE_N\n"
						"[3:0]:MP_REV3-0\n"
						"[0]:CMC_SLOT_ID\n"
						"[0]:CMCF_CMCL_PRES_N\n"
						"[0]:SYSPWR_GOOD\n"
						"[0]:VBAT_PWR_GOOD\n";

t_regs   reg_fpga_gpr[] = 	{	/*FPGA General Purpose registers*/
	{ "rev",		FPGA_REG_REV			},
	{ "scratch1",	FPGA_REG_SCRATCH1			},
	{ "scratch0",	FPGA_REG_SCRATCH0			},
	{ "mode1",		FPGA_REG_MODE1				},
	{ "mode0",		FPGA_REG_MODE0				},
	{ "id",			FPGA_REG_ID					},
	{ "fpga_vendor",FPGA_REG_VENDOR_MODE_N		},
	{ "mp_rev",		FPGA_REG_MP_REV				},
	{ "cmc_id",		FPGA_REG_CMC_SLOT_ID		},
	{ "cmcf_l_pres",FPGA_REG_CMCF_CMCL_PRES		},
	{ "syspwrgood",	FPGA_REG_SYSPWR_GOOD		},
	{ "vbat_pwr_good", FPGA_REG_VBAT_PWR_GOOD	},
	{ "therm_trip", FPGA_REG_THERM_TRIP			},
	{ "cpld_rev",   FPGA_REG_CPLD_REV			},
	{ 0, 0}	};


	char hdr_fpga_blade[] =	"BLADE INT and MASK:\n"
							"[15]:BLD16_INT - [0]:BLD1_INT R/w1c\n"
							"[15]:BLD16_INT_MSK - [0]:BLD1_INT_MSK\n\n"

							"BLADE_IN:\n"
							"[15]:BLD_PSNT\n"
							"[14]:BLD_BLNK_PSNT\n"
							"[13-6]:RES\n"
							"[5]:ALERT\n"
							"[4]:LINK_STAT\n"  
							"[3:1]:SPARE\n"
							"[0]:ACK\n\n"
							
							"BLADE POWER OUT:\n"
							"[15]:BLADE16_PWR_N - [0]:BLADE1_PWR_N\n"
							
							"BLADE OUT:\n"
							"[15-9]:RES\n"
							"[8]:IMC_RST\n"
							"[7]:BLD_RST\n"
							"[6]:THROTTLE\n"
							"[5]:BLD_PWRDN\n"
							"[4]:NTWRK_ISO\n"
							"[3:0]:SPARE1-4\n";

t_regs   reg_fpga_blade[] = 	{	/*FPGA blade INPUT registers*/
	{ "bld_int",	FPGA_REG_GPI_BLADE_INT },
	{ "bld_mask",	FPGA_REG_GPI_BLADE_MASK },

	{ "bld1_in",	FPGA_REG_BLADE1_GPI		},
	{ "bld2_in",	FPGA_REG_BLADE2_GPI		},
	{ "bld3_in",	FPGA_REG_BLADE3_GPI		},
	{ "bld4_in",	FPGA_REG_BLADE4_GPI		},
	{ "bld5_in",	FPGA_REG_BLADE5_GPI		},
	{ "bld6_in",	FPGA_REG_BLADE6_GPI		},
	{ "bld7_in",	FPGA_REG_BLADE7_GPI		},
	{ "bld8_in",	FPGA_REG_BLADE8_GPI		},
	{ "bld9_in",	FPGA_REG_BLADE9_GPI		},
	{ "bld10_in",	FPGA_REG_BLADE10_GPI	},
	{ "bld11_in",	FPGA_REG_BLADE11_GPI	},
	{ "bld12_in",	FPGA_REG_BLADE12_GPI	},
	{ "bld13_in",	FPGA_REG_BLADE13_GPI	},
	{ "bld14_in",	FPGA_REG_BLADE14_GPI	},
	{ "bld15_in",	FPGA_REG_BLADE15_GPI	},
	{ "bld16_in",	FPGA_REG_BLADE16_GPI	},

	{ "bld_pwr",	FPGA_REG_BLADE_PWR_OUT	},

	{ "bld1_out",	FPGA_REG_BLADE1_GPO		},
	{ "bld2_out",	FPGA_REG_BLADE2_GPO		},
	{ "bld3_out",	FPGA_REG_BLADE3_GPO		},
	{ "bld4_out",	FPGA_REG_BLADE4_GPO		},
	{ "bld5_out",	FPGA_REG_BLADE5_GPO		},
	{ "bld6_out",	FPGA_REG_BLADE6_GPO		},
	{ "bld7_out",	FPGA_REG_BLADE7_GPO		},
	{ "bld8_out",	FPGA_REG_BLADE8_GPO		},
	{ "bld9_out",	FPGA_REG_BLADE9_GPO		},
	{ "bld10_out",	FPGA_REG_BLADE10_GPO	},
	{ "bld11_out",	FPGA_REG_BLADE11_GPO	},
	{ "bld12_out",	FPGA_REG_BLADE12_GPO	},
	{ "bld13_out",	FPGA_REG_BLADE13_GPO	},
	{ "bld14_out",	FPGA_REG_BLADE14_GPO	},
	{ "bld15_out",	FPGA_REG_BLADE15_GPO	},
	{ "bld16_out",	FPGA_REG_BLADE16_GPO	},
	{ 0, 0}	};							
	

	char hdr_fpga_psu[] = "PSU INT and ALERT MASK:\n"
						"[15]:I2C PSU123_ALERT R/w1c\n"
						"[14]:I2C PSU456_ALERT R/w1c\n"
						"[13]:I2C PSU123_INT RO\n"
						"[12]:I2C PSU456_INT RO\n"
						"[11]:I2C PS123_SMTO RO\n"
						"[12]:I2C PS456_SMTO RO\n"
						"[9:6]:RES\n"
						"[5]:GPI_PSU6_INT -[0]:GPI_PSU1_INT R/w1c\n\n"

						"[15]:I2C PSU123_ALERT_MSK\n"
						"[14]:I2C PSU456_ALERT_MSK\n"
						"[13]:I2C PSU123_INT_MSK\n"
						"[12]:I2C PSU456_INT_MSK\n"
						"[11]:I2C PS123_SMTO_MSK\n"
						"[12]:I2C PS456_SMTO_MSK\n"
						"[9:6]:RES\n"
						"[5]:GPI_PSU6_INT_MSK -[0]:GPI_PSU1_INT_MSK\n\n"
	
						"PSU IN:\n"
						"[15:3]:RES\n"
						"[2]:PSU_PRES\n"
						"[1]:PSU ACGOOD\n"
						"[0]:PSU DCGOOD\n\n"
						
						"PSU OUT:\n"
						"[15:6]:res\n"
						"[5]:PS6_ON_N - [0]:PS1_ON_N\n";


t_regs   reg_fpga_psu[] = 	{	/*FPGA PSU INPUT registers*/
	{ "psu_int",	FPGA_REG_GPI_PSU_INT	},
	{ "psu_mask",	FPGA_REG_GPI_PSU_MASK	},

	{ "psu1_in",	FPGA_REG_GPI_PSU1_IN	},
	{ "psu2_in",	FPGA_REG_GPI_PSU2_IN	},
	{ "psu3_in",	FPGA_REG_GPI_PSU3_IN	},
	{ "psu4_in",	FPGA_REG_GPI_PSU4_IN	},
	{ "psu5_in",	FPGA_REG_GPI_PSU5_IN	},
	{ "psu6_in",	FPGA_REG_GPI_PSU6_IN	},

	{ "psu_out",	FPGA_REG_PSU_OUT		},
	{ 0, 0}	};



	char hdr_fpga_iom[] = "IOM INT and MASK:\n"
						"[15]:I2C IOM123_ALERT R/w1c\n"
						"[14]:I2C IOM456_ALERT R/w1c\n"
						"[13]:I2C IOM123_INT RO\n"
						"[12]:I2C IOM456_INT RO\n"
						"[11]:I2C IOM123_SMTO RO\n"
						"[12]:I2C IOM456_SMTO RO\n"
						"[9:6]:RES\n"
						"[5]:GPI_IOM6_INT -[0]:GPI_IOM1_INT R/w1c\n\n"

						"[15]:I2C IOM123_ALERT_MSK\n"
						"[14]:I2C IOM456_ALERT_MSK\n"
						"[13]:I2C IOM123_INT_MSK\n"
						"[12]:I2C IOM456_INT_MSK\n"
						"[11]:I2C IOM23_SMTO_MSK\n"
						"[12]:I2C IOM456_SMTO_MSK\n"
						"[9:6]:RES\n"
						"[5]:GPI_IOM6_INT_MSK -[0]:GPI_IOM1_INT_MSK\n\n"

						"IOM IN:\n"
						"[15:14]:RES\n"
						"[13]:IOM6_BLANK_N - [8]:IOM1_BLANK_N\n"
						"[7:6]:REC\n" 
						"[5]:IOM6_PRES_N - [5]:IOM6_PRES_N\n";


t_regs   reg_fpga_iom[] = 	{	/*FPGA IOM INPUT registers*/
	{ "iom_int",	FPGA_REG_GPI_IOM_INT	},
	{ "iom_mask",	FPGA_REG_GPI_IOM_MASK	},
	{ "iom_in",		FPGA_REG_GPI_IOM_IN		},
	{ 0, 0}	};



char hdr_fpga_fan[] = 
			"FAN INT and MASK:\n"
			"[15]:TACH2_THRSHLD_INT\n"
			"[14]:TACH1_THRSHLD_INT\n"
			"[13:9]:RES\n"
			"[8]:GPI_FAN9_TYPE_INT - [0]:GPI_FAN0_TYPE_INT\n\n"

			"[15]:TACH2_THRSHLD_INT_MSK\n"
			"[14]:TACH1_THRSHLD_INT_MSK\n"
			"[13:9]:RES\n"
			"[8]:GPI_FAN9_TYPE_INT_MSK - [0]:GPI_FAN0_TYPE_INT_MSK\n\n"

			"FAN TYPE:\n"
			"[15:3]:RES\n"
			"[2:0]:FAN_TYPE\n\n"

			"FAN LED STATE:\n"
			"[15:3]:RES\n"
			"[2]:FAN_STATE2 - [0]:FAN_STATE0\n\n"
			
			"FAN PWM Conrol\n"
			"[15:8]:RES\n"
			"[7:0]:Duty cycle\n\n"
			
			"FAN TACH_CNT:\n"
			"[15-0]:TACH_CNT\n\n"
			
			"FAN TACH THRESHOLD:\n"
			"[15:0]:TACH_THRSHLD\n\n"
			
			"FAN INT status  (TACH1 and 2):\n"
			"[15-9]:RES\n"
			"[8]:FAN9_TACH1_INT - [0]:FAN1_TACH1_INT\n"
			"[8]:FAN9_TACH2_INT - [0]:FAN1_TACH2_INT\n";


t_regs   reg_fpga_fan[] = 	{	/*FPGA FAN INPUT registers*/
	{ "fan_int",	FPGA_REG_FAN_INT		},
	{ "fan_mask",	FPGA_REG_FAN_INT_MASK	},

	{ "fan1_type",	FPGA_REG_FAN1_TYPE		},
	{ "fan2_type",	FPGA_REG_FAN2_TYPE		},
	{ "fan3_type",	FPGA_REG_FAN3_TYPE		},
	{ "fan4_type",	FPGA_REG_FAN4_TYPE		},
	{ "fan5_type",	FPGA_REG_FAN5_TYPE		},
	{ "fan6_type",	FPGA_REG_FAN6_TYPE		},
	{ "fan7_type",	FPGA_REG_FAN7_TYPE		},
	{ "fan8_type",	FPGA_REG_FAN8_TYPE		},
	{ "fan9_type",	FPGA_REG_FAN9_TYPE		},

	{ "fan1_state",	FPGA_REG_FAN1_STATE		},
	{ "fan2_state",	FPGA_REG_FAN2_STATE		},
	{ "fan3_state",	FPGA_REG_FAN3_STATE		},
	{ "fan4_state",	FPGA_REG_FAN4_STATE		},
	{ "fan5_state",	FPGA_REG_FAN5_STATE		},
	{ "fan6_state",	FPGA_REG_FAN6_STATE		},
	{ "fan7_state",	FPGA_REG_FAN7_STATE		},
	{ "fan8_state",	FPGA_REG_FAN8_STATE		},
	{ "fan9_state",	FPGA_REG_FAN9_STATE		},

	{ "fan1_pwm",	FPGA_REG_FAN1_PWM_CTRL	},
	{ "fan2_pwm",	FPGA_REG_FAN2_PWM_CTRL	},
	{ "fan3_pwm",	FPGA_REG_FAN3_PWM_CTRL	},
	{ "fan4_pwm",	FPGA_REG_FAN4_PWM_CTRL	},
	{ "fan5_pwm",	FPGA_REG_FAN5_PWM_CTRL	},
	{ "fan6_pwm",	FPGA_REG_FAN6_PWM_CTRL	},
	{ "fan7_pwm",	FPGA_REG_FAN7_PWM_CTRL	},
	{ "fan8_pwm",	FPGA_REG_FAN8_PWM_CTRL	},
	{ "fan9_pwm",	FPGA_REG_FAN9_PWM_CTRL	},

	{ "fan1_t1",	FPGA_REG_FAN1_TACH1	},
	{ "fan1_t2",	FPGA_REG_FAN1_TACH2	},
	{ "fan2_t1",	FPGA_REG_FAN2_TACH1	},
	{ "fan2_t2",	FPGA_REG_FAN2_TACH2	},
	{ "fan3_t1",	FPGA_REG_FAN3_TACH1	},
	{ "fan3_t2",	FPGA_REG_FAN3_TACH2	},
	{ "fan4_t1",	FPGA_REG_FAN4_TACH1	},
	{ "fan4_t2",	FPGA_REG_FAN4_TACH2	},
	{ "fan5_t1",	FPGA_REG_FAN5_TACH1	},
	{ "fan5_t2",	FPGA_REG_FAN5_TACH2	},
	{ "fan6_t1",	FPGA_REG_FAN6_TACH1	},
	{ "fan6_t2",	FPGA_REG_FAN6_TACH2	},
	{ "fan7_t1",	FPGA_REG_FAN7_TACH1	},
	{ "fan7_t2",	FPGA_REG_FAN7_TACH2	},
	{ "fan8_t1",	FPGA_REG_FAN8_TACH1	},
	{ "fan8_t2",	FPGA_REG_FAN8_TACH2	},
	{ "fan9_t1",	FPGA_REG_FAN9_TACH1	},
	{ "fan9_t2",	FPGA_REG_FAN9_TACH2	},

	{ "fan1_t1_thrs",	FPGA_REG_FAN1_TACH1_THRSHLD	},
	{ "fan1_t2_thrs",	FPGA_REG_FAN1_TACH2_THRSHLD	},
	{ "fan2_t1_thrs",	FPGA_REG_FAN2_TACH1_THRSHLD	},
	{ "fan2_t2_thrs",	FPGA_REG_FAN2_TACH2_THRSHLD	},
	{ "fan3_t1_thrs",	FPGA_REG_FAN3_TACH1_THRSHLD	},
	{ "fan3_t2_thrs",	FPGA_REG_FAN3_TACH2_THRSHLD	},
	{ "fan4_t1_thrs",	FPGA_REG_FAN4_TACH1_THRSHLD	},
	{ "fan4_t2_thrs",	FPGA_REG_FAN4_TACH2_THRSHLD	},
	{ "fan5_t1_thrs",	FPGA_REG_FAN5_TACH1_THRSHLD	},
	{ "fan5_t2_thrs",	FPGA_REG_FAN5_TACH2_THRSHLD	},
	{ "fan6_t1_thrs",	FPGA_REG_FAN6_TACH1_THRSHLD	},
	{ "fan6_t2_thrs",	FPGA_REG_FAN6_TACH2_THRSHLD	},
	{ "fan7_t1_thrs",	FPGA_REG_FAN7_TACH1_THRSHLD	},
	{ "fan7_t2_thrs",	FPGA_REG_FAN7_TACH2_THRSHLD	},
	{ "fan8_t1_thrs",	FPGA_REG_FAN8_TACH1_THRSHLD	},
	{ "fan8_t2_thrs",	FPGA_REG_FAN8_TACH2_THRSHLD	},
	{ "fan9_t1_thrs",	FPGA_REG_FAN9_TACH1_THRSHLD	},
	{ "fan9_t2_thrs",	FPGA_REG_FAN9_TACH2_THRSHLD	},

	{ "fan_t1_int",		FPGA_REG_FAN_TACH1_INT	},
	{ "fan_t2_int",		FPGA_REG_FAN_TACH2_INT	},

	{ 0, 0}	};


char hdr_fpga_kvm[] =	"KVM INT:\n"
						"[15]:I2C_KVM_ALERT (R/w1c)\n"
						"[14]:I2C_KVM_INT\n"
						"[13]:I2C_KVM_SMTO\n"
						"[12:1]:RES\n"
						"[0]:KVM_INT\n\n"
						
						"KVM INT MASK:\n"
						"[15]:I2C_KVM_ALERT_MSK\n"
						"[14]:I2C_KVM_INT_MSK\n"
						"[13]:I2C_KVM_SMTO_MSK\n"
						"[12:1]:RES\n"
						"[0]:KVM_INT_MSK\n\n"
						
						"KVM In Status:\n"
						"[15:2]:RES\n"
						"[1]:KVM_BLANK_N\n"
						"[0]:KVM_PRES_N\n\n"
						
						"KVM Out:\n"
						"[15:1]:RES\n"
						"[0]:I2C_KVM_PWR_N\n";

t_regs   reg_fpga_kvm[] = 	{	/*FPGA General Purpose registers*/
	{ "kvm_int",	FPGA_REG_GPI_KVM_INT		},
	{ "kvm_mask",	FPGA_REG_GPI_KVM_MASK		},
	{ "kvm_in",		FPGA_REG_GPI_KVM_IN			},
	{ "kvm_out",	FPGA_REG_I2C_KVM_PWR_OUT	},

	
	{ 0, 0}	};




char hdr_fpga_cp[] =	"CP INT:\n"
						"[15:4]:RES\n"
						"[3]:CP_PWR_INT\n"
						"[2]:I2C_CP_ALERT\n"
						"[1]:CP_TFT_PRES_INT\n"
						"[0]:CP_IO_PRES_INT\n\n"
						
						"CP INT MASK:\n"
						"[15:4]:RES\n"
						"[3]:CP_PWR_INT_MSK\n"
						"[2]:I2C_CP_ALERT_MSK\n"
						"[1]:CP_TFT_PRES_INT_MSK\n"
						"[0]:CP_IO_PRES_INT_MSK\n\n"
						
						"CP Status:\n"
						"[15:4]:RES\n"
						"[3]:CP_PWR_INT\n"
						"[2]:I2C_CP_ALERT_N\n"
						"[1]:CP_TFT_PRES_N_B\n"
						"[0]:CP_IO_PRES_N_B\n";

t_regs   reg_fpga_cp[] = 	{	/*FPGA General Purpose registers*/
	{ "cp_int",		FPGA_REG_GPI_CP_INT		},
	{ "cp_mask",	FPGA_REG_GPI_CP_MASK	},
	{ "cp_in",		FPGA_REG_GPI_CP_IN		},
	{ 0, 0}	};
	

char hdr_fpga_cmc[] =	"Foreign CMC Reset Protect:\n"
						"[15:0]:CMCF_RST_PROTECT (0xDE11)\n\n"
						
						
						"Foreign CMC Reset Protect:\n"
						"[15:1]:RES\n"
						"[0]:CMCF_RST\n";

t_regs   reg_fpga_cmc[] = 	{	/*FPGA General Purpose registers*/
	{ "cmcf_prot",		FPGA_REG_CMCF_RST_PROTECT	},
	{ "cmcf_rst",		FPGA_REG_CMCF_RST_OUT		},
	{ 0, 0}	};

char hdr_fpga_wdog[] =	"Watch Dog ctrl:\n"
						"[15:4]:RES\n"
						"[3]:WDT_CLR_EXT_LATCH\n"
						"[2]:WDT_EXPIRED\n"
						"[1]:WDT_RESET\n"
						"[0]:WDT_ARM_DISABLE\n\n"					
						
						"FWatch Dog Duration:\n"
						"[15:0]:WDT_DUR\n";

t_regs   reg_fpga_wdog[] = 	{	/*FPGA General Purpose registers*/
	{ "wdog_ctrl",		FPGA_REG_WDT_CTRL	},
	{ "wdog_dur",		FPGA_REG_WDT_DUR	},
	{ 0, 0}	};
	
/*	
*	Define type ID's used to determine how to access the registers.
*	Some DCR's are indexed, some memory mapped, etc
*/
#define REGGRP_DCREBC	1
#define REGGRP_DCRSDR	2
#define REGGRP_GPIO		3
#define REGGRP_SDRAM	4
#define REGGRP_CLK		5
#define REGGRP_RAW_DCR	6
#define REGGRP_FPGA		7
#define REGGRP_NAND		8
#define REGGRP_SPR		9
#define REGGRP_UIC		10

/*	
*	This is the "register list" array that contains the register groupings.
*	Walk this array to access supported registers.
*/
t_reg_groups  rlist[] = 	{
{"PPC",  "ebc",	32, REGGRP_DCREBC,	EBC_DCR_BASE,	reg_ebc,		hdr_generic		},
{"PPC",  "sdr",	32, REGGRP_DCRSDR,	EBC_DCR_BASE,	reg_sdr,		hdr_generic		},
{"PPC",  "gpio",32, REGGRP_GPIO,	CFG_PERIPHERAL_BASE, reg_gpio,	hdr_generic		},
{"PPC",  "sdram",32, REGGRP_SDRAM,	SDRAM_DCR_BASE,	reg_sdram,		hdr_generic		},
{"PPC",  "clk",	32, REGGRP_CLK,		CPR0_CFGADDR,	reg_clk,		hdr_generic		},
{"PPC",  "mal",	32, REGGRP_RAW_DCR,	EBC_DCR_BASE,	reg_mal0,		hdr_generic		},
{"PPC",	 "dcr",	32, REGGRP_RAW_DCR,	EBC_DCR_BASE,	reg_dcrnull,	hdr_generic		},
{"PPC",	 "spr",	32, REGGRP_SPR,		0,				reg_spr,		hdr_generic		},
{"PPC",	 "uic",	32, REGGRP_UIC,		0,				reg_uic,		hdr_generic		},
{"NAND", "nand",32, REGGRP_NAND,	CFG_NAND_ADDR,	reg_nand,		hdr_generic		},
{"FPGA", "gpr",	16, REGGRP_FPGA,	CFG_FPGA_BASE,  reg_fpga_gpr,	hdr_fpga_gpr	},
{"FPGA", "blade",16, REGGRP_FPGA,	CFG_FPGA_BASE,	reg_fpga_blade,	hdr_fpga_blade	},
{"FPGA", "psu",	16, REGGRP_FPGA,	CFG_FPGA_BASE,	reg_fpga_psu,	hdr_fpga_psu	},
{"FPGA", "iom",	16, REGGRP_FPGA,	CFG_FPGA_BASE,	reg_fpga_iom,	hdr_fpga_iom	},
{"FPGA", "fan",	16, REGGRP_FPGA,	CFG_FPGA_BASE,	reg_fpga_fan,	hdr_fpga_fan	},
{"FPGA", "kvm",	16, REGGRP_FPGA,	CFG_FPGA_BASE,	reg_fpga_kvm,	hdr_fpga_kvm	},
{"FPGA", "cp",	16, REGGRP_FPGA,	CFG_FPGA_BASE,	reg_fpga_cp,	hdr_fpga_cp		},
{"FPGA", "cmc",	16, REGGRP_FPGA,	CFG_FPGA_BASE,	reg_fpga_cmc,	hdr_fpga_cmc	},
{"FPGA", "wdog",16, REGGRP_FPGA,	CFG_FPGA_BASE,	reg_fpga_wdog,	hdr_fpga_wdog	},
{ 0, 0, 0, 0, 0, 0}	};

/******************************************************************************
*
*	FUNCTION: util_reg()
*	DATE:	11-29-06
*
******************************************************************************/
s32 util_reg(int argc, char *argv[])
{
u32  rc = 0;
u32  index = 0;
u32  i;
char msg[MAX_MSG_LEN];			/*message buffer*/

//	Get any options
mcoptind = 2;	
while( (index = mcgetopt(argc, argv, 
	  "-help -r:2 -w:3 -dump:1 -mm:2")) != MCGETOPT_EOF)
	{
	switch(index) 
		{
		case 0 :		/*-help*/
			printf("\nRegister Groups supported:\n");
			for(i = 0; rlist[i].name != 0; ++i)
				printf("%-6s 0x%08lx \"%s\" \n", 
							rlist[i].subsys, rlist[i].base, rlist[i].name);

			printf("\n");
			return(0);
			break;

		case 1 :		/*-r Read a register*/
			if((rc = reg_read(mcoptarg[0], mcoptarg[1], msg)) != 0)
				{
				printf("%s %s:%s\n", argv[0], argv[1], msg);
				return(rc);
				}
			break;

		case 2 :		/*-w write a register*/
			if((rc = reg_write(mcoptarg[0], mcoptarg[1], mcoptarg[2], msg)) != 0)
				{
				printf("%s %s:%s\n", argv[0], argv[1], msg);
				return(rc);
				}
			break;

		case 3:		/*Dump a supported register group*/
			if(!strcmp(mcoptarg[0], "help"))
				{
				printf("Register groups supported:\n");

				for(i = 0; rlist[i].name != 0; ++i)
					printf("0x%08lx %s\n", rlist[i].base, rlist[i].name );

				printf("\n");
				return(0);
				}

			if((rc = dump_regs(mcoptarg[0], msg)) != 0)
				{
				printf("%s %s:%s\n", argv[0], argv[1], msg);
				return(rc);
				}
			break;

		case 4 :		/*-mm Read and allow modification to a register*/
			if((rc = reg_mod(mcoptarg[0], mcoptarg[1], msg)) != 0)
				{
				printf("%s %s:%s\n", argv[0], argv[1], msg);
				return(rc);
				}
			break;

		default:		/*not good*/
			rc = mcgetopt_error(index, msg, mcoptarg[0]);
			printf("%s %s:%s\n", argv[0], argv[1], msg);
			return(rc);
			break;
		}
	}/*end mcgetopt while*/

return(0);
}/*end FUNCTION: util_reg()*/
	



/*****************************************************************************
*	FUNCTION:	reg_read()
*	Date:	11-29-06
*
*	Read a specified register from a register group. 
*
*	INPUT:  group_name - register group to access.
*			reg_str    - specific reg within gropup.
*			msg		- error message buffer 
*
*	RETURN: 0 = Read was successful
*			non-0 = error locating/parsing register
*****************************************************************************/ 
static u32 reg_read(char *group_name, char *reg_str, char *msg)
{
u32		rc;
u32		gindex	= 0;	/*index pointing to the group within t_reg_groups */
u32		rindex	= 0;	/*index pointing to the register within t_regs */
u32		data	= 0;
u32		reg		= 0;

/*	Get index of register group within t_reg_groups*/ 
if((rc = get_group_index(group_name, &gindex, msg)) != 0)
   return(rc);

/*	Get address of desired register name*/ 
if((rc = get_reg_asc2hex(reg_str, gindex, &reg, &rindex, msg)) != 0)
   return(rc);


/*	
*	The user could have entered a 0x<val> hex string and I have no idea
*   if it is a valid register within the specified group.
*	I do not want to prevent the user from operating on a register that
*	we have not yet added.  You have the gun...fire away.
*/
data = single_read(rlist[gindex].type, reg);
if(16 == rlist[gindex].width)
	printf("Register \"%s\" at offset 0x%04lx = 0x%04lx\n", 
								rlist[gindex].name, reg, data); 
else
	printf("Register \"%s\" at offset 0x%04lx = 0x%08lx\n", 
								rlist[gindex].name, reg, data); 

return(0);
}/*end FUNCTION: reg_read*/



/*****************************************************************************
*	FUNCTION:	reg_write()
*	Date:	11-29-06
*
*	Write a specified register from a selected register group. 
*
*	INPUT:  group_name - register group to access.
*			reg_str    - specific reg within gropup.
*			wdata      - data to write
*			msg		- error message buffer 
*
*	RETURN: 0 = Write was successful
*			non-0 = error locating/parsing register
*****************************************************************************/ 
static u32 reg_write(char *group_name, char *reg_str, char *wdata, char *msg)
{
u32		rc;
u32		gindex = 0;		/*index pointing to the group within t_reg_groups */
u32		rindex = 0;		/*index pointing to the register within t_regs */
u32		data;
u32		reg = 0;
char	*cptr1;

/*	Get index of register group within t_reg_groups*/ 
if((rc = get_group_index(group_name, &gindex, msg)) != 0)
   return(rc);

/*	Get address of desired register name*/ 
if((rc = get_reg_asc2hex(reg_str, gindex, &reg, &rindex, msg)) != 0)
   return(rc);

/*	Make sure we have a vaild hex data value*/
data = (u32)simple_strtol( wdata, &cptr1, 0 );
if( (s32)cptr1 < (strlen(wdata) + (s32)wdata) )
	{	
	sprintf(msg, "ERROR: \"%s\" cannot be converted to int.\n", data);
	return(E_REG_INT_CONVERSION);
	}


/*	
*	The user could have entered a 0x<val> hex string and I have no idea
*   if it is a valid register within the specified group.
*	I do not want to prevent the user from operating on a register that
*	we have not yet added.  You have the gun...fire away.
*/
single_write(rlist[gindex].type, reg, data);
if(rlist[gindex].width == 16)
	printf("Write reg \"%s\" at offset 0x%04lx = 0x%04lx\n", 
									rlist[gindex].name, reg, data); 
else
	printf("Write reg \"%s\" at offset 0x%04lx = 0x%08lx\n", 
									rlist[gindex].name, reg, data); 
return(0);
}/*end FUNCTION: reg_write*/


/*****************************************************************************
*	FUNCTION:	reg_mod()
*	Date:	12-11-06
*
*	Read and display a specified register from a register group.  Then allow
*	the user to modify and write it back.
*
*	INPUT:  group_name - register group to access.
*			reg_str    - specific reg within gropup.
*			msg		- error message buffer 
*
*	RETURN: 0 = Read was successful
*			non-0 = error locating/parsing register
*****************************************************************************/ 
static u32 reg_mod(char *group_name, char *reg_str, char *msg)
{
u32		rc;
u32		gindex = 0;		/*index pointing to the group within t_reg_groups */
u32		rindex = 0;		/*index pointing to the register within t_regs */
u32		data = 0;
u32		reg = 0;
u32		rbytes;
char	*cptr1;
char	buf[100];
extern char console_buffer[];

/*	Get index of register group within t_reg_groups*/ 
if((rc = get_group_index(group_name, &gindex, msg)) != 0)
   return(rc);

/*	Get address of desired register name*/ 
if((rc = get_reg_asc2hex(reg_str, gindex, &reg, &rindex, msg)) != 0)
   return(rc);

do
	{
	/*	Read the register, display to the user, query for a change*/
	data = single_read(rlist[gindex].type, rlist[gindex].regs[rindex].addr);
	printf("Group \"%s\" at 0x%04lx = 0x%08lx", 
				rlist[gindex].name, rlist[gindex].regs[rindex].addr, data); 

	rbytes = readline (" ? 0x");

	/*	exit upon a 'q'*/
	if( ('q' == console_buffer[0]) || ('Q' == console_buffer[0]) )
		{
		printf("\n");
		return(0);
		}

	/*	No change, go to the next register offset*/
	if(0 == rbytes)
		{
		++rindex;
		continue;
		}

	/*	Pre-pend the "0x" char for the user*/
	sprintf(buf, "0x");
	strcat(buf, console_buffer);

	/*	Make sure we have valid hex data*/
	data = (u32)simple_strtol( buf, &cptr1, 0 );
	if( (s32)cptr1 < (strlen(buf) + (s32)buf) )
		{	
		sprintf(msg, "ERROR: \"%s\" cannot be converted to int.\n", buf);
		rc = E_REG_REGMOD_INT_CONVERSION;
		}

	/*	Write the new value out*/
	single_write(rlist[gindex].type, rlist[gindex].regs[rindex].addr, data);
	++rindex;	/*go to next register on the list*/
	}
while(rlist[gindex].regs[rindex].addr);

return(0);
}/*end FUNCTION: reg_mod*/



/*****************************************************************************
*	FUNCTION:	dump_regs()
*	Date:	11-29-06
*
*	Dump all the registers within a specified group. 
*
*	INPUT:  group_name - register group to dump.
*
*	RETURN: 0 = Dump was successful
*			non-0 = error locating/parsing group
*****************************************************************************/ 
u32 dump_regs(char *group_name, char *msg)
{
u32 i;
u32	rc;
u32 data;
u32 first;
u32 gindex = 0;		/*index pointing to the group within t_reg_groups */

/*	Get index of register group within t_reg_groups*/ 
if((rc = get_group_index(group_name, &gindex, msg)) != 0)
   return(rc);


printf("\n%s register list for \"%s\":base 0x%08lx: \n%s\n",
								rlist[gindex].subsys, rlist[gindex].name,  
								rlist[gindex].base, rlist[gindex].hdr_msg);

for(i = 0, first = 1; rlist[gindex].regs[i].name != 0; ++i)
	{
	data = single_read(rlist[gindex].type, rlist[gindex].regs[i].addr);

	if(16 == rlist[gindex].width)
		{/*16 bit register*/
		printf("%-11s 0x%04lx=0x%04lx     ", 
		rlist[gindex].regs[i].name, rlist[gindex].regs[i].addr, data);
		}
	else
		{/*32 bit register*/
		printf("%-7s 0x%04lx=0x%08lx     ", 
		rlist[gindex].regs[i].name, rlist[gindex].regs[i].addr, data);
		}
	
	if(first)
		first = 0;
	else
		{
		printf("\n");
		++first;
		}
	}

printf("\n");
/*	Dump out a text line to help brain dead people count bits, PPC is 0-31*/ 
if(16 == rlist[gindex].width)
	printf("\n15141312 111098 7654 3210\n\n");
else
	printf("\n0123 4567 891011 12131415 - 16171819  20212223 24252627 28293031\n\n");
	

return(0);
}/*end FUNCTION: dump_regs() */


/*****************************************************************************
*	FUNCTION:	single_read()
*	Date:	11-29-06
*
*	Read a single register location.
*
*	INPUT:  type - how to access the register, indexed, memory mapped, etc.
*			reg  - address/offset of register
*
*	RETURN: data = data value read
*****************************************************************************/ 
static u32 single_read(u32 type, u32 reg)
{
u32 data = 0;
unsigned long get_dcr (unsigned short);
	

switch(type)
	{
	case REGGRP_DCREBC:
		mfebc(reg, data);
		break;

	case REGGRP_DCRSDR:
		mfsdr(reg, data);
		break;

	case REGGRP_GPIO:
		data = *(u32*) (CFG_PERIPHERAL_BASE + reg);
		break;

	case REGGRP_SDRAM:
		mfsdram(reg, data);
		break;

	case REGGRP_CLK:
		mfclk(reg, data);
		break;

	case REGGRP_RAW_DCR:
		data = get_dcr (reg);
		break;	

	case REGGRP_FPGA:
		data = *(u16*)(reg);	/*full register address, not just an offset*/
		break;

	case REGGRP_SPR:			/*Special Purpose Registers*/
		data = get_spr(reg);
		break;

	case REGGRP_UIC:			/*Universal Interrupt Controller*/
		data = get_uic(reg);
		break;

	case REGGRP_NAND:
		data = in32(reg);
		break;
	}
return(data);
}/*end FUNCTION: single_read() */

/*****************************************************************************
*	FUNCTION:	single_write()
*	Date:	11-29-06
*
*	Write a single register location.
*
*	INPUT:  type - how to access the register, indexed, memory mapped, etc.
*			reg  - address/offset of register
*			data - data to write
*
*	RETURN: data = data value read
*****************************************************************************/ 
static u32 single_write(u32 type, u32 reg, u32 data)
{
unsigned long set_dcr (unsigned short, unsigned long);

switch(type)
	{
	case REGGRP_DCREBC:
		mtebc(reg, data);
		break;

	case REGGRP_DCRSDR:
		mtsdr(reg, data);
		break;

	case REGGRP_GPIO:
		*(u32*) (CFG_PERIPHERAL_BASE + reg) = data;
		break;

	case REGGRP_SDRAM:
		mtsdram(reg, data);
		break;

	case REGGRP_CLK:
		mtclk(reg, data);
		break;

	case REGGRP_RAW_DCR:
		set_dcr (reg, data);
		break;		

	case REGGRP_FPGA:			/*full register address, not just an offset*/
		*(u16*)(reg) = data;
		break;

	case REGGRP_SPR:			/*Special Purpose Registers*/
		set_spr(reg, data);
		break;

	case REGGRP_UIC:			/*Universal Interrupt Controller*/
		set_uic(reg, data);
		break;

	case REGGRP_NAND:
		out32(reg, data);
		break;
	}
return(0);
}/*end FUNCTION: single_write() */


/*****************************************************************************
*	FUNCTION:	get_group_index()
*	Date:	11-29-06
*
*	Get index of register group within t_reg_groups.  Make sure it is a 
*	supported register group.
*
*	INPUT:  group_name - text string of register group.
*			gindex	- returned index into t_reg_groups struct
*			msg		- error message buffer 
*
*	RETURN: 0 = index found
*			non-0 = group name was not found
*****************************************************************************/ 
static u32 get_group_index(char *group_name, u32 *gindex, char *msg)
{
u32 i;

for(i = 0; rlist[i].name != 0; ++i)
	{
	if( !strcmp(rlist[i].name, group_name) )
		{
		*gindex = i;
		return(0);		/*found it, exit now*/
		}
	}

/*	Never found the register name, error*/
sprintf(msg, "ERROR: unsupported register group \"%s\"\n", group_name);
return(E_REG_UNSUPPORTED_GROUP);
}/*end FUNCTION: get_group_index*/


/*****************************************************************************
*	FUNCTION:	get_reg_asc2hex()
*	Date:	11-29-06
*
*	Convert a text register name into its hex value. 
*
*	INPUT:  reg_str - text string of register name, or 0x<reg> string.
*			reg 	- returned hex address of register
*			rindex  - returned index of register itself, into t_regs
*			gindex  - index into t_reg_groups struct, points to group
*			msg		- error message buffer 
*
*	RETURN:	0 = valid hex value, "reg" and "rindex" are valid
*			non-0 = error during conversion
*****************************************************************************/ 
static u32 get_reg_asc2hex(char *reg_str, u32 gindex, u32 *reg, u32 *rindex, 
																	char *msg)
{
u32 i;
u32 rc = 0;
char	*cptr1;

/*
*	The user can enter the register offset address as a a 0xHex 
*	value, or as a text register name.
*/
if( strncmp(reg_str, "0x", 2) ) 
	{/*reg is a text name*/
	for(i = 0; rlist[gindex].regs[i].name != 0; ++i)
		{
		if( !strcmp(rlist[gindex].regs[i].name, reg_str) )
			{
			*reg = rlist[gindex].regs[i].addr;
			*rindex = i;
			return(0);	/*got it, exit*/
			}
		}
	/* not in the list, error*/
	sprintf(msg, "ERROR: unsupported register name \"%s\"\n", reg_str);
	rc = E_REG_GETREG_UNSUPPORTED_REG;
	}
else
	{/*reg is a hex value, make sure it is valid*/
	*reg = (u32)simple_strtol( reg_str, &cptr1, 0 );
	if( (s32)cptr1 < (strlen(reg_str) + (s32)reg_str) )
		{	
		sprintf(msg, "ERROR: reg \"%s\" cannot be converted to int.\n", reg_str);
		rc = E_REG_GETREG_INT_CONVERSION;
		}
	/*	We still have to find the index of the reg within its table*/
	for(i = 0; rlist[gindex].regs[i].name != 0; ++i)
		{
		if(rlist[gindex].regs[i].addr == *reg)
			{
			*rindex = i;
			return(0);	/*got it, exit*/
			}
		}
	*rindex = 0;		/*force to the beginnig of the list*/
	}

return(rc);
}/*end FUNCTION: get_reg_asc2hex*/


/*****************************************************************************
*	FUNCTION: get_spr()  set_spr()
*	DATE:	6-14-07
*
*	I just want to do simple special purpose register read/writes.  I cannot 
*	get the macros to work.  The register address cannot exceed 10 bits.  In
*	C we have 8,16,32 bit types. For now this will have to work.
*
*****************************************************************************/
static u32 get_spr(u32 reg)
{
switch(reg)
	{
	case 0x00000190: return(mfspr(0x190)); break;	/*IVOR0*/
	case 0x00000191: return(mfspr(0x191)); break;	
	case 0x00000192: return(mfspr(0x192)); break;	
	case 0x00000193: return(mfspr(0x193)); break;	
	case 0x00000194: return(mfspr(0x194)); break;	
	case 0x00000195: return(mfspr(0x195)); break;	
	case 0x00000196: return(mfspr(0x196)); break;	
	case 0x00000197: return(mfspr(0x197)); break;	
	case 0x00000198: return(mfspr(0x198)); break;
	case 0x00000199: return(mfspr(0x199)); break;	
	case 0x0000019a: return(mfspr(0x19a)); break;	
	case 0x0000019b: return(mfspr(0x19b)); break;	
	case 0x0000019c: return(mfspr(0x19c)); break;	
	case 0x0000019d: return(mfspr(0x19d)); break;	
	case 0x0000019e: return(mfspr(0x19e)); break;	
	case 0x0000019f: return(mfspr(0x19f)); break;	/*IVOR15*/
	case 0x0000003f: return(mfspr(0x03f)); break;	/*IVPR*/
	}

return(0);	/*should never get here*/
}/*end FUNCTION: get_spr()*/

static u32 set_spr(u32 reg, u32 data)
{
switch(reg)
	{
	case 0x00000190: mtspr(0x190, data); break;	/*IVOR0*/
	case 0x00000191: mtspr(0x191, data); break;	
	case 0x00000192: mtspr(0x192, data); break;	
	case 0x00000193: mtspr(0x193, data); break;	
	case 0x00000194: mtspr(0x194, data); break;	
	case 0x00000195: mtspr(0x195, data); break;	
	case 0x00000196: mtspr(0x196, data); break;	
	case 0x00000197: mtspr(0x197, data); break;	
	case 0x00000198: mtspr(0x198, data); break;
	case 0x00000199: mtspr(0x199, data); break;	
	case 0x0000019a: mtspr(0x19a, data); break;	
	case 0x0000019b: mtspr(0x19b, data); break;	
	case 0x0000019c: mtspr(0x19c, data); break;	
	case 0x0000019d: mtspr(0x19d, data); break;	
	case 0x0000019e: mtspr(0x19e, data); break;	
	case 0x0000019f: mtspr(0x19f, data); break;	/*IVOR15*/
	case 0x0000003f: mtspr(0x03f, data); break;	/*IVPR*/
	}

return(0);
}/*end FUNCTION: set_spr()*/

				 
/*****************************************************************************
*	FUNCTION: get_uic()  set_uic()
*	DATE:	6-14-07
*
*	I just want to do simple UIC register read/writes.  I cannot 
*	get the macros to work.
*
*****************************************************************************/
static u32 get_uic(u32 reg)
{

switch(reg)
	{
	/*UIC0*/
	case 0x00000c0: return(mfdcr(0xc0)); break;	/*Status register*/
	case 0x00000c2: return(mfdcr(0xc2)); break;	/*Enable register*/
	case 0x00000c3: return(mfdcr(0xc3)); break;	/*Critical register*/
	case 0x00000c4: return(mfdcr(0xc4)); break; /*Polarity register*/
	case 0x00000c5: return(mfdcr(0xc5)); break; /*Triggering register*/
	case 0x00000c6: return(mfdcr(0xc6)); break; /*Masked Statue register*/
	case 0x00000c7: return(mfdcr(0xc7)); break; /*Vector register*/

	/*UIC1*/
	case 0x00000d0: return(mfdcr(0xd0)); break;	/*Status register*/
	case 0x00000d2: return(mfdcr(0xd2)); break;	/*Enable register*/
	case 0x00000d3: return(mfdcr(0xd3)); break;	/*Critical register*/
	case 0x00000d4: return(mfdcr(0xd4)); break; /*Polarity register*/
	case 0x00000d5: return(mfdcr(0xd5)); break; /*Triggering register*/
	case 0x00000d6: return(mfdcr(0xd6)); break; /*Masked Statue register*/
	case 0x00000d7: return(mfdcr(0xd7)); break; /*Vector register*/

	/*UIC2*/
	case 0x00000e0: return(mfdcr(0xe0)); break;	/*Status register*/
	case 0x00000e2: return(mfdcr(0xe2)); break;	/*Enable register*/
	case 0x00000e3: return(mfdcr(0xe3)); break;	/*Critical register*/
	case 0x00000e4: return(mfdcr(0xe4)); break; /*Polarity register*/
	case 0x00000e5: return(mfdcr(0xe5)); break; /*Triggering register*/
	case 0x00000e6: return(mfdcr(0xe6)); break; /*Masked Statue register*/
	case 0x00000e7: return(mfdcr(0xe7)); break; /*Vector register*/
	}

return(0);	/*should never get here*/
}/*end FUNCTION: get_uic()*/


static u32 set_uic(u32 reg, u32 data)
{
switch(reg)
	{
	/*UIC0*/
	case 0x00000c0: mtdcr(0xc0, data); break;	/*Status register*/
	case 0x00000c2: mtdcr(0xc2, data); break;	/*enable register*/
	case 0x00000c3: mtdcr(0xc3, data); break;	/*Critical register*/
	case 0x00000c4: mtdcr(0xc4, data); break;	/*Polarity register*/
	case 0x00000c5: mtdcr(0xc5, data); break;	/*Triggering register*/
	case 0x00000c6: mtdcr(0xc6, data); break;	/*Masked Statue register*/
	case 0x00000c7: mtdcr(0xc7, data); break;	/*Vector register*/
	/*UIC1*/
	case 0x00000d0: mtdcr(0xd0, data); break;	/*Status register*/
	case 0x00000d2: mtdcr(0xd2, data); break;	/*enable register*/
	case 0x00000d3: mtdcr(0xd3, data); break;	/*Critical register*/
	case 0x00000d4: mtdcr(0xd4, data); break;	/*Polarity register*/
	case 0x00000d5: mtdcr(0xd5, data); break;	/*Triggering register*/
	case 0x00000d6: mtdcr(0xd6, data); break;	/*Masked Statue register*/
	case 0x00000d7: mtdcr(0xd7, data); break;	/*Vector register*/
	/*UIC2*/
	case 0x00000e0: mtdcr(0xe0, data); break;	/*Status register*/
	case 0x00000e2: mtdcr(0xe2, data); break;	/*enable register*/
	case 0x00000e3: mtdcr(0xe3, data); break;	/*Critical register*/
	case 0x00000e4: mtdcr(0xe4, data); break;	/*Polarity register*/
	case 0x00000e5: mtdcr(0xe5, data); break;	/*Triggering register*/
	case 0x00000e6: mtdcr(0xe6, data); break;	/*Masked Statue register*/
	case 0x00000e7: mtdcr(0xe7, data); break;	/*Vector register*/
	}

return(0);
}/*end FUNCTION: set_uic()*/



	
#endif /*CONFIG_CMC_10G*/


