/****************************************************************************

  Copyright (c) 2006, Dell Inc
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

  - Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
    
  - Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution. 

  - Neither the name of Dell Inc nor the names of its contributors
    may be used to endorse or promote products derived from this software 
    without specific prior written permission. 

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. 

*****************************************************************************/


/*
 * Utility support
 */
#include <common.h>
#include <command.h>
#include <asm/cache.h>

#ifdef CONFIG_CMC_10G	/*only compile in for a CMC_10G build*/


/******************************************************************************
*
*	FILE: util_reset.c
*
******************************************************************************/


/******************************************************************************
*  Include files
******************************************************************************/
#include <util_defs.h>
#include <util_fpga.h>
#include <util_ret_codes.h>


/******************************************************************************
*  Local Defines
******************************************************************************/



/******************************************************************************
*  Globals
******************************************************************************/


/******************************************************************************
*  Prototypes/Externs
******************************************************************************/
extern s32  mcoptind;
extern char	*mcoptarg[];

static s32 cause_fpga_wdog(char *msg);


/******************************************************************************
*	List all structures defines
******************************************************************************/
t_options util_reset_options[] = 
	{
	/*Options*/
	{"core",	"Reset PPC core(NOT recomended)"},
	{"chip",	"Reset PPC chip(NOT recomended)"},
	{"system",	"Reset CMC board via CPLD"},
	{"wdog",	"FPGA watch dog"},
	{"dbsr",	"Display the debug status register"},

	{ NULL, NULL }
	};


t_usage util_reset_usage = { 
	"reset", 
					
	/*Syntax*/
	"reset  <-core | -chip> | -wdog | -system>\n",

	/*Description*/
	"With no options the default is a \"board\" reset via the FPGA watch dog.\n"
	"Force a PowerPC core reset or a PowerPC internal chip reset.\n"
	"The -wdog is a way to cause a FPGA watch dog.  The -system resets the CMC\n"
	"board via the CPLD.\n", 
	
	util_reset_options
	}; /*end t_usage util_reset_usage*/


/******************************************************************************
*
*	FUNCTION: util_reset()
*	DATE:	2-20-07
*
*	NOTE: PowerPC system reset is not supported with CMC HW.
*
******************************************************************************/
s32 util_reset(int argc, char *argv[])
{
u32  index = 0;
s32  rc = 0;
u32  dbg_status = 0;
char msg[MAX_MSG_LEN];			/*message buffer*/


//	Get any options
mcoptind = 2;	
while( (index = mcgetopt(argc, argv, "-core -chip -system -dbsr -wdog")) != MCGETOPT_EOF)
	{
	switch(index) 
		{
		case 0 :		/*core */			
			reset_core();
			break;

		case 1 :		/*chip */
			reset_chip();
			break;

		case 2 :		/*system */
			reset_system();
			break;

		case 3 :		/*status */
			dbg_status = mfspr(dbsr);
			printf("Debug Status Register = 0x%08lx\n", dbg_status);
			return(0);
			break;

		case 4 :		/*wdog (not a forced)*/
			if( (rc = cause_fpga_wdog(msg)) != 0)
				{
				printf("%s %s:%s\n", argv[0], argv[1], msg);
				return(rc);
				}
			break;

		default:		/*not good*/
			rc = mcgetopt_error(index, msg, mcoptarg[0]);
			printf("%s %s:%s\n", argv[0], argv[1], msg);
			return(rc);
			break;
		}
	}/*end mcgetopt while*/


/*	Default to a forced wdog via the FPGA resetting the CMC board*/
if( (rc = reset_board(msg)) != 0)
	{
	printf("%s %s:%s\n", argv[0], argv[1], msg);
	return(rc);
	}

return(0);
}/* end FUNCTION: util_reset()*/

/****************************************************************************
*	Reset functions
****************************************************************************/
void reset_core(void)
{
mtspr (dbcr0, DBCR0_RESET_CORE);
while(1)
	;
}


void reset_chip(void)
{
mtspr (dbcr0, DBCR0_RESET_CHIP);
while(1)
	;
}


/*	
*	If the CPLD is < rev 4, we should NOT issue a system reset causing the PPC 
*	to drive this line. HOWEVER, if the FPGA fails to load we cannot read the 
*	CPLD rev.  The corner case of wanting to issue a system reset IS because 
*	the FPGA is not loaded and we cannot do a board_reset().  For ST level 
*	CMC's and later the CPLD will be > 4, and all is ok.
*
*	We are going to address the end result and NOT the x01/x02 proto's.	
*/
void reset_system(void)
{
mtspr (dbcr0, DBCR0_RESET_SYSTEM);
while(1)
	;
}




/*	Cause a FPGA watch dog reset, set the force bit, this is a board reset*/
s32 reset_board(char *msg)
{

if( ABS_read_fpga(FPGA_REG_ID) == FPGA_ID)
	{
	/*	Indicate it is a forced wdog*/
	set_mode_bits(MODE_FPGA_FORCE_WDOG);

	/*	Load one tick and start the timer*/
	ABS_write_fpga(FPGA_REG_WDT_DUR, 1);	
	ABS_write_fpga(FPGA_REG_WDT_CTRL, FPGA_WDT_ARM_DISABLE);

	/*
	*	Wait 1sec, then fall to 2nd choice, system reset via CPLD.
	*	This sill only be the case if the FPGA fails for some strange reason.
	*	The CPLD rev must be > 4.
	*/
	udelay(1000000); 

	reset_system();
	}

sprintf(msg, "FPGA ID failure, cannot force a WDOG reset");
return(E_FPGA_FORCE_RESET_FAILURE);
}


/*	Cause a FPGA watch dog reset but do not set the force bit*/
static s32 cause_fpga_wdog(char *msg)
{

if( ABS_read_fpga(FPGA_REG_ID) == FPGA_ID)
	{
	/*	Load one tick and start the timer*/
	ABS_write_fpga(FPGA_REG_WDT_DUR, 1);	
	ABS_write_fpga(FPGA_REG_WDT_CTRL, FPGA_WDT_ARM_DISABLE);
	while(1)
		;	
	}

sprintf(msg, "FPGA ID failure, cannot cause a WDOG reset");
return(E_FPGA_FORCE_RESET_FAILURE);
}/* end FUNCTION: cause_fpga_wdog() */



/******************************************************************************
*	DF144413 After recover or fwupdate we did a reset_core().  This reset does
*	NOT reset the NOR flash chip, change to board reset.
******************************************************************************/
void reset_after_fwupd(void)
{
udelay(1000000);

/*	Cause board reset via FPGA*/
if( ABS_read_fpga(FPGA_REG_ID) == FPGA_ID)
	{
	/*	Indicate it is a forced wdog*/
	set_mode_bits(MODE_FPGA_FORCE_WDOG);

	/*	Load one tick and start the timer*/
	ABS_write_fpga(FPGA_REG_WDT_DUR, 1);	
	ABS_write_fpga(FPGA_REG_WDT_CTRL, FPGA_WDT_ARM_DISABLE);

	udelay(1000000);					/*should reset now*/
	}

/*
*	Drop here IF the FPGA is not loaded, last choice is a system reset.
*	(after CPLD rev 4 this will also be a board reset)
*	DF144413 A core reset does NOT reset the DRAM controller and was causing a 
*	problem in fwupdate.
*/
reset_system();
}/*end FUNCTION: reset_after_fwupd()*/
#endif /*CONFIG_CMC_10G*/



