/****************************************************************************

  Copyright (c) 2006, Dell Inc
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

  - Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
    
  - Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution. 

  - Neither the name of Dell Inc nor the names of its contributors
    may be used to endorse or promote products derived from this software 
    without specific prior written permission. 

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. 

*****************************************************************************/

/*
 * Utility support
 */
#include <common.h>		/*must be first in file*/
#include <command.h>
#include <asm/processor.h>
#include <ppc4xx.h>

#ifdef CONFIG_CMC_10G	/*only compile in for a CMC_10G build*/

/******************************************************************************
*
*	FILE: util_script.c
*	This command supports options to allow the user to run a serices or 
*   a script of u-boot commands.
*
******************************************************************************/


/******************************************************************************
*  Include files
******************************************************************************/
#include <util_defs.h>
#include <util_ret_codes.h>


/******************************************************************************
*  Local Defines
******************************************************************************/


/******************************************************************************
*  Globals
******************************************************************************/


/******************************************************************************
*  Prototypes
******************************************************************************/
extern s32	mcoptind;
extern char	*mcoptarg[];

extern int run_command(const char *cmd, int flag);

static u32 util_run_builtin(char *cmd_string, u32 lpcnt, char *msg);


/******************************************************************************
*	List all structures defines
******************************************************************************/

/******************************************************************************
*	List all structures defines
******************************************************************************/
t_options util_script_options[] = 
	{
	//Options
	{"help",	"Display all of the available script names" },
	{"list",	"List/display all of the lines in the specified script" },
	{"run",		"run a specified scripted script" },
	{ "l",	    "Number of times to loop on the script\n"},
	
	{ NULL, NULL }
	};


t_usage util_script_usage = {  
	"script", 
					
	/*Syntax*/
	"script [-help]\n"
	"       [-list <script name>]\n"
	"       [-run  <script name>]\n"
	"       [-l    <loopcount>]",

	/*Description*/
	"Run a list of commands that are built in.\n"
	"Examples:\n"
	"util script -help\n"
	"util script -list burn_test\n"
	"util script -run burn_test -l -1\n",
	
	util_script_options
	}; //end t_usage util_script_usage    

/******************************************************************************
*  built-in script lists
******************************************************************************/
typedef struct {					
	char *name;					/*name of the script*/
	char **script_p;			/*pointer to the script*/
	} t_util_script;




char *quick_test[] = 
	{
	"util fpga -load",
	"util flash -erase -s 0:10",
	"util flash -erase -b 0:10",

	NULL
	};

char *test[] = 
	{
	"util fpga -load",
	"util flash -erase -s 0:10",
	"util flash -erase -b 0:10",

	NULL
	};

char *burn_test[] = 
	{
	"util fpga -load -l 100",
	"util flash -erase -s 0:10",
	"util flash -b 0:1023 -erase -verify",
	"util flash -b 0:1023 -pat 0x55555555",
	"util flash -b 0:1023 -rand 1",

	NULL
	};

t_util_script built_in_scripts[] ={
	{ "quick_test",		quick_test },
	{ "test",			test },
	{ "burn_test",		burn_test },
	{ NULL,		NULL } };



/******************************************************************************
*
*	FUNCTION: util_script()
*	DATE:	1-15-07
*
******************************************************************************/
s32 util_script(int argc, char *argv[])
{
u32  	i, j, rc, index;
u32		runflag = 0, listflag = 0, helpflag = 0;
char	msg[MAX_MSG_LEN];				/*misc message buffer*/
char	*cptr1;
char	cmd_string[MAX_MSG_LEN];
u32  	lpcnt = 1;


//	Get any options
mcoptind = 2;	
while( (index = mcgetopt(argc, argv, "-run:1 -list:1 -help -l:1")) != MCGETOPT_EOF)
	{
	switch(index) {
		case 0 :		// run a specified script list
			runflag = 1;
			strcpy(cmd_string, mcoptarg[0]);
			break;

		case 1 :		// list a specified script
			listflag = 1;
			strcpy(cmd_string, mcoptarg[0]);
			break;

		case 2 :		// show all of the available scripts
			helpflag = 1;
			break;

		case 3 :		// loop count
			lpcnt = (u32)simple_strtol( mcoptarg[0], &cptr1, 0 );

			if( (s32)cptr1 < (strlen(mcoptarg[0]) + (s32)mcoptarg[0]) ) 
				{	
				printf("%s %s:ERROR: -l argument \"%s\" cannot be converted "
						"to int.\n", argv[0], argv[1], mcoptarg[0]);
				return(E_OPTARG_INT_CONVERSION);
				}
			break;

		default:		//not good
			rc = mcgetopt_error(index, msg, mcoptarg[0]);
			printf("%s %s:%s\n", argv[0], argv[1], msg);
			return(rc);
			break;
	}
}//end mcgetopt while

//	User friendly check
if( (helpflag == 0) && (listflag == 0) && (runflag == 0) )
	{
	printf("%s %s:ERROR: must have -help | -list | -run option\n", 
															argv[0], argv[1]);
	return(E_SCRIPT_MISSING_OPTION);
	}

/*****************************************************************************
*	Help will display the available script names
*****************************************************************************/
if(helpflag)
	{
	printf("\n\nAvailable built-in scripts:\n");
	for(i = 0; built_in_scripts[i].name != NULL; ++i)
		printf("%s\n", built_in_scripts[i].name);

	printf("\n");
	return(0);
	}

/*****************************************************************************
*	List will display all of the commands in a specified lists.
*****************************************************************************/
if(listflag)
	{
	for(i = 0; built_in_scripts[i].name != NULL; ++i)
		{
		if( !strcmp(cmd_string, built_in_scripts[i].name))
			{
			printf("\nCommand list for script \"%s\":\n", cmd_string);
			for(j = 0;  built_in_scripts[i].script_p[j] != NULL; ++j)
				printf("%s \n", built_in_scripts[i].script_p[j]);

			printf("\n");
			return(0);
			}
		}

	printf("\n%s %s:ERROR: cannot find script \"%s\" \n", 
											argv[0], argv[1], cmd_string);
	return(E_SCRIPT_NOT_FOUND);
	}


/*****************************************************************************
*	Run a specified command list.
*****************************************************************************/
if(runflag)
	{
	rc = util_run_builtin(cmd_string, lpcnt, msg);
	printf("%s %s:%s\n", argv[0], argv[1], msg);
	return(rc);
	}

return(0);
}//end FUNCTION: util_run()




/*****************************************************************************
*	FUNCTION:	util_run_buildin()
*	DATE:	1-15-07
*
*****************************************************************************/
static u32 util_run_builtin(char *cmd_string, u32 lpcnt, char *msg)
{
u32	i, j, rc;
u32	cur_loop = 1;

for(i = 0; built_in_scripts[i].name != NULL; ++i)
	{
	if( !strcmp(cmd_string, built_in_scripts[i].name))
		break;	// i is the offset into the script table
	}

//	Script name is not found
if(built_in_scripts[i].name == NULL)
	{
	sprintf(msg, "\nERROR: cannot find script \"%s\" \n", cmd_string);
	return(E_SCRIPT_NOT_FOUND);
	}


printf("\n");
while(	(cur_loop <= lpcnt) || (lpcnt == -1) )
	{
	printf("Script execution: \"%s\"...loop %d\n", 
										built_in_scripts[i].name, cur_loop);

	for(j = 0;  built_in_scripts[i].script_p[j] != NULL; ++j)
		{
		//	Go execute he command line
		rc = run_command( built_in_scripts[i].script_p[j], 0 );
printf("HACK rc=0x%x\n", rc);
		if(rc == EXIT_BY_CTRLC)
			{
			sprintf(msg, 
				"Script execution: \"%s\"...loop %d, terminated by CTRL-C\n",
				built_in_scripts[i].name, cur_loop);
			return(EXIT_BY_CTRLC);
			}

		if( rc < 0 )
			{	
			//	Command line failed, get out
			sprintf(msg, "Script terminated:ERROR: rc=0x%x\n", rc);
			return(rc);
			}
		}

       ++cur_loop;
	}//end looping while

return(0);
}//end FUNCTION: util_run_builtin()


#endif /*CONFIG_CMC_10G*/




