/****************************************************************************

  Copyright (c) 2006, Dell Inc
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

  - Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
    
  - Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution. 

  - Neither the name of Dell Inc nor the names of its contributors
    may be used to endorse or promote products derived from this software 
    without specific prior written permission. 

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. 

*****************************************************************************/



#include <common.h>		/*must be first in file*/
#include <command.h>
#include <linux/ctype.h>

#ifdef CONFIG_CMC_10G	/*only compile in for a CMC build*/
/******************************************************************************
*
*	FILE: util_support.c
*
*	This file contains functions used to support various utility commands. 
*
******************************************************************************/


/******************************************************************************
*  Include files
******************************************************************************/
#include <util_defs.h>
#include <util_ret_codes.h>
#include <util_fpga.h>
#include <ppc440.h>

/******************************************************************************
*  Local Defines
******************************************************************************/
DECLARE_GLOBAL_DATA_PTR;

#define STR_MAX_LEN_IP 16 

/******************************************************************************
*  Globals
******************************************************************************/

/******************************************************************************
*  Prototypes/Externs
******************************************************************************/
extern u32	 g_post_rc;
extern int i2c_read(uchar chip, uint addr, int alen, uchar *buffer, int len);

/******************************************************************************
*	List all structures defines
******************************************************************************/




/****************************************************************************
*	FUNCTION: calc_tstamp_str()
*	DATE:	  9-20-06
*
*	Return a string of sec:millsec, min:sec:millisec, or hr:min:sec:millsec
*	in the form 00:000, 00:00:000, or 00:00:00:000 respectivly.
*
*	INPUT:	base -	Passing in a 0 will generate a string from the beginning
*					of u-boot time.  A non-0 value will generate a string of
*					elapsed time with respect of a previously saved "base".
*			str -	A char array of MAX_TSTAMP_STR_LEN where the string will
*					be copied.  
*	RETURN:	Newly read timestamp in milliseconds.
*
*****************************************************************************/
u32 calc_tstamp_str(u32 base, char *str)
{
u32  current, milli, sec, min, hr;
char tstamp_str[MAX_TSTAMP_STR_LEN];

current	= ABS_get_timer(0);
milli	= current - base;			/*elapsed time*/
sec   = milli / 1000;
milli = milli % 1000;	
min   = sec / 60;
sec	  = sec % 60;
hr	  = min / 60;
min   = min % 60;

if( (hr == 0) && (min == 0) )
	sprintf(tstamp_str, "%.2d:%.3d", sec, milli);
else if(hr == 0)
	sprintf(tstamp_str, "%.2d:%.2d:%.3d", min, sec, milli);
else
	sprintf(tstamp_str, "%.2d:%.2d:%.2d:%.3d", hr, min, sec, milli);
	
strcpy(str, tstamp_str);
return(current);
}//end FUNCTION: calc_tstamp_str()



/****************************************************************************
*
*	FUNCTION: get_file_addr_and_size()
*	DATE:	10-18-06
*
*	Get the env variable "fileaddr" and "filesize" and convert to hex.
*	These two vars will be creatd after a tftp of a file.
*
*	INPUT:	addr_p - Location to put the mem addr where "fileaddr" points
*			size_p - Location to put the size of "filesize"
*			msg     - Buffer used to return any messages.
*
*	RETURN:	0 - Both values were found and converted.
*			non-0 - error retreving or converting the values
*
*****************************************************************************/
s32 get_file_addr_and_size(u32 *addr_p, u32 *size_p, char *msg)
{
char *s;
char *cptr1;

/*	Get the starting memory address of a downloaded binary*/
if((s = (char*)getenv("fileaddr")) == 0)
	{
	*addr_p = CFG_LOAD_ADDR;	/*serial download does not set fileaddr*/
	}
else
	{
	*addr_p = (u32)simple_strtol( s, &cptr1, 16 ); /*u-boot does not prepend 0x*/
	if( (s32)cptr1 < (strlen(s) + (s32)s) )
		{	
		sprintf(msg, "\"fileaddr\" cannot be converted to int.\n");
		return(1);
		}
	}


/*	Get the size in bytesa of a downloaded binary*/
if((s = (char*)getenv("filesize")) == 0)
	{
	sprintf(msg, "\"filesize\" does not exists, nothing downloaded\n");
	return(1);
	}

*size_p = (u32)simple_strtol( s, &cptr1, 16 ); /*u-boot does not prepend 0x*/
if( (s32)cptr1 < (strlen(s) + (s32)s) ) 
	{	
	sprintf(msg, "\"filesize\" cannot be converted to int.\n");
	return(1);
	}

return(0);
}/*end FUNCTION: get_file_addr_and_size()*/


/*****************************************************************************
*  Pseudo-random number generation 
*****************************************************************************/    
static u32 next32bit = 1;
static u16 next16bit = 1;

u32 rand32(void)
{
u32 tmp, tmp1;

next32bit = next32bit * 1103515245 + 12345;
tmp = (next32bit/65536) % 32768;

next32bit = next32bit * 1103515245 + 12345;
tmp1 = (next32bit/65536) % 32768;
return( ((tmp << 16) | tmp1) );
}

u16 rand16(void)
{
next16bit = next16bit * 1103515245 + 12345;
return((next16bit/65536) % 32768);
}

void srand32(u32 seed)
{
next32bit = seed;
}

void srand16(u16 seed)
{
next16bit = seed;
}



/*****************************************************************************
*	Get 64 bits from the GPIO Input Register
*****************************************************************************/

#if 0
uint64_t get_gpio_ir(void)
{
uint64_t gpio64 = in32(GPIO0_IR);
gpio64  = gpio64 << 32;
gpio64 |= in32(GPIO1_IR);
return(gpio64);
}/*end FUNCTION: get_gpio_ir()*/
#endif

/*****************************************************************************
*	Set bits in the 64 bit GPIO Output Register
*****************************************************************************/
s32 sethigh_gpio_or(u32 gpio_bank, u32 data)
{
u32 tmp;
if(gpio_bank == GPIO_BANK1)
	{
	tmp = in32(GPIO1_OR);
	out32(GPIO1_OR, tmp | data);
	}
else
	{
	tmp = in32(GPIO0_OR);
	out32(GPIO0_OR, tmp | data);
	}
return(0);
}/*end FUNCTION: sethigh_gpio_or()*/

/*****************************************************************************
*	Get the GPIO IR register
*****************************************************************************/
u32 get_gpio_ir(u32 gpio_bank)
{
if(gpio_bank)
	return(in32(GPIO1_IR));
else
	return(in32(GPIO0_IR));
}/*end FUNCTION: get_gpio_ir()*/

/*****************************************************************************
*       Get the GPIO OR register 
*****************************************************************************/
u32 get_gpio_or(u32 gpio_bank)
{
if(gpio_bank)   
        return(in32(GPIO1_OR));
else
        return(in32(GPIO0_OR));   
}/*end FUNCTION: get_gpio_or()*/



/*****************************************************************************
*	Set bits in the 64 bit GPIO Output Register
*****************************************************************************/
s32 clearlow_gpio_or(u32 gpio_bank, u32 data)
{
u32 tmp;

if(gpio_bank == GPIO_BANK1)
	{
	tmp = in32(GPIO1_OR);
	out32(GPIO1_OR, tmp & ~data);
	}
else
	{
	tmp = in32(GPIO0_OR);
	out32(GPIO0_OR, tmp & ~data);
	}
return(0);
}/*end FUNCTION: clearlow_gpio_or()*/


s32 is_hard_vendor_mode(void)
	{	
	return( (get_gpio_ir(GPIO_BANK1) & GPIO_MFG_MODE_N) ? 0 : 1 );  
	}

s32 is_hard_recover_mode(void)
	{	return( (get_gpio_ir(GPIO_BANK1) & GPIO_FLASH_RECOVER_N) ? 0 : 1 );  }

s32 is_hard_password_mode(void)
	{	return( (get_gpio_ir(GPIO_BANK0) & GPIO_PWORD_N) ? 0 : 1 );  }


s32 is_soft_vendor_mode(void)
{
if( ABS_read_fpga(FPGA_REG_ID) == FPGA_ID)
	{
	if( ABS_read_fpga(FPGA_REG_MODE0) & MODE_VENDOR_SOFT)
		return(1);
	}

return(0);			/*soft vendor mode not set*/
}


s32 is_fpga_vendor_mode(void)
{
if( ABS_read_fpga(FPGA_REG_ID) == FPGA_ID)
	{
	if( !(ABS_read_fpga(FPGA_REG_VENDOR_MODE_N) & FPGA_VENDOR_MODE_N) )
		return(1);
	}

return(0);			/*soft vendor mode not set*/
}

s32 is_soft_recover_mode(void)
{
if( ABS_read_fpga(FPGA_REG_ID) == FPGA_ID)
	{
	if( ABS_read_fpga(FPGA_REG_MODE0) & MODE_RECOVER_SOFT)
		return(1);
	}

return(0);			/*soft vendor mode not set*/
}

s32 is_fwupd_req_from_peer(void)
{
return(is_soft_recover_mode());
}/*end FUNCTION: is_fwupd_req_from_peer()*/

s32 clear_fwupd_req_from_peer(void)
{
clear_mode_bits(MODE_RECOVER_SOFT);  
return(0);
}/*end FUNCTION: clear_fwupd_req_from_peer()*/


s32 is_recover_mode(void)
{
if(is_hard_recover_mode())		
	return(1);				/*no need to look at the soft bit*/

if(is_soft_recover_mode())		
	return(1);	
	
return(0);  
}/*end FUNCTION: is_recover_mode()*/


s32 is_fwupd_mode_set(void)
{
if( ABS_read_fpga(FPGA_REG_ID) == FPGA_ID)
	{
	if( ABS_read_fpga(FPGA_REG_MODE0) & MODE_FWUPD_REQUEST)
		return(1);
	}

return(0);
}


s32 clear_fwupd_mode(void)
{
u16 saved_mode0;

if( ABS_read_fpga(FPGA_REG_ID) == FPGA_ID)
	{
	saved_mode0 = ABS_read_fpga(FPGA_REG_MODE0);
	ABS_write_fpga(FPGA_REG_MODE0, (saved_mode0 & ~MODE_FWUPD_REQUEST) );
	}

return(0);	
}


s32 is_vendor_mode(void)
{
if(is_hard_vendor_mode())		
	return(1);				/*no need to look at the soft bit*/

if(is_soft_vendor_mode())		
	return(1);	

if(is_fpga_vendor_mode())
	return(1);	
	
return(0);  
}/*end FUNCTION: is_vendor_mode()*/




s32 is_fpga_done(void)
	{	return( (get_gpio_ir(GPIO_BANK0) & GPIO_PPC_GPIO_DONE) ? 1 : 0 );  }

s32 is_fpga_init_b(void)
	{	return( (get_gpio_ir(GPIO_BANK0) & GPIO_PPC_GPIO_INIT_B) ? 1 : 0 );  }

s32 is_fpga_busy(void)
	{	return( (get_gpio_ir(GPIO_BANK0) & PPC_GPIO_BUSY) ? 1 : 0 );  }


s32 fpga_program_enable(void)
	{	return(	sethigh_gpio_or(GPIO_BANK1, PPC_GPIO_PROG_B) ); }

s32 fpga_program_disable(void)
	{	return(	clearlow_gpio_or(GPIO_BANK1, PPC_GPIO_PROG_B) ); }

s32 get_cmc_rev(void)
	{	
	s32 rev, tmp;
	tmp  = get_gpio_ir(GPIO_BANK0);
	rev  =  (tmp & GPIO_CMC_REV0) >> CMC_REV0_SHIFTCNT;
	rev  |= (tmp & GPIO_CMC_REV1) >> CMC_REV1_SHIFTCNT;
	rev  |= (tmp & GPIO_CMC_REV2) >> CMC_REV2_SHIFTCNT;
	return(rev);
	}

s32 get_micrel_en(void)              
{                       
        s32 en;
        en = get_gpio_or(GPIO_BANK1);
        return (en&GPIO_PPC_MICREL_EN);
}
s32 set_micrel_en(u32 dir)
{
        if(dir!=0) {

                sethigh_gpio_or(GPIO_BANK1,GPIO_PPC_MICREL_EN);
        } else {
                clearlow_gpio_or(GPIO_BANK1,GPIO_PPC_MICREL_EN);
        }
        return 0;
}
s32 set_switch_reset(void)
{
	clearlow_gpio_or(GPIO_BANK1,GPIO_SWITCH_RESET_N);
	return 0;
}
s32 clear_switch_reset(void)
{
	sethigh_gpio_or(GPIO_BANK1,GPIO_SWITCH_RESET_N);
	return 0;
}

s32 is_cmc_buf_en(void)
	{	return( (get_gpio_ir(GPIO_BANK0) & GPIO_CMC_BUF_EN) ? 1 : 0 );  }

s32 is_local_active(void)
	{	
	return( (in32(GPIO0_OR) & GPIO_CMCL_CMCF_ACTIVE) ? 1 : 0 );  
	}

s32 is_foreign_active(void)
	{	
	return( (get_gpio_ir(GPIO_BANK0) & GPIO_CMCF_CMCL_ACTIVE) ? 1 : 0 );  
	}


s32 set_local_active(void)
	{	
	sethigh_gpio_or(GPIO_BANK0, (GPIO_LED_BLUE | GPIO_CMCL_CMCF_ACTIVE) );
	return(0); 
	}

s32 clear_local_active(void)
	{	
	clearlow_gpio_or(GPIO_BANK0, (GPIO_LED_BLUE | GPIO_CMCL_CMCF_ACTIVE) );
	return(0); 
	}

s32 set_cmc_buf_en(void)
	{	return(	sethigh_gpio_or(GPIO_BANK0, GPIO_CMC_BUF_EN) ); }

s32 clear_cmc_buf_en(void)
	{	return( clearlow_gpio_or(GPIO_BANK0, GPIO_CMC_BUF_EN) ); }


s32 set_mode_bits(u16 mode)
{
u16 current;

current = ABS_read_fpga(FPGA_REG_MODE0);	
ABS_write_fpga(FPGA_REG_MODE0, current | mode);	
return(0);
}

s32 clear_mode_bits(u16 mode)
{
u16 current;

current = ABS_read_fpga(FPGA_REG_MODE0);	
ABS_write_fpga(FPGA_REG_MODE0, current & ~mode);	
return(0);
}



/****************************************************************************
*
*	FUNCTION: get_prompt(char *str, u32 max_len)
*	DATE:	2-1-07
*
*	The command prompt for the CMC u-boot is dynamic so a #define will not
*	provide all of the necessary information.  The following information is 
*	used to determing what the prompt is. 
*
*	Read the FPGA slot Id register to determine slot 1 or 2.
*	Read the FPGA ID register. If the FPGA did not load we cannot determine
*	  the slot, however we will be in recover as this is a selftest failure.
*	Read the necessary GPIO to determine if this CMC is active/passive.
*	From the mode determine if we are in recover.  
*
*	The following example shows valid prompts for CMC 1.
*	cmc1[act]=>			CMC is in slot 1 and is active
*	cmc1[pas]=>			CMC is in slot 1 and is passive
*	cmc[recover]=>		CMC FPGA failed to load, unknown slot, recover/ST fail
*	cmc1[recover]=>		CMC is is slot 1 and in recover mode other than FPGA
*
*	INPUT:	str, - Location to build the propt string
*				Worst case str len must be "cmc1[recover]=>" + null = 16
*				or CMC_PROMPT_MAX_LEN
*
*	RETURN:	0 
*
*****************************************************************************/
s32 get_cmcprompt(char *str)
{
u16  slot;
u16  rec_id;
char *mode_str = "";
u32  forced = 0;
u32  mode;


/*	Special case, check ID reg. if cannot read then bad FPGA load*/
rec_id = ABS_read_fpga(FPGA_REG_ID);
if(FPGA_ID != rec_id)
	{
	strcpy(str, "recover[self test]=>");
	return(0);
	}

/*Get CMC slot ID*/
slot =  ABS_read_fpga(FPGA_REG_CMC_SLOT_ID) & FPGA_CMC_SLOT_ID;

/*	Check if a self test failure, OR a self test RECOVER*/
forced = is_recover_mode();
mode = ABS_read_fpga(FPGA_REG_MODE0);
if( (mode & (MODE_SELF_TEST_ERROR | MODE_IMAGE_RECOVER_CONDITION)) || forced)
	{
	if(mode & MODE_SELF_TEST_ERROR)
		mode_str = "self test";
	else if(mode & MODE_IMAGE_RECOVER_CONDITION)
		mode_str = "Bad FW images";
	else if(forced)
		mode_str = "forced";

	sprintf(str, "recover%d[%s]=>", slot+1, mode_str);
	return(0);
	}

/*	To get here we do not have an error*/
if(is_local_active())
	mode_str = "act";
else
	mode_str = "pas";


/*	Do not exceed CMC_PROMPT_MAX_LEN*/
sprintf(str, "cmc%d[%s]=>", slot+1, mode_str);
return(0);
}/*end FUNCTION: get_prompt()*/


/****************************************************************************
*
*	FUNCTION: is_cmcf_reset_asserted()
*	DATE:	2-1-07
*
*	Test to see if this CMC is holding the peer/foreign CMC in reset. 
*
*	INPUT:	none
*	RETURN:	0 = Foreign CMC is NOT held in reset
*			1 = Foreign CMC IS being held in reset
*
*****************************************************************************/
s32 is_cmcf_reset_asserted(void)
{
if(ABS_read_fpga(FPGA_REG_CMCF_RST_OUT) & FPGA_CMCF_RST_OUT)
	return(1);
else
	return(0);
}/*end FUNCTION: is_cmcf_reset_asserted()*/


/****************************************************************************
*
*	FUNCTION: set_cmcf_reset()
*	DATE:	2-1-07
*
*	Hold the peer/foreign CMC in reset by writing the 16 bit protect token
*	to the protect register, then on the NEXT write set reset out low.
*
*	INPUT:	none
*	RETURN:	none
*
*****************************************************************************/
s32 set_cmcf_reset(void)
{
ABS_write_fpga(FPGA_REG_CMCF_RST_PROTECT, FPGA_CMCF_RST_PROTECT);
ABS_write_fpga(FPGA_REG_CMCF_RST_OUT, FPGA_CMCF_RST_OUT);
return(0);
}/*end FUNCTION: set_cmcf_reset()*/

/****************************************************************************
*
*	FUNCTION: release_cmcf_reset()
*	DATE:	2-1-07
*
*	Hold the peer/foreign CMC in reset by writing the 16 bit protect token
*	to the protect register, then on the NEXT write set reset out low.
*
*	INPUT:	none
*	RETURN:	none
*
*****************************************************************************/
s32 release_cmcf_reset(void)
{
ABS_write_fpga(FPGA_REG_CMCF_RST_PROTECT, FPGA_CMCF_RST_PROTECT);
ABS_write_fpga(FPGA_REG_CMCF_RST_OUT, 0);
return(0);
}/*end FUNCTION: release_cmcf_reset()*/


/****************************************************************************
*	FUNCTION: swizzle()
*	DATE:	3-7-07
*
*	Call this funcion to convert a 32 bit value between little and big endian.
*
*	INPUT:	data - 32 bit value
*	RETURN:	swapped/swizzled data
*
*****************************************************************************/
s32 swizzle(u32 data)
{
u32 tmp = 0;

tmp |= (data >> 24) & 0x000000ff;
tmp |= (data >> 8 ) & 0x0000ff00;
tmp |= (data << 8 ) & 0x00ff0000;
tmp |= (data << 24) & 0xff000000;
return(tmp);
}/*end FUNCTION: swizzle() */


/****************************************************************************
*	FUNCTION:	set_save_silent_mode()
*	DATE:	3-9-07
*
*	Call to disable output to the serial port, return pre-disabled setting.
*
*	INPUT:	none
*	RETURN:	previous global flag value for silent only
*
*****************************************************************************/
s32 set_save_silent_mode(void)
{
s32 saved = gd->flags & GD_FLG_SILENT;
gd->flags |= GD_FLG_SILENT;
return(saved);
}/*end FUNCTION: set_save_silent_mode() */


/****************************************************************************
*	FUNCTION:	clear_save_silent_mode()
*	DATE:	3-22-07
*
*	Call to enable output to the serial port, return pre-disabled setting.
*
*	INPUT:	none
*	RETURN:	previous global flag value for silent only
*
*****************************************************************************/
s32 clear_save_silent_mode(void)
{
s32 saved = gd->flags & GD_FLG_SILENT;
gd->flags &= ~GD_FLG_SILENT;
return(saved);
}/*end FUNCTION: clear_save_silent_mode() */

/****************************************************************************
*	FUNCTION:	restore_silent_mode()
*	DATE:	3-9-07
*
*	Or restore a previousl saved value.
*
*	INPUT:	Value to place into the silent bit location
*	RETURN:	0
*
*****************************************************************************/
s32 restore_silent_mode(s32 saved_silent)
{
if(saved_silent)
	gd->flags |= GD_FLG_SILENT;
else
	gd->flags &= ~GD_FLG_SILENT;
	
return(0);
}/*end FUNCTION: restore_silent_mode() */


/******************************************************************************
*	FUNCTION:	ip_verify()
*	DATE:	5-8-07
*
*	Verifies whether the input IP string qualifies the dotted quad notation
*
*	INPUT:	IP string
*	RETURN:	0 - ip string is correct
*			non-zero - ip string contains a fault.
*
*****************************************************************************/
s32 ip_verify(char *str)
{
  char buffer[STR_MAX_LEN_IP];
  int  field_num ,len, i = 0, x = 0, dot_count =0, field_count =0;
  char field[3];
  char *fp;
    
  fp = &field[0];
  len = strlen(str);
  if(len > STR_MAX_LEN_IP)
  {
    return(1);
  }
  if((strchr(str,'.') == NULL))
  {
    return(1);
  }

strcpy(buffer, str);
/*
  while(i < len)
  {
    buffer[i] = *str;
    i++;
    str++;
  }
*/
  len++;
  buffer[len-1] = '.';
  i = 0;
  
  while(i < len) 
  {
    if(isdigit(buffer[i]) != 0)
    {
      if(x==0)
      {
        field_count++;
      }
      x++;
      if(x > 3)
      {
        return(1);
      }
      else
      {
        *fp = buffer[i];
        fp++;  
      }
    }
    else if(buffer[i] == '.')
    {
      dot_count++;
      if(dot_count > 4)
      {
        return(1);
      }
      x = 0;
      *fp = '\0';
      fp = &field[0];
      field_num = simple_strtol(fp,0,10);
      if((field_num < 0) || (field_num > 255))
      {
        return(1);
      }
      memset(field,0,3);  
    }
    else
    {
      return(1);;
    }
    i++;
  }
 
  if(dot_count!= 4)
  {
    return(1);
  }

  if(field_count != 4)
  {
    return(1);
  }

return(0);
}/*end FUNCTION: ip_verify()*/


/***************************************************************************
*	Function: get_rtc_tod()
*	DATE:	5-12-07
*
*   Read the 4 byte time-of-day from the RTC.  
*
*	INPUT:	tod - pointer to return the 32 bit Time of Day counter
*	RETURN:	0 - read of I2C returned a value.
*			non-zero - rtc tick was NOT returned.
*
****************************************************************************/
s32 get_rtc_tod(u32 *tod)
{
char rtc_data[4];
u32  counter;
s32	 j;

if(0 != i2c_read(CHIP_ADDR_RTC, 0, 1, rtc_data, 4))
	return(E_I2C_RTC_ID_FAILURE);

for(j = 3, counter = 0; j >= 0; --j)
	{
	counter <<= 8;
	counter |= rtc_data[j];
	}

*tod = counter;
return(0);
}/*end FUNCTION: get_rtc_tod*/


/***************************************************************************
*	Function: mac_verify_colon()
*	DATE:	07-02-07
*
*  Verifies whether the string is a valid MAC string.
*  This function requires that the input string be in the format specified below:
*   XX:XX:XX:XX:XX:XX -  6 colon separated fields, where X is a Hex digit.
*   
*	INPUT:	MAC string
*	RETURN:	0 - MAC string is Valid
*			non-zero - MAC string contains a fault.
*
****************************************************************************/
s32 mac_verify_colon(char *str, char *msg)
{
s32		rc = 0;
s32		error = 0;
s32		i = 0;
char	mac_str[MAX_MAC_STR_LEN];
char	*p1, *p2, *eptr;
s32     len;
u32     tempint = 0;

    //printf("string entered = %s\n", str);
    len = strlen(str);
    //printf("len = %d\n",len);
    if(len != (MAX_MAC_STR_LEN-1))
	{
        sprintf(msg,"Invalid MAC String Length");
        return(E_POST_MAC_STR_TOO_LONG);
	}
    strcpy(mac_str, str);
    strcat(mac_str, ":");
    strcat(mac_str,"\0");
    i = 0;
    p2 = mac_str;
    error = 0;
    while((i<= 5) && (!error))
    {
        if((p1 = strchr(p2, ':')) == 0)
			{
			++error;
            rc = E_POST_MAC_MISSING_COLON;
            break;
			}
        *p1 = '\0';
        tempint = simple_strtol(p2, &eptr, 16);
        if(eptr < p1)
        {
            printf("Invalid Hex number\n");
            ++error;
            rc = E_POST_MAC_ILLEGAL_CHAR;
            break;
        }
        //printf("tempint = %d\n",tempint);
        //*mac_buf_ptr = tempint;
        //mac_buf_ptr++;
        if(tempint < 0 || tempint > 255)
        {
            ++error;
            rc = E_POST_MAC_INVALID_MAC_FIELD;
            break;
        }
        //mac_buf[i] = tempint;
        p2 = ++p1;
        i++;
    }
           
    if(error)
    {   sprintf(msg,"Invalid MAC String");
        return(rc);
    }
        

return(0);
}/*end FUNCTION: mac_verify*/


/***************************************************************************
*	Function: mac_verify()
*	DATE:	07-02-07
*
*  Verifies whether the string is a valid MAC string.
*  This function requires that the input string be in the format specified below:
*   XXXXXXXXXXXX -  string of 12 characters, where X is a Hex digit.
*
*	INPUT:	MAC string
*	RETURN:	0 - MAC string is Valid
*			non-zero - MAC string contains a fault.
*
****************************************************************************/

s32 mac_verify(char *str, s32 *mac_buf_ptr)
{
s32		error;
s32		i;
char	mac_str[MAX_MAC_STR_LEN];
char	*p1, *p2, *eptr;
s32     len;
u32     tempint = 0;
char    mac_buf[3];

    p1 = mac_buf;
    //printf("string entered = %s\n", str);
    len = strlen(str);
    //printf("len = %d\n",len);
    
    if(len != 12)
	{
        printf("Invalid String Length\n");
        return(E_FRU_INVALID_MAC_STR_LEN);
	}
    strcpy(mac_str, str);
    //strcat(mac_str, ":");
    strcat(mac_str,"\0");
    i = 0;
    p2 = mac_str;
    error = 0;
    while((i<= 5) && (!error))
    {
        strncpy(p1, p2, 2);
        strcat(mac_buf, "\0");
        tempint = simple_strtol(p1, &eptr, 16);
        if(*eptr != '\0')
        {
            printf("Invalid Hex number\n");
            ++error;
            break;
        }
        //printf("tempint = %d\n",tempint);
       /* if(tempint < 0 || tempint > 255)
        {
            ++error;
            break;
        }*/
        *mac_buf_ptr = tempint;
        mac_buf_ptr++;
        //mac_buf[i] = tempint;
        p2 = p2+2;
        i++;
    }
           
    if(error)
        return(E_FRU_INVALID_MAC_STR);

return(0);
}/*end FUNCTION: mac_verify*/


/****************************************************************************
*	FUNCTION: get_cmc_slot()
*	DATE: 9-10-08
****************************************************************************/
s32 get_cmc_slot(u32 *slot, char *msg)
{

if(FPGA_ID != ABS_read_fpga(FPGA_REG_ID))
	{
	sprintf(msg, "cannot read slot ID via FPGA");
	return(E_CANNOT_READ_FPGA_SLOT_ID);
	}

/*Get CMC slot ID*/
*slot = ABS_read_fpga(FPGA_REG_CMC_SLOT_ID) & FPGA_CMC_SLOT_ID;
sprintf(msg, "This CMC is slot %d", slot+1);
return(0);
}/*end FUNCTION: get_cmc_slot()*/



/****************************************************************************
*	FUNCTION: is_peer_cmc_present()
*	DATE: 9-10-08
****************************************************************************/
s32 is_peer_cmc_present(u32 *pres, char *msg)
{
*pres = 0;		/*default to not present*/
sprintf(msg, "Peer CMC is NOT present");

if(FPGA_ID != ABS_read_fpga(FPGA_REG_ID))
	{
	sprintf(msg, "Cannot read peer present via FPGA");
	return(E_CANNOT_READ_FPGA_PEER_PRES);
	}
	
	
if(!(ABS_read_fpga(FPGA_REG_CMCF_CMCL_PRES) & FPGA_CMCF_CMCL_PRES))
	{
	*pres = 1;
	sprintf(msg, "Peer CMC is present");
	}

return(0); 
}/*FUNCTION: is_peer_cmc_present()*/




/*******************************************************************************
**
**  FUNCTION:       format_print 
**  ___________________________________________________________________________
**
**  DESCRIPTION:    This function takes in a binary buffer and formats it in 
**                  standard looking addr, data, ascii format:
**    000: 04 00 01 05 01 00 00 01 00 00 00 00 00 3C 1D 98  .............<..
**    010: 00 27 BD 10 00 3C 1C 98 00 27 9C 10 00 00 00 F0  .'...<...'......
**    020: 25 27 9C 40 00 27 9C 40 00 00 00 68 25 3C 01 00  %'.@.'.@...h%<..
**    030: 00 24 21 00 00 02 01 60 2A 15 80 00 06 3C 01 00  .$!....`*....<..
**    040: 00 24 21 00 00 00 30 60 2A 15 80 00 02 00 00 00  .$!...0`*.......
**
**  INPUTS:         data         -   pointer to the byte string
**                  maxlength    -   the number of bytes to print
**
**
**  RETURNS:        None
**
*******************************************************************************/
void format_print(unsigned char *data,long maxlength)
{
int  i;
char ascii[22];    
long length = 0;
int padd;

    length = maxlength;
    ascii[16] = 0;
    for( i=0; i<length; i++ ) {
        if( (i & 0x0F) == 0 ) {
            printf("\n%03X:", i);
        }
        printf(" %02X", data[i]);
        ascii[i & 0x0F] = isgraph(data[i]) ? data[i] : '.';
        if( (i & 0x0F) == 0x0F ) {
            printf("  %s", ascii);
        }
    }
    padd = i & 0x0F;
    if(padd != 0x00){
        ascii[padd] = 0;    //terminate the string
        for(i = 0x0f;i >= padd; i--)
            printf("   ");
        printf("  %s", ascii);
    }           

    printf("\n");
}

#endif /*CONFIG_CMC_10G*/



