#include "xiic_l.h"


u32     xiic_init(u32 dev)
{

	/* BLR - Disable all I2C Interrupts */
    *XIIC_IIER_REG_OFFSET(dev) = 0;

	/* BLR - Clear ISRs */
	*XIIC_IISR_REG_OFFSET(dev) = (0xFFFF) & *XIIC_IISR_REG_OFFSET(dev);

	/* BLR - Enable IIC Device by setting CR bit to 1*/
	*XIIC_CR_REG_OFFSET(dev) = (XIIC_CR_START_CFG_MASK | XIIC_CR_ENABLE_DEVICE_MASK) | *XIIC_CR_REG_OFFSET(dev);

    /* BLR - Set RX Occupancy register to 0, triggering interrupt on any byte*/
	*XIIC_RFD_REG_OFFSET(dev) = 0;

	/* BLR - Set up appropriate interrupts, taken from xilinx IIC drv.
		 Again, I think this is really only for slaves or multimaster busses */
	*XIIC_IISR_REG_OFFSET(dev) = (XIIC_INTR_AAS_MASK | XIIC_INTR_ARB_LOST_MASK) &
                                     *XIIC_IISR_REG_OFFSET(dev);

	*XIIC_IIER_REG_OFFSET(dev) = (XIIC_INTR_AAS_MASK | XIIC_INTR_ARB_LOST_MASK) |
                                     *XIIC_IIER_REG_OFFSET(dev);

	/* BLR - Xilinx code enables global IPIF interrupt, but ours is by default*/

	return 1;
}

u32     xiic_iic_send(u32 dev, u8 address, u8 *data, u8 len, u32 rs)
{
	u16 val;
	u16 tAddr;
	s32 avail, numtosend;
	s32 i;

	if(len == 0) return 0; /* BLR - Do not allow 0 length writes */
	/* BLR - Do not allow access to an uninitialized bus */
    if((*XIIC_CR_REG_OFFSET(dev) & XIIC_CR_ENABLE_DEVICE_MASK)==0) return 0;

	/* Set TX and repeated start mask */
	val = *XIIC_CR_REG_OFFSET(dev);
	if(val & XIIC_CR_MSMS_MASK) {
		val &= ~XIIC_CR_NO_ACK_MASK;
		val |= (XIIC_CR_DIR_IS_TX_MASK | XIIC_CR_REPEATED_START_MASK);
		*XIIC_CR_REG_OFFSET(dev) = val;
	}

	/* Load device address */
	tAddr = address << 1;
	tAddr &= 0xFE;
	*XIIC_DTR_REG_OFFSET(dev) = tAddr;


	/* Fill TX FIFO w/Data */
	if(len > 1) {
		avail = IIC_TX_FIFO_DEPTH - (*XIIC_TFO_REG_OFFSET(dev) + 1);
		if(len > avail) { /* If more data than FIFO Holds*/
			numtosend = avail; /* Fill the fifo */
		} else {
			numtosend = len-1; /* Otherwise leave one byte */
		}
		for(i=0;i<numtosend;i++) { /* Fill the FIFO */
			*XIIC_DTR_REG_OFFSET(dev)=*data++;
			len--;
		}
	}

	if(len > 1) { /* data left to send (but not the last byte) */
		*XIIC_IISR_REG_OFFSET(dev) &= XIIC_INTR_TX_HALF_MASK; /* Enable half empty Intr
*/
		*XIIC_IIER_REG_OFFSET(dev) |= XIIC_INTR_TX_HALF_MASK;
	}

	/* Also enable interrupts for errors and tx fifo (i.e. done) */
	*XIIC_IISR_REG_OFFSET(dev) &= (XIIC_INTR_TX_ERROR_MASK |
				       XIIC_INTR_TX_EMPTY_MASK);
        *XIIC_IIER_REG_OFFSET(dev) |= (XIIC_INTR_TX_ERROR_MASK |
				       XIIC_INTR_TX_EMPTY_MASK);

	val = *XIIC_CR_REG_OFFSET(dev); /* Set control reg for transfer */
	if( (val & XIIC_CR_MSMS_MASK) == 0) {
		val &= ~XIIC_CR_NO_ACK_MASK;
		val |= XIIC_CR_MSMS_MASK | XIIC_CR_DIR_IS_TX_MASK;
		*XIIC_CR_REG_OFFSET(dev) = val;
	}

	/* BLR - At this point data is ready to ship, several things may happen
		 1) Half empty interrupt may trigger, for large xfers. Fill it.
		 2) TX Empty interrupt may trigger - step stop flag, fill last byte,
		    clear all TX interrupts, enable Bus Busy interrupt.
		 3) NACK - Invalid address, etc. Abort transaction
		 4) Bus Busy interrupt - Transaction Complete, clean up and bail.
	*/

	while(1) {

		val = *XIIC_IISR_REG_OFFSET(dev);
		while( (val & (XIIC_INTR_BNB_MASK  |
		           XIIC_INTR_TX_ERROR_MASK |
			       XIIC_INTR_TX_EMPTY_MASK |
			       XIIC_INTR_TX_HALF_MASK)) == 0 ) {
			val = *XIIC_IISR_REG_OFFSET(dev);
		}

        /* BLR - 04/30/07 - Changed this to only look for TX error, may miss Dell
                abort conditions. Not important for test code.*/
		if( (val & XIIC_INTR_TX_ERROR_MASK) != 0) {
			/* BLR - Bus error. Abort */
			/* BLR - Flush TX FIFO */
			*XIIC_CR_REG_OFFSET(dev) |= XIIC_CR_TX_FIFO_RESET_MASK;
			/* BLR - Turn off and clear all interrupts */
			*XIIC_IIER_REG_OFFSET(dev) &= ~(    XIIC_INTR_BNB_MASK |
                                                XIIC_INTR_RX_FULL_MASK |
                                                XIIC_INTR_TX_ERROR_MASK |
                                                        XIIC_INTR_TX_EMPTY_MASK |
                                                        XIIC_INTR_TX_HALF_MASK);
			*XIIC_IISR_REG_OFFSET(dev) &= (            XIIC_INTR_BNB_MASK |
                                                        XIIC_INTR_RX_FULL_MASK |
                                                        XIIC_INTR_TX_ERROR_MASK |
                                                        XIIC_INTR_TX_EMPTY_MASK |
                                                        XIIC_INTR_TX_HALF_MASK);
			return 0; /* Bail out */
		}

		/* BLR - If we are here, all is going well, play the FIFO game */
		if(len > 1) {
        		/* Fill TX FIFO w/Data */
        		avail = IIC_TX_FIFO_DEPTH - (*XIIC_TFO_REG_OFFSET(dev) + 1);
        		if(len > avail) { /* If more data than FIFO Holds*/
                		numtosend = avail; /* Fill the fifo */
        		} else {
                		numtosend = len-1; /* Otherwise leave one byte */
        		}
        		for(i=0;i<numtosend;i++) { /* Fill the FIFO */
                		*XIIC_DTR_REG_OFFSET(dev)=*data++;
                		len--;
        		}
			*XIIC_IISR_REG_OFFSET(dev) &= (XIIC_INTR_TX_HALF_MASK |
						       XIIC_INTR_TX_EMPTY_MASK);
		} else if((len == 1) && (val & XIIC_INTR_TX_EMPTY_MASK)) {
		    if(rs) {
		        *XIIC_DTR_REG_OFFSET(dev) = *data++;
		        len--;
		    } else {
                *XIIC_CR_REG_OFFSET(dev) &= ~XIIC_CR_MSMS_MASK;
                *XIIC_DTR_REG_OFFSET(dev) = *data++;
                len--;

                *XIIC_IISR_REG_OFFSET(dev) &= (XIIC_INTR_BNB_MASK);
                *XIIC_IIER_REG_OFFSET(dev) |= (XIIC_INTR_BNB_MASK);

                *XIIC_IISR_REG_OFFSET(dev) &= (XIIC_INTR_TX_ERROR_MASK |
                                           XIIC_INTR_TX_HALF_MASK |
                                           XIIC_INTR_TX_EMPTY_MASK);
                *XIIC_IIER_REG_OFFSET(dev) &= ~(XIIC_INTR_TX_ERROR_MASK |
                                                        XIIC_INTR_TX_EMPTY_MASK |
                                                        XIIC_INTR_TX_HALF_MASK);
		    }
		} else if((len == 0) && (val & XIIC_INTR_BNB_MASK)) {
		    /*By arriving here, the stop condition has been generated, and we can exit*/
			*XIIC_IISR_REG_OFFSET(dev) &= (XIIC_INTR_BNB_MASK);
            *XIIC_IIER_REG_OFFSET(dev) &= ~(XIIC_INTR_BNB_MASK);
		    return 1;
		} else if ((rs==1) && (len == 0) && (val & XIIC_INTR_TX_EMPTY_MASK)) {
		    /* This means we've sent the last byte, and we want a repeated start:
                - Set repeated start bit
                - Clear all transmit interrupts
                - Return to caller, so it can prepare the next phase of the transaction
            */
            *XIIC_CR_REG_OFFSET(dev) |= XIIC_CR_REPEATED_START_MASK;
            *XIIC_IISR_REG_OFFSET(dev) &= (XIIC_INTR_TX_ERROR_MASK |
                                           XIIC_INTR_TX_HALF_MASK |
                                           XIIC_INTR_TX_EMPTY_MASK);
            *XIIC_IIER_REG_OFFSET(dev) &= ~(XIIC_INTR_TX_ERROR_MASK |
                                           XIIC_INTR_TX_EMPTY_MASK |
                                           XIIC_INTR_TX_HALF_MASK);
            return 1;
		}
	}
}

u32     xiic_iic_recv(u32 dev, u8 address, u8 *data, u8 len, u32 rs)
{
	u16 val;
	u16 temp, togo;
	u16 tAddr;
	u16 i;

	if(len == 0) return 0; //Do not allow 0 byte reads
	if((*XIIC_CR_REG_OFFSET(dev) & XIIC_CR_ENABLE_DEVICE_MASK)==0) return 0;

	/* BLR - Set up RX_FULL interrupt */
    *XIIC_IISR_REG_OFFSET(dev) &= XIIC_INTR_RX_FULL_MASK;
    *XIIC_IIER_REG_OFFSET(dev) |= (XIIC_INTR_RX_FULL_MASK);

	/* BLR - Generate repeated start if needed, don't think this applies to us but... */
	val = *XIIC_CR_REG_OFFSET(dev);

    /* BLR - If MSMS bit still set, this means we're doing a repeated start */
	if(val & XIIC_CR_MSMS_MASK) {
	    val |= (XIIC_CR_REPEATED_START_MASK); /* Set repeated start flag */
		val &= ~(XIIC_CR_NO_ACK_MASK | XIIC_CR_DIR_IS_TX_MASK); /* Clear out previous xmit bits */
		if(len == 1)
			val |= XIIC_CR_NO_ACK_MASK;
		*XIIC_CR_REG_OFFSET(dev) = val; /* Write out*/
	}

 	/* BLR - Set the RX occupancy register for how many bytes to receive */
	if(len == 1)
		temp = 0; /* BLR - RX Occupancy is 1 below actual # to read */
	else {
		if (len <= IIC_RX_FIFO_DEPTH) /* BLR - 04/30/07 - Bug Fix, changed
                                             to less than equal, b/c reads = FIFO_DEPTH
                                             would miss a No Ack and hang the bus*/
			temp = len -2; /* One below actual # to read, one more to ensure No Ack*/
		else
			temp = IIC_RX_FIFO_DEPTH - 1; /* BLR - Fill RX FIFO */
	}
	*XIIC_RFD_REG_OFFSET(dev) = temp;

	/* BLR - Form bus address by shifting dev address left 1, and appending direction*/
	tAddr = (address<<1);
	tAddr = (tAddr & 0xFE) | 0x1; /* BLR - 0x1 = Read */
	*XIIC_DTR_REG_OFFSET(dev) = tAddr; /* BLR - Put Address out on bus */

	/* BLR - Enable to detect NACK */
    *XIIC_IISR_REG_OFFSET(dev) &= XIIC_INTR_TX_ERROR_MASK;
    *XIIC_IIER_REG_OFFSET(dev) |= XIIC_INTR_TX_ERROR_MASK;

    /* BLR - If MSMS not set, we're starting a new transaction, enable it*/
	val = *XIIC_CR_REG_OFFSET(dev);
	if((val & XIIC_CR_MSMS_MASK) == 0) {
		val |= XIIC_CR_MSMS_MASK;
		val &= ~(XIIC_CR_NO_ACK_MASK | XIIC_CR_DIR_IS_TX_MASK);
                if(len == 1)
                        val |= XIIC_CR_NO_ACK_MASK;
                *XIIC_CR_REG_OFFSET(dev) = val;
	}

	/* BLR - At this point we've sent out an address, one of three things can happen:
                 1) We receive data and get a fifo full message
                    a) If more than 1 byte left, clear FIFO and repeat
                    b) If 1 byte left, set to generate No ACK, repeat
                    c) No bytes left, clean up HW state, and return true.
                 2) We get a TXER indicating a nack - Abort
                 3) Some form of bus error such as the STRT_ERRx statuses - Abort

                 For our purposes, unless we get an RX FULL, we will return 0.
        */

	while(1) {
		val = *XIIC_IISR_REG_OFFSET(dev);
		/* BLR - Wait for bus to not be busy */
		while( (val & (XIIC_INTR_RX_FULL_MASK|XIIC_INTR_TX_ERROR_MASK)) == 0) {
			val = *XIIC_IISR_REG_OFFSET(dev);
		}

		if( ((val & XIIC_INTR_RX_FULL_MASK) == 0) &&
                     (*XIIC_SR_REG_OFFSET(dev) & XIIC_SR_RX_FIFO_EMPTY_MASK)) {
                       /* BLR - Bus error. Abort */
                        /* BLR - Flush TX FIFO */
			val = *XIIC_CR_REG_OFFSET(dev);
			*XIIC_CR_REG_OFFSET(dev) |= XIIC_CR_TX_FIFO_RESET_MASK;
			*XIIC_CR_REG_OFFSET(dev) = val;

                        /* BLR - Turn off and clear all interrupts */
                        *XIIC_IIER_REG_OFFSET(dev) &= ~(XIIC_INTR_RX_FULL_MASK |
                                                        XIIC_INTR_TX_ERROR_MASK |
                                                        XIIC_INTR_TX_EMPTY_MASK |
                                                        XIIC_INTR_TX_HALF_MASK);
                        *XIIC_IISR_REG_OFFSET(dev) &= ~(XIIC_INTR_RX_FULL_MASK |
                                                        XIIC_INTR_TX_ERROR_MASK |
                                                        XIIC_INTR_TX_EMPTY_MASK |
                                                        XIIC_INTR_TX_HALF_MASK);

			*XIIC_CR_REG_OFFSET(dev) &= ~XIIC_CR_MSMS_MASK; /* Generate stop */
			*XIIC_RFD_REG_OFFSET(dev) = 0;

                        return 0; /* Bail out */
		}

		/* BLR - If we're here, theoretically we have data to receive */

		temp = *XIIC_RFO_REG_OFFSET(dev) + 1; /* BLR - Get # of bytes in fifo*/
		val = *XIIC_CR_REG_OFFSET(dev);


		if((len - temp) == 1) {
			*XIIC_IIER_REG_OFFSET(dev) &= ~XIIC_INTR_TX_ERROR_MASK;
			*XIIC_IISR_REG_OFFSET(dev) &=  XIIC_INTR_TX_ERROR_MASK;

			*XIIC_CR_REG_OFFSET(dev) = val | XIIC_CR_NO_ACK_MASK;
			*data++ = *XIIC_DRR_REG_OFFSET(dev);
			len --;
		} else if ((len - temp) == 0) {
            if(rs)
                val |= XIIC_CR_REPEATED_START_MASK;
            else
                val &= ~XIIC_CR_MSMS_MASK;

            *XIIC_CR_REG_OFFSET(dev) = val;

             for(i=0;i<temp;i++){
                 *data++ = *XIIC_DRR_REG_OFFSET(dev);
                 len--;
            }
            *XIIC_RFD_REG_OFFSET(dev)=0;

            *XIIC_IIER_REG_OFFSET(dev) &= ~(XIIC_INTR_RX_FULL_MASK |
			 				XIIC_INTR_TX_ERROR_MASK);
            *XIIC_CR_REG_OFFSET(dev) = val & ~XIIC_CR_NO_ACK_MASK;
            return 1; /* BLR - Message Successfully Received! */
		} else {
			togo = len - temp - 1;
			if(togo > IIC_RX_FIFO_DEPTH)
				togo = IIC_RX_FIFO_DEPTH;

			for(i=0;i<togo;i++) {
				*data++=*XIIC_DRR_REG_OFFSET(dev);
				len--;
			}
		}
	}
}

